/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.bert;

/**
* A node editor for PAM. 
* Started life as a node editor used by Bert.
* 
* @author Nazario Irizarry, Jr.
*/
import java.util.Vector;
import java.util.Enumeration;
import java.awt.MenuItem;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;

import org.mitre.pam.util.draw.Block;
import org.mitre.pam.util.draw.BlockGroup;
import org.mitre.pam.util.pdifHashtables.PDIFNode;
import org.mitre.pam.util.pdifHashtables.IpamPDIFNode;
import org.mitre.pam.util.pdifHashtables.NodeEditor;
import org.mitre.pam.util.pdifHashtables.ParentServices;

final class BertGraphicBlock extends Block implements WindowListener {
	private BertPDIFNode pNode = null;
	private NodeEditor nodeEditor;
	private boolean onScreen;
	private MenuItem menuItem;
	private static ParentServices parentServices;
	static BlockGroup blockGroup = new BlockGroup();
/**
* BertGraphicBlock is a graphical object that has an associated BertPDIFNode 
*/
	public BertGraphicBlock(BertPDIFNode pNode) {
		setBlockGroup(blockGroup); // for now all belong to same group for all instances of bert
		this.pNode = pNode;
		pNode.setGraphicNode(this);
		setKind(pNode.getType());
		setEnabled(pNode.isEnabled());
		if (pNode.hasPos()) {
			setOnce(pNode.xPos(), pNode.yPos(), pNode.getName());
		}
		else {
			setOnce(5, 5, pNode.getName());
		}
		menuItem = new MenuItem(pNode.getName());
		menuItem.setEnabled(false);
	}

	public static void setParentServices(ParentServices services) {
		parentServices = services;
	}
	
	public MenuItem getMenuItem() {
		return menuItem;
	}

	/**
	 * Checks if the node has been disabled
	 */
	 
	 public boolean disabled() {
	 	return !pNode.isEnabled();
	 }
	
	public void updatePresentation() {
		setEnabled(pNode.isEnabled());
		setName(pNode.getName());
		menuItem.setLabel(pNode.getName());
		setKind(pNode.getType());
		setHasInputs(pNode.hasInputs());
		setHasOutputs(pNode.hasOutputs());
/*		System.out.print("update graghic node on "+this);
		if (parent == null)
			System.out.println(" parent is null");
		else
			System.out.println();
*/
	}
	
	public BertPDIFNode getData() {
		return pNode;
	}

	public void setData(BertPDIFNode pNode)	{
		this.pNode = pNode;
	}	
	/**
	 * This method was created by a SmartGuide.
	 */
	public String toString ( ) {
		return "BertGraphicBlock " + getName() ;
	}


/**
 * if there is an editor open already, don't open
 * another one.
 */
	public synchronized void editProperties() {
		if (nodeEditor == null) {
			nodeEditor = new NodeEditor(getData(), parentServices);
			nodeEditor.addWindowListener(this);
		} 
		else {
			nodeEditor.show();
			nodeEditor.toFront();
		}
	}
    /**
     * Invoked the first time a window is made visible.
     */
    public void windowOpened(WindowEvent e){}

    /**
     * Invoked when the user attempts to close the window
     * from the window's system menu.  If the program does not 
     * explicitly hide or dispose the window while processing 
     * this event, the window close operation will be cancelled.
     */
    public void windowClosing(WindowEvent e){}

    /**
     * Invoked when a window has been closed as the result
     * of calling dispose on the window.
     */
    public void windowClosed(WindowEvent e){
		nodeEditor.removeWindowListener(this);
		nodeEditor = null;
	}

    /**
     * Invoked when a window is changed from a normal to a
     * minimized state. For many platforms, a minimized window 
     * is displayed as the icon specified in the window's 
     * iconImage property.
     * @see Frame#setIconImage
     */
    public void windowIconified(WindowEvent e){}

    /**
     * Invoked when a window is changed from a minimized
     * to a normal state.
     */
    public void windowDeiconified(WindowEvent e){}

    /**
     * Invoked when the window is set to be the user's
     * active window, which means the window (or one of its
     * subcomponents) will receive keyboard events.
     */
    public void windowActivated(WindowEvent e){}

    /**
     * Invoked when a window is no longer the user's active
     * window, which means that keyboard events will no longer
     * be delivered to the window or its subcomponents.
     */
    public void windowDeactivated(WindowEvent e){}

/**
* identifies whether the node belongs to the owner under test
*/
	public boolean isOwnedBy(String testOwner) {
		return pNode.isOwnedBy(testOwner);
	}
/**
* identifies whether the node belongs to the owner under test
*/
	public boolean isOwned() {
		return pNode.isOwned();
	}
/**
* is this node on screen?
*/
	public boolean isOnScreen() {
		return onScreen;
	}
/**
* is this node on screen?
*/
	public void setOnScreen(boolean onScreen) {
		this.onScreen = onScreen;
		if(!onScreen) {
			pNode.withdrawFromLayout();
		}
		menuItem.setEnabled(!onScreen);
	}
/**
* Set position on screen
*/
	public void setPos(int x, int y){
		super.setPos(x,y);
		pNode.setPos(x,y);
	}
/**
* Set position on screen
*/
	public void setPosFromPDIF(){
		super.setPos(pNode.xPos(), pNode.yPos());
	}

/* only for debugging purposes
	public void setEnabled(boolean enabled) {
		System.out.println(this+" "+ (enabled?"enabled":"disabled"));
		if (parent == null)
			System.out.println("\tparent is null");
			
		super.setEnabled(enabled);
	}
*/

}