/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.bert;
import java.awt.MenuItem;
import java.net.URLEncoder;
import java.util.Hashtable;

import org.mitre.pam.util.PAMDate;
import org.mitre.pam.store.SourceSinkInfo;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.interfaces.Sink;
import org.mitre.pam.interfaces.ConnectedAgent;
import org.mitre.pam.interfaces.Schedulable;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.interfaces.Agent;

import org.mitre.pam.util.pdifHashtables.IpamPDIFNode;
import org.mitre.pam.util.MetaData;

/**
 *
 * @version 1, 28 May 1998
 * @version 2, 28 January 1999
 * @version 2.1, 20 May 1999
 * @author Nazario Irizarry
 */
class BertPDIFNode extends IpamPDIFNode {
	BertGraphicBlock graphicNode;
	int autoPlaced;
	String layoutId = "bertUnassigned";
	String xTag = "bertUnassigned_x";
	String yTag = "bertUnassigned_y";

	BertPDIFNode(String type, String id, Reporter reporter) {
		super(type, id, reporter);
	}
/**
* filter out bad chars
*/
	private String massageName(String name) {
		if (name == null)
			return null;
		
		StringBuffer betterName = new StringBuffer(80);
		int len = name.length();
		for (int i = 0; i<len; i++) {
			char c = name.charAt(i);
			if (c == ' ')
				c = '_';
			else if (c == '%')
				c = '_';
			betterName.append(c);
		}
		return betterName.toString();
	}

/**
* The layout id allows a node to have multiple x and y positions
* in different layouts
*/
	static String lastLayoutId;
	static String lastMassagedId;
	static String lastXTag;
	static String lastYTag;
	void setLayoutId(String layoutId) {
		if (lastLayoutId != layoutId) {
			lastLayoutId = layoutId;
			lastMassagedId = massageName(layoutId);
			lastXTag = "bert_"+lastMassagedId + "_x";
			lastYTag = "bert_"+lastMassagedId + "_y";
		}
		this.layoutId = lastMassagedId;
		this.xTag = lastXTag;
		this.yTag = lastYTag;
	}
/**
* take out of the current layout
*/
	void withdrawFromLayout() {
		removePropertyAndRemoteCopies(xTag);
		removePropertyAndRemoteCopies(yTag);
	}
/**
* set the x and y position for current layout
*/
	void setPos(int x, int y) {
		setProperty(xTag, Integer.toString(x));
		setProperty(yTag, Integer.toString(y));
	}
/**
* the x position
*/
	int xPos() {
		return getIntegerProperty(xTag, 5);
	}
/**
* the x position
*/
	int yPos() {
		return getIntegerProperty(yTag, 5);
	}
/**
* does it have position in our layout?
*/
	public boolean hasPos() {
		return getStringProperty(xTag, "") != "" ;
	}
/**
* this is how we make something disappear from this layout,
* just remove its postion since it has to have position in
* order to be shown on the layout
*/
	public void removePos() {
		removePropertyAndRemoteCopies(xTag);
		removePropertyAndRemoteCopies(yTag);
	}
	


	
	public void setGraphicNode(BertGraphicBlock graphicNode) {
		this.graphicNode = graphicNode;
	}
	
	public BertGraphicBlock getGraphicNode() {
		return graphicNode;
	}

	public void setName(String name) {
		super.setName(name);
		if (graphicNode == null) {
			throw new RuntimeException("got graphicNode==null");
		}
		graphicNode.updatePresentation(); // update the label in the menu to match new name
	}

/**
* does not copy name or type or graphicNode
*/
	public void copyAttributesNotConnections(BertPDIFNode otherNode) {
		super.copyAttributesNotConnections(otherNode);
		layoutId = otherNode.layoutId;
		xTag = otherNode.xTag;
		yTag = otherNode.yTag;
		metaData = otherNode.metaData; // assume that metadata does not change
	}
	public void touch() {
		super.touch();
		getGraphicNode().updatePresentation();
	}

}