/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.bert;
import java.awt.Frame;
import java.awt.FileDialog;
import java.net.URL;
import java.net.URLEncoder;
import java.io.*;

import org.mitre.pam.monitor.MultipleTextBoxDlg;
import org.mitre.pam.util.Codecs;
import org.mitre.pam.interfaces.Reporter;

/**
* Write state to a file. 
* This probably would not take that long but this
* way it is consistent with the threads that
* read and write to a server.
*/
class PostStateToFile extends Thread {
	LoadCallbacks callbacks;
	ConnectionInfo connectionInfo;
	Frame parentFrame;
	Reporter reporter;

	PostStateToFile(LoadCallbacks callbacks, ConnectionInfo connectionInfo, Frame parentFrame, Reporter reporter) {
		this.callbacks = callbacks;
		this.connectionInfo = connectionInfo;
		this.parentFrame = parentFrame;
		this.reporter = reporter;
	}

	public void run() {
		try {
			callbacks.setUserControls(false);
			FileDialog askStateFileName = new FileDialog(parentFrame, "Save State As", FileDialog.SAVE);
			if (connectionInfo.stateFileName != null) {
				askStateFileName.setFile(connectionInfo.stateFileName);
				askStateFileName.setDirectory(connectionInfo.stateDirectoryName);
			}	
			askStateFileName.setVisible(true);
			String tmpStateFileName;
			if (askStateFileName.getFile() != null) {
				tmpStateFileName = askStateFileName.getDirectory() + askStateFileName.getFile();
				try {
					callbacks.getStateStructure().writeState(new PrintWriter(new FileOutputStream(tmpStateFileName)), false, true);
	//				PAMstate.writeState(new PrintWriter(new FileOutputStream(tmpStateFileName+"ft")), false, true);
	//				PAMstate.writeState(new PrintWriter(new FileOutputStream(tmpStateFileName+"tf")), true, false);
				} catch (IOException ioEx) {
					callbacks.setStatus ("Unable to write output state.: " + ioEx);
					return;
				}
				connectionInfo.stateFileName = askStateFileName.getFile();
				connectionInfo.stateDirectoryName = askStateFileName.getDirectory();
			}
			callbacks.setStatus("Save done.");
			connectionInfo.lastLoadSaveTime = System.currentTimeMillis();
		}
		finally {
			callbacks.setUserControls(true);
		}
	}
}
