// Copyright (C) 1996 by Jef Poskanzer <jef@acme.com>.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit the ACME Labs Java page for up-to-date versions of this and other
// fine Java utilities: http://www.acme.com/java/
package Acme.Serve;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
* @author Jef Poskanzer, Copyright (C) 1996 by Jef Poskanzer <jef@acme.com>.  All rights reserved.
* @author Nazario Irizarry, <naz@mitre.org> many modifications to original code, 1 Jan 1998
*/

class ServeConnection extends Thread implements  HttpServletRequest, HttpServletResponse
    {

    private Socket socket;
    private Serve serve;
    private ServerSocket serverSocket;

    private ServletInputStream in;
    private ServletOutputStream out;
    private boolean continueRunning;

	

    // Methods from Runnable.

    private String reqMethod = null;
    private String reqUriPath = null;
    private String reqProtocol = null;
    private boolean oneOne;		// HTTP/1.1 or better
    private boolean reqMime;
    String reqQuery = null;
    private Vector reqHeaderNames = new Vector();
    private Vector reqHeaderValues = new Vector();
    Hashtable queryHash = null;
    private boolean headersWritten = false;
//------
    private int resCode = -1;
    private String resMessage = null;
    
    private Properties resHeaderNV = new Properties();

	public void initVariables() {
	    reqMethod = null;
	    reqUriPath = null;
	    reqProtocol = null;
	    oneOne = false;		// HTTP/1.1 or better
	    reqMime=false;
	    reqQuery = null;
	    reqHeaderNames = new Vector();
	    reqHeaderValues = new Vector();
	    queryHash = null;
	    headersWritten = false;
	    
	    resCode=-1;
	    resMessage= null;
	    resHeaderNV = new Properties();
	}
/**
* Constructor that creates a thread and starts listening for connection.
*/
    public ServeConnection( ServerSocket serverSocket, Serve serve ) {
    	super("Http Listener");
		this.serve = serve;
		this.serverSocket = serverSocket;
		continueRunning = true;
		serve.newServerSocketUser();
//		setPriority(Thread.NORM_PRIORITY - 1);
		start();
	}
/**
* Stop running.
* Close the server socket if requested
*/
	public void stopRunning() {
		continueRunning = false;
	}
/**
* Assumes that the serverSocket had a timeout in it.  When inputs timeout
* we halt if a call has been made to stopRunning()
*/
    public void run()
	{
		while (continueRunning) {
			try {
				initVariables();
				socket = serverSocket.accept();

				try {
					socket.setSoLinger(true, 10000); // wait up to 10 seconds on close before doing a forced close
					in = new ServeInputStream( socket.getInputStream() );
					out = new ServeOutputStream( new BufferedOutputStream(socket.getOutputStream(), 400), this );
				}
				catch ( IOException e ){
					problem( "Getting streams: " + e.getMessage(), e, SC_BAD_REQUEST );
				}
			
				try {
					parseRequest();
				}
				catch (Exception e) {
					problem( "Somewhere: " + e.getMessage(), e, SC_INTERNAL_SERVER_ERROR );
				}
			
			/*
			* Run down any remaining input.
			* Keeps linux happy!
			*/
				try {
				    while (in.available() > 0) {
				    	in.skip(in.available());
				    	}
				    }
				catch ( IOException e ) {  }

				try
				    {
				    out.close();
				    }
				catch ( IOException e ) { }
			
				try
				    {
				    in.close();
				    }
				catch ( IOException e ) { }
			

				try
				    {
				    socket.close();
				    }
				catch ( IOException e ) {  }

			}
			catch (InterruptedIOException exint) {
//				System.out.println("ServeConnection timeout");
			}
			catch (IOException exint) {
				serve.reportException(exint);
			}
		}
		serve.expiredServerSocketUser();
	}


	private void slowOut(String s) {
//		System.out.println(s);
//		try { Thread.sleep(1000); } catch (InterruptedException e) {}
	}
	
    private void parseRequest() {
		byte[] lineBytes = new byte[4096];
		int len;
		String line;
	
		try
		    {
		    // Read the first line of the request.
		    len = in.readLine( lineBytes, 0, lineBytes.length );
		    if ( len == -1 || len == 0 ) {
				problem( "Empty request", SC_BAD_REQUEST );
				return;
			}
			
		    line = new String( lineBytes, 0, 0, len );

		    String[] tokens = Acme.Utils.splitStr( line );
		    switch ( tokens.length )
				{
				case 2:
					
					// Two tokens means the protocol is HTTP/0.9.
					reqProtocol = "HTTP/0.9";
					oneOne = false;
					reqMime = false;
					break;
				
				case 3:
					
					reqProtocol = tokens[2];
					oneOne = ! reqProtocol.toUpperCase().equals( "HTTP/1.0" );
					reqMime = true;
					// Read the rest of the lines.
					while ( true )
					    {
//					    slowOut("parseRequest 5");
					    
					    len = in.readLine( lineBytes, 0, lineBytes.length );
					    if ( len == -1 || len == 0 )
						break;
					    line = new String( lineBytes, 0, 0, len );
					    int colonBlank = line.indexOf( ": " );
					    if ( colonBlank != -1 )
						{
						String name = line.substring( 0, colonBlank );
						String value = line.substring( colonBlank + 2 );
						if (value == null) {
							value = "";
//							System.out.println(" unexpected line in header -> "+line);
							log(" unexpected line in header -> "+line);
						}
						reqHeaderNames.addElement( name.toLowerCase() );
						reqHeaderValues.addElement( value );
						}
					    }
					break;
					
				default:
					problem( "Malformed request line", SC_BAD_REQUEST );
					return;
				}
		    reqMethod = tokens[0].toUpperCase();
		    reqUriPath = tokens[1];
	
			
		    // Check Host: header in HTTP/1.1 requests.
		    if ( oneOne ) {
				String host = getHeader( "host" );
				if ( host == null ) {
				    problem( "Host header missing on HTTP/1.1 request", SC_BAD_REQUEST );
				    return;
				}
				// !!!
			}
	
		    // Split off query string, if any.
		    int qmark = reqUriPath.indexOf( '?' );
		    if ( qmark >= 0 ) {
				reqQuery = reqUriPath.substring( qmark + 1 );
				reqUriPath = reqUriPath.substring( 0, qmark );
			}
	
		    // Decode %-sequences.
		    reqUriPath = decode( reqUriPath );
	
		    Servlet servlet = (Servlet) serve.registry.get( reqUriPath );


		    if ( servlet != null )				
				runServlet( (HttpServlet) servlet );
		    }
		catch ( IOException e ) {
		    problem( "Reading request: " + e.getMessage(), e, SC_BAD_REQUEST );
		}
		catch ( Exception e ) {
		    problem( "Reading request: " + e.getMessage(), e, SC_INTERNAL_SERVER_ERROR );
		}		
	}

    private void runServlet( HttpServlet servlet )
	{
	// Set default response fields.
	setStatus( SC_OK );
	setDateHeader( "Date", System.currentTimeMillis() );
	setHeader(
	    "Server", ServeUtils.serverName + "/" + ServeUtils.serverVersion );

	setHeader( "MIME-version", "1.0" );
		
	try
	    {
	    servlet.service( this, this );
	    }
	catch ( IOException e )
	    {
	    problem(
		"IO problem running servlet: " + e.toString(), e, SC_BAD_REQUEST );
	    }
	catch ( ServletException e )
	    {
	    problem(
		"problem running servlet: " + e.toString(), e, SC_BAD_REQUEST );
	    }
	catch ( Exception e )
	    {
	    problem(
		"unexpected problem running servlet: " + e.toString(), e, SC_INTERNAL_SERVER_ERROR );
	    }
	}

    private void problem( String logMessage, int resCode )
	{
	serve.log( logMessage );
	try
	    {
	    sendError( resCode );
	    }
	catch ( IOException e ) { /* ignore */ }
	}

    private void problem( String logMessage, Exception e, int resCode )
	{
		problem(logMessage, resCode);
		serve.reportException(e);
	}

    private String decode( String str )
	{
	StringBuffer result = new StringBuffer();
	int l = str.length();
	for ( int i = 0; i < l; ++i )
	    {
	    char c = str.charAt( i );
	    if ( c == '%' && i + 2 < l )
		{
		char c1 = str.charAt( i + 1 );
		char c2 = str.charAt( i + 2 );
		if ( isHexit( c1 ) && isHexit( c2 ) )
		    {
		    result.append( (char) ( hexit( c1 ) * 16 + hexit( c2 ) ) );
		    i += 2;
		    }
		else
		    result.append( c );
		}
	    else
		result.append( c );
	    }
	return result.toString();
	}

    private boolean isHexit( char c )
	{
	String legalChars = "0123456789abcdefABCDEF";
	return ( legalChars.indexOf( c ) != -1 );
	}

    private int hexit( char c )
	{
	if ( c >= '0' && c <= '9' )
	    return c - '0';
	if ( c >= 'a' && c <= 'f' )
	    return c - 'a' + 10;
	if ( c >= 'A' && c <= 'F' )
	    return c - 'A' + 10;
	return 0;	// shouldn't happen, we're guarded by isHexit()
	}


    // Methods from HttpServletRequest.

    /// Returns the method with which the request was made. This can be "GET",
    // "HEAD", "POST", or an extension method.
    // Same as the CGI variable REQUEST_METHOD.
    public String getMethod()
	{
	return reqMethod;
	}

    /// Returns the size of the request entity data, or -1 if not known.
    // Same as the CGI variable CONTENT_LENGTH.
    public int getContentLength()
	{
	return getIntHeader( "content-length", -1 );
	}

    /// Returns the MIME type of the request entity data, or null if
    // not known.
    // Same as the CGI variable CONTENT_TYPE.
    public String getContentType()
	{
	return getHeader( "content-type" );
	}

    /// Returns the full request URI.
    public String getRequestURI()
	{
	String portPart = "";
	int port = getServerPort();
	if ( port != 80 )
	    portPart = ":" + port;
	String queryPart = "";
	String queryString = getQueryString();
	if ( queryString != null && queryString.length() > 0 )
	    queryPart = "?" + queryString;
	return "http://" + getServerName() + portPart + reqUriPath + queryPart;
	}

    ///  Returns the part of the request URI that corresponds to the
    // servlet path plus the optional extra path information, if any
/*
    public String getRequestPath()
	{
	return reqUriPath;
	}
*/
    /// Returns the part of the request URI that referred to the servlet being
    // invoked.
    // Analogous to the CGI variable SCRIPT_NAME.
    public String getServletPath()
	{
	// In this server, the entire path is regexp-matched against the
	// servlet pattern, so there's no good way to distinguish which
	// part refers to the servlet.
	return reqUriPath;
	}

    /// Returns optional extra path information following the servlet path, but
    // immediately preceding the query string.  Returns null if not specified.
    // Same as the CGI variable PATH_INFO.
    public String getPathInfo()
	{
	// In this server, the entire path is regexp-matched against the
	// servlet pattern, so there's no good way to distinguish which
	// part refers to the servlet.
	return null;
	}

    /// Returns extra path information translated to a real path.  Returns
    // null if no extra path information was specified.
    // Same as the CGI variable PATH_TRANSLATED.
    public String getPathTranslated()
	{
	// In this server, the entire path is regexp-matched against the
	// servlet pattern, so there's no good way to distinguish which
	// part refers to the servlet.
	return null;
	}


	public String getRealPath(String path) {

		return null;

	}

	
    /// Returns the query string part of the servlet URI, or null if not known.
    // Same as the CGI variable QUERY_STRING.
    public String getQueryString()
	{
	return reqQuery;
	}

    /// Returns the value of the specified query string parameter, or null
    // if not found.
    // @param name the parameter name
    public String getParameter( String name )
	{
		String[] vals = (String[]) getParameters().get( name );
		if (vals == null)
			return null;
		return vals[0];
	}


	public String[] getParameterValues(String name) {
		return (String[]) getParameters().get( name );

	}


	private void log(String s) {
		serve.log(s);
	}
	
    /// Returns a hash table of query string parameter values.
	public Hashtable getParameters() {
		if ( queryHash == null )  {
			if (reqMethod.equals("POST")) {
				int len = getContentLength();
				if (len == -1)
					log("ServeConnection detected -1 content length on post");
					queryHash = HttpUtils.parsePostData(len,  getInputStream());
				}
				else {
					String qs = getQueryString();
					if (qs == null) 
						queryHash = new Hashtable();
					else
						queryHash = HttpUtils.parseQueryString(getQueryString());
				}
			}
		return queryHash;
	}


	public Object getAttribute(String name) {

		log("ServeConnection.getAttribute not implmented");

		return null;

	}

	

	public Enumeration getParameterNames() {

		return getParameters().keys();

	}
    /// Returns the protocol and version of the request as a string of
    // the form <protocol>/<major version>.<minor version>.
    // Same as the CGI variable SERVER_PROTOCOL.
    public String getProtocol()
	{
	return reqProtocol;
	}


	public String getScheme() {

		int endslash = reqProtocol.indexOf('/');

		return reqProtocol.substring(0,endslash).toLowerCase();

	}

	
    /// Returns the host name of the server as used in the <host> part of
    // the request URI.
    // Same as the CGI variable SERVER_NAME.
    public String getServerName()
	{
	try
	    {
	    return InetAddress.getLocalHost().getHostName();
	    }
	catch ( UnknownHostException e )
	    {
	    return null;
	    }
	}

    /// Returns the port number on which this request was received as used in
    // the <port> part of the request URI.
    // Same as the CGI variable SERVER_PORT.
    public int getServerPort()
	{
	return socket.getLocalPort();
	}

    /// Returns the name of the user making this request, or null if not known.
    // Same as the CGI variable REMOTE_USER.
    public String getRemoteUser()
	{
	// This server does not support authentication, so even if a username
	// is supplied in the headers we don't want to look at it.
	return null;
	}

    /// Returns the IP address of the agent that sent the request.
    // Same as the CGI variable REMOTE_ADDR.
    public String getRemoteAddr()
	{
	return socket.getInetAddress().toString();
	}

    /// Returns the fully qualified host name of the agent that sent the
    // request.
    // Same as the CGI variable REMOTE_HOST.
    public String getRemoteHost()
	{
	return socket.getInetAddress().getHostName();
	}

    /// Returns the authentication scheme of the request, or null if none.
    // Same as the CGI variable AUTH_TYPE.
    public String getAuthType()
	{
	// This server does not support authentication.
	return null;
	}


	public boolean containsHeader(String name) {

		String value = getHeader(name);

		return value != null;

	}

	
    /// Returns the value of a header field, or null if not known.
    // Same as the information passed in the CGI variabled HTTP_*.
    // @param name the header field name
    public String getHeader( String name )
	{
	int i = reqHeaderNames.indexOf( name.toLowerCase() );
	if ( i == -1 )
	    return null;
	return (String) reqHeaderValues.elementAt( i );
	}

    /// Returns the value of a long header field.
    // @param name the header field name
    // @param def the long value to return if header not found or invalid
    public long getLongHeader( String name, long def )
	{
	String val = getHeader( name );
	if ( val == null )
	    return def;
	try
	    {
	    return Long.parseLong( val );
	    }
	catch ( Exception e )
	    {
	    return def;
	    }
	}

    /// Returns the value of an integer header field.
    // @param name the header field name
    // @param def the integer value to return if header not found or invalid
    public int getIntHeader( String name, int def )
	{
	String val = getHeader( name );
	if ( val == null )
	    return def;
	try
	    {
	    return Integer.parseInt( val );
	    }
	catch ( Exception e )
	    {
	    return def;
	    }
	}

    /// Returns the value of a date header field.
    // @param name the header field name
    // @param def the date value to return if header not found or invalid
    public long getDateHeader( String name, long def )
	{
	String val = getHeader( name );
	if ( val == null )
	    return def;
	try
	    {
	    return Date.parse( val );
	    }
	catch ( Exception e )
	    {
	    return def;
	    }
	}

    /// Returns an Enumeration of the header names.
    public Enumeration getHeaderNames()
	{
	return reqHeaderNames.elements();
	}

    /// Returns the name of the nth header field, or null if there are fewer
    // than n fields. This can be used to iterate through all the headers in
    // the message.
    public String getHeaderName( int n )
	{
	if ( n < 0 || n >= reqHeaderNames.size() )
	    return null;
	return (String) reqHeaderNames.elementAt( n );
	}

    /// Returns an input stream for reading request data.
    public ServletInputStream getInputStream()
	{
		return in;
	}


	public int getIntHeader(String name){
		String value = getHeader(name);
		if (value == null)
			return -1;
		try {
			int val = Integer.parseInt(value);
			return val;
		}
		catch (NumberFormatException e) {
		}
		return -1;
	}

	public long getDateHeader(String name){

		return getDateHeader(name, 0);
	}



    // Methods from HttpServletResponse. -------------------------
    
    /// Sets the status code and message for this response.
    // @param resCode the status code
    // @param resMessage the status message
    public void setStatus( int resCode, String resMessage )
	{
	this.resCode = resCode;
	this.resMessage = resMessage;
	}

    /// Sets the status code and a default message for this response.
    // @param resCode the status code
    public void setStatus( int resCode )
	{
	switch ( resCode )
	    {
//	    case SC_CONTINUE: setStatus( resCode, "Continue" ); break;
//	    case SC_SWITCHING_PROTOCOLS: setStatus( resCode, "Switching protocols" ); break;
	    case SC_OK: setStatus( resCode, "Ok" ); break;
	    case SC_CREATED: setStatus( resCode, "Created" ); break;
	    case SC_ACCEPTED: setStatus( resCode, "Accepted" ); break;
//	    case SC_NON_AUTHORITATIVE: setStatus( resCode, "Non-authoritative" ); break;
	    case SC_NO_CONTENT: setStatus( resCode, "No content" ); break;
//	    case SC_RESET_CONTENT: setStatus( resCode, "Reset content" ); break;
//	    case SC_PARTIAL_CONTENT:   setStatus( resCode, "Partial content" ); break;
//	    case SC_MULTIPLE_CHOICES:    setStatus( resCode, "Multiple choices" ); break;
	    case SC_MOVED_PERMANENTLY:    setStatus( resCode, "Moved permanentently" ); break;
	    case SC_MOVED_TEMPORARILY:    setStatus( resCode, "Moved temporarily" ); break;
//	    case SC_SEE_OTHER: setStatus( resCode, "See other" ); break;
	    case SC_NOT_MODIFIED: setStatus( resCode, "Not modified" ); break;
//	    case SC_USE_PROXY: setStatus( resCode, "Use proxy" ); break;
	    case SC_BAD_REQUEST: setStatus( resCode, "Bad request" ); break;
	    case SC_UNAUTHORIZED: setStatus( resCode, "Unauthorized" ); break;
//	    case SC_PAYMENT_REQUIRED: setStatus( resCode, "Payment required" ); break;
	    case SC_FORBIDDEN: setStatus( resCode, "Forbidden" ); break;
	    case SC_NOT_FOUND: setStatus( resCode, "Not found" ); break;
//	    case SC_METHOD_NOT_ALLOWED: setStatus( resCode, "Method not allowed" ); break;
//	    case SC_NOT_ACCEPTABLE:   setStatus( resCode, "Not acceptable" ); break;
//	    case SC_PROXY_AUTH_REQUIRED:    setStatus( resCode, "Proxy auth required" ); break;
//	    case SC_REQUEST_TIMEOUT:   setStatus( resCode, "Request timeout" ); break;
//	    case SC_CONFLICT: setStatus( resCode, "Conflict" ); break;
//	    case SC_GONE: setStatus( resCode, "Gone" ); break;
//	    case SC_LENGTH_REQUIRED:  setStatus( resCode, "Length required" ); break;
//	    case SC_PRECONDITION_FAILED:    setStatus( resCode, "Precondition failed" ); break;
//	    case SC_REQUEST_ENTITY_TOO_LARGE:    setStatus( resCode, "Request entity too large" ); break;
//	    case SC_REQUEST_URI_TOO_LARGE:    setStatus( resCode, "Request URI too large" ); break;
//	    case SC_UNSUPPORTED_MEDIA_TYPE:    setStatus( resCode, "Unsupported media type" ); break;
	    case SC_INTERNAL_SERVER_ERROR:    setStatus( resCode, "Internal server error" ); break;
	    case SC_NOT_IMPLEMENTED:    setStatus( resCode, "Not implemented" ); break;
	    case SC_BAD_GATEWAY: setStatus( resCode, "Bad gateway" ); break;
	    case SC_SERVICE_UNAVAILABLE:    setStatus( resCode, "Service unavailable" ); break;
//	    case SC_GATEWAY_TIMEOUT:    setStatus( resCode, "Gateway timeout" ); break;
//	    case SC_HTTP_VERSION_NOT_SUPPORTED:  setStatus( resCode, "HTTP version not supported" ); break;
	    default: setStatus( resCode, "" ); break;
	    }
	}

    /// Sets the content length for this response.
    // @param length the content length
    public void setContentLength( int length )
	{
	setIntHeader( "Content-length", length );
	}

    /// Sets the content type for this response.
    // @param type the content type
    public void setContentType( String type )
	{
	setHeader( "Content-type", type );
	}

    /// Sets the value of a header field.
    // @param name the header field name
    // @param value the header field value
    public void setHeader( String name, String value )
	{
//	resHeaderNames.addElement( name );
//	resHeaderValues.addElement( value );
		resHeaderNV.put(name, value);
	}

    /// Sets the value of an integer header field.
    // @param name the header field name
    // @param value the header field integer value
    public void setIntHeader( String name, int value )
	{
	setHeader( name, Integer.toString( value ) );
	}

    /// Sets the value of a long header field.
    // @param name the header field name
    // @param value the header field long value
    public void setLongHeader( String name, long value )
	{
	setHeader( name, Long.toString( value ) );
	}

    /// Sets the value of a date header field.
    // @param name the header field name
    // @param value the header field date value
    public void setDateHeader( String name, long value )
	{
	setHeader( name, to1123String( new Date( value ) ) );
	}

    private static final String[] weekdays =
	{ "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };

    /// Converts a Date into an RFC-1123 string.
    private  String to1123String( Date date )
	{
	// We have to go through some machinations here to get the
	// correct day of the week in GMT.  getDay() gives the day in
	// local time.  getDate() gives the day of the month in local
	// time.  toGMTString() gives a formatted string in GMT.  So, we
	// extract the day of the month from the GMT string, and if it
	// doesn't match the local one we change the local day of the
	// week accordingly.
	//
	// The Date class sucks.
	int localDay = date.getDay();
	int localDate = date.getDate();

	String gmtStr = date.toGMTString();
	int blank = gmtStr.indexOf( ' ' );
	int gmtDate = Integer.parseInt( gmtStr.substring( 0, blank ) );
	int gmtDay;
	if ( gmtDate > localDate || ( gmtDate < localDate && gmtDate == 1 ) )
	    gmtDay = ( localDay + 1 ) % 7;
	else if ( localDate > gmtDate || ( localDate < gmtDate && localDate == 1 ) )
	    gmtDay = ( localDay + 6 ) % 7;
	else
	    gmtDay = localDay;
	return weekdays[gmtDay] + ( gmtDate < 10 ? ", 0" : ", " ) + gmtStr;
	}

    /// Unsets the value of a header field.
    // @param name the header field name
    public void unsetHeader( String name )
	{
		resHeaderNV.remove(name);
	}

    /// Returns an output stream for writing response data.
    public ServletOutputStream getOutputStream()
	{
	return out;
	}


    /// Writes the status line and message headers for this response to the
    // output stream.
    // @exception IOException if an I/O error has occurred
    void writeHeaders() throws IOException {
		if ( headersWritten )
		    return;
		headersWritten = true;
		if ( reqMime )
		    {
		    out.println( reqProtocol + " " + resCode + " " + resMessage );
		    
		    Enumeration e = resHeaderNV.keys();
		    
		    while ( e.hasMoreElements() ) {
				String name = (String) e.nextElement();
				String value = resHeaderNV.getProperty( name );
				if ( value != null )	// just in case
				    out.println( name + ": " + value );
			}
		    out.println( "" );
//		    out.flush(); // flush later, like on close
	    }
	}

    /// Writes an error response using the specified status code and message.
    // @param resCode the status code
    // @param resMessage the status message
    // @exception IOException if an I/O error has occurred
    public void sendError( int resCode, String resMessage ) throws IOException
	{
	setStatus( resCode, resMessage );
	realSendError();
	}

    /// Writes an error response using the specified status code and a default
    // message.
    // @param resCode the status code
    // @exception IOException if an I/O error has occurred
    public void sendError( int resCode ) throws IOException
	{
	setStatus( resCode );
	realSendError();
	}

    private void realSendError() throws IOException
	{
	setContentType( "text/html" );
	out.println( "<HTML><HEAD>" );
	out.println( "<TITLE>" + resCode + " " + resMessage + "</TITLE>" );
	out.println( "</HEAD><BODY BGCOLOR=\"#a0c0a0\">" );
	out.println( "<H2>" + resCode + " " + resMessage + "</H2>" );
	out.println( "<HR>" );
	ServeUtils.writeAddress( out );
	out.println( "</BODY></HTML>" );
	}

    /// Sends a redirect message to the client using the specified redirect
    // location URL.
    // @param location the redirect location URL
    // @exception IOException if an I/O error has occurred
    public void sendRedirect( String location ) throws IOException
	{
	setHeader( "Location", location );
	sendError( SC_MOVED_TEMPORARILY );
	}

    }