/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.daemon;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Vector;

import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.util.smtp.SendMail;
import org.mitre.pam.util.smtp.MIMEc;

/**
* This class creates a Daemon that uses the Alarm class to filter
* the log.txt file and, based on date, emails all the new found
* exceptions and alerts to an administrator.
*
* @author John Fantry, Technical Co-op (D510)
* @author Jason Mathews Use default smtp host instead of extracting
*                       host from mail address.
*/
public class ExceptionChecker extends DaemonBase {
  protected final static String MAIL_TO = "mailExceptionsTo";
  protected final static String HOUR = "hoursBack";
  protected final static String MINUTE = "minutesBack";

  public ExceptionChecker() {
    setProperty(MAIL_TO, "");
    setProperty(HOUR, "1");
    setProperty(MINUTE, "0");
    setPeriod(3600);
  }

  public void doDaemonTask() {

    long hour = getIntegerProperty(HOUR, 1);
    long minute = getIntegerProperty(MINUTE, 0);
    if (hour==0 && minute==0) return;

    String mailTo = getStringProperty(MAIL_TO, "");
    int pos = mailTo.indexOf('@');
    if (pos == -1)
	throw new IllegalArgumentException("Invalid email address in " + MAIL_TO);

    String mailHost = getSmtpHostName();
    SendMail mailer = new SendMail(mailHost);

    File tempFile = new File(getWorkingDir() + File.separator + "exceptionLog.txt");
    if(tempFile.exists()) {
      tempFile.delete();
    }

    // longTime will be subtracted from the current time in order to determine how far back to look
    // for new entries
    long longTime = (hour * 60 + minute) * 60 * 1000;
    ((ExecToController)exec).filterExceptions(longTime, tempFile);

    if(tempFile.length() > 0) {
	MIMEc message = new MIMEc();
	message.setTo(mailTo);
		message.setFrom("ExceptionChecker");
		message.setSubject("Log exceptions");
		message.setBody("Exceptions found in the log file (enclosed)");
		message.addAttachment("text/plain", tempFile);
	try {
		mailer.sendMail(message);
      } catch(IOException ioe) {
        reportException(ioe);
      }
    }
  }
}