/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Doer;
import org.mitre.pam.interfaces.Tasker;
import org.mitre.pam.interfaces.Task;
import org.mitre.pam.interfaces.ConnectedAgent;
import org.mitre.pam.interfaces.Sink;

import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.interfaces.Product;

/**
* A basic unscheduled command for the exec. 
*
* @author Nazario Irizarry, Jr.
*/
public class CommandParallelInputs extends SetFinisher implements Sink {
	Enumeration sources;
	Product product;
	VectorProduct vectorProduct;
/**
* Create a command that executes its inputs in parallel on an
* executer to be supplied at runtime by the exec.  Also acts as 
* a proxy for data sinks. It gets product and passes it on.
*
* @param requestingAgent is the agent that wants its inputs activated
*/
/*	public CommandParallelInputs (Tasker requestingAgent, Sink sink, Task token) {
		super(requestingAgent);
		sinkArg = (Sink) sink;
		putToken = token;
		reporter.debugMsg(this,"new()",toShortString(),null,ExecGlobal.TRACE_MSG_FLOW);
		if (sink == null)
			throw new RuntimeException("null sink");
		if (sink == null)
			throw new RuntimeException("null sink");
	}
*/
/**
* Create a command that executes its inputs in parallel on an
* executer to be supplied at runtime by the exec.  Also acts as 
* a proxy for data sinks. It gets product and passes it on.
*
* @param requestingAgent is the agent that wants its inputs activated
* @param source are the places to get data from
*/
	public CommandParallelInputs (Agent requestingAgent, Sink sink, Task token, Enumeration sources) {
		super(requestingAgent);
		sinkArg = (Sink) sink;
		putToken = token;
		this.sources = sources;
		if (sources == null)
			throw new IllegalArgumentException("null sources");
		if (token == null)
			throw new IllegalArgumentException("null token");
		if (sink == null)
			throw new IllegalArgumentException("null sink");
		if (requestingAgent == null)
			throw new IllegalArgumentException("null requestingAgent");
		reporter.debugMsg(this,"new()",toShortString(),null,ExecGlobal.TRACE_MSG_FLOW);
	}
/**
* A list of sinks
*/
	public void setSources(Enumeration sources) {
		this.sources = sources;
		if (reporter.isDebugKey(ExecGlobal.TRACE_MSG_FLOW)) { //avoid unneeded complex string building
			reporter.debugMsg(this,"setSources() ",toShortString(),null,ExecGlobal.TRACE_MSG_FLOW);
		}
	}
/**
* the object as a string
*/
	public String toString(){
		return "CmdPIn"+Integer.toString(idNumber)+" "+command;
	}
/**
* the object as a string
*/
	public String toShortString(){
		return "("+command+", from= "+doer+", deliverTo= "+sinkArg+")";
	}

/**
* Accepts product as a Sink "proxy".
* When all inputs are done it forwards the input to the "real" sink.
*/
	public synchronized void pleasePutProduct(Product newProduct, Task token){
		if (newProduct == null)
			return;
		if (product == null) {
			product = newProduct;
			vectorProduct = null;
		}
		else {
			if (vectorProduct == null) {
				vectorProduct = new VectorProduct("Multiple products","");
				vectorProduct.addElement(product);
				vectorProduct.addElement(newProduct);
				product = vectorProduct;
			}
			else {
				vectorProduct.addElement(newProduct);
			}
		}
	}
/**
* Pass product to our Sink (if we have any)
* then signal our tasker that we are done.  The tasker
* probably is the Sink, but we are not assuming that.
*/
	public void signalDone(){
		if (product != null)
			getSink().pleasePutProduct(product, putToken);
		super.signalDone();
		product = null;
	}
/**
* Get the sequence of tasks, we currently assume that this is only
* called ONCE between activations.  In other words, tasks are allowed
* to execute and complete before this method is called again.
*/
	public Enumeration getTaskEnumeration(){
		if (sources == null)
			throw new RuntimeException("null sources");
		while(sources.hasMoreElements()) {
			waitFor(new GetCommand(this, (Doer) sources.nextElement(), this, putToken));
		}
		/*
		* clone so that the enumeration is valid even if tasks start
		* finishing before the numeration is all used up.  In that
		* case we would attempt to shorten our taskList while it is being
		* enumerated.  This was an actual bug in our code
		* and a known problem with changing vectors that are being
		* enumerated.
		*/
		return ((Vector)taskList.clone()).elements();
	}
}
