/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import org.mitre.pam.interfaces.Doer;
import org.mitre.pam.interfaces.Tasker;
import org.mitre.pam.interfaces.Task;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.Source;

/**
* A basic unscheduled command for the exec. 
*
* @author Nazario Irizarry, Jr.
*/
public class CommandParallelOutputs extends SetFinisher {
	Enumeration sinks;
/**
* Create a command that executes its inputs in parallel on an
* executer to be supplied at runtime by the exec
*/
	public CommandParallelOutputs (Tasker tasker, Enumeration sinks, Product product) {
		super(tasker);
		this.sinks = sinks;
		this.productArg = product;
		if (product == null)
			throw new IllegalArgumentException("null product");
		if (sinks == null)
			throw new IllegalArgumentException("null sinks");
		if (tasker == null)
			throw new IllegalArgumentException("null tasker");
		reporter.debugMsg(this,"new()",toShortString(),null,ExecGlobal.TRACE_MSG_FLOW);
	}
/**
* Create a command that executes its inputs in parallel on an
* executer to be supplied at runtime by the exec.
* This version of the constructor allows the sinks to be null so
* that they can be set later via setSinks().
*/
	public CommandParallelOutputs (Tasker tasker) {
		super(tasker);
		this.sinks = null;
		this.productArg = null;
		reporter.debugMsg(this,"new()",toShortString(),null,ExecGlobal.TRACE_MSG_FLOW);
	}
/**
* A list of sinks
*/
	public void setSinks(Enumeration sinks) {
		if (sinks == null)
			throw new IllegalArgumentException("null sinks");
		this.sinks = sinks;
		if (reporter.isDebugKey(ExecGlobal.TRACE_MSG_FLOW)) { //avoid unneeded complex string building
			reporter.debugMsg(this,"setSinks(), ",toShortString(),null,ExecGlobal.TRACE_MSG_FLOW);
		}
	}
/**
* the object as a string
*/
	public String toString(){
		return "CmdPOut"+Integer.toString(idNumber)+" "+command;
	}
/**
* the object as a string
*/
	public String toShortString(){
		return "("+command+", from= "+tasker+", product= "+productArg+" )";
	}
/**
* Get the sequence of tasks, we currently assume that this is only
* called ONCE between activations.  In other words, tasks are allowed
* to execute and complete before this method is called again.
*/
	public Enumeration getTaskEnumeration(){
		if (sinks == null)
			throw new RuntimeException("null sinks");
		while(sinks.hasMoreElements()) {
			Doer doer = (Doer) sinks.nextElement();
			if (((Agent)doer).getActivator() == null) {
				waitFor(new SendCommand(this, doer, getProduct())); 
			}

		}
		/*
		* clone so that the enumeration is valid even if tasks start
		* finishing before the enumeration is all used up.  In that
		* case we would attempt to shorten our taskList while it is being
		* enumerated.  This was an actual bug in our code
		* and a known problem with changing vectors that are being
		* enumerated.
		*/
		return ((Vector)taskList.clone()).elements();
	}
}
