/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.*;
import java.io.File;

import org.mitre.pam.interfaces.*;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.executive.Module;
import org.mitre.pam.executive.Command;

/**
* A type of sink that acts as the completion agent for a Juggler
*
* @author Nazario Irizarry, Jr.
*/
final class CompletionSink implements Sink, Tasker {
	private Module parent;
	Sink productRecepient;
	Task putToken;
	boolean productAlreadyBuilt;
	Reporter reporter;

	

	CompletionSink(Module parent, Sink productRecepient, Task putToken) {
		this.parent = parent;
		this.productRecepient = productRecepient;
		this.putToken = putToken;
		if (parent == null)
			throw new RuntimeException("null parent");
		if (productRecepient == null)
			throw new RuntimeException("null productRecepient");
		if (putToken == null)
			throw new RuntimeException("null putToken");
		reporter = parent.getReporter();
	}



	public boolean getTaskingPermission(Task ignoreThisToken){
		return true;
	}

	public void pleasePutProduct(Product p, Task ignoreThisToken){

		if (p == null)
			return;
		parent.declareStart(Juggler.STATE_DIMS_OUTSTANDING);
		Product gJugglersProduct=null;
		try {
			gJugglersProduct = productAlreadyBuilt? p : parent.buildProduct(p);
			parent.setProduct(gJugglersProduct);
			if (gJugglersProduct == null) {
				parent.declareEnd(false);
				return;
			}
			parent.copyAttributesToProduct(parent.getMemento(), gJugglersProduct);
			reporter.debugMsg(this, "put product to 1, product=",gJugglersProduct,productRecepient,ExecGlobal.TRACE_MSG_FLOW);
			productRecepient.pleasePutProduct(gJugglersProduct, putToken); // direct call assumes we have the correct token
		}
		catch (Error e) {
			reporter.reportException(e);
		}
		parent.declareEnd(gJugglersProduct != null);
		gJugglersProduct = null; // for quicker, I hope, gc
	}

	public void taskDone(Task task){
		reporter.debugMsg(this,"output 1 command done",null,null,ExecGlobal.TRACE_MSG_FLOW);
		parent.declareEnd(true);
	}


	public String toString() {
		return "CompletionSink for "+parent.toString();
	}
}