/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.*;
import java.io.File;

import org.mitre.pam.interfaces.*;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.executive.Module;
import org.mitre.pam.executive.CommandParallelOutputs;

/**
* A type of sink that acts as the completion agent for a Juggler
*
* @author Nazario Irizarry, Jr.
*/
public final class CompletionSinkN implements Sink, Tasker {

	private Module parent;
	Enumeration clientList;
	boolean productAlreadyBuilt;
	Reporter reporter;
	
	CompletionSinkN(Module parent, Enumeration clientList) {
		this.parent = parent;
		this.clientList = clientList;
		if (parent == null)
			throw new RuntimeException("null parent");
		if (clientList == null)
			throw new RuntimeException("null clientList");
		reporter = parent.getReporter();
	}

	public boolean getTaskingPermission(Task ignoreThisToken){
		return true;
	}

	public void pleasePutProduct(Product p, Task ignoreThisToken) {
		if (p == null)
			return;
		parent.declareStart(Juggler.STATE_DIMS_OUTSTANDING);
		Product gJugglersProduct = null;
		try {
			gJugglersProduct = productAlreadyBuilt? p : parent.buildProduct(p);
			parent.setProduct(gJugglersProduct);
			if (gJugglersProduct == null) {
				parent.declareEnd(false);
				return;
			}
			parent.copyAttributesToProduct(parent.getMemento(), gJugglersProduct);
			reporter.debugMsg(this,"put product to N, product=",gJugglersProduct,null,ExecGlobal.TRACE_MSG_FLOW);

			if ( clientList.hasMoreElements() ) {
				parent.notifyMyClients(new CommandParallelOutputs(this, clientList, gJugglersProduct));
			}
			else {
				parent.declareEnd(gJugglersProduct != null);
			}		
		}
		catch (Error e) {
			reporter.reportException(e);
			parent.declareEnd(gJugglersProduct != null);
			gJugglersProduct = null; // for quicker, I hope, gc
		}
	}

	
	public void taskDone(Task task){
		reporter.debugMsg(this,"output N command done",null,null,ExecGlobal.TRACE_MSG_FLOW);
		parent.declareEnd(true);
	}


	public String toString() {
		return "CompletionSinkN for "+parent.toString();
	}
	
	public Agent getParent() {
		return parent;
	}
}