/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;
import java.io.File;
import java.util.Enumeration;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.interfaces.Tasker;
import org.mitre.pam.interfaces.Task;
import org.mitre.pam.interfaces.FlowListener;
/**
* An extension of Exec00 so the exec can masquerade as an agent.<P>
*
* @author Nazario Irizarry, Jr.
*/
class Exec01 extends Exec00 implements Agent{


/**
* Consturctor just makes a reference to this as the agent "face"
* because the super doesn't actually know what will masquerade as
* the "agent" view of the exec
*/
	Exec01() {
		agentFace = this;
	}
	
/**
* Get the reporter for the Agent
*/
	final public Reporter getReporter(){
		return reporter;
	}
/**
* This is an empty method so that the exec can masquerade as an Agent.
*/
	final public void setEnabled(boolean enableState){
//		dontUse();
	}
/**
* Implemented so the exec can act as an agent
* @return true
*/
	final public boolean isEnabled() {
		return true;
	}
/**
* @return "exec"
*/
	final public String getId(){
		return "exec";
	}
/*
* Sets an attribute of the exec.
*/
//	public void setProperty(String attribute, String value); implemented by super

/**
* Set a boolean-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	final public void setProperty(String property, boolean value){
		setProperty(property, value?"true":"false");
	}
/**
* Set a single integer-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	final public void setProperty(String property, int value){
		setProperty(property, Integer.toString(value));
	}
/**
* Set a single double-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	final public void setProperty(String property, double value){
		setProperty(property, Double.toString(value));
	}
/**
* Set a single long-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	final public void setProperty(String property, long value){
		setProperty(property, Long.toString(value));
	}
/*
* Gets an attribute of the exec.
*
*/
//	public String getProperty(String attribute); implemented by super

/**
* Get a boolean-valued property
*/
	final public boolean getBooleanProperty(String property, boolean defaultValue){
		return execOptions.getBooleanProperty(property, defaultValue);
	}
/**
* Get a double-valued property
*/
	final public double getDoubleProperty(String property, double defaultValue){
		return execOptions.getDoubleProperty(property, defaultValue);
	}
/**
* Get a integer-valued property
*/
	final public int getIntegerProperty(String property, int defaultValue){
		return execOptions.getIntegerProperty(property, defaultValue);
	}
/**
* Get a single string-valued property
*/
	final public String getStringProperty(String property, String defaultValue){
		return execOptions.getStringProperty(property, defaultValue);
	}
/**
* Get a long-valued property
*/
	final public long getLongProperty(String property, long defaultValue){
		return execOptions.getLongProperty(property, defaultValue);
	}
/**
* @return true if the property exists
*/
	final public boolean isProperty(String propertyName){
		return execOptions.isProperty(propertyName);
	}
/*
* Remove an attribute of the exec.
*/
//	public void removeProperty(String attribute); implemented by super

/**
* @return 0 if property does not exist, has been deleted, or is an empty valuelist
* @return 1 if string or valuelist with 1 object
* @return >1 if valuelist with more than one value
*/
	final public int getCurrentDimension(String property){
		return execOptions.getCurrentDimension(property);
	}
/*
* Copy the property and value potentially with a different name
*/
//	public void copyPropertyFromAs(String fromName, HasProperties fromOther, String newName); implemented by super

/**
* Remove a property but tag it so that remote, dependent
* data copies can synchronize with this deletion
*
*/
	final public void removePropertyAndRemoteCopies(String property){
		removeProperty(property);
	}
/**
* This is an empty method so that the exec can masquerade as an Agent.
*/
	final public File getWorkingDir(){
		dontUse();
		return null;
	}
/*
* Add a property-value list
*/
//	public void setPropertyValues(String property, ValueList value); implemented by super

/**
* Get a multivalued property.
* @return always of type ValueList, never null
*/
//	public ValueList getPropertyValues(String property); implemented by super

/**
* Get list of exec property names
*/
	final public Enumeration getProperties() {
		return execOptions.getProperties();
	}
/**
* Get a generic object that holds the value of the key.
* @return an object of String, Numeric, or ValueList type, or null
*/
	public Object getPropertyAsObject(String property){
		return execOptions.getPropertyAsObject(property);
	}
/**
* @return "exec"
*/
	final public String getKind(){
		return "exec";
	}
/**
* This is an empty method so that the exec can masquerade as an Nameable.
*/
	final public void setId(String name){
		dontUse();
	}
/**
* This is an empty method so that the exec can masquerade as an Agent.
*/
	final public void setKind(String name){
		dontUse();
	}
/**
* Get exec's memento
*/
	final public Memento getMemento (){
		updateDebugKeyList();
		return execOptions;
	}
/**
* This is an empty method so that the exec can masquerade as an Agent.
*/
	final public Product getPersistentProduct(){
		dontUse();
		return null;
	}
/**
* This is an empty method so that the exec can masquerade as an Agent.
*/
	final public void setPersistentProduct(Product p){
		dontUse();
	}
/**
* Log an error message in the context of the Agent if an assertion fails.
* These are programming assumptions. If these occur
* the software support facility should be notified.
*/
	private void dontUse(){
		reporter.assert(false, "not implemented", null, null, null);
	}
/**
* Does nothing.
*/
	final public void addFlowListener(FlowListener listener){

	}
/**
* This is an empty method so that the exec can masquerade as Agent
*/
	final public void doCommittedTask(){
		dontUse();
	}	
/**
* This is an empty method so that the exec can masquerade as Agent
*/
	final public void taskDone(Task task){
	}	
/**
* Does nothing.  Implemented so Exec can be a Doer.
* @return false;
*/
	final public boolean getTaskingPermission(Task task){
		dontUse();
		return false;
	}
/**
* Does nothing.  Implemented so Exec can be a Doer.
*/
	final public void releaseTaskingPermission(Task task){
	}
/**
* Does nothing.  Implemented so Exec can be a Doer.
*
* @return true
*/
	final public boolean isIdle(){
		dontUse();
		return false;
	}
/**
* Does nothing.  Implemented so Exec can be a Doer.
*
* @return true
*/
	final public void waitTillIdle(){
		dontUse();
	}
/*
* get exec state
*/
//	public String getState(); implemented by super

/**
* @return "the executive"
*/
	final public String getDescription(){
		return "the executive";
	}
/**
* An activator is the thing that gets an agent to do something.  
* The activator may be a schedule.
*/
	final public Object getActivator(){
		return null;
	}
}
