/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import java.io.PrintWriter;
import java.io.IOException;

import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.Memento;

import org.mitre.pam.interfaces.Sink;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.ConnectedAgent;
import org.mitre.pam.interfaces.Source;
import org.mitre.pam.util.EnabledTable;
import org.mitre.pdif.eventParser.PDIFWriter;


/**
* Exec functions to support monitoring.<P>
*
* @author Nazario Irizarry, Jr.
*/
class Exec02 extends Exec01 implements ExecToMonitor{
/**
* Get list of currently active modules of the specified type.
*
* @param type one of ExecToMonitor.DIM, ExecToMonitor.PIM, etc
* @return an Enumeration of active objects per request
*/
	public Enumeration getListOfActiveModules(int type) {
	
		if (type < 0 || type > 3) {
			reporter.assert(false,"Argument out of range, type="+type,null,null,null);
			return (new java.util.Vector()).elements();
		}
		
		return moduleList[type].elements();
	}
/**
* Get list of currently active agents that belong to the
* group designated by the attribute value pair.  One possible
* attribute would be "group" in which case the value would identify a
* group's name or identifier. <p>
*
* @param attribute the attribute to look for in each agent, if null then all agents are returned
* @param value if null any value is acceptable,
*  if not null then this is the String value that each attribute must have,
* @return an Enumeration of type Agent of active agents that meet the criterion
*/
	public Enumeration getListOfActiveAgents(String attribute, String value){
//		reporter.assert(attribute != null,"attribute=null",null,null,null);
//		reporter.assert(value != null,"value=null",null,null,null);
		
		Vector findings = new Vector(100);
		for (int i = 0; i<= 3; i++) {
			Enumeration enum = moduleList[i].elements();
			while (enum.hasMoreElements()) {
				if (attribute == null) {
					findings.addElement((Agent)enum.nextElement());
				}
				else {
					Agent agent = (Agent)enum.nextElement();
					String itsValue = agent.getProperty(attribute);
					if (value == null) {
						if (itsValue != null)
							findings.addElement(agent);
					}
					else {
						if (itsValue != null && itsValue.equals(value))
							findings.addElement(agent);
					}
				}
			}
		}
		return findings.elements();
	}

/**
* Get list of available modules of the specified type.  
* 
* @param type one of ExecToMonitor.DIM, ExecToMonitor.PIM, etc
* @return an enumeration of unused instances of DIM for use as a template
*/
	public Enumeration getListOfAvailableModules(int type){
		return sourceSinkInfo.getListOfAvailableModules(type);
	}

/**
* Get a specific (unused,unistantiated) agent.
* 
* @param id of requested agent
* @return null if not found
*/
	public Agent getAvailableAgent(String id){
		return sourceSinkInfo.getUnusedInstanceFromCommonName(id);
	}

/**
* Get item memento
* @param id the it of the item
* @param properties [in] an empty vector; [out] a vector of Strings
* @param values [in] an empty vector; [out] a vector of Strings
*/
	public void getItemStateList(String id, Vector properties, Vector values){
		Memento memento = null;
		if (id.equalsIgnoreCase("exec")){
			memento = getMemento();
		}
		else {
		
			try {
				Agent it = getActiveAgent(id);
				if (it != null)
					memento = it.getMemento();
			}
			catch (Exception e) {
				reporter.reportException(e);
			}
		}
		
		if (memento == null)
			return;
		
		Enumeration e = memento.getProperties();
		while (e.hasMoreElements()) {
			String property = (String) e.nextElement();
			String value =  memento.getProperty(property);
			if (value == null)
				value = "-";
			properties.addElement(property);
			values.addElement(value);
		}
	}
/**
* Get status of queue
*/
	public void getQueState(int whichQueue, Vector running, Vector pending){
		if ( whichQueue == ExecToMonitor.PIM ) {
			PIMExecuter.getQueueLists(running, pending);
		}
		if ( whichQueue == ExecToMonitor.DIM ) {
			DIMExecuter.getQueueLists(running, pending);
		}		
		if ( whichQueue == ExecToMonitor.JUGGLER ) {
			jugglerScheduler.getQueueLists(running, pending);
		}		
	}
/**
* Get list of an agents sources. 
* 
* @param agentId
* @return an enumeration of agents of type Source, never null
*/
	public Enumeration getListOfAgentSources(String agentId){
		try {
			Agent agent =  getActiveAgent(agentId);
			if (agent == null || !(agent instanceof Sink))
				return (new Vector()).elements();
			EnabledTable et = (EnabledTable) ((ConnectedAgent) agent).getSourceList();
			return et.allElements();
		}
		catch (ClassCastException e){
			return (new Vector()).elements();
		}
	}
/**
* Get list of an agents sinks. 
* 
* @param agentId
* @return an enumeration of agents of type Sink, never null
*/
	public Enumeration getListOfAgentSinks(String agentId){
		try {
			Agent agent = getActiveAgent(agentId);
			if (agent == null || !(agent instanceof Source))
				return (new Vector()).elements();
			EnabledTable et = (EnabledTable) ((ConnectedAgent) agent).getSinkList();
			return et.allElements();
		}
		catch (ClassCastException e){
			return (new Vector()).elements();
		}
	}
/**
* Get state as a stream
* @param p the print writer to use to write the output
* @param selectors null, or a set of "query" options
*/
	public void getPAMState(PrintWriter p, Hashtable selectors) throws IOException {
		PDIFWriter pout = new PDIFWriter(p, "CDA", "ipam");
		String configurationName = null;
		boolean useCompression = false;
		if (selectors != null) {
			configurationName = (String) selectors.get(SELECTOR_CONFIGURATION_NAME);
			useCompression = selectors.get(SELECTOR_NO_REDUNDANCY_WITH_PARENT) != null;
		}

		PAMSave pamSave = new PAMSave(this, execOptions, sourceSinkInfo, listOfThingsWithoutProduct, listOfThingsWithProduct);
		pamSave.setSuppressRedundantAttributes(useCompression);
		pamSave.saveConfiguration(pout, configurationName);	
		pout.flush();

	}
/**
* Get state as a PDIF stream
*
* @param p the print writer to use to write the output
*/
/*
	public void getPAMState(PrintWriter p) throws IOException{
		getPAMState(p, null);
	}
*/
}