/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.*;

import java.io.File;

//import org.mitre.pam.executive.PropertySet;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.util.alarm.Alarm;
import org.mitre.pam.store.FileNotLocalException;
import org.mitre.pam.interfaces.ExecToController;


/**
* Services provided to all <P>
* @author Nazario Irizarry, Jr.
*/
public class ExecGlobal extends Observable {
/**
* 
*/
	public static final String AVAILABLE_DEBUG_KEYS = "availableDebugKeys";
/**
* a verbose debug key to trace executive "message" flow
*/
	public static final String TRACE_MSG_FLOW = "debugMessageFlow";
/**
* The keyword used when and agent is created.  The exec property
* by this name "lastNewAgent" , is set to the id of the new module.
* This allows something that is monitoring exec memento changes
* to easily detect the name of the id of the new module.
*/
	public static String LAST_NEW_AGENT = "lastNewAgent";
/**
* The keyword used when and agent is created.  The exec property
* by this name "lastDeletedAgent" , is set to the id of the new module.
* This allows something that is monitoring exec memento changes
* to easily detect the name of the id of the new module.
*/
	public static String LAST_DELETED_AGENT = "lastDeletedAgent";
/**
* The keyword used when and agent is created.  The exec property
* by this name "lastAgentCreated" , is set to the id of the new module.
* This allows something that is monitoring exec memento changes
* to easily detect the name of the id of the new module.
*/
	public static String LAST_CONNECTION_CREATE_DESTROY_LONG = "lastMakeOrBreakConnectionLong";
/**
* Names a resource scheduler or queue and can
* be used to notify the exec and its clients of changes
* to the respective queues (future capablity)
*/
	public static String JUGGLER_SCHEDULER = "JugglerScheduler";
/**
* Names a resource scheduler or queue and can
* be used to notify the exec and its clients of changes
* to the respective queues (future capablity)
*/
	public static String PIM_EXECUTER = "PIM Executor";
/**
* Names a resource scheduler or queue and can
* be used to notify the exec and its clients of changes
* to the respective queues (future capablity)
*/
	public static String DIM_EXECUTER = "DIM Executor";
/**
* Names a resource scheduler or queue and can
* be used to notify the exec and its clients of changes
* to the respective queues (future capablity)
*/
	public static String EXECUTIVE = "Executive";
/**
* These strings are used to notify the exec and its clients of changes
* to exec state (implemented)
*/
	public static String START_GC = "executive.start gc";
/**
* These strings are used to notify the exec and its clients of changes
* to exec state (implemented)
*/
	public static String END_GC = "executive.end gc";
/**
* These strings are used to notify the exec and its clients of changes
* to exec state (implemented)
*/
	public static String START_CHECKPOINT = "executive.start checkpoint";
/**
* These strings are used to notify the exec and its clients of changes
* to exec state (implemented)
*/
	public static String END_CHECKPOINT = "executive.end checkpoint";
/**
* Attribute of exec and agents that identifies the run state
*/
	public static String STATE = "runState";
/**
* Attribute of exec and agents that identifies the exec is starting up
* no agents should be running
*/
	public static String STARTING = "executive.starting";
/**
* Attribute of exec and agents that identifies things should be running
*/
	public static String WORKING = "executive.working";
/**
* Attribute of exec and agents that identifies nothing is going
* on at the moment
*/
	public static String IDLE = "executive.idle";
/**
* Attribute of exec that says things are shutting down
*/
	public static String SHUTTING_DOWN = "executive.shutting down";
/**
* This is a heartbeat tick generated by the exec and
* available to observers
*/
	public static String TICK = "executive.tick";
/**
* Exec attribute for the root of all working directories,
* read only
*/
	public static final String WORKING_DIR_ROOT = "workingDirRoot";
/**
* Exec attribute for the root of all working directories,
* as a URL request to the built in web server
*/
	public static final String WORKING_DIR_URL_ROOT_NAME = "workingDirURLRoot";
/**
* Exec attribute for the root of all support directories,
* as a URL request to the built in web server
*/
	public static final String SUPPORT_DIR_URL_ROOT_NAME = "supportDirURLRoot";
/**
* Exec attribute for the root of all support directories
*/
	public static final String SUPPORT_DIR_ROOT = "supportDirRoot";
/**
*  Attribute used by exec to tag the id of persistent products in
*  persistent storage
*/
	public static final String PERSISTENT_PRODUCT = "persistentProduct";
/**
*  Name of checkpoint file created by the exec
*/
	public static final String DEF_CKPT_FILE_NAME = "ckpt.txt";
/**
*  Exec attribute that identifies the checkpoint directory (read only)
*/
	public static final String CHECK_POINT_DIR="CheckPointDirectory";
/**
*  Exec attribute that tells exec max number of PIM threads to use (r/w)
*/
	public static final String PIM_THREAD_COUNT="PIMThreadCount";
/**
*  Exec attribute that tells exec max number of DIM threads to use (r/w)
*/
	public static final String DIM_THREAD_COUNT="DIMThreadCount";
/**
*  Exec attribute that tells DIM and PIM queues the limit on how many
*  simultaneous activations of the SAME instance of a DIM in multiple threads.
*  For experimentaiton, recommended value is 1
*/
	public static final String CONCURRENCY_LIMIT="concurrencyLimit";
/**
*  Exec attribute that is updated after product garbage collection
* to indicate how many FileRefs are currently in use (read only, future feature)
*/
	public static final String ACTIVE_FILEREF_COUNT="activeFileRefCount";
/**
*  Exec attribute that identifies the internal web servers protocol and host
* (something like http://me.here.now).  Automatically set during startup,
* can be changed for example if you machine report numbers instead of machine names.
*/
	public static final String PROTOCOL_AND_HOST="protocolAndHost";
/**
* Exec attribute set by exec so you can tell when the last checkpoint was
* performed (not too useful to humans since it is in Java milliseconds)
*/ 
	public static final String LAST_CHECKPOINT_LONG="lastCheckpointLong";
/**
* Exec attribute set by exec so you can tell when the last checkpoint was
* performed (human readable)
*/ 
	public static final String LAST_CHECKPOINT_DATE="lastCheckpointDate";
/**
* Exec attribute to control the TCP port number used by the internal web server
*/ 
	public static final String SERVER_PORT_NUMBER="serverPortNumber";
/**
* Exec attribute to enable the output of otherwise suppressed debugging messages,
* Works in conjunction with debugTerse and debugVerbose method calls.
*/ 
	public static final String DEBUG_KEY="debugKey";
/**
* Exec attribute to enable the output of otherwise suppressed debugging messages,
* Works in conjunction with debugTerse and debugVerbose method calls.
*/ 
	public static final String DEBUG_VERBOSE_KEY="debugVerboseKey";
/**
* Exec attribute to set a brief identification string stamped on exec output windows.
*/ 
	public static final String EXEC_ID="id";
/**
* The default SMTP host name.
*/ 
	public static final String DEFAULT_SMTP_HOST_NAME="defaultSmtpHostName";
/**
* Proxy info just like Java proxyHost parameter
*/
	public static final String PROXY_HOST="proxyHost";
/**
* Proxy info just like Java proxyPort parameter
*/
	public static final String PROXY_PORT="proxyPort";
/*
* A peer that serves up our info so we don't have to be up all the time
*/
	public static final String PEER_PROTOCOL_HOST_PORT="peerProtocolHostAndPort";
/*
* A String that indicates which file the exec was started from (or empty)
*/
	public static final String EXEC_STARTED_FROM_FILE="startedFromConfigFile";
/**
* A reference to an exec singleton
*/
	static protected ExecToController execSingleton;
/**
* Executive options
*/
	static Memento execOptions;
/**
* Attribute of exec and agents that identifies nothing is going
* on at the moment
*/
	public static final String COMMITTED = "executive.committed";
/**
* Exec property that causes file deletions to be entered into the log file
*/
	public static final String LOG_DELETED_FILES="logFileDeletions";
/**
* Exec property
*/
	public static final String MIN_BERT_VERSION="minimumBertVersion";
/**
* Exec property
*/
	public static final String BERT_ADMINS_NAME="bertAdministratorName";
/**
* Exec property that identifies who has access to the iPAM server
*/
	public static final String IPAM_ACCESS_FOLKS="pamAccess";

/**
* A way to access the single copy of the exec
*/
	static public ExecToController getExec() {
/*
		if (execSingleton==null)
			throw new Error("ExecGlobal.getExec() request for an undefined exec");
*/
		return execSingleton;
	}

/**
* Get the port that the server should operate on
*/
	static public int getServerPortNumber() {
		try {
			return Integer.parseInt(execOptions.getProperty(SERVER_PORT_NUMBER));
		}
		catch (NumberFormatException e) {
			return 0;
		}
	}
/**
* Get the logical name of the working directory root for use
* in servlets
*/
	static public String getWorkingDirURLRoot() {
		return execOptions.getProperty(WORKING_DIR_URL_ROOT_NAME);
	}
/**
* Get the logical name of the support directory root for use
* in servlets
*/
	static public String getSupportDirURLRoot() {
		return execOptions.getProperty(SUPPORT_DIR_URL_ROOT_NAME);
	}
/**
* Get the partial url for us such as "http:my.server.addr"
*/
	static public String getProtocolHostNoPort() {
		return execOptions.getProperty(PROTOCOL_AND_HOST) ;
	}
/**
* Get the partial url for us such as "http:my.server.addr:8080"
*/
	static public String getProtocolHostAndPort() {
		return execOptions.getProperty(PROTOCOL_AND_HOST)+":"+execOptions.getProperty(SERVER_PORT_NUMBER) ;
	}
/**
* Get the partial url for a server that is up all the time
* and serving our info, such as "http:peer.server.addr:8080"
*/
	static public String getPeerProtocolHostAndPort() {
		return execOptions.getProperty(PEER_PROTOCOL_HOST_PORT) ;
	}
/**
* Get the  name of the http url for a peer that is up all the time for service
*/
	static public File getCheckPointDir() {
		File checkPtDir = new File(execOptions.getProperty(CHECK_POINT_DIR));
		if (!checkPtDir.isDirectory())
			checkPtDir.mkdirs();
		return checkPtDir;
	}

/**
* Get the PIM, DIM, JUGGLER working directory root.
* @param subDir client specifies their subdirectory unique id.
*/
	public static File g_getWorkingDir(String subDir) {
		File wd = new File(execOptions.getProperty(WORKING_DIR_ROOT));
		if (!wd.isDirectory())
			wd.mkdirs();
		
		File dir = new File(wd, subDir);
		if (!dir.isDirectory())
			dir.mkdirs();
		return dir;
	}

/**
* Get the PIM, DIM, JUGGLER working directory root.
* @param subDir client specifies their subdirectory unique id.
*/
	public static File g_getWorkingDir() {
		File wd = new File(execOptions.getProperty(WORKING_DIR_ROOT));
		if (!wd.isDirectory())
			wd.mkdirs();
		
		return wd;
	}

/**
* Get the checkpoint file
*/
	static File getCheckPointFile() {

		return new File(getCheckPointDir(), DEF_CKPT_FILE_NAME);
	}
/**
* The support directory is the directory that PIM, DIM, and Juggler
* code can turn to for "fixed" readonly access to support files
* needed to get the job done.  This includes html template files
* as required.  The module should use a subdirectory
* that is clearly identifiable (by a human
* being) with the PIM DIM or Juggler class (maybe use the PIM DIM or Juggler class name).
*/
	public static File getSupportDir() {
		File supportDir = new File(execOptions.getProperty(SUPPORT_DIR_ROOT));
		if (!supportDir.isDirectory())
			supportDir.mkdirs();
		return supportDir;
	}
/**
* The support directory is the directory that PIM, DIM, and Juggler
* code can turn to for "fixed" readonly access to support files
* needed to get the job done.  This includes html template files
* as required.  The module should use a subdirectory
* that is clearly identifiable (by a human
* being) with the PIM DIM or Juggler class (maybe use the PIM DIM or Juggler class name).
*
* @param subDir client specifies their subdirectory unique id.
*/
	public static File getSupportDir(String subDir) {
		File sd = getSupportDir();
		
		File dir = new File(sd, subDir);
		if (!dir.isDirectory())
			dir.mkdirs();
		return dir;
	}

  /**
   * Returns a relative URL (doesn't include protocol or host name) for the
   * file provided. Note that this method will return file names
   * with spaces in them, which are not legal URL characters in files.  They
   * need to be converted using the % convention.
   *
   * @return A string that specifies the URL for the given file (minus the
   *  protocol and host - Call getProtocolAndHost() to get that portion).
   *  Null is returned if the file is not in a file system tree served by the
   *  web server.
   * @param file  The File object that the caller wants a URL for.
   * @see getProtocolHostAndPort
   */
  static public String getURLFromFile(File file) throws FileNotLocalException {
    String path = file.getAbsolutePath();
    String wdPath = g_getWorkingDir().getAbsolutePath();
    String sdPath = getSupportDir().getAbsolutePath();
    StringBuffer urlBuf = null;
    int startPos = 0;

    if(path.startsWith(wdPath)) {
      // File is in the working directory tree...
      urlBuf = new StringBuffer(getWorkingDirURLRoot());
      startPos = wdPath.length();
    } else if(path.startsWith(sdPath)) {
      // File is in the support directory tree...
      urlBuf = new StringBuffer(getSupportDirURLRoot());
      startPos = sdPath.length();
    } else {
      // We can't serve this file from the web server...
      throw new FileNotLocalException(file);
    }
    startPos++;                         // To skip over delimiter

    // Append rest of path to the URL we are building
    urlBuf.append('/');
    urlBuf.append(path.substring(startPos));
    if(file.isDirectory()) urlBuf.append('/');

    // Substitute '/' for the file system directory seperator in our URL...
    String fileUrl = urlBuf.toString();
    if(File.separatorChar != '/') {
      fileUrl = fileUrl.replace(File.separatorChar, '/');
    }
    /*
    * now go through and urlencode the directory and file names
    */
    
    return fileUrl;
  }

/**
* Identifies whether a file can have a URL (is servicable via http). <p>
*/
	static public boolean fileHasURL(File file) {
		String path = file.getAbsolutePath();
		String wdPath = g_getWorkingDir().getAbsolutePath();
		String sdPath = getSupportDir().getAbsolutePath();
		
		if(path.startsWith(wdPath)) {
		  return true;
		} else if (path.startsWith(sdPath)) {
		  return true;
		}
		return false;
	}

}
