/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.Observable;
import java.util.Enumeration;
import org.mitre.pam.interfaces.Doer;
import org.mitre.pam.interfaces.Tasker;
import org.mitre.pam.interfaces.Task;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.ExecCallbacks;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.Sink;

/**
* A basic unscheduled command for the exec. 
*
* @author Nazario Irizarry, Jr.
*/
public class SendCommand extends Command {

/**
* Create a SEND_PRODUCT command as a task to be committed.
* No assumption that we already have permission to send this.
* Our command may get queued somewhere.
*
* @param tasker the Tasker that receives the done callback
* @param doer the Doer
* @param product the Product to put
* @param sink the Sink to call if the putToken is valid (may be null)
* @param putToken the Task that has "authority" to send to the Sink, may be null
*/
	public SendCommand (Tasker tasker, Doer doer, Product product) {
		super(tasker, doer, SEND_PRODUCT);
		this.productArg = product;
		if (reporter.isDebugKey(ExecGlobal.TRACE_MSG_FLOW)) { //avoid unneeded complex string building
			reporter.debugMsg(this,"new (),",toShortString(),null,ExecGlobal.TRACE_MSG_FLOW);
		}
		if (product == null)
			throw new RuntimeException("null product");
		if (doer == null)
			throw new RuntimeException("null doer");
		if (tasker == null)
			throw new RuntimeException("null tasker");
	}

/**
* For use by subclasses
*/
	SendCommand (Tasker tasker) {
		super(tasker, null, SEND_PRODUCT);
		if (reporter.isDebugKey(ExecGlobal.TRACE_MSG_FLOW)) { //avoid unneeded complex string building
			reporter.debugMsg(this,"new (),",toShortString(),null,ExecGlobal.TRACE_MSG_FLOW);
		}
	}

/**
* the object as a string
*/
	public String toShortString(){
		return "("+command+", product= "+productArg+", to=" + doer +" )";
	}
/**
* the object as a string
*/
	public String toString(){
		return "SendCommand"+Integer.toString(idNumber);
	}
}
