/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.executive;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import java.io.File;
import org.mitre.pam.interfaces.*;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.store.FileRefList;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.util.SortedHash;
/**
* A single instance of this class runs
* on behalf of the exec.  Scans through agents to mark
* their persistent product.
* It used to also delete unused product, but
* we finally got java finalization to do that for us.
* Adds a debugging tag to each product as a side-effect.
*  <p>
*
* @author Nazario Irizarry, Jr.
*/

final class SweepAndGC /*implements Runnable*/ {

	SortedHash[] listOfThingsWithProduct;
	ExecCallbacks exec;
//	Thread thread;
	static File tempFile = new File("pamTempFile");
	
/**
* @param list1 Hashtable whose elements are of type PIM
* @param list2 Hashtable whose elements are of type Juggler
* @param list3 Hashtable whose elements are of type DIM
*/
	public SweepAndGC(SortedHash[] listOfThingsWithProduct, ExecCallbacks exec) {
		this.listOfThingsWithProduct = listOfThingsWithProduct;
		this.exec = exec;
//		thread = new Thread(this, "SweepAndGC");
//		thread.setPriority(Thread.NORM_PRIORITY -1);
//		thread.start();
	}
/**
* The thread sleeps for a while and then does garbage collection
*/
/*
	public void run() {
		while (exec.getState() != ExecGlobal.SHUTTING_DOWN) {
			try {
				Thread.sleep(30013);
				if (exec.getState() != ExecGlobal.START_CHECKPOINT)
					sweep();				
			}
			catch (Exception e) {
				exec.reportException("exec",e);
			}
		}
	}
*/
/**
* Tickle the system to do garbage collection.
* Collect the memory statistics.
* Use a mark approach to aid in determining used
* files because Java gc will not necessarily
* clean up expired files until shutdown.
*/
	public void sweep() {
		exec.update(null, ExecGlobal.START_GC);	
		Runtime runtime = Runtime.getRuntime();
		long freeBefore = runtime.freeMemory();
		long totalMem = runtime.totalMemory();
		runtime.runFinalization();
		runtime.gc();
		long freeAfter = runtime.freeMemory();
		Agent execAgent = exec.getExecAsAgent();
		execAgent.setProperty("memFreeBefore",freeBefore);
		execAgent.setProperty("memFreeAfter",freeAfter);
		execAgent.setProperty("memTotal",totalMem);
		Reporter reporter = execAgent.getReporter();
		if (reporter.isDebugKey("logGC")) {
			reporter.log("memory\ttotal = "+totalMem+"\tfree before gc = "+freeBefore+"\tfree after gc = "+freeAfter+"\tused after gc = "+(totalMem-freeAfter));
		}
		markUsedFileRefs();
		exec.update(null, ExecGlobal.END_GC);		
	}
/**
* Beware of products with circular references
*/
	private void markProduct(Product p, String agentId) {
		if (p == null)
			return;

//		p.setProperty(Product.PROPERTY_USED_BY_AGENT, agentId);
		
		Object content = p.getContent();
		if (content == null) {
			return;
		}
		if (content instanceof FileRef) {
			((FileRef) p.getContent()).mark();
		}
		else if (content instanceof Enumeration) {
			Enumeration e = (Enumeration) p.getContent();
			while (e.hasMoreElements()) {
				markProduct((Product) e.nextElement(), agentId);
			}
		}
	}

/**
* Scans all products for marking.  Products
* that have not been marked can be purged via
* FileRefList.purgeAll().
*/
	public void markUsedFileRefs() {
//		System.out.println("begin sweep");
		FileRefList.advanceHoldToken();	
		Enumeration e;
		
		for (int i = 0; i< listOfThingsWithProduct.length; i++) {
			SortedHash list = listOfThingsWithProduct[i];
			e = list.elements();
			while (e.hasMoreElements()) {
				Agent c = (Agent) e.nextElement();
				markProduct(c.getPersistentProduct(), c.getId());
			}
		}


//		System.out.println("  end sweep");
	}
}