/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter;

import java.util.*;
import java.net.*;
import java.io.*;



import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.executive.Command;
import org.mitre.pam.executive.Module;
import org.mitre.pam.util.EnabledTable;
import org.mitre.pam.executive.CommandParallelOutputs;
import org.mitre.pam.interfaces.PIM;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.Task;
import org.mitre.pam.interfaces.Tasker;
import org.mitre.pam.interfaces.Sink;
import org.mitre.pam.interfaces.Schedulable;
import org.mitre.pam.interfaces.Doer;

/**
* A simple PIM shell that you might want to use as a base class. <P>
*
* @author Nazario Irizarry, Jr.
*/
public abstract class PIMShell1 extends Module implements PIM {
/**
* List of this pim's Sinks
*/
	protected EnabledTable sinkList = new EnabledTable();
		
	public PIMShell1 () {
		setState(ExecGlobal.IDLE);
		setProperty(HAS_OUTPUTS,"true");
		setProperty(Schedulable.HAS_SCHEDULE, "false");
	}

/**
* @deprecated
*/
	final public void doCommand(String command, Object arg){
	}
/**
* Callback to receive a command.  There
* is only one thing we can do - fetch.
*/
	public synchronized void doCommittedTask(){
		Task task = taskHelper.getCommittedTask();
		String command = task.getCommand();
		if ( command == Task.GET_PRODUCT) {
			Product prod=null;
			declareStart(null);
			try {
				prod = gAgentsProduct = doFetch();
				if (prod != null) {
					copyAttributesToProduct(getMemento(), prod);
					task.getSink().pleasePutProduct(prod, task.getPutToken());
				}
			}
			catch (Exception e) {
				reportException(e);
			}
			finally {
				declareEnd(prod != null);
			}
		}
		else {
			assert(false,"PIMShell1 received unknown command ",command,null,null);
			setState(ExecGlobal.IDLE); // clears committed task
		}
		
	}
/**
* identify that we are starting
*/
	final public void declareStart(String ignored) {
		String state = getState();
		if ((state == ExecGlobal.IDLE || state == ExecGlobal.COMMITTED) && logActivity) {
			reporter.log("begin pim");
		}
		setState(ExecGlobal.WORKING);
	}
/**
* set the conditions at end of update
*/
	final public void declareEnd(boolean hasProduct) {
		if (logActivity) {
			if (hasProduct) {
				reporter.log("end pim,\tproduct");
			}
			else {
				reporter.log("end pim,\t[]");
			}
		}
		setState(ExecGlobal.IDLE); // clears committed task
	}
/**
* Do a "query" and return the product
*/
	abstract public Product doFetch();
/**
* @return a table of enabled Sinks.
*/
	public Hashtable getSinkList() {
		return sinkList;
	}
/**
* get enumeration of all Sinks including disabled items
*/
	public Enumeration getFullSinkElementEnumeration(){
		return sinkList.allElements();
	}

/**
* An activator is the thing that gets an agent to do something.  
* A PIMs activator is actually any juggler that it is connected to.
* This is only marginally implemented.
*/
	public Object getActivator(){
		return this;
	}
/**
* Notify an agents clients that there is new product.
* Only used by agents that have output.
* The agents are notified asynchronously.  They
* will be run by the exec at a later time.
*/
	public void notifyMyClients(Task task) {
		dispatcher.notifyMyClients(this, task);
	}
}
