/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;

import java.util.Enumeration;

/**
* Generic interface to access an iPAM standard product. 
* ContainerProducts are Products that contain Product.<p>
*
* Note that there is no explicit enumeration method
* because when you call getContent() (a method of the parent class)
* you will get an enumeration.
*
* @see Product
*
* @author Nazario Irizarry, Jr.
*/
public interface ContainerProduct extends Product{
/**
* Get size of vector
*/
	public int size();
/**
* Add to body of container.
*/
	public void addElement(Product element);
/**
* Remove from body of container.
*/
	public void removeElement(Product element);
/**
* is the container empty?
*/
	public boolean isEmpty();
/**
* Do we have the specified element in the container?
*/
	public boolean containsElement(Product element);

/**
* Add an item that has a particular role.
* @deprecated
*/
	public void addElement(String role, Product element);

/**
* Add a root item to the contents of container.
* A root item is the top of the information hierarchy for
* hierachical info.
*/
	public void addRootElement(Product element);
/**
* Return the root of the product if it exists.
*/
	public Product getRoot();
/**
* Tells containers to eliminate branches that have no children
* @return true if this ContainerProduct should be kept; false if it should be removed
*/
	public boolean prune();
/**
* Get n'th element
*/
	public Product elementAt(int n);
}
