/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;

import java.io.File;
import java.io.IOException;
import javax.servlet.http.HttpServlet;

/**
* Methods that may be invoked
* by a controller of the executive <p>
*
* @author Nazario Irizarry, Jr.
*/
public interface ExecToController extends ExecToEditor {

/**
* tell exec to checkpoint
*/
	void checkPoint();
/**
* Halt the system permanently, causing a checkpoint.
*/
	void halt();
/**
* Modifies the runtime environment from a pdif string
* @return null if successfull
* @return error string if not successfull
*/
	public String modifyFromPDIFString (String modString);
/**
* Looks through the log file to see if there are
* any new exceptions.  An exception is considered new if its time is recent
* based on the milliSecondsBack. Exceptions that are new are recorded in file whose handle
* is also supplied as an argument.
*/
	public void filterExceptions(long milliSecondsBack, File file);
/**
* Performs two layer filtering of the log file in order to
* keep the file manageable.  Two intervals (in units of days)
* are provided as arguments in order to perform
* the filtering.  Any entry whose date is before the eraseAllBeforeDays interval is disgarded, and any
* entry that is before the eraseAllButExceptionsBeforeDays interval is also disgarded UNLESS the entry
* is for an exception or assertion.  All entries that are made after the eraseAllButException
* date are saved.
*/
  public void filterLog(int eraseAllBeforeDays, int eraseAllButExceptionsBeforeDays);
/**
* Define a URL path for a servlet
*/
	public void addServlet(String path, HttpServlet servlet);
/**
* Start listening for connections web service
* @exception IOException if the port is in use or if there is some other problem
*/
	public void listenForWebConnections() throws IOException;
/**
* Gets the default checkpoint file
*/
	public File getDefaultCheckPointFile();
/**
* Add an admin tool
* @param toolClassName full java class name for a tool that implements org.mitre.pam.interfaces.AdminTool
* @param displayName the short string to be used on the tool picker's gui
*/
	public void addAdminTool(String toolClassName, String displayString);
/**
* Show or hide the tool picker
*/
	public void setToolPickerVisibility(boolean visible);
/**
* Start exec from a given configuration file.
* @param configFile the plain text file with the configuration
* @param useGui false to disable all gui's
* @param commandLineArgs are the command line arguements passed to main() 
*/
	public void startFromConfig (File configFile, boolean useGui, String[] commandLineArgs);
/**
* Checks one directory to see if it exsits as an iPAM support directory and has
* a minimum number of files.
*
* @param dirName the simple name of the directory to check
* @param count the minimum number of files that must exist
* @exception Exception if there is something wrong -- the exception message explains why
*/
	public void isAValidSupportDir( String dirName, int count) throws Exception;
}
