/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;

import java.util.Hashtable;
/**
* Methods useful for 
* editors of the agent configuration. <p>
*
* @author Nazario Irizarry, Jr.
*/
import org.mitre.pam.store.IntegrityException;

public interface ExecToEditor extends ExecToMonitor {
	static final long serialVersionUID = -5398274395395704408L;
	/**
	* Create a node if it does not exist, modify it if it does. 
	*
	* @param nodeType kind of agent
	* @param nodeId of agent
	* @param attributes may be null to start with default properties
	*
	* @exception IntegrityException may be generated if a node of the same id already exists
	*/
	public void declareNode(String nodeType, String nodeId, Hashtable attributes) 
			throws IntegrityException;
			
	/**
	* Delete a node (module).  First disable node, then wait for it to go idle
	* before deleting and disconnecting it.
	* @exception IntegrityException reserved for future, currently, none is thrown
	*/
	public void deleteNode(String nodeId) 
			throws IntegrityException;

	/**
	* Connect nodes.
	* @exception IntegrityException if source or sink can't be found
	*/
	public void connectNodes(String fromId, String toId) 
			throws IntegrityException;

	/**
	* Disconnect two nodes (modules).<p>
	*
	* @exception IntegrityException thrown if the fromId is not a Source or toId is not a Sink
	*/
	public void disconnectNodes(String fromId, String toId) 
			throws IntegrityException;

	/**
	* Delete all nodes (modules) and their connections.  The nodes are first disabled and then
	* allowed to go idle. <p>
	*
	* @exception IntegrityException is currently a placeholder (not really generated)
	*/
	public void deleteAll() 
			throws IntegrityException;

	/**
	* Delete all nodes (modules) and their connections except for daemons.
	* The nodes are first disabled and then
	* allowed to go idle. <p>
	*
	* @exception IntegrityException is currently a placeholder (not really generated)
	*/
	public void deleteAllButDaemons() 
			throws IntegrityException;

	/**
	* Delete all nodes (modules) that have the specified attribute with the specified
	* value.  This allows the editor to set up logical groups of various kinds.
	* The nodes are first disabled and then
	* allowed to go idle. <p>
	*
	* @exception IntegrityException is currently a placeholder (not really generated)
	*/
	public void deleteGroup(String attribute, String value) 
			throws IntegrityException;

	/**
	* Enable or disable all nodes (modules) that have the specified 
	* attribute with the specified
	* value.  This allows the editor to enable or disable all members of a group. <p>
	*
	* @param attribute the name of the module attribute that defines the group
	* @param value the value of the module attribute that defines the group
	* @param enableState true to enable; false to disable
	*
	*/
	public void setGroupEnabled(String attribute, String value, boolean enableState);

	/**
	* Enable or disable a list of nodes, probably in preparation for deletes.
	* All nodeIds must exist in order for any to be disabled.
	*
	* @exception IntegrityException if one of the nodeIds does not exist.
	*/
	public void setNodesEnabled(String nodeIds[], boolean enabled) 
			throws IntegrityException;
	/**
	* Allows a controller to define aliases for pims, dims, or jugglers
	*
	* @param moduleType PIM, DIM, juggler, daemon
	* @param alias the name to be used as an alias (has pdif visibility)
	* @param javaClassName must match the javaClassName, used for dynamic loading
	*/
	public void defineModuleAlias(String alias, String javaClassName);



	/**
	* Add listener to monitor config events
	*/
	public void addConfigurationListener(ConfigurationListener listener);
	/**
	* Remove listener 
	*/
	public void removeConfigurationListener(ConfigurationListener listener);
}
