/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;
import org.mitre.pdif.eventParser.ValueList;
import java.util.Enumeration;

/**
* Generic interface describing an object with attributes. <p>
*
* @author Nazario Irizarry, Jr.
*/
public interface HasProperties {
/**
* Get a property
*/
	public String getProperty(String propertyName);
/**
* Set a property
*/
	public void setProperty(String propertyName, String value);
/**
* Remove a property without regard for other remote
* copies of the data
*
*/
	public void removeProperty(String property);
/**
* Remove a property but tag it so that remote, dependent
* data copies can synchronize with this deletion
*
*/
	public void removePropertyAndRemoteCopies(String property);
/**
* Get a property
*/
	public ValueList getPropertyValues(String propertyName);
/**
* Set a property
*/
	public void setPropertyValues(String propertyName, ValueList values);
/**
* Get a property list
*/
	public Enumeration getProperties();
/**
* Get a boolean-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	public boolean getBooleanProperty(String property, boolean defaultValue);
/**
* Get a single string-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public String getStringProperty(String property, String defaultValue);
/**
* Get a single integer-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public int getIntegerProperty(String property, int defaultValue);
/**
* Get a single long-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public long getLongProperty(String property, long defaultValue);
/**
* Get a single double-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public double getDoubleProperty(String property, double defaultValue);

/**
* Set a boolean-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return true or false, if the property does not exist false is returned
*/
	public void setProperty(String property, boolean value);
/**
* Set a single integer-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public void setProperty(String property, int value);
/**
* Set a single double-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public void setProperty(String property, double value);
/**
* Set a single long-valued property
* @param property name of the property
* @param defaultValue value to use if the property does not exist
* @return the value, if the property does not exist false is returned
*/
	public void setProperty(String property, long value);
/**
* @return true if the property exists
*/
	public boolean isProperty(String propertyName);
/**
* @return 0 if property does not exist or is an empty valuelist
* @return 1 if string or valuelist with 1 object
* @return >1 if valuelist with more than one value
*/
	public int getCurrentDimension(String propertyName);
/**
* Copy the property and value potentially with a different name
*/
	public void copyPropertyFromAs(String fromName, HasProperties fromOther, String newName);
/**
* Get a generic object that holds the value of the key.
* @return an object of String, Numeric, or ValueList type, or null
*/
	public Object getPropertyAsObject(String property);
}
