/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.interfaces;
import org.mitre.pdif.eventParser.ValueList;

/**
* Identifies basic information about agent fields.
*/
public interface PropertyMetaData {
    /** Constant for Boolean datatype */
    public final static Class BOOLEAN = Boolean.TYPE;

    /** Constant for Integer datatype */        
    public final static Class INTEGER = Integer.TYPE;
    
    /** Constant for Long datatype */
    public final static Class LONG = Long.TYPE;
    
    /** Constant for Float datatype */
    public final static Class FLOAT = Float.TYPE;
    
    /** Constant for Double datatype */    
    public final static Class DOUBLE = Double.TYPE;
    
    /** Constant for String datatype */
    public final static Class STRING = "".getClass();
/**
* @eturn the field name for a given index
*/
    public String getName();
/**
* @return true if the field can have multiple values simultaneously
*/
    public boolean isMultivalued();
/**
* @return a java.Class object that represents the type of the object.
* If it is multivalued then this is the type of its values
*/
    public Class getDataType();
/**
* @return an object that is multivalued, may return null
*/
    public Object[] getDefaultValues();
/**
* @return either a String or a Number or a Boolean type
*/
    public Object getDefaultValue();
/**
* @return an array of two elements or a multiple of two elements.
* The first element of each pair is the lower end of the range (inclusive).
* The second element of each pair is the upper end of the range (inclusive).
* @return null if there is no specified range.
*/
    public Object[] getRange();
/**
* @return An array of the allowed values.
* @return null if the property does not have enumerated values.
*/
    public Object[] getEnumeratedRange();
/**
* @return A string with a help message.
*/
    public String getShortHelpString();
/**
* @return true if the field value should not be displayed during editing
*/
    public boolean isPassword();
/**
* @return true if the field has value(s) from an enumerated range
*/
	public boolean hasEnumeratedRange();
}