/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.monitor;


/**
 * This class shows a dialog with any number of text fields in it (up to 100)
 * the dialog has ok and cancel buttons on it, and returns an array which is the
 * contents of each of the text fields.
 */
import java.awt.Label;
import java.awt.TextField;
import java.awt.Button;
import java.awt.Panel;
import java.awt.Frame;
import java.awt.Component;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Font;
import java.awt.Color;
import java.util.Vector;

public class MultipleTextBoxDlg extends java.awt.Dialog implements java.awt.event.ActionListener, java.awt.event.KeyListener {
	
	private String[] returnVal = null;
	private Vector labels = new Vector();
	private Vector txtFields = new Vector();
	private int numFields = 0;
	private Button OKButton = new Button("OK");
	private Button CancelButton = new Button("Cancel");

	private Font myFont;
/**
 * creates a modal dialog over the specified frame
 */
private MultipleTextBoxDlg(java.awt.Frame parent, String title) {
	super(parent, title, true);
	myFont = new Font("Dialog",Font.PLAIN,12);
	setFont(myFont);
	setBackground(Color.white);
	setLayout(new GridBagLayout());

}
	public void actionPerformed(java.awt.event.ActionEvent e) {
		if (e.getSource() == OKButton) {
			returnVal = new String[txtFields.size()];
			for (int i = 0; i < txtFields.size(); i++) {
				returnVal[i] = ((TextField)txtFields.elementAt(i)).getText();
			}	
		}	else {
			// must be cancel
			returnVal = null;
		}	
		setVisible(false);
	}	
	
   public void keyPressed(java.awt.event.KeyEvent e) {
   	// do nothing..
   }
   
   public void keyReleased(java.awt.event.KeyEvent e) {
		//System.out.println("Key Released"+e);
		if (e.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER) {
			//System.out.println("got Return");
			returnVal = new String[txtFields.size()];
			for (int i = 0; i < txtFields.size(); i++) {
				returnVal[i] = ((TextField)txtFields.elementAt(i)).getText();
			}
			setVisible(false);			
		}		
   }
   
	public void keyTyped(java.awt.event.KeyEvent e) {
   	// do nothing...
	}
	
	
	private String[] showDialog(String messages[], String defTxt[], int xpos, int ypos) {
		
		int firstEmptyField = -1;
		
		/*
		* modified by naz (5/26/1998)
		* needed to make NEW gridBagConstraints for every item, can not share
		*/
		for (int i = 0; i < messages.length; i++) {
			TextField tmpTxtFld;
			Label tmpLabel;
			if (defTxt == null || defTxt.length <= i || defTxt[i]==null || defTxt[i].equals("")) {
				tmpTxtFld = new TextField(30);
				if (firstEmptyField == -1)
					firstEmptyField = i;
			} else
				tmpTxtFld = new TextField(defTxt[i], 30);
			tmpLabel = new Label(messages[i], Label.RIGHT);

			tmpLabel.setFont(myFont);
			add(tmpLabel, newLeftConstraint(i));
			labels.addElement(tmpLabel);
			
			tmpTxtFld.setFont(myFont);
			add(tmpTxtFld, newRightConstraint(i));
			txtFields.addElement(tmpTxtFld);
		}	
		
	/*
	* Add bottom panel with ok cancel buttons
	*/
		GridBagConstraints constraints = new GridBagConstraints();
		OKButton.setSize(66, 30);
		OKButton.addActionListener(this);	
		CancelButton.setSize(66, 30);
		CancelButton.addActionListener(this);
		Panel bottomPanel = new Panel();
		bottomPanel.add(OKButton);
		bottomPanel.add(CancelButton);
		constraints.fill = GridBagConstraints.NONE;
		constraints.gridx = 0;
		constraints.gridy = messages.length;
		constraints.gridwidth = 2;
		constraints.gridheight = 1;
		constraints.anchor = GridBagConstraints.CENTER;
		add(bottomPanel, constraints);
		
		// added by Seenu (4/22/98) to handle the return key
		if (messages.length > 0)
			((Component)txtFields.elementAt(messages.length-1)).addKeyListener(this);
		//
		
	/*
	* set resizable must be done after the pack() to keep solaris happy
	* otherwise the pack() will not work
	*/
		pack();
		setResizable(false);

		if (firstEmptyField != -1) {
			((TextField)txtFields.elementAt(firstEmptyField)).requestFocus(); 
		} else {
			OKButton.requestFocus();
		}
		setLocation (xpos - getSize().width/2, ypos - getSize().height/2);
		setVisible(true); // blocks
		return returnVal;
	}
	
	private GridBagConstraints newLeftConstraint(int pos) {
		GridBagConstraints leftConstraints = new GridBagConstraints();
		leftConstraints.fill = GridBagConstraints.HORIZONTAL;
		leftConstraints.weightx = 1.0;
		leftConstraints.weighty = 1.0;
		leftConstraints.gridy = 0;
		leftConstraints.gridwidth = 1;
		leftConstraints.gridheight = 1;
		leftConstraints.anchor = GridBagConstraints.EAST;
		leftConstraints.insets.left = 3;
		leftConstraints.insets.right = 3; 
		leftConstraints.insets.top = 1; 
		leftConstraints.insets.bottom = 1; 
		leftConstraints.gridx = 0;
		leftConstraints.gridy = pos;
		return leftConstraints;
	}

	private GridBagConstraints newRightConstraint(int pos) {
		GridBagConstraints rightConstraints = new GridBagConstraints();
		rightConstraints.fill = GridBagConstraints.HORIZONTAL;
		rightConstraints.weightx = 5.0;
		rightConstraints.weighty = 1.0;
		rightConstraints.gridy = 0;
		rightConstraints.gridwidth = 1;
		rightConstraints.gridheight = 1;
		rightConstraints.anchor = GridBagConstraints.WEST;
		rightConstraints.insets.left = 3;
		rightConstraints.insets.right = 3;
		rightConstraints.insets.top = 1; 
		rightConstraints.insets.bottom = 1; 
		rightConstraints.gridx = 1;
		rightConstraints.gridy = pos;
		return rightConstraints;
	}

	public void dispose() {
		OKButton.removeActionListener(this);
		CancelButton.removeActionListener(this);
		super.dispose();
	}
	
	public static String[] showMultipleTextBoxDlg(Frame targetComp, String msgs[], String defTxt[], String title) {
		MultipleTextBoxDlg mtxtdlg = new MultipleTextBoxDlg((Frame)targetComp, title);
		
/*
		while (!(targetComp instanceof Frame))
			targetComp = (Component)targetComp.getParent();
		if (!(targetComp instanceof Frame)) {
			System.err.println("MultipleTextBoxDlg oops- couldn't find parent frame");
			return null;
		}	
*/
		int xpos = targetComp.getLocation().x + targetComp.getSize().width / 2;
		int ypos = targetComp.getLocation().y + targetComp.getSize().height / 2;
//		mtxtdlg.setLocation(xpos, ypos);

		String[] answer = mtxtdlg.showDialog(msgs, defTxt, xpos, ypos); // this blocks
		mtxtdlg.dispose();
		return answer;
	}

	public static String[] showMultipleTextBoxDlg(Frame targetComp, String msgs[], String title) {
		return showMultipleTextBoxDlg(targetComp, msgs, null, title);
	}	
	
	public static String showTextBoxDlg(Frame targetComp, String msg, String defTxt) {
		String msgs[] = new String[1];
		String defTxts[] = new String[1];
		
		msgs[0] = msg;
		defTxts[0] = defTxt;
		String[] answers = showMultipleTextBoxDlg(targetComp, msgs, defTxts, "?");
		return (answers != null) ? answers[0] : null;
	}
}