/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.product.view;

import java.util.*;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.ProductFilter;

import org.mitre.pam.store.FileRef;
import org.mitre.pdif.eventParser.PDIFWriter;
import org.mitre.pam.executive.ExecGlobal;


/**
* An enumerated product contains other products.<p>
*
* @author Nazario Irizarry, Jr.
*/
abstract public class EnumeratedProduct extends ProductShell implements ContainerProduct {
/**
* only need id and description to have a product of some sort
*/
	public EnumeratedProduct (String id, String description) {
		super(id, description);
	}
/**
* used only by persistence subsystem
*/
	protected EnumeratedProduct () {
		super();
	}
/**
* 
*/
	protected void loadContent(String attribute, String value) {
	}
/**
* An enumerated product has no immediate content but instead has
* references to other products.  Here we don't output content
* to the the PDIF output stream but we do register our references
* to other products.
*/
	protected void storeContent(PDIFWriter pdifOut) {
		Enumeration e = (Enumeration) getContent();
		if (!e.hasMoreElements())
			return;
					
		while (e.hasMoreElements()) {
			ProductShell prod = (ProductShell)e.nextElement();
			assert(prod != null, "EnumeratedProduct.storeContent found null in "+getUid()+" "+getId());
		}
	}
	
	

/*
* Return an enumeration that gives the index of the product if they exist.
* This was an experimental feature that never got used.
*/
/*
	Enumeration getIndexItems() {
		Vector index = new Vector();
		Enumeration e = (Enumeration) getContent();
		while(e.hasMoreElements()) {
			Product prod = (Product) e.nextElement();
			if ( prod.getProperty(PROPERTY_INDEX) != null)
				index.addElement(prod);
		}
		return index.elements();
	}
*/
/**
* Return the root of the product if it exists.
*/
	public Product getRoot(){
		Enumeration e = (Enumeration) getContent();
		while(e.hasMoreElements()) {
			Product prod = (Product) e.nextElement();
			if ( prod.getProperty(PROPERTY_ROOT) != null)
				return prod;
		}
		return null;
	}

/**
* @return Enumeration of PDIFItems or null
*/
	public Enumeration getRelatedPDIFItems(){
		return (Enumeration) getContent();
	}
/*
*
*/
	public String toString() {
		StringBuffer out = new StringBuffer(1000);
		out.append("\n(");
		out.append(getId());
		out.append(" ");
		out.append(super.toString());
		Enumeration e = (Enumeration) getContent();
		while(e.hasMoreElements()) {
			Product p = (Product)e.nextElement();
			out.append(p.toString());
		}
		out.append(")");
		return out.toString();
	}
/**
* A product that contains other products
*/
	public boolean canContainProducts(){
		return true;
	}

}
