/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.putter;

import java.util.Hashtable;
import java.util.Enumeration;

import java.io.File;

import org.mitre.pam.interfaces.*;

import org.mitre.pam.executive.ExecGlobal;

import org.mitre.pam.executive.Module;
import org.mitre.pam.util.EnabledTable;
import org.mitre.pam.executive.Command;

/**
* An abstract class designed use
* as a DIM superclass.  <P>
*
* @author Nazario Irizarry, Jr.
*/
abstract public class DIMShell1 extends Module implements DIM {
	
/**
* List of sources
*/
	protected EnabledTable sourceList = new EnabledTable();
	
	public DIMShell1 () {
		setProperty(HAS_INPUTS,"true");
		setProperty(Schedulable.HAS_SCHEDULE, "false");
	}
/**
* @deprecated
*/
	final public void doCommand(String command, Object arg){
	}
/**
* @deprecated
*/
	final protected void doSend(Source theSource){};
/**
* Callback to receive a command.  There
* is only one thing we can do - send product.
*/
	public synchronized void doCommittedTask(){
		Task task = taskHelper.getCommittedTask();
		String command = task.getCommand();
		if ( command == Command.SEND_PRODUCT) {
			declareStart(null);
			try {
				pleasePutProduct(task.getProduct(), task);
			}
			catch (Exception e) {
				reportException(e);
			}
			declareEnd(false);
		}
		else {
			assert(false,"DIMShell1 received unknown command ",command,null,null);
		}
	}
/**
* identify that we are starting
*/
	public void declareStart(String ignored) {
		String state = getState();
		if ((state == ExecGlobal.IDLE || state == ExecGlobal.COMMITTED) && logActivity) {
			reporter.log("begin dim");
		}
		setState(ExecGlobal.WORKING);
	}
/**
* set the conditions at end of update
*/
	public void declareEnd(boolean ignored) {

		if (logActivity) {
			reporter.log("end dim");
		}
		setState(ExecGlobal.IDLE); //clears committed task
	}
	
/**
* @return a table of enabled sources.
*/
	public Hashtable getSourceList(){
		return sourceList;
	}
	
/**
* get enumeration of all Sources including disabled items
*/
	public Enumeration getFullSourceElementEnumeration(){
		return sourceList.allElements();
	}
/**
* set the source to get 1 page from
*/
	final protected void setTargetSpec(String url){
//		setProperty(TARGET_SPEC, url);
		throw new RuntimeException("don't use");
	}
/**
* get the source url
*/
	protected String getTargetSpec(){
//		return getProperty(TARGET_SPEC);
		throw new RuntimeException("don't use");
	}
/**
* Make a string that gets appended to the assertion, log, exception, etc to explain
* what the agent was doing.
*/
	final protected String makeDoingWhatWhenErrorString() {
		Task task = taskHelper.getCommittedTask();
		if (task != null )
			return "(on DIM: "+getId()+", on task "+task.toString()+")";
		else
			return "(on DIM: "+getId()+", on no task invocation)";
	}
/**
* An activator is the thing that gets an agent to do something.  
* A DIM has no activator, always returns null
* This is only marginally implemented.
*/
	public Object getActivator(){
		return null;
	}
/**
* Checks the specified putToken
* to see if it is valid, before putting the product.
* This prevents asyncrhonouse sources from sneaking product
* in while the sink is in the middle of another put or task.
*
* @param product to put
* @param the token that grants authorization to put
*/
	final public void pleasePutProduct(Product product, Task token){
		taskHelper.pleasePutProduct(product, token);
	}
}
