/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
/*--------------------------------------------------------------------------
  Copyright (c) 1998 by The MITRE Corporation.  All rights reserved.
--------------------------------------------------------------------------*/
package org.mitre.pam.rmi.editor;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.Naming;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.ServerException;

import org.mitre.pam.interfaces.AsynchronousAgent;
import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.interfaces.ScheduleAttributes;
import org.mitre.pam.daemon.DaemonBase;
import org.mitre.pam.executive.ExecGlobal;


/**
 * This daemon instantiates an instance of an Agent Editor RMI
 * service object.
 *
 * @author Michael E. Los, MITRE
*/
public class EditorDaemon extends DaemonBase implements AsynchronousAgent
{
  //--------------------------------------------
  // PUBLIC STATIC DATA MEMBERS
  //--------------------------------------------
  public static final String RMI_OBJ_NAME = "remoteObjectName";

  //--------------------------------------------
  // PRIVATE DATA MEMBERS
  //--------------------------------------------
  private boolean zSvcStarted = false;
  private Object zSvcStartedLock = new Object();
  private String zsBoundRemoteEditorName;

  private Registry zRMIRegistry;

  //--------------------------------------------
  // CONSTRUCTORS
  //--------------------------------------------
  public EditorDaemon()
  {
    setProperty(RMI_OBJ_NAME, "remoteAgentEditor");
    setDescription("Implement RMI Access to Editor Function");
	setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_NONE);
  }

  //--------------------------------------------
  // PUBLIC METHODS THAT IMPLEMENT AsynchronousAgent
  //--------------------------------------------
  public void startService()
  {
    synchronized (zSvcStartedLock)
    {
      if (!zSvcStarted)
      {
        try
        {
          setProgress("Starting RMI Agent Editor Service");
          // search for the registry in which the remote object will
          // be advertised.
          zRMIRegistry = LocateRegistry.getRegistry();

          // Instantiate remote object
          RmtExecToEditorImpl remoteEditor = new RmtExecToEditorImpl((ExecToEditor)exec, reporter);

          zsBoundRemoteEditorName = getProperty(RMI_OBJ_NAME);
          setProgress("About to bind to " + zsBoundRemoteEditorName);
          try {
            // Bind the remote object with the registry
            zRMIRegistry.rebind(zsBoundRemoteEditorName, remoteEditor);
          } catch (Exception se)
          {
            se.printStackTrace();
            reportException(se);
            System.out.println("Editor Daemon: try restarting rmiregistry");
          }
          setProgress("Finished binding");

          zSvcStarted = true;
        }
        catch( Exception e )
        {
          e.printStackTrace();
          reportException(e);
        }
      }
      setProgress("");
    }//end synch block
  }//startService()

  public void stopService()
  {
    synchronized (zSvcStartedLock)
    {
      try
      {
//System.out.println("Stop service called");
        // unbind the remote object from the registry
        if (zsBoundRemoteEditorName != null)
        {
          setProgress("about to unbind: " + zsBoundRemoteEditorName);
          zRMIRegistry.unbind(zsBoundRemoteEditorName);
        }
      }
      catch (AccessException ae)
      { reportException(ae); }
      catch (NotBoundException nbe)
      { reportException(nbe); }
      catch (RemoteException re)
      { reportException(re); }
      finally {
        zSvcStarted = false;
      }
    }//end synch block
  }//stopService()

  public void pauseService()
  {
  }

  public void resumeService()
  {
  }

  /**
   * This daemon
  */
  public void doDaemonTask()
  {
	}

/**
* Can't set a schedule on this one.
*/
	public void setProperty(String property, String value) {
		if (property.equals(ScheduleAttributes.SCHEDULE_VERB)){
			return;
		}
		super.setProperty(property, value);
	}
}