/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
/*--------------------------------------------------------------------------
  Copyright (c) 1998 by The MITRE Corporation.  All rights reserved.
--------------------------------------------------------------------------*/
package org.mitre.pam.rmi.editor;

import java.rmi.RemoteException;
import java.util.Hashtable;

/**
 * Implementations of this interface are typically used to get
 * information about the agents in the system.<p>
 *
 * @see         org.mitre.pam.interfaces.ExecToMonitor
 *
 * @author Michael E. Los, MITRE, D510
 * @author Nazario Irizarry, Jr., MITRE
*/
public interface RemoteExecToMonitor extends RemoteExecCallbacks
{
  /**
   * Returns an array of the identifiers of all active agents
   * <P>
   *
   * @return  Array of String objects that denote the
   *          identifiers of active agents
   * @exception   RemoteException
   */
  public String [] getActiveAgentIDs()
                      throws RemoteException;

  /**
   * Returns an array of Hashtable objects that represent
   * the active agents.
   * <P>
   *
   * @return  Array of Hashtable objects that contain the
   *          attribute-value pairs for each of the
   *          active agents
   * @exception   RemoteException
   */
  public Hashtable [] getActiveAgents()
                      throws RemoteException;

  /**
   * Returns an array of the identifiers of all active agents
   * that contain the given attribute-value pair.
   * <P>
   * @param attribj  agent attribute anme
   * @param value    value associated with the agent attribute
   *
   * @return  Array of String objects that denote the
   *          identifiers of active agents
   * @exception   RemoteException
   */
  public String [] getActiveAgentIDs(String attrib, String value)
                      throws RemoteException;
  /**
   * Returns an array of Hashtable objects that represent
   * the active agents that contain the given attribute-value pair.
   * <P>
   * @param attribj  agent attribute anme
   * @param value    value associated with the agent attribute
   *
   * @return  Array of Hashtable objects that contain the
   *          attribute-value pairs for each of the matching
   *          active agents
   * @exception   RemoteException
   */
  public Hashtable [] getActiveAgents(String attrib, String value)
                      throws RemoteException;
  /**
   * Returns an array of the identifiers of all available agents
   * <P>
   *
   * @return  Array of String objects that denote the
   *          identifiers of available agents
   * @exception   RemoteException
   */
  public String [] getAvailableAgentIDs()
                      throws RemoteException;

  /**
   * Returns an array of Hashtable objects that represent
   * the available agents.
   * <P>
   *
   * @return  Array of Hashtable objects that contain the
   *          attribute-value pairs for each of the
   *          available agents
   * @exception   RemoteException
   */
  public Hashtable [] getAvailableAgents()
                      throws RemoteException;

  /**
   * Returns an array of the identifiers of the agents for
   * which the specified agent is a sink. (i.e., This routine
   * returns the identifiers of those agents that serve as sources
   * for the specified agent.)
   * <P>
   * @param agentID  agent identifier
   *
   * @return an array of String objects that contain
   *         the identifiers of the source agents.
   *         This routine never returns null.
   * @exception   RemoteException
   */
  public String [] getAgentSourceIDs(String agentId)
                      throws RemoteException;

  /**
   * Returns an array of the identifiers of the agents for
   * which the specified agent is a source. (i.e., This routine
   * returns the identifiers of those agents that serve as sinks
   * for the specified agent.)
   * <P>
   * @param agentID  agent identifier
   *
   * @return an array of String objects that contain
   *         the identifiers of the sink agents.
   *         This routine never returns null.
   * @exception   RemoteException
   */
  public String [] getAgentSinkIDs(String agentId)
                      throws RemoteException;
  /**
   * Retrieves the attribute-value pairs that describe the
   * active agent associated with the given identifier.
   * <P>
   * @param agentID  agent identifier
   * @return a Hashtable object containing the attribute-value pairs.
   *         If the agent could not be found, null is returned.
   * @exception   RemoteException
   */
  public Hashtable getActiveAgent(String agentId)
                      throws RemoteException;

  /**
   * Retrieve the attribute-value pairs that describe the
   * available agent (unused,unistantiated) associated with
   * the given identifier.
   * <P>
   * @param agentID  agent identifier
   * @return a Hashtable object containing the attribute-value pairs.
   *         If the agent could not be found, null is returned.
   * @exception   RemoteException
   */
  public Hashtable getAvailableAgent(String agentId)
                      throws RemoteException;

  /**
   * Checks if the agent associated with the given identifer
   * is active.
   * <P>
   * @param agentID  agent identifier
   * @return        true if the agent is active; false, otherwise
   * @exception   RemoteException
   */
  public boolean isActiveAgent(String agentId)
                      throws RemoteException;

  /**
   * Checks if the agent associated with the given identifer
   * is available.
   * <P>
   * @param agentID  agent identifier
   * @return        true if the agent is available; false, otherwise
   *
   * @exception   RemoteException
   */
  public boolean isAvailableAgent(String agentId)
                      throws RemoteException;
}