/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
/*--------------------------------------------------------------------------
  Copyright (c) 1998 by The MITRE Corporation.  All rights reserved.
--------------------------------------------------------------------------*/
package org.mitre.pam.rmi.editor;

import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;

import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.interfaces.Reporter;


public class RmtExecCallbacksImpl extends UnicastRemoteObject
                                    implements RemoteExecCallbacks
{
	private static final String DEBUG_KEY = "remoteExecCalls";
  //--------------------------------------------
  // PRIVATE DATA MEMBERS
  //--------------------------------------------
  ExecToEditor exec;
  Reporter reporter;

  //--------------------------------------------
  // CONSTRUCTORS
  //--------------------------------------------
  public RmtExecCallbacksImpl(ExecToEditor exec, Reporter reporter) throws RemoteException
  {
  	this.exec = exec;
  	this.reporter = reporter;
  }//RemoteExecCallbacksImpl()

  //--------------------------------------------
  // PUBLIC METHODS IMPLEMENTED FOR RemoteExecCallbacks
  //--------------------------------------------
  /**
  * Returns the build number of the running executive.
  * These numbers are guaranteed
  * to increase as new versions of the exec come out.  Users
  * are encouraged to use whichNumber==0
  *
  * @param whichNumber 0 - get the build number
  * @param whichNumber 1 - get the major version
  * @param whichNumber 2 - get the minor version
  *
  * @return       integer denoting the build number
  * @exception    java.rmi.RemoteException
  */
  public int getBuildNumber(int whichNumber)
                      throws RemoteException
  {
  	try {
	  	debug("getBuildNumber called");
	    return exec.getBuildNumber(whichNumber);
	} catch (Exception e) {
		reporter.reportException(e);
	}
		return 0;
  }

	protected void debug(String msg1) {
  		reporter.debugMsg(msg1,null,null,null, DEBUG_KEY);
	}
	protected void debug(String msg1, Object msg2) {
  		reporter.debugMsg(msg1,msg2,null,null, DEBUG_KEY);
	}
	protected void debug(String msg1, Object msg2, Object msg3) {
  		reporter.debugMsg(msg1,msg2,msg3,null, DEBUG_KEY);
	}
}