/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
/*--------------------------------------------------------------------------
  Copyright (c) 1998 by The MITRE Corporation.  All rights reserved.
--------------------------------------------------------------------------*/
package org.mitre.pam.rmi.editor;

import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.ExecToEditor;

public class RmtExecToMonitorImpl extends RmtExecCallbacksImpl
                                    implements RemoteExecToMonitor
{
  //--------------------------------------------
  // PRIVATE DATA MEMBERS
  //--------------------------------------------

  //--------------------------------------------
  // CONSTRUCTORS
  //--------------------------------------------
  public RmtExecToMonitorImpl(ExecToEditor exec, Reporter reporter) throws RemoteException
  {
  	super(exec,reporter);
  }//RemoteExecToMonitorImpl()

  //--------------------------------------------
  // PUBLIC METHODS
  //--------------------------------------------

  //--------------------------------------------
  // PUBLIC METHODS IMPLEMENTED FOR RemoteExecToMonitor
  //--------------------------------------------
  /**
   * Returns an array of the identifiers of all active agents
   * <P>
   *
   * @return  Array of String objects that denote the
   *          identifiers of active agents
   * @exception   RemoteException
   */
  public String [] getActiveAgentIDs()
                      throws RemoteException
  {
  	try {
  		debug("getActiveAgentIDs begin");
		Vector agentVector = new Vector();
		Enumeration enum;
		
		// Temporarily store the agents in a vector so we can
		// determine how many there are
			for (int i = 0; i<= 3; i++) {
		  enum = exec.getListOfActiveModules(i);
				while (enum.hasMoreElements()) {
					Agent agent = (Agent)enum.nextElement();
		    agentVector.addElement(agent);
		  }//while
		}//for
		
		int numAgents = agentVector.size();
		String [] agentIds = new String[numAgents];
		for (int i = 0; i < numAgents; i++)
		{
		  String agentId = ((Agent) agentVector.elementAt(i)).getId();
		  if (agentId != null)
		    agentIds[i] = agentId;
		  else
		    System.out.println("null agent ID!!");
		}
  		debug("getActiveAgentIDs end");
		return agentIds;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new String[0];
  }//getActiveAgentIDs()

  /**
   * Returns an array of Hashtable objects that represent
   * the active agents.
   * <P>
   *
   * @return  Array of Hashtable objects that contain the
   *          attribute-value pairs for each of the
   *          active agents
   * @exception   RemoteException
   */
  public Hashtable [] getActiveAgents()
                      throws RemoteException
  {
	try {
  		debug("getActiveAgents begin");
		Enumeration enum;
		Vector agentVector = new Vector(50);
		
		// Temporarily store the agents in a vector so we can
		// determine how many there are
			for (int i = 0; i<= 3; i++) {
		  enum = exec.getListOfActiveModules(i);
				while (enum.hasMoreElements()) {
					Agent agent = (Agent)enum.nextElement();
		    agentVector.addElement(agent);
		  }//while
		}//for
		
		int numAgents = agentVector.size();
		Hashtable [] agentInfo = new Hashtable[numAgents];
		enum = agentVector.elements();
		int i = 0;
		while ( enum.hasMoreElements() )
		{
		  // Get the memento assoc. w/ the agent. Memento stores agent properties
		  Memento mem = ((Agent)enum.nextElement()).getMemento();
		  // Query the memento for the property attributes
		  Enumeration propsEnum = mem.getProperties();
		  agentInfo[i++] = copyPropsToHashtable(propsEnum, mem);
		}//for
		
  		debug("getActiveAgents end");
		return agentInfo;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new Hashtable[0];
  }//getActiveAgents()

  /**
   * Returns an array of the identifiers of all active agents
   * that contain the given attribute-value pair.
   * <P>
   * @param attribj  agent attribute anme
   * @param value    value associated with the agent attribute
   *
   * @return  Array of String objects that denote the
   *          identifiers of active agents
   * @exception   RemoteException
   */
  public String[] getActiveAgentIDs(String attrib, String value)
                      throws RemoteException
  {
	try {
		debug("getActiveAgentIDs begin");

	    Vector agentIdVector = new Vector();
	
	    //System.out.println("RemoteObject: about to call getListOfActiveAgents: " + attrib + " " + value);
	
	    // Temporarily store the agent Ids in a vector so we can
	    // determine how many there are
	    Enumeration agentsEnum = exec.getListOfActiveAgents(attrib, value);
	    while ( agentsEnum.hasMoreElements() )
	    {
	      Agent agent = (Agent) agentsEnum.nextElement();
	
	      String agentId = agent.getId();
	      if (agentId != null)
	        agentIdVector.addElement(agentId);
	       else
	        System.out.println("null agent ID!!");
	    }//while
	
	    // Copy elements from the vector into the String array
	    String [] agentIds = copyVectorToStringArray(agentIdVector);
	
		debug("getActiveAgentIDs end");
	    return agentIds;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new String[0];
  }//getActiveAgentIDs

  /**
   * Returns an array of Hashtable objects that represent
   * the active agents that contain the given attribute-value pair.
   * <P>
   * @param attribj  agent attribute anme
   * @param value    value associated with the agent attribute
   *
   * @return  Array of Hashtable objects that contain the
   *          attribute-value pairs for each of the matching
   *          active agents
   * @exception   RemoteException
   */
  public Hashtable [] getActiveAgents(String attrib, String value)
                      throws RemoteException
  {
	try {
		debug("getActiveAgents begin");
	    Enumeration enum;
	    Vector agentVector = new Vector(50);
	
	    // Temporarily store the agents in a vector so we can
	    // determine how many there are
	    Enumeration agentsEnum = exec.getListOfActiveAgents(attrib, value);
	    while ( agentsEnum.hasMoreElements() )
	    {
	      Agent agent = (Agent) agentsEnum.nextElement();
	      agentVector.addElement(agent);
	    }//while
	
	
	    int numAgents = agentVector.size();
	    Hashtable [] agentInfo = new Hashtable[numAgents];
	    enum = agentVector.elements();
	    int i = 0;
	    while ( enum.hasMoreElements() )
	    {
	      // Get the memento assoc. w/ the agent. Memento stores agent properties
	      Memento mem = ((Agent)enum.nextElement()).getMemento();
	      // Query the memento for the property attributes
	      Enumeration propsEnum = mem.getProperties();
	      agentInfo[i++] = copyPropsToHashtable(propsEnum, mem);
	    }//for
	
		debug("getActiveAgents end");
	    return agentInfo;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new Hashtable[0];
  }//getActiveAgents()


  /**
   * Returns an array of the identifiers of all available agents
   * <P>
   *
   * @return  Array of String objects that denote the
   *          identifiers of available agents
   * @exception   RemoteException
   */
  public String [] getAvailableAgentIDs()
                      throws RemoteException
  {
	try {
		debug("getAvailableAgentIDs begin");
	    Vector agentVector = new Vector();
	    Enumeration enum;
	
	    // Temporarily store the agents in a vector so we can
	    // determine how many there are
			for (int i = 0; i<= 3; i++) {
	      enum = exec.getListOfAvailableModules(i);
				while (enum.hasMoreElements()) {
					Agent agent = (Agent)enum.nextElement();
	        agentVector.addElement(agent);
	      }//while
	    }//for
	
	    int numAgents = agentVector.size();
	    String [] agentIds = new String[numAgents];
	    for (int i = 0; i < numAgents; i++)
	    {
	      String agentId = ((Agent) agentVector.elementAt(i)).getId();
	      if (agentId != null)
	        agentIds[i] = agentId;
	      else
	        System.out.println("null agent ID!!");
	    }
		debug("getAvailableAgentIDs end");
	    return agentIds;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new String[0];
  }//getAvailableAgentIDs()

  /**
   * Returns an array of Hashtable objects that represent
   * the available agents.
   * <P>
   *
   * @return  Array of Hashtable objects that contain the
   *          attribute-value pairs for each of the
   *          available agents
   * @exception   RemoteException
   */
  public Hashtable [] getAvailableAgents()
                      throws RemoteException
  {
	try {
		debug("getAvailableAgents begin");
	    Enumeration enum;
	    Vector agentVector = new Vector(50);
	
	    // Temporarily store the agents in a vector so we can
	    // determine how many there are
			for (int i = 0; i<= 3; i++) {
	      enum = exec.getListOfAvailableModules(i);
				while (enum.hasMoreElements()) {
					Agent agent = (Agent)enum.nextElement();
	        agentVector.addElement(agent);
	      }//while
	    }//for
	
	    int numAgents = agentVector.size();
	    Hashtable [] agentInfo = new Hashtable[numAgents];
	    enum = agentVector.elements();
	    int i = 0;
	    while ( enum.hasMoreElements() )
	    {
	      // Get the memento assoc. w/ the agent. Memento stores agent properties
	      Memento mem = ((Agent)enum.nextElement()).getMemento();
	      // Query the memento for the property attributes
	      Enumeration propsEnum = mem.getProperties();
	      agentInfo[i++] = copyPropsToHashtable(propsEnum, mem);
	    }//for
	
		debug("getAvailableAgents end");
	    return agentInfo;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new Hashtable[0];
  }

  /**
   * Returns an array of the identifiers of the agents for
   * which the specified agent is a sink. (i.e., This routine
   * returns the identifiers of those agents that serve as sources
   * for the specified agent.)
   * <P>
   * @param agentID  agent identifier
   *
   * @return an arran of String objects that contain
   *         the identifiers of the source agents.
   *         This routine never returns null.
   * @exception   RemoteException
   */
  public String [] getAgentSourceIDs(String agentId)
                      throws RemoteException
  {
	try {
		debug("getAgentSourceIDs begin");
	    Vector agentIdVector = new Vector();
	
	    Enumeration agentsEnum = exec.getListOfAgentSources(agentId);
	
	    // Temporarily store the agents in a vector so we can
	    // determine how many there are
	    while ( agentsEnum.hasMoreElements() )
	    {
	      Agent agent = (Agent) agentsEnum.nextElement();
	
	      String sourceId = agent.getId();
	      if (agentId != null)
	        agentIdVector.addElement(sourceId);
	       else
	        System.out.println("null agent ID!!");
	    }//while
	
	    // Copy elements from the vector into the String array
	    String [] agentIds = copyVectorToStringArray(agentIdVector);
	
		debug("getAgentSourceIDs begin");
	    return agentIds;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new String[0];
  }//getAgentSourceIDs()

  /**
   * Returns an array of the identifiers of the agents for
   * which the specified agent is a source. (i.e., This routine
   * returns the identifiers of those agents that serve as sinks
   * for the specified agent.)
   * <P>
   * @param agentID  agent identifier
   *
   * @return an array of String objects that contain
   *         the identifiers of the sink agents.
   *         This routine never returns null.
   * @exception   RemoteException
   */
  public String [] getAgentSinkIDs(String agentId)
                      throws RemoteException
  {
	try {
		debug("getAgentSinkIDs begin");
	    Vector agentIdVector = new Vector();
	
	    Enumeration agentsEnum = exec.getListOfAgentSinks(agentId);
	
	    // Temporarily store the agents in a vector so we can
	    // determine how many there are
	    while ( agentsEnum.hasMoreElements() )
	    {
	      Agent agent = (Agent) agentsEnum.nextElement();
	
	      String sinkId = agent.getId();
	      if (agentId != null)
	        agentIdVector.addElement(sinkId);
	       else
	        System.out.println("null agent ID!!");
	    }//while
	
	    // Copy elements from the vector into the String array
	    String [] agentIds = copyVectorToStringArray(agentIdVector);
	
		debug("getAgentSinkIDs end");
	    return agentIds;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new String[0];
  }//getAgentSinkIDs()

  /**
   * Retrieves the attribute-value pairs that describe the
   * active agent associated with the given identifier.
   * <P>
   * @param agentID  agent identifier
   * @return a Hashtable object containing the attribute-value pairs.
   *         If the agent could not be found, null is returned.
   * @exception   RemoteException
   */
  public Hashtable getActiveAgent(String agentId)
                      throws RemoteException
  {
	try {
		debug("getActiveAgent begin");
	    Hashtable agentAttribs = null;
	    Agent agent = exec.getActiveAgent(agentId);
	    if (agent != null)
	    {
	      // Get the memento assoc. w/ the agent. Memento stores agent properties
	      Memento mem = ((Agent)agent).getMemento();
	      // Query the memento for the property attributes
	      Enumeration enum = mem.getProperties();
	      agentAttribs = copyPropsToHashtable(enum, mem);
	    }
		debug("getActiveAgent end");
	    return agentAttribs;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new Hashtable();
  }//getActiveAgent()


  /**
   * Retrieve the attribute-value pairs that describe the
   * available agent (unused,unistantiated) associated with
   * the given identifier.
   * <P>
   * @param agentID  agent identifier
   * @return a Hashtable object containing the attribute-value pairs.
   *         If the agent could not be found, null is returned.
   * @exception   RemoteException
   */
  public Hashtable getAvailableAgent(String agentId)
                      throws RemoteException
  {
	try {
		debug("getAvailableAgent begin");
	    Hashtable agentAttribs = null;
	    Agent agent = exec.getAvailableAgent(agentId);
	    if (agent != null)
	    {
	      // Get the memento assoc. w/ the agent. Memento stores agent properties
	      Memento mem = ((Agent)agent).getMemento();
	      // Query the memento for the property attributes
	      Enumeration enum = mem.getProperties();
	      agentAttribs = copyPropsToHashtable(enum, mem);
	    }
		debug("getAvailableAgent end");
	    return agentAttribs;
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return new Hashtable();
  }

  /**
   * Checks if the agent associated with the given identifer
   * is active.
   * <P>
   * @param agentID  agent identifier
   * @return        true if the agent is active; false, otherwise
   * @exception   RemoteException
   */
  public boolean isActiveAgent(String agentId)
                      throws RemoteException
  {
	try {
		debug("isActiveAgent begin -",agentId);
	    Agent agent = exec.getActiveAgent(agentId);
		debug("isActiveAgent   end -",agentId);
	    return (agent != null ? true : false);
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return false;
  }

  /**
   * Checks if the agent associated with the given identifer
   * is available.
   * <P>
   * @param agentID  agent identifier
   * @return        true if the agent is available; false, otherwise
   *
   * @exception   RemoteException
   */
  public boolean isAvailableAgent(String agentId)
                      throws RemoteException
  {
	try {
		debug("isAvailableAgent begin -",agentId);
	    Agent agent = exec.getAvailableAgent(agentId);
		debug("isAvailableAgent   end -",agentId);
	    return (agent != null ? true : false);
	}
	catch (Exception e) {
		reporter.reportException(e);
	}
	return false;
  }

  //--------------------------------------------
  // PROTECTED METHODS
  //--------------------------------------------
  protected Hashtable copyPropsToHashtable(Enumeration vEnum, Memento vMemento)
  {
    Hashtable h = new Hashtable(vMemento.size());
    while ( vEnum.hasMoreElements() )
    {
      String sKey = (String) vEnum.nextElement();
      Object val = vMemento.getProperty(sKey);
      h.put(sKey, val);
    }//while
    return h;
  }

  protected String [] copyVectorToStringArray(Vector v)
  {
    String [] a = new String[v.size()];
    Enumeration enum = v.elements();
    int i = 0;
    while ( enum.hasMoreElements() )
    {
      a[i++] = (String) enum.nextElement();
    }//while
    return a;
  }//copyVectorToStringArray()

}