/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.store;

import java.util.*;

import java.io.InputStream;
import java.io.OutputStream;

import org.mitre.pam.interfaces.*;
import org.mitre.pam.util.JoinEnumeration;
import org.mitre.pdif.eventParser.PDIFWriter;
import org.mitre.pam.executive.PropertySet;
/**
* Allows a hashtable to extend a memento's contents without needing
* to be a memento.  This is used by channel's that may need to maintain
* additional state that is best kept as a hashtable of objects instead of
* as a hashtable of strings. <p>
*
* @author Nazario Irizarry, Jr.
*/

public class JoinMemento extends PropertySet {
/**
* @param addOn the hashtable that is to "extend" the main memento
*/
	public JoinMemento(String parentKind, String parentId, Hashtable addOn, Reporter reporter) {
		super(parentKind, parentId, reporter);
		this.addOn = addOn;
	}
/**
* The keys enumeration must be the concatenation of both
* sets of keys
*/
	public Enumeration keys(){
		if (addOn != null)
			return new JoinEnumeration(super.keys(), addOn.keys());
		else
			return super.keys();
	}
/**
* The keys enumeration must be the concatenation of both
* sets of keys
*/
	public Enumeration properties(){
		if (addOn != null)
			return new JoinEnumeration(super.getProperties(), addOn.keys());
		else
			return super.getProperties();
	}
/**
* Gets invisibly span both this and the addOn hashtable
*/
	public Object get(Object key) {
		if (super.containsKey(key))
			return super.getProperty(key.toString());
			
		if (addOn != null)
			return addOn.get(key);
			
		return null;
	}
/**
* Get property must span both, we use the toString method on
* whatever element is returned by the addOn hashtable.
*/
	public String getProperty(String key) {
		if (super.containsKey(key))
			return super.getProperty(key);
		
		if (addOn == null)
			return null;
			
		Object property =  addOn.get(key);
		if (property != null)
			return property.toString();
		
		return null;
	}

}

