/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.PropertyMetaData;
import java.util.Enumeration;
import java.util.Vector;


/**
* A structure with field metadata
*/
public class MetaData implements PropertyMetaData {
	String name;
	Object[] defaultValues;
	Object defaultValue;
	String[] enumeratedRange;
	String helpText="";
	
	boolean isPass;
	Enumeration parseEnumeration;
	String peekToken;
	String lastToken;
	boolean multivalued;
	Class dataType;
	public final static MetaData UNSPECIFIED_STRING_META_DATA = new MetaData(1);
	

/**
* Create default string metadata
*/
	private MetaData (int ignore) {
		this.name = "genericString";
		this.defaultValue = "";
		dataType = STRING;
	}
/**
* Create from string representation 
*/
	public MetaData (String name, ValueList vList) {
		if (name == null || name.length() == 0) {
			throw new IllegalArgumentException("null or empty name in constructor");
		}
		this.name = name;
		parseValueList(vList);
	}
/**
* Parse the meta data that is coded into the ValueList object.
* This must match the "coding" used in org.mitre.pam.exec.Module .
*/
	private void parseValueList(ValueList vList) {
		try {
			parseEnumeration = vList.elements();
			while (moreTokens()) {
				String token = nextToken();
//				System.out.println(" token: "+token);
			/*
			* type of parameter
			*/
				if (token.equals(Agent.META_ATTRIBUTE_VALUE_TYPE)) {
					String type = nextToken();
				/*
				* password
				*/
					if (type.equals(Agent.META_ATTRIBUTE_TYPE_PASSWORD)) {
						isPass = true;
						dataType = STRING;
						nextToken(); // eat the algorithm for now
					}
				/*
				* boolean
				*/
					else if (type.equals(Agent.META_ATTRIBUTE_TYPE_BOOLEAN)) {
						dataType = BOOLEAN;
					}
				/*
				* double
				*/
					else if (type.equals(Agent.META_ATTRIBUTE_TYPE_DOUBLE)) {
						dataType = DOUBLE;
					}
				/*
				* integer
				*/
					else if (type.equals(Agent.META_ATTRIBUTE_TYPE_INTEGER)) {
						dataType = INTEGER;
					}
				/*
				* long
				*/
					else if (type.equals(Agent.META_ATTRIBUTE_TYPE_LONG)) {
						dataType = LONG;
					}
				/*
				* string
				*/
					else if (type.equals(Agent.META_ATTRIBUTE_TYPE_STRING)) {
						dataType = STRING;
					}
				/*
				* enumeration
				*/
					else if (type.equals(Agent.META_ATTRIBUTE_TYPE_ENUMERATION)) {
						dataType = STRING;
					}
					else {
						throw new RuntimeException(" unexpected type: "+type);
					}
				}
			/*
			* multivalued
			*/
				else if (token.equals(Agent.META_ATTRIBUTE_MULTIVALUED)) {
					multivalued = true;
				}
			/*
			* Help part
			*/
				else if (token.equals(Agent.META_ATTRIBUTE_HELP_SHORT)) {
					helpText = nextToken();
				}
			/*
			* default value
			*/
				else if (token.equals(Agent.META_ATTRIBUTE_VALUE_DEFAULT)) {
					if (dataType == INTEGER)
						defaultValue = Integer.getInteger(nextToken());
					else if (dataType == LONG)
						defaultValue = Long.getLong(nextToken());
					else if (dataType == DOUBLE)
						defaultValue = Double.valueOf(nextToken());
					else if (dataType == STRING)
						defaultValue = nextToken();
					else if (dataType == BOOLEAN)
						defaultValue = new Boolean(nextToken().charAt(0) == 't');
					else {
						nextToken(); // eat it even if we don't understand it
						throw new RuntimeException(" unexpected default value type ");
					}
				}
			/*
			* default values
			*/
				else if (token.equals(Agent.META_ATTRIBUTE_VALUE_DEFAULTS)) {
					int n = Integer.parseInt(nextToken());
					Object[] list = new Object[n];
					for (int i = 0; i< n; i++) {
						Object v=null;
						if (dataType == INTEGER)
							v = Integer.getInteger(nextToken());
						else if (dataType == LONG)
							v = Long.getLong(nextToken());
						else if (dataType == DOUBLE)
							v = Double.valueOf(nextToken());
						else if (dataType == STRING)
							v = nextToken();
						else if (dataType == BOOLEAN)
							v = new Boolean(nextToken().charAt(0) == 't');
						else {
							v = nextToken(); // eat it even if we don't understand it
							throw new RuntimeException(" unexpected multi values type ");
						}
						list[i] = v;
					}
					defaultValues = list;
				}
			/*
			* enumerated range
			*/
				else if (token.equals(Agent.META_ATTRIBUTE_ENUMERATED_VALUES)) {
					int n = Integer.parseInt(nextToken());
					String[] list = new String[n];
					for (int i = 0; i< n; i++) {
						list[i] = nextToken();
					}
					enumeratedRange = list;
				}
			}
		}
		catch (Exception e){
			e.printStackTrace();
			System.err.println("last token for property "+name+" was: "+lastToken);
		}
		parseEnumeration = null;
		peekToken = null;
	}

	private boolean moreTokens() {
		if (peekToken != null)
			return true;
		return parseEnumeration.hasMoreElements();
	}
	
	private String nextToken() {
		if (peekToken != null) {
			lastToken = peekToken;
			peekToken = null;
		}
		else {
			lastToken = (String) parseEnumeration.nextElement();
		}
		return lastToken;
	}
	
	private String peek() {
		if (peekToken != null)
			return peekToken;
		peekToken = (String) parseEnumeration.nextElement();
		return peekToken;
	}
/**
* @eturn the field name for a given index
*/
    public String getName() {
    	return name;
    }
/**
* @return true if the field can have multiple values simultaneously
*/
    public boolean isMultivalued() {
    	return multivalued;
    }
/**
* @return a java.Class object that represents the type of the object.
* If it is multivalued then this is the type of its values
*/
    public Class getDataType(){
    	return dataType;
    }
/**
* @return an object that is multivalued, may return null
*/
    public Object[] getDefaultValues(){
    	return defaultValues;
    }
/**
* @return either a String or a Number or a Boolean type, may return null
*/
    public Object getDefaultValue(){
    	return defaultValue;
    }
/**
* NOT IMPLEMENTED, ALWAYS RETURNS NULL <p>
*
* @return an array of two elements or a multiple of two elements.
* The first element of each pair is the lower end of the range (inclusive).
* The second element of each pair is the upper end of the range (inclusive). <p>
* @return null if there is no specified range.
*/
    public Object[] getRange(){
    	return null;
    }
/**
* @return An array of the allowed values.
* @return null if the property does not have enumerated values.
*/
    public Object[] getEnumeratedRange(){
    	return enumeratedRange;
    }
/**
* @return A string with a help message.
*/
    public String getShortHelpString(){
    	return helpText;
    }
/**
* @return true if the field value should not be displayed during editing
*/
    public boolean isPassword(){
    	return isPass;
    }
/**
* @return true if the field has value(s) from an enumerated range
*/
	public boolean hasEnumeratedRange() {
		return enumeratedRange != null;
	}
}
