/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util;

import java.io.*;

/**
 * This class is similar to java.io.PrintWriter by extending PrintCRLFWriter
 * except that it uses LF for all newlines regardless of platform. Typically used
 * for string/text data stored in a Java String object or widget.
 *
 * @version 	1, 14 January 1999
 * @author	Jason Mathews
 */

public class PrintLFWriter extends PrintCRLFWriter {

    public PrintLFWriter(Writer out) {
	this(out, false);
    }

    /**
     * Create a new PrintLFWriter, without automatic line flushing, from an
     * existing OutputStream.  This convenience constructor creates the
     * necessary intermediate OutputStreamWriter, which will convert characters
     * into bytes using the default character encoding.
     *
     * @param  out        An output stream
     *
     * @see java.io.OutputStreamWriter#OutputStreamWriter(java.io.OutputStream)
     */
    public PrintLFWriter(OutputStream out) {
	this(out, false);
    }

    /**
     * Create a new PrintLFWriter from an existing OutputStream.  This
     * convenience constructor creates the necessary intermediate
     * OutputStreamWriter, which will convert characters into bytes using the
     * default character encoding.
     *
     * @param  out        An output stream
     * @param  autoFlush  A boolean; if true, the println() methods will flush
     *                    the output buffer
     *
     * @see java.io.OutputStreamWriter#OutputStreamWriter(java.io.OutputStream)
     */
    public PrintLFWriter(OutputStream out, boolean autoFlush) {
	this(new BufferedWriter(new OutputStreamWriter(out)), autoFlush);
    }

    /**
     * Create a new PrintLFWriter.
     *
     * @param  out        A character-output stream
     * @param  autoFlush  A boolean; if true, the println() methods will flush
     *                    the output buffer
     */
    public PrintLFWriter(Writer out, boolean autoFlush) {
		super(out, autoFlush);
		lineSeparator = "\n";
    }

}
