/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.NoSuchElementException;


/**
* Used by the SortedHash class
* @author Nazario Irizarry
*/



class SortedEnumerator implements Enumeration {
	int entry;
	boolean keys;
	boolean entireEntry;
	Vector list;

/**
* This form of the constructor exists to allow the entire
* key-value bucket to be returned, so that a list
* may be traversed to easily access both key and value.
*
* @param list the vector which has a reference to the SortedLink items
* @param keys true means return the keys, false means return the values
* @param entireEntry true means ignore second argument and return the whole SortedLink bucket
*/
    SortedEnumerator(Vector list, boolean keys, boolean entireEntry) {
		this.keys = keys;
		this.entireEntry = entireEntry;
		this.list = list;
		entry = 0;
    }
/**
* This form of the constructor allows the list to be
* traversed returning the key or the value, depending
* on the value of the second argument
*
* @param list the vector which has a reference to the SortedLink items
* @param keys true means return the keys, false means return the values
*/
    SortedEnumerator(Vector list, boolean keys) {
		this.keys = keys;
		this.entireEntry = false;
		this.list = list;
		entry = 0;
    }
	
    public boolean hasMoreElements() {
    	if (list == null)
    		return false;
    		
    	if (entry < list.size()) {
    		return true;
    	}

		list = null; // helps gc ?
		return false;
    }

    public Object nextElement() {
    	if (list == null || entry >= list.size())
			throw new NoSuchElementException("SortedEnumerator");
		SortedLink link = (SortedLink) list.elementAt(entry++);
		if (entireEntry)
			return link;
		else
    		return keys ? link.key : link.value;
    }
}
