/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.draw;

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Reporter;

public class DrawParent extends Canvas implements 
		MouseMotionListener, MouseListener, Runnable/*, ActionListener*/{

	Reporter reporter;
	int highlightMagicNumber = 1;
	Vector highlightLines = new Vector(20);
	boolean wasDragging;
	long nextUpdateTime;
	Hashtable lines = new Hashtable(500);
	Hashtable blocks = new Hashtable(500);
	Rectangle cumulativeRect;
	Label statusLine;
	Hashtable componentMoves = new Hashtable();
	
	boolean running = true;

	Block mousePressedOnBlock;
	boolean mouseIsDown;
	Block hilitedBlock;
	
	Thread refreshThread;

	private RubberBand rubberBand;
	private GhostBlock ghostBlock;
	private boolean inPaint;
	
	PopupMenu blockPopup;
	Block blockThatIsPoped;

	PopupMenu connectorPopup;
	BlockConnector connectorThatIsPoped;
	
	PopupMenu openAreaPopup;
	int openAreaClickX, openAreaClickY;
	
	Vector subscribers = new Vector();
	
	private volatile Image offScreenImageArea;
	private boolean useOffScreen = true;
	private MediaTracker mediaTracker;

	DrawParent() {
		setBackground(Color.white);
		addMouseMotionListener(this);
		addMouseListener(this);
		offScreenImageArea = createImage(400, 400);
		refreshThread = new Thread(this, "drawRefresh");
		refreshThread.setDaemon(true);
		refreshThread.setPriority(Thread.NORM_PRIORITY -1);
		refreshThread.start();
		
	}
	
	void setReporter(Reporter reporter) {
		this.reporter = reporter;
	}
	
	void setStatusLine(Label statusLine) {
		this.statusLine = statusLine;
	}
	
	
	Object lockCumulative = new Object();
	
	protected void needsRefresh(Rectangle rect){
		synchronized (lockCumulative) {
			if (cumulativeRect == null)
				cumulativeRect = new Rectangle(rect.x, rect.y, rect.width, rect.height);
			else {
				cumulativeRect = cumulativeRect.union(rect);
			}
		}
	}
	public void needsRefresh(){
		synchronized (lockCumulative) {
			cumulativeRect = new Rectangle(0, 0, Integer.MAX_VALUE-1, Integer.MAX_VALUE-1);
		}
	}
	protected void needsRefresh(Block block){
		needsRefresh(block.boundingRect);
	}
	protected void needsRefresh(BlockConnector bc){
		needsRefresh(bc.boundingRect);
	}

	Rectangle getAndClearCumulativeRect(){
		synchronized (lockCumulative) {
			Rectangle retRect = cumulativeRect;
			cumulativeRect = null;
			return retRect;
		}
	}
	
/**
* Add a block
*/
	synchronized public void add(Block block) {
		block.setParent(this);
		blocks.put(block.getName(), block);
		block.adjustSizeForName();
	}
/**
* Add a block connector
*/
	synchronized public void add(BlockConnector line) {
		if (lines.containsKey(line)) {
			System.err.println("DrawParent.add already contains line: "+line);
			return;
		}
		line.setParent(this);
		lines.put(line,line); 
		needsRefresh(line.boundingRect);
	}
	
	public boolean containsBlock(Block block) {
		return blocks.contains(block);
	}

	public boolean containsBlock(String name) {
		return blocks.containsKey(name);
	}
	
/**
* Remove a specific block
*/
	public void remove(Block block) {
		if (!blocks.contains(block))
			return;
		blocks.remove(block.getName());
		Rectangle refreshArea = block.boundingRect;
//	System.out.println("deleting block "+block+" "+block.boundingRect);
		
		
		synchronized(manipulateLines) {
			for (int i = block.sources.size()-1; i>= 0; i--) {
				BlockConnector line = (BlockConnector) block.sources.elementAt(i);
				block.sources.removeElementAt(i);
				lines.remove(line);
				refreshArea = refreshArea.union(line.boundingRect);
				line.from.sinks.removeElement(line);
				line.to = null;
				line.from = null;
			}
			for (int i = block.sinks.size()-1; i>= 0; i--) {
				BlockConnector line = (BlockConnector) block.sinks.elementAt(i);
				block.sinks.removeElementAt(i);
				lines.remove(line);
				refreshArea = refreshArea.union(line.boundingRect);
				line.to.sources.removeElement(line);
				line.from = null;
				line.to = null;
			}
		}
		needsRefresh(refreshArea);
	}

/**
* Remove a specific BlockConnector.
* The arg may be a BlockConnector with the same endpoints
* as the one on screen.  If so, the one on screen is removed.
*/
	public void remove(BlockConnector requestedLine) {
		synchronized(manipulateLines) {
			BlockConnector line = (BlockConnector) lines.remove(requestedLine); //get actual on screen
			if (line != null) {
				needsRefresh(line.boundingRect);
				line.to.sources.removeElement(line);
				line.from.sinks.removeElement(line);
				line.to = null;
				line.from = null;
			}
		}
	}

	public Object manipulateLines = new Object();
	
	private void hiliteBlock(Block b) {
		if (hilitedBlock != b)
			unhiliteBlock(hilitedBlock);
		
		if (b == null || b == hilitedBlock)
			return;
			
		synchronized (manipulateLines) {
		/*
		* hilite sources
		*/
			highlightMagicNumber++;
			if (highlightMagicNumber == 0)
				highlightMagicNumber++; // don't let it be zero
			Enumeration e = b.sources.elements();
			while (e.hasMoreElements()) {
				BlockConnector bc = (BlockConnector) e.nextElement();
				bc.setColor(Color.green);
				bc.setHighlighted(highlightMagicNumber);
				needsRefresh(bc);
				bc.from.setOutlineColor(Color.green);
				needsRefresh(bc.from);
			}
		/*
		* hilite sinks
		*/
			e = b.sinks.elements();
			while (e.hasMoreElements()) {
				BlockConnector bc = (BlockConnector) e.nextElement();
				bc.setColor(Color.green);
				bc.setHighlighted(highlightMagicNumber);
				needsRefresh(bc);
				bc.to.setOutlineColor(Color.green);
				needsRefresh(bc.to);
			}
		/*
		* hilite the block itself
		*/
			hilitedBlock = b;
			hilitedBlock.setOutlineColor(Color.green);
			needsRefresh(hilitedBlock);
		}
	}

	private void unhiliteBlock(Block b) {
		if (b == null)
			return;
		/*
		* restore the block itself
		*/
			hilitedBlock.setOutlineColor(Color.black);
			needsRefresh(hilitedBlock);
		/*
		* restore sources
		*/
		Enumeration e = b.sources.elements();
		while (e.hasMoreElements()) {
			BlockConnector bc = (BlockConnector) e.nextElement();
			bc.setColor(Color.black);
			needsRefresh(bc);
			bc.from.setOutlineColor(Color.black);
			needsRefresh(bc.from);
		}
		/*
		* restore sinks
		*/
		e = b.sinks.elements();
		while (e.hasMoreElements()) {
			BlockConnector bc = (BlockConnector) e.nextElement();
			bc.setColor(Color.black);
			needsRefresh(bc);
			bc.to.setOutlineColor(Color.black);
			needsRefresh(bc.to);
		}
		hilitedBlock = null;
	}
	

/**
* @return a list of Blocks
*/
	public Enumeration getListOfNodes() {
		return blocks.elements();
	}

/**
* Remove all blocks and connector lines
*/
	public void removeAll() {
		blocks.clear();
		lines.clear();
		needsRefresh(getBounds());
	}
	
	public void update(Graphics g) {
		paint(g);
	}

	public void paint(Graphics g) {
		if (inPaint) {
			System.err.println("already in paint");
			return;
		}

		inPaint = true;

		if (offScreenImageArea != null) {
		/*
		* Check and update size of offscreen buffer if necessary
		*/
			Rectangle updateRect = g.getClipBounds(); 
			int currentWidth = offScreenImageArea.getWidth(null);
			int currentHeight = offScreenImageArea.getHeight(null);
			if ( updateRect.width > currentWidth ||
					updateRect.height > currentHeight) {
				int newWidth = Math.max(currentWidth, updateRect.width+50);
				int newHeight = Math.max(currentHeight, updateRect.height+50);
				offScreenImageArea = createImage(newWidth, newHeight);
//				System.out.println(" new width,height="+newWidth+", "+newHeight);
			}

		/*
		* Draw into offscreen image area
		*/
			int updateX = updateRect.x;
			int updateY = updateRect.y;
			Graphics gOffscreen = offScreenImageArea.getGraphics();
			gOffscreen.translate(updateX, updateY);
			gOffscreen.setClip(updateRect);
			paintImpl(gOffscreen);
			g.drawImage(offScreenImageArea, updateX, updateY, null);
//	System.out.println("draw bounds="+g.getClipBounds()+" i="+i);
		}
		else {
			paintImpl(g);
		}
		if (ghostBlock!= null) 
			ghostBlock.paint(g);
		if (rubberBand != null) 
			rubberBand.paint(g);

		inPaint = false;
	}

	public void paintImpl(Graphics g) {
		Rectangle clipRect = g.getClipBounds();
//	System.out.println(" paintImpl r="+clipRect);
	/*
	* We just want to clear the region, but unless
	* We erase, draw, and erase it does nothing on WindowNT under jre 1.1.8
	* 
	*/
		g.setColor(Color.white);
		g.fillRect(0, 0, 8, 2 ); 
		g.setColor(Color.black);
		g.drawLine(1,1,5,1);
		g.setColor(Color.white);
		g.fillRect(clipRect.x, clipRect.y, clipRect.width, clipRect.height );
		
		/*
		* Draw lines
		*/
		synchronized (manipulateLines) {
			Color lineColor = Color.black;

			/*
			* all lines
			*/
			highlightLines.setSize(0); 
			g.setColor(lineColor);
			Enumeration enumOfLines = lines.elements();
			while (enumOfLines.hasMoreElements()) {
				BlockConnector aline = (BlockConnector) enumOfLines.nextElement();
				if (clipRect.intersects(aline.boundingRect)) {
					aline.paint(g);
//					if (aline.isHighlighted(highlightMagicNumber))
					if (aline.highlighted == highlightMagicNumber) // faster
						highlightLines.addElement(aline);
				}
			}

			/*
			* hilite lines
			*/
			int lineLen = highlightLines.size();
			for (int i = 0; i<lineLen; i++) {
				BlockConnector aline = (BlockConnector) highlightLines.elementAt(i);
				if (lineColor != aline.color){
					lineColor = aline.color;
					g.setColor(lineColor);
				}
				aline.paint(g);
			}
		}

		/*
		* Draw blocks
		*/
		synchronized(blocks) {
			Enumeration e  = blocks.elements();
			while (e.hasMoreElements()) {
				Block ablock = (Block) e.nextElement();
				if (clipRect.intersects(ablock.boundingRect)) {
					ablock.paint(g);
				}
			}
		}
	}
	
	void setStatus(String status) {
		if (statusLine != null) {
			statusLine.setText(status);
		}
	}
	
	boolean processingMove;
	
	public void mouseMoved(MouseEvent e) {
		if (processingMove)
			return;
		processingMove = true;
		long time = System.currentTimeMillis();
		if (time > nextUpdateTime) {
			nextUpdateTime = time + 100;
			Block theBlock = findContainingBlock(e.getX(), e.getY());
			if (theBlock != null) {
				if (theBlock.hasOutputs && theBlock.isRubberbandLoc(e.getX(), e.getY())) {
					setStatus("Click to connect to another block");
				}
				else {
					setStatus(theBlock.getInfoString());
				}
			}
			else {
				BlockConnector theLine = findContainingLine(e.getX(), e.getY());
				if (theLine != null)
					setStatus(theLine.from.getName()+" to "+theLine.to.getName());
			}
			
			if (rubberBand != null)
				rubberBand.setPos(e.getX(), e.getY());
		}
		processingMove = false;
	}
	
	private Block findContainingBlock(int x, int y) {
		Enumeration e = blocks.elements();
		while (e.hasMoreElements()) {
			Block b = (Block) e.nextElement();
			if (b.boundingRect.contains(x,y))
				return b;
		}

		return null;
	}

	private BlockConnector findContainingLine(int x, int y) {
		Enumeration e = lines.elements();
		BlockConnector shortestLine = null;
		double shortestDist = Double.MAX_VALUE;
		while (e.hasMoreElements()) {
			BlockConnector l = (BlockConnector) e.nextElement();
			double dist;
			if (l.boundingRect.contains(x,y) && (dist=l.distanceTo(x,y)) < shortestDist) {
				shortestLine = l;
				shortestDist = dist;
			}
		}
		return (shortestDist < 20) ? shortestLine : null;
	}

	public void mouseDragged(MouseEvent e) {
		if (mousePressedOnBlock == null)
			return;
			
		if (rubberBand != null)
			return;
			
		mousePressedOnBlock.moveToX = e.getX();
		mousePressedOnBlock.moveToY = e.getY();
		
		if (ghostBlock == null) {
			ghostBlock = new GhostBlock(this, mousePressedOnBlock);
			componentMoves.put(mousePressedOnBlock.getName(), mousePressedOnBlock);
			mousePressedOnBlock.select();
		}
		else {
			ghostBlock.moveToX = e.getX();
			ghostBlock.moveToY = e.getY();
		}
		
		
	}

	public void mouseReleased(MouseEvent e) {
		mouseIsDown = false;
		
		if (ghostBlock != null) {
			needsRefresh(ghostBlock.boundingRect);
			ghostBlock = null;
			return;
		}

		if (rubberBand != null) {
			rubberBand.setPos(e.getX(), e.getY());
			rubberBand.delete();
			Block startBlock = rubberBand.getStartBlock();
			Block endBlock = findContainingBlock(e.getX(), e.getY());
			if (endBlock == null) {
			}
			else if (!endBlock.hasInputs) {
				setStatus("Can't connect");
			}
			else if (startBlock.hasOutputs && 
						!connectionExists(startBlock, endBlock)) {
				reportNewConnection(startBlock, endBlock);
			}
			rubberBand = null;
		}
		else if (e.isPopupTrigger()) { 
			doMousePopupTrigger(e);
		}
		else if (e.getClickCount() == 2) {
			Block block = findContainingBlock(e.getX(), e.getY());
			if (block != null) {
				if (block.isRubberbandLoc(e.getX(), e.getY())) {
					rubberbandFrom(block);
				}
				else {
					block.editProperties();
				}
			}
			else {
				BlockConnector line = findContainingLine(e.getX(), e.getY());
				if (line != null) {
					line.editProperties();
				}
			}
		}
		else if (e.getClickCount() == 1) {
			Block block = findContainingBlock(e.getX(), e.getY());
			if (block != null && block.isRubberbandLoc(e.getX(), e.getY())) {
				rubberbandFrom(block);
			}
			else {
				hiliteBlock(block); // works even if block==null
			}
			
		}

	}
	
	boolean processingMousePressed;
	public void mousePressed(MouseEvent e) {
		if (e.isPopupTrigger()) { 
			doMousePopupTrigger(e);
			return;
		}

		if (processingMousePressed)
			return;
		processingMousePressed = true;
		mouseIsDown = true;
		mousePressedOnBlock = findContainingBlock(e.getX(), e.getY());
		if (mousePressedOnBlock != null)
			mousePressedOnBlock.markForMoveRelative(e.getX(), e.getY());
		processingMousePressed = false;
	}
	public void mouseExited(MouseEvent e) {
	}
	public void mouseEntered(MouseEvent e) {
	}
	
	boolean processingMouseClick;
	
	private void doMousePopupTrigger(MouseEvent e) {
		Block block;
		BlockConnector line;
		if ((blockPopup != null) &&
				((block = findContainingBlock(e.getX(), e.getY())) != null)) {
			blockThatIsPoped = block;
			connectorThatIsPoped = null;
			blockPopup.show(this, e.getX(), e.getY());
		}
		else if ((connectorPopup != null) &&
				((line = findContainingLine(e.getX(), e.getY())) != null)){
			blockThatIsPoped = null;
			connectorThatIsPoped = line;
			connectorPopup.show(this, e.getX(), e.getY());
		}
		else {
			blockThatIsPoped = null;
			connectorThatIsPoped = null;
			if (openAreaPopup != null) {
				openAreaClickX = e.getX();
				openAreaClickY = e.getY();
				openAreaPopup.show(this, openAreaClickX, openAreaClickY);
			}
		}
	}
	public void mouseClicked(MouseEvent e) {
		if (processingMouseClick)
			return;
		processingMouseClick = true;
		

		processingMouseClick = false;
	}
	
	
	private void rubberbandFrom(Block startBlock) {
		if (rubberBand != null)
			return;
		rubberBand = new RubberBand(startBlock, this);
//		add(rubberBand);
		
	}
/**
* 
* The thread body to update the paint surface.  Should not be overridden
*/

	public void run() {
		Thread.yield();
		
		while (running) {
			try {
				do {
					Thread.sleep(80);
				} while (inPaint);
								
				Block gb = ghostBlock;
				RubberBand band = rubberBand;
				if (gb != null) {
					gb.moveRelative(); // will cause cumulativeRect to grow
					Rectangle rect = getAndClearCumulativeRect(); // also clears any mouse created rect
					repaint(rect.x, rect.y, rect.width+1, rect.height);
				}
				else if (band != null) {
					band.moveTo(); // will cause cumulativeRect to grow
					Rectangle rect = getAndClearCumulativeRect(); // also clears any mouse created rect
					repaint(rect.x, rect.y, rect.width+1, rect.height);
				}
				else {
					if ( (componentMoves.size() > 0) ) {
						Enumeration e = componentMoves.elements();
						while(e.hasMoreElements()) {
							Block b = (Block)e.nextElement();
							b.moveRelative();
							notifyOfMove(new DrawEvent(b));
						}
						componentMoves.clear();
					}

					Rectangle rect = getAndClearCumulativeRect(); // also clears any mouse created rect
					if (rect != null) {
						repaint(rect.x, rect.y, rect.width+1, rect.height);
					}
				}
			}
			catch (InterruptedException e) {}
			catch (Error e) {
				running = false;
				if (reporter != null)
					reporter.reportException(e);
				else 
					e.printStackTrace();
			}
		}
		refreshThread = null;
		subscribers = null;
//		System.err.println("refresh thread exited");
	}
/**
* terminate the refresh thread
*/
	public void finish() {
		running = false;
		removeMouseMotionListener(this);
		removeMouseListener(this);
	}
/*
	public void actionPerformed(ActionEvent aEvent) {
//		String command = aEvent.getActionCommand();
	}
*/
/**
* Accept a popup menu to be displayed over the drawing area
* @param pm may be null
*/
	public void setOpenAreaPopup(PopupMenu pm) {
		if (openAreaPopup != null) {
			remove(openAreaPopup);
			}
		if (pm != null) {
			openAreaPopup = pm;
			add(pm);
		}
	}

/**
* Accept a popup menu to be displayed over blocks
* @param popup may be null
*/
	public void setBlockPopup(PopupMenu popup) {
		if (blockPopup != null) {
			remove(blockPopup);
			}
		if (popup != null) {
			blockPopup = popup;
			add(blockPopup);
		}
	}

/**
* Accept a popup menu to be displayed over connecting lines
* @param popup may be null
*/
	public void setConnectorPopup(PopupMenu popup) {
		if (connectorPopup != null) {
			remove(connectorPopup);
		}
		if (popup != null) {
			connectorPopup = popup;
			add(connectorPopup);
		}
	}

/**
* 
* @return the block that received the popup request
*/
	public Block getBlockThatIsPoped() {
		return blockThatIsPoped;
	}

/**
* 
* @return the block connector that received the popup request
*/
	public BlockConnector getConnectorThatIsPoped() {
		return connectorThatIsPoped;
	}

/**
* 
* @return the mouse position where the user clicked
*/
	public int getOpenAreaClickX() {
		return openAreaClickX;
	}

/**
* 
* @return the mouse position where the user clicked
*/
	public int getOpenAreaClickY() {
		return openAreaClickY;
	}
	
	synchronized void notifyOfCommand(DrawEvent evt) {
		Enumeration e = subscribers.elements();
		while (e.hasMoreElements()) {
			DrawListener dl = (DrawListener) e.nextElement();
			dl.doCommand(evt);
		}
	}

	private void notifyOfMove(DrawEvent evt) {
		Enumeration e = subscribers.elements();
		while (e.hasMoreElements()) {
			DrawListener dl = (DrawListener) e.nextElement();
			dl.blockMoveTo(evt);
		}
	}

	private boolean connectionExists(Block from, Block to) {
		for (int i = 0; i<from.sinks.size(); i++) {
			if (((BlockConnector) from.sinks.elementAt(i)).to == to)
				return true;
		}
		return false;
	}
	
	private void reportNewConnection(Block from, Block to) {
		if (from == to)
			return;
		DrawEvent de = new DrawEvent(from,to);

		Enumeration e = subscribers.elements();
		while (e.hasMoreElements()) {
			DrawListener dl = (DrawListener) e.nextElement();
			dl.blockConnectRequest(de);
		}
	}
	
/**
* 
* Add a DrawEvent listener.  There may be more than one listener.
*/
	public synchronized void addDrawListener(DrawListener dl) {
		if (subscribers.contains(dl))
			return;
		subscribers.addElement(dl);
	}
/**
* 
* Remove a DrawEvent listener. 
*/
	public synchronized void removeDrawListener(DrawListener dl) {
		subscribers.removeElement(dl);
	}
/**
* @return max width of block (bold names + anything else that maximizes block width)
*/
	public int getBlockWidth(String str, BlockGroup group) {
		return getFontMetrics(group.enabledFont).stringWidth(str);
	}
}