/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.draw;

import java.awt.*;

public class RubberBand extends BlockConnector {
	int xNew, yNew;
	
	public RubberBand(Block from, DrawParent parent) {
		this.from = from;
		this.to = from; // need to have a from and a to in order for BlockConnector.hashCode to work
		this.parent = parent;
		updateBounds();
	}

	void setPos(int x, int y) {
		xNew = x;
		yNew = y;
	}
	
	void moveTo() {
		parent.needsRefresh(boundingRect);
		x2 = xNew;
		y2 = yNew;
		updateBounds();
		parent.needsRefresh(boundingRect);
	}
	
	Block getStartBlock() {
		return from;
	}
	
	public void delete() {
		parent.lines.remove(this);
		parent.needsRefresh(boundingRect);
	}
		
	void updateBounds() {
		x1 = from.boundingRect.x + from.boundingRect.width;
		y1 = from.boundingRect.y + from.boundingRect.height/2;
		
		boundingRect.x = Math.min(x1,x2);
		boundingRect.y = Math.min(y1,y2);
		boundingRect.width = Math.abs(x1-x2)+1;
		boundingRect.height = Math.abs(y1-y2)+1;
	}
	
	void paint(Graphics g) {
		g.drawLine(	x1, y1, x2, y2);
	}
	
	double distanceTo(int x, int y) {		
		return Double.MAX_VALUE;
	}
/**
* The hashcode for a rubberband only depends on where it is from.
*
* @return  a hash code value for this object. 
*/
    public int hashCode() {
    	return from.hashCode();
    }

/**
* @return true if and only if the other object == this.
*/
    public boolean equals(Object anObject) {
    	return this == anObject;
    }

}