/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.pdifHashtables;

import java.awt.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.awt.event.MouseEvent;

import org.mitre.pam.util.SortedHash;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.interfaces.Juggler;
import org.mitre.pam.interfaces.ScheduleAttributes;
import org.mitre.pam.interfaces.Sink;
import org.mitre.pam.interfaces.Source;
import org.mitre.pam.interfaces.HasProperties;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.PropertyMetaData;
import org.mitre.pam.store.SourceSinkInfo;
import org.mitre.pam.executive.PropertySet;
import org.mitre.pam.util.PAMDate;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.util.MetaData;
import org.mitre.pam.monitor.MultipleTextBoxDlg;
import org.mitre.pam.monitor.MessageDlg;

/**
* A node editor for iPAM. 
* Started life as a node editor used by Bert.
* 
* @author Nazario Irizarry, Jr.
*/
public final class NodeEditor extends Frame
{
	private PopupResponder aPopupResponder = new PopupResponder();
	private static final String ADD_PROPERTY = "Add Property ...";
	private static final String REMOVE_PROPERTY = "Remove Property";
	private static final String ADD_PRODUCT_ID = "Add product_id";
	private static final String ADD_PRODUCT_DESCRIPTION = "Add product_description";
	private MenuItem removeMenuItem;
	private HasProperties parentAttributes;

	private ParentServices parentApp;
	private PopupMenu addRemovePopup = new PopupMenu("Attribute");
//	private static Font myFont = new Font("Dialog", Font.PLAIN, 12);

//	static private BinaryPredicate orderFunction = new LessUString();
	private SortedHash attributeList = new SortedHash();

	private boolean haveModifyAccess;
	private boolean schedulePaneChanged;
	private Hashtable changeableItems;
	private SortedHash displayAttributes;
	private Hashtable scheduleAttributes;
	private static Hashtable noDisplayAttributes;
	private static Hashtable knownScheduleAttributes;
	private static String[] ignoreAttributeList = 
		{ScheduleAttributes.NEXT_UPDATE_LONG, ScheduleAttributes.LAST_UPDATE_LONG,
		"runState", "javaClass", "mementoUid", "mementoKind", "persistentProduct",
		IpamPDIFNode.EXPOSURE_VERB, "lastModifiedByBertTheGui", Agent.ENABLED
		};

	private final IpamPDIFNode pnode;
	
	//{{DECLARE_CONTROLS
	java.awt.Panel wholePanel;
	java.awt.Panel topPanel;
	java.awt.TextField name;
	java.awt.Label kind;
	java.awt.Checkbox enabledBox;
	java.awt.Choice exposureChoice;
	java.awt.Panel okPanel;
	java.awt.Button OKButton;
	java.awt.Button cancelButton;
	java.awt.Label statusLine;
	java.awt.Panel schedulePanel;
	java.awt.Checkbox onceSchedRadio;
	CheckboxGroup Group1;
	java.awt.Checkbox noSchedRadio;
	java.awt.Checkbox periodicSchedRadio;
	java.awt.Checkbox fixedSchedRadio;
	java.awt.Panel fixedSchedStuff;
	java.awt.Panel fixed1row1;
	java.awt.Choice scheduleChoice;
	java.awt.Choice dayOfWeek;
	java.awt.Panel fixed1row2;
	java.awt.Label hourLabel;
	java.awt.Choice hourOfDay;
	java.awt.Label minuteLabel;
	java.awt.TextField minOfHour;
	java.awt.Panel periodicSchedStuff;
	java.awt.TextField periodField;
	java.awt.Choice periodUnitsChoice;
	java.awt.ScrollPane attributesScrollPane;
	java.awt.Panel attributesPanel;
	//}}


	public NodeEditor(IpamPDIFNode bpNode, ParentServices parentApp)
	{
		if (bpNode == null)
			throw new IllegalArgumentException("1st arg null");
		if (parentApp == null)
			throw new IllegalArgumentException("2nd arg null");
		this.pnode = bpNode;
		this.parentApp = parentApp;
		
		//{{INIT_CONTROLS
		setLayout(new BorderLayout(0,0));
		setVisible(false);
		setSize(390,399);
		setFont(new Font("Dialog", Font.PLAIN, 10));
		setBackground(new Color(16777215));
		wholePanel = new java.awt.Panel();
		GridBagLayout gridBagLayout;
		gridBagLayout = new GridBagLayout();
		wholePanel.setLayout(gridBagLayout);
		wholePanel.setBounds(0,0,390,399);
		wholePanel.setFont(new Font("Dialog", Font.PLAIN, 10));
		wholePanel.setForeground(new Color(0));
		wholePanel.setBackground(new Color(16777215));
		add("Center", wholePanel);
		topPanel = new java.awt.Panel();
		gridBagLayout = new GridBagLayout();
		topPanel.setLayout(gridBagLayout);
		topPanel.setBounds(0,1,390,44);
		GridBagConstraints gbc;
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridheight = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)wholePanel.getLayout()).setConstraints(topPanel, gbc);
		wholePanel.add(topPanel);
		name = new java.awt.TextField();
		name.setText("nameHere");
		name.setBounds(95,22,234,22);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 1;
		gbc.weightx = 20.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)topPanel.getLayout()).setConstraints(name, gbc);
		topPanel.add(name);
		kind = new java.awt.Label("(text)",Label.CENTER);
		kind.setBounds(95,0,234,22);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)topPanel.getLayout()).setConstraints(kind, gbc);
		topPanel.add(kind);
		enabledBox = new java.awt.Checkbox("enabled");
		enabledBox.setBounds(329,23,59,19);
		gbc = new GridBagConstraints();
		gbc.gridx = 2;
		gbc.gridy = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)topPanel.getLayout()).setConstraints(enabledBox, gbc);
		topPanel.add(enabledBox);
		exposureChoice = new java.awt.Choice();
		exposureChoice.addItem("private");
		exposureChoice.addItem("public");
		exposureChoice.setBounds(10,24,81,17);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,8,0,4);
		((GridBagLayout)topPanel.getLayout()).setConstraints(exposureChoice, gbc);
		topPanel.add(exposureChoice);
		okPanel = new java.awt.Panel();
		gridBagLayout = new GridBagLayout();
		okPanel.setLayout(gridBagLayout);
		okPanel.setBounds(0,356,390,39);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 10;
		gbc.gridheight = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.SOUTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(4,0,2,0);
		((GridBagLayout)wholePanel.getLayout()).setConstraints(okPanel, gbc);
		wholePanel.add(okPanel);
		OKButton = new java.awt.Button();
		OKButton.setLabel("Accept");
		OKButton.setBounds(210,0,58,17);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 4.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,4,0,4);
		((GridBagLayout)okPanel.getLayout()).setConstraints(OKButton, gbc);
		okPanel.add(OKButton);
		cancelButton = new java.awt.Button();
		cancelButton.setLabel("Cancel");
		cancelButton.setBounds(276,0,58,17);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,4,0,4);
		((GridBagLayout)okPanel.getLayout()).setConstraints(cancelButton, gbc);
		okPanel.add(cancelButton);
		statusLine = new java.awt.Label("");
		statusLine.setBounds(4,17,386,22);
		statusLine.setFont(new Font("Dialog", Font.PLAIN, 10));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridwidth = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.SOUTHWEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,4,0,0);
		((GridBagLayout)okPanel.getLayout()).setConstraints(statusLine, gbc);
		okPanel.add(statusLine);
		schedulePanel = new java.awt.Panel();
		gridBagLayout = new GridBagLayout();
		schedulePanel.setLayout(gridBagLayout);
		schedulePanel.setBounds(0,231,390,110);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 6;
		gbc.gridheight = 5;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)wholePanel.getLayout()).setConstraints(schedulePanel, gbc);
		wholePanel.add(schedulePanel);
		Group1 = new CheckboxGroup();
		onceSchedRadio = new java.awt.Checkbox("Schedule Once", Group1, false);
		onceSchedRadio.setBounds(8,0,90,19);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,8,0,0);
		((GridBagLayout)schedulePanel.getLayout()).setConstraints(onceSchedRadio, gbc);
		schedulePanel.add(onceSchedRadio);
		noSchedRadio = new java.awt.Checkbox("No Schedule", Group1, false);
		noSchedRadio.setBounds(8,19,79,19);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,8,0,0);
		((GridBagLayout)schedulePanel.getLayout()).setConstraints(noSchedRadio, gbc);
		schedulePanel.add(noSchedRadio);
		periodicSchedRadio = new java.awt.Checkbox("Periodic", Group1, false);
		periodicSchedRadio.setBounds(8,39,59,19);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,8,0,0);
		((GridBagLayout)schedulePanel.getLayout()).setConstraints(periodicSchedRadio, gbc);
		schedulePanel.add(periodicSchedRadio);
		fixedSchedRadio = new java.awt.Checkbox("Fixed", Group1, false);
		fixedSchedRadio.setBounds(8,60,46,19);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,8,0,0);
		((GridBagLayout)schedulePanel.getLayout()).setConstraints(fixedSchedRadio, gbc);
		schedulePanel.add(fixedSchedRadio);
		fixedSchedStuff = new java.awt.Panel();
		fixedSchedStuff.setLayout(new GridLayout(2,1,0,0));
		fixedSchedStuff.setBounds(118,60,191,50);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 3;
		gbc.gridheight = 2;
		gbc.weightx = 4.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)schedulePanel.getLayout()).setConstraints(fixedSchedStuff, gbc);
		schedulePanel.add(fixedSchedStuff);
		fixed1row1 = new java.awt.Panel();
		fixed1row1.setLayout(new FlowLayout(FlowLayout.LEFT,1,4));
		fixed1row1.setBounds(0,0,191,25);
		fixedSchedStuff.add(fixed1row1);
		scheduleChoice = new java.awt.Choice();
		scheduleChoice.addItem("Weekly");
		scheduleChoice.addItem("Daily");
		scheduleChoice.addItem("Hourly");
		scheduleChoice.setBounds(1,4,84,17);
		fixed1row1.add(scheduleChoice);
		dayOfWeek = new java.awt.Choice();
		dayOfWeek.addItem("Sunday"); //ni42
		dayOfWeek.addItem("Monday");
		dayOfWeek.addItem("Tuesday");
		dayOfWeek.addItem("Wednesday");
		dayOfWeek.addItem("Thursday");
		dayOfWeek.addItem("Friday");
		dayOfWeek.addItem("Saturday");
		dayOfWeek.setBounds(86,4,104,17);
		fixed1row1.add(dayOfWeek);
		fixed1row2 = new java.awt.Panel();
		fixed1row2.setLayout(new FlowLayout(FlowLayout.LEFT,1,0));
		fixed1row2.setBounds(0,25,191,25);
		fixedSchedStuff.add(fixed1row2);
		hourLabel = new java.awt.Label("hr:",Label.RIGHT);
		hourLabel.setBounds(1,0,30,22);
		fixed1row2.add(hourLabel);
		hourOfDay = new java.awt.Choice();
		hourOfDay.addItem("0");
		hourOfDay.addItem("1");
		hourOfDay.addItem("2");
		hourOfDay.addItem("3");
		hourOfDay.addItem("4");
		hourOfDay.addItem("5");
		hourOfDay.addItem("6");
		hourOfDay.addItem("7");
		hourOfDay.addItem("8");
		hourOfDay.addItem("9");
		hourOfDay.addItem("10");
		hourOfDay.addItem("11");
		hourOfDay.addItem("12");
		hourOfDay.addItem("13");
		hourOfDay.addItem("14");
		hourOfDay.addItem("15");
		hourOfDay.addItem("16");
		hourOfDay.addItem("17");
		hourOfDay.addItem("18");
		hourOfDay.addItem("19");
		hourOfDay.addItem("20");
		hourOfDay.addItem("21");
		hourOfDay.addItem("22");
		hourOfDay.addItem("23");
		try {
			hourOfDay.select(8);
		}
		catch (IllegalArgumentException e) { }
		hourOfDay.setBounds(32,2,59,17);
		fixed1row2.add(hourOfDay);
		minuteLabel = new java.awt.Label("min:",Label.RIGHT);
		minuteLabel.setBounds(92,0,39,22);
		fixed1row2.add(minuteLabel);
		minOfHour = new java.awt.TextField();
		minOfHour.setText("00");
		minOfHour.setBounds(132,0,24,22);
		fixed1row2.add(minOfHour);
		periodicSchedStuff = new java.awt.Panel();
		gridBagLayout = new GridBagLayout();
		periodicSchedStuff.setLayout(gridBagLayout);
		periodicSchedStuff.setBounds(118,38,150,22);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 2;
		gbc.weightx = 4.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)schedulePanel.getLayout()).setConstraints(periodicSchedStuff, gbc);
		schedulePanel.add(periodicSchedStuff);
		periodField = new java.awt.TextField();
		periodField.setText("3600");
		periodField.setBounds(0,0,60,22);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		gbc.ipadx = 12;
		((GridBagLayout)periodicSchedStuff.getLayout()).setConstraints(periodField, gbc);
		periodicSchedStuff.add(periodField);
		periodUnitsChoice = new java.awt.Choice();
		periodUnitsChoice.addItem("seconds");
		periodUnitsChoice.addItem("minutes");
		periodUnitsChoice.addItem("hours");
		periodUnitsChoice.setBounds(64,2,86,17);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,4,0,0);
		((GridBagLayout)periodicSchedStuff.getLayout()).setConstraints(periodUnitsChoice, gbc);
		periodicSchedStuff.add(periodUnitsChoice);
		attributesScrollPane = new java.awt.ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		attributesScrollPane.setBounds(4,56,382,168);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.gridheight = 4;
		gbc.weightx = 1.0;
		gbc.weighty = 6.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(4,4,4,4);
		((GridBagLayout)wholePanel.getLayout()).setConstraints(attributesScrollPane, gbc);
		wholePanel.add(attributesScrollPane);
		attributesPanel = new java.awt.Panel();
		gridBagLayout = new GridBagLayout();
		attributesPanel.setLayout(gridBagLayout);
		attributesPanel.setBounds(0,0,380,166);
		attributesPanel.setFont(new Font("SansSerif", Font.PLAIN, 10));
		attributesScrollPane.add(attributesPanel);
		//}}

	
		//{{REGISTER_LISTENERS
		SymMouse aSymMouse = new SymMouse();
		SymItem lSymItem = new SymItem();
		fixedSchedRadio.addItemListener(lSymItem);
		periodicSchedRadio.addItemListener(lSymItem);
		exposureChoice.addItemListener(lSymItem);
		enabledBox.addItemListener(lSymItem);
		SymAction lSymAction = new SymAction();
		OKButton.addActionListener(lSymAction);
		cancelButton.addActionListener(lSymAction);
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		scheduleChoice.addItemListener(lSymItem);
		dayOfWeek.addItemListener(lSymItem);
		hourOfDay.addItemListener(lSymItem);
		SymText lSymText = new SymText();
		minOfHour.addTextListener(lSymText);
		periodField.addTextListener(lSymText);
		periodUnitsChoice.addItemListener(lSymItem);
		schedulePanel.addMouseListener(aSymMouse);
		topPanel.addMouseListener(aSymMouse);
		attributesPanel.addMouseListener(aSymMouse);
		noSchedRadio.addItemListener(lSymItem);
		onceSchedRadio.addItemListener(lSymItem);
		//}}
	/*
	* Attributes that should never be displayed
	*/
		if (noDisplayAttributes == null) {
			noDisplayAttributes = new Hashtable(100);
			for (int i = 0; i< ignoreAttributeList.length; i++) {
				noDisplayAttributes.put(ignoreAttributeList[i], ignoreAttributeList[i]);
			}
			
			knownScheduleAttributes = new Hashtable(100);
			for (int i = 0; i< ScheduleAttributes.attributeList.length; i++) {
				knownScheduleAttributes.put(ScheduleAttributes.attributeList[i], ScheduleAttributes.attributeList[i]);
			}
		}
		// start customizing for this node

		

	/*
	* check whether the node is owned
	*/
		haveModifyAccess = parentApp.isAdminSession() || pnode.isOwnedBy(parentApp.getUserID()); // used below to disable things
	
	/*
	* The popupmenu
	*/

		if (haveModifyAccess) {
			addRemovePopup.add(new MenuItem(ADD_PRODUCT_ID));
			addRemovePopup.add(new MenuItem(ADD_PRODUCT_DESCRIPTION));
			addRemovePopup.add(new MenuItem(ADD_PROPERTY));
			addRemovePopup.add(removeMenuItem = new MenuItem(REMOVE_PROPERTY));
			addRemovePopup.addActionListener(aPopupResponder);
			attributesPanel.add(addRemovePopup);
			attributesPanel.addMouseListener(aPopupResponder);
		}
		
	/*
	* init workspace
	*/
		changeableItems = new Hashtable(50);
		displayAttributes = new SortedHash();
		scheduleAttributes = new Hashtable();
	/*
	* set title, kind, name, enabled
	*/
		setTitle(pnode.getName());
		kind.setText("("+pnode.getType()+")");
		name.setText(pnode.getName());
		name.setEnabled(true);
		enabledBox.setState(pnode.isEnabled());
		enabledBox.setEnabled(haveModifyAccess);
		exposureChoice.select(pnode.isPublic()?"public":"private");
		exposureChoice.setEnabled(haveModifyAccess);
		if (haveModifyAccess) {
			changeableItems.put(Agent.ENABLED,enabledBox);
			changeableItems.put(IpamPDIFNode.EXPOSURE_VERB, exposureChoice);
		}
		if (haveModifyAccess && parentApp.isRenameSupported()) {
			name.setEditable(true);
			changeableItems.put("~Name",name);
		}
		else {
			name.setEditable(false);
		}
	/*
	* copy attributes 
	*/
		copyAttributes(pnode, displayAttributes);
		
	/*
	* identify the parents attributes
	*/
		String type = pnode.getType();
		if (type != null) {
			parentAttributes = parentApp.getPDIFNode(type);
		}
	/*
	* show schedule pane based on type
	*/
/*
		setContainerEnable(schedulePanel, false);
		if (type != null && (type == SourceSinkInfo.JUGGLER_STR) || (type == SourceSinkInfo.DAEMON_STR)) {
			setContainerEnable(schedulePanel, true);
			getScheduleAttributes(displayAttributes, scheduleAttributes);
		}
*/
		schedulePanel.setVisible(pnode.hasSchedule());
		setContainerEnable(schedulePanel, pnode.hasSchedule());
		if (pnode.hasSchedule()) {
			getScheduleAttributes(displayAttributes, scheduleAttributes);
		}
	/*
	* build attributes panel
	*/
		buildAttributePanel();
	/**
	* Adjust sizes
	*/
		attributesPanel.validate();
		int width = attributesPanel.getMaximumSize().width;
		if (width > 500)
			width = 500;
		resize(width+10, getSize().height);	

		setVisible(true);
	}

	
/**
* Enable everything in the schedule panel
*/
	private void setContainerEnable(Container container, boolean enabled) {
		int count = container.countComponents();
		for (int i = 0; i< count; i++) {
			Component component = container.getComponent(i);
			if (component instanceof Container) {
				setContainerEnable((Container)component, enabled);
			}
			else {
				component.setEnabled(enabled);
			}
		}
	}
/**
* Build the list of attributesPanel
*/
	private void buildAttributePanel() {
	/*
	* remove anything already in the panel in case this is not the initial build
	*/
		attributesPanel.removeAll();
		
		int size = attributesPanel.countComponents();
		for (int i = 0; i< size; i++) {
			Object item = attributesPanel.getComponent(i);
			if (item instanceof Label) {
				((Label)item).removeMouseListener(aPopupResponder);
			}
		}
	/*
	* Add each attribute in order to the view
	*/
		int vPos = 0;
		Enumeration enum = displayAttributes.keys();
		while (enum.hasMoreElements()) {
			String propName = (String) enum.nextElement();
			String txtVal = (String)displayAttributes.get(propName);
			PropertyMetaData meta = getMetaDataFor(propName);

			if ( txtVal == null || txtVal == PropertySet.DELETED) {
				changeableItems.put(propName, PropertySet.DELETED);
			}
			else {
				Label aLabel = new Label(propName, Label.RIGHT);
				attributesPanel.add(aLabel, newLeftConstraint(vPos));
				if (haveModifyAccess)
					aLabel.addMouseListener(aPopupResponder);
				
				
				// use a checkbox if it's a true/false value
				if (!meta.isPassword() && (txtVal.equals("true") || txtVal.equals("false"))) {
					Checkbox tmpCheckbox = new Checkbox("", txtVal.equals("true"));
					attributesPanel.add(tmpCheckbox, newRightConstraint(vPos++));
					tmpCheckbox.setEnabled(haveModifyAccess);
					if (haveModifyAccess) {
						changeableItems.put(propName, tmpCheckbox);
					}
					String helpString = meta.getShortHelpString(); // help text
					if (helpString.length() > 0) {
						tmpCheckbox.addMouseListener(new ValueFieldHelper(helpString)); 
					}
				}
				else if (haveModifyAccess && meta.hasEnumeratedRange()) {
					Choice choice = new Choice();
					Object[] range = meta.getEnumeratedRange();
					int choiceIndex = 0;
					for (int i = 0; i< range.length; i++) {
						String entry = range[i].toString();
						if (entry.equals(txtVal))
							choiceIndex = i;
						choice.add(entry);
					}
					choice.select(choiceIndex);
					attributesPanel.add(choice, newRightConstraint(vPos++));
					choice.setEnabled(haveModifyAccess);
					if (haveModifyAccess) {
						changeableItems.put(propName, choice);
					}
					String helpString = meta.getShortHelpString(); // help text
					if (helpString.length() > 0) {
						choice.addMouseListener(new ValueFieldHelper(helpString)); 
					}
				} 
				else {
					TextField tmpTxtFld = new TextField(txtVal);
					if (meta.isPassword())
						tmpTxtFld.setEchoChar('*');
					attributesPanel.add(tmpTxtFld, newRightConstraint(vPos++));
					if (propName.equals(Juggler.CREATOR) || 
						propName.equals(ScheduleAttributes.LAST_UPDATE_TIME) ||
						propName.equals(ScheduleAttributes.NEXT_UPDATE_TIME) 
						)
						tmpTxtFld.setEnabled(false);
					else {
						tmpTxtFld.setEnabled(haveModifyAccess);
						if (haveModifyAccess) {
							changeableItems.put(propName, tmpTxtFld);
						}
					}
					String helpString = meta.getShortHelpString(); // help text
					if (helpString.length() > 0) {
						tmpTxtFld.addMouseListener(new ValueFieldHelper(helpString)); 
					}

				}	
			}
		}
	}
/**
* Get the metadata descriptor for a field
*/
	private PropertyMetaData getMetaDataFor(String name) {
		Hashtable hash = pnode.metaData;
		if (hash == null)
			return MetaData.UNSPECIFIED_STRING_META_DATA;
		PropertyMetaData metaData = (PropertyMetaData)hash.get(name);
		if (metaData == null)
			return MetaData.UNSPECIFIED_STRING_META_DATA;
		return metaData;
	}
/**
* Make a local copy of attributesPanel from a pdif node
*/
	private void copyAttributes(IpamPDIFNode pnode, SortedHash displayAttributes) {
		Enumeration names = pnode.getProperties();
		while (names.hasMoreElements()) {
			try {
				String name = (String) names.nextElement();
				if (!noDisplayAttributes.containsKey(name)) {
					Object value = pnode.getProperty(name);
					String stringValue = null;
					if (value instanceof String ) {
						stringValue = (String)value;
					}
					else if (value instanceof ValueList) {
						stringValue = ((ValueList)value).toString();
					}
					if (stringValue != null && 
						!name.startsWith("bert_") && 
						!name.startsWith("meta_") && 
						!name.startsWith("hash.") && 
						(stringValue != PropertySet.DELETED)) {
//							System.out.println(" ca     name="+name+"   value="+stringValue);
							displayAttributes.put(name, stringValue);
					}
				}
//				else {
//					System.out.println(" ca ign name="+name);
//				}
			} 
			catch (ClassCastException e) {} // we expect casts where key == PDIFConn
		}
	}
	
/**
* Make a local copy of attributesPanel from a pdif node.
* Remove any any schedule attributes that are found.
*/
	private void getScheduleAttributes(SortedHash displayAttributes, Hashtable scheduleAttributes){
		periodicSchedRadio.setState(true);
//		displayAttributes.dumpLists();
		Enumeration names = displayAttributes.keys();
		while (names.hasMoreElements()) {
			String name = (String) names.nextElement();
//			System.out.println(" gsa name="+name);
			if (knownScheduleAttributes.containsKey(name)) {
				String value = (String)displayAttributes.remove(name);
				scheduleAttributes.put(name,value);
//				System.out.println("     name="+name+"   value="+value);
				if (name == ScheduleAttributes.SCHEDULE_VERB) {
					if (ScheduleAttributes.SCHEDULE_ONCE.equalsIgnoreCase(value)) {
						onceSchedRadio.setState(true);
						onceSchedRadio_ItemStateChanged(null);
					}
					else if (ScheduleAttributes.SCHEDULE_NONE.equalsIgnoreCase(value)) {
						noSchedRadio.setState(true);
						noSchedRadio_ItemStateChanged(null);
					}
					else if (ScheduleAttributes.SCHEDULE_PERIODIC.equalsIgnoreCase(value)) {
						periodicSchedRadio.setState(true);
						periodicSchedRadio_ItemStateChanged(null);
					}
					else if (ScheduleAttributes.SCHEDULE_HOURLY.equalsIgnoreCase(value)) {
						fixedSchedRadio.setState(true);
						scheduleChoice.select("Hourly");
						fixedSchedRadio_ItemStateChanged(null);
					}
					else if (ScheduleAttributes.SCHEDULE_DAILY.equalsIgnoreCase(value)) {
						fixedSchedRadio.setState(true);
						scheduleChoice.select("Daily");
						fixedSchedRadio_ItemStateChanged(null);
					}
					else if (ScheduleAttributes.SCHEDULE_WEEKLY.equalsIgnoreCase(value)) {
						fixedSchedRadio.setState(true);
						scheduleChoice.select("Weekly");
						fixedSchedRadio_ItemStateChanged(null);
					}
				}
				else if (name == ScheduleAttributes.MINUTE_OF_HOUR) {
					minOfHour.setText(value);
				}
				else if (name == ScheduleAttributes.HOUR_OF_DAY) {
					hourOfDay.select(value);
				}
				else if (name == ScheduleAttributes.DAY_OF_WEEK) {
					dayOfWeek.select("Monday");
					for (int i = 0; i< ScheduleAttributes.dayNames.length; i++) {
						if (value.equalsIgnoreCase(ScheduleAttributes.dayNames[i])) {
							dayOfWeek.select(ScheduleAttributes.dayNames[i]);
							break;
						}
					}
				}
				else if (name == ScheduleAttributes.PERIOD) {
					setPeriod(value);
				}
			}
		}
		schedulePaneChanged = false;
	}
	/**
	* Create grid bag constraint for an attribute on the left
	*/
	private GridBagConstraints newLeftConstraint(int pos) {
		GridBagConstraints leftConstraints = new GridBagConstraints();
		leftConstraints.fill = GridBagConstraints.HORIZONTAL;
		leftConstraints.weightx = 1.0;
		leftConstraints.weighty = 1.0;
		leftConstraints.gridwidth = 1;
		leftConstraints.gridheight = 1;
		leftConstraints.anchor = GridBagConstraints.WEST;
		leftConstraints.insets.left = 3;
		leftConstraints.insets.right = 3; 
		leftConstraints.insets.top = 1; 
		leftConstraints.insets.bottom = 1; 
		leftConstraints.gridx = 0;
		leftConstraints.gridy = pos;
		return leftConstraints;
	}

	/**
	* Create grid bag constraint for an attribute value on the right
	*/
	private GridBagConstraints newRightConstraint(int pos) {
		GridBagConstraints rightConstraints = new GridBagConstraints();
		rightConstraints.fill = GridBagConstraints.HORIZONTAL;
		rightConstraints.weightx = 5.0;
		rightConstraints.weighty = 1.0;
		rightConstraints.gridwidth = 1;
		rightConstraints.gridheight = 1;
		rightConstraints.anchor = GridBagConstraints.WEST;
		rightConstraints.insets.left = 3;
		rightConstraints.insets.right = 3;
		rightConstraints.insets.top = 1; 
		rightConstraints.insets.bottom = 1; 
		rightConstraints.gridx = 1;
		rightConstraints.gridy = pos;
		return rightConstraints;
	}

/*	public void addNotify()
	{
	    Dimension d = getSize();
	    
		super.addNotify();

		if (fComponentsAdjusted)
			return;

		setSize(insets().left + insets().right + d.width, insets().top + insets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(insets().left, insets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}
*/
    // Used for addNotify check.
	boolean fComponentsAdjusted = false;


	class SymMouse extends java.awt.event.MouseAdapter
	{
		public void mouseClicked(java.awt.event.MouseEvent event)
		{
		}

		public void mouseExited(java.awt.event.MouseEvent event)
		{
		}

		public void mouseEntered(java.awt.event.MouseEvent event)
		{
			Object object = event.getSource();
			if (object == schedulePanel)
				schedulePanel_MouseEntered(event);
			else if (object == topPanel)
				topPanel_MouseEntered(event);
			else if (object == attributesPanel)
				attributesPanel_MouseEntered(event);
		}

		public void mousePressed(java.awt.event.MouseEvent event)
		{
		}

		public void mouseReleased(java.awt.event.MouseEvent event)
		{
		}
	}

	class ValueFieldHelper extends java.awt.event.MouseAdapter
	{
		String helpText;
		
		ValueFieldHelper(String helpText) {
			this.helpText = helpText;
		}
		public void mouseExited(java.awt.event.MouseEvent event)
		{
//			setHelp("");
		}

		public void mouseEntered(java.awt.event.MouseEvent event)
		{
			setHelp(helpText);
		}

	}

	class PopupResponder extends java.awt.event.MouseAdapter implements java.awt.event.ActionListener 
	{
		String componentToRemove;
		
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			
			if (event.getActionCommand() == ADD_PROPERTY)
				queryToAddAttribute();
			if (event.getActionCommand() == ADD_PRODUCT_ID)
				addAttribute("product_"+Product.PROPERTY_ID); // prefix product_ identifies a product property
			if (event.getActionCommand() == ADD_PRODUCT_DESCRIPTION)
				addAttribute("product_"+Product.PROPERTY_DESCRIPTION);
			else if (componentToRemove != null && event.getActionCommand() == REMOVE_PROPERTY)
				removeAttribute(componentToRemove);

		}

		public void mousePressed(java.awt.event.MouseEvent event)
		{
			componentToRemove = null;
			if (event.isPopupTrigger() || event.isMetaDown()) {
				doPopUp(event);
			}
		}
		

		public void mouseEntered(java.awt.event.MouseEvent event)
		{
			setStatus("Right mouse click to add or remove attributes");
		}
		
		private void doPopUp(MouseEvent event) {
			Component component = event.getComponent();
			if (component instanceof Label) {
				String attribute = ((Label)component).getText();
				if ( changeableItems.containsKey(attribute)) {
				/*
				* You can delete attributes added by the user.
				* Don't delete standard attributes that the parent has.
				*/
					if (parentAttributes == null || parentAttributes.getStringProperty(attribute,"...") == "...")
						componentToRemove = attribute;
				}
			}
			boolean showIt = componentToRemove != null;
			removeMenuItem.setEnabled(showIt);
			addRemovePopup.show((Component)event.getSource(), event.getX(), event.getY());
		}
	}

	

	class SymItem implements java.awt.event.ItemListener
	{
		public void itemStateChanged(java.awt.event.ItemEvent event)
		{
			Object object = event.getSource();
			if (object == fixedSchedRadio)
				fixedSchedRadio_ItemStateChanged(event);
			else if (object == periodicSchedRadio)
				periodicSchedRadio_ItemStateChanged(event);
			else if (object == exposureChoice)
				exposureChoice_ItemStateChanged(event);
			else if (object == enabledBox)
				enabledBox_ItemStateChanged(event);
			else if (object == scheduleChoice)
				scheduleChoice_ItemStateChanged(event);
			else if (object == dayOfWeek)
				dayOfWeek_ItemStateChanged(event);
			else if (object == hourOfDay)
				hourOfDay_ItemStateChanged(event);
			else if (object == periodUnitsChoice)
				periodUnitsChoice_ItemStateChanged(event);
			else if (object == noSchedRadio)
				noSchedRadio_ItemStateChanged(event);
			else if (object == onceSchedRadio)
				onceSchedRadio_ItemStateChanged(event);
		}
	}

	void periodicSchedRadio_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		schedulePaneChanged = true;
		boolean state = periodicSchedRadio.getState();
		
		setContainerEnable(periodicSchedStuff, state);
		if (state)
			setContainerEnable(fixedSchedStuff, false);
	}

	void fixedSchedRadio_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		schedulePaneChanged = true;
		boolean state = fixedSchedRadio.getState();
		if (state) {
			setContainerEnable(periodicSchedStuff, false);
			scheduleChoice_ItemStateChanged(null);
		}
		else
			setContainerEnable(fixedSchedStuff, false);
	}

	

	void exposureChoice_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		// to do: code goes here.
	}

	void enabledBox_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		// to do: code goes here.
	}


	/**
	* Display message on bottom of window
	*/
	private void setStatus(String status) {
		statusLine.setText(status);
	}
	
	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == OKButton)
				OKButton_ActionPerformed(event);
			else if (object == cancelButton)
				cancelButton_ActionPerformed(event);
		}
	}

	

	static int attributeNumber = 1;
	

	void queryToAddAttribute()
	{
//		System.out.println("request to add ");
		int iter = 0;
		String newName = "";
		do {
			newName = MultipleTextBoxDlg.showTextBoxDlg(this, 
							0==iter++?"Enter name for new attribute":"Atribute "+newName+" exists, use different name for new attribute", 
							"newAttribute"+attributeNumber++);
			if (newName == null)
				return;
		} while (pnode.getProperty(newName) != null);
		
		addAttribute(newName);
	}

	void addAttribute(String newName)
	{
/*
		if (pnode.getProperty(newName) != null)
			return;
*/
		displayAttributes.put(newName,""); 
		buildAttributePanel();
		attributesPanel.invalidate();
		attributesScrollPane.validate();
	}

	void removeAttribute(String name)
	{
//		System.out.println("request to remove: "+name);
	
		displayAttributes.put(name,PropertySet.DELETED);
		buildAttributePanel();
		attributesPanel.invalidate();
		attributesScrollPane.validate();
		
	}

	
	private void setPeriod(String sPeriod) {
		int period;
		try {
			period = Integer.parseInt(sPeriod);
		}
		catch (NumberFormatException e) {
			period = 3600;
		}
		
		if (period %60 != 0) {
			periodField.setText(Integer.toString(period));
			periodUnitsChoice.select("seconds");
		}
		else if (period %3600 != 0) {
			periodField.setText(Integer.toString(period / 60));
			periodUnitsChoice.select("minutes");
		}
		else {
			periodField.setText(Integer.toString(period / 3600));
			periodUnitsChoice.select("hours");
		}
	}

	private int getPeriod() {
		int multiplier = 1;
		if (periodUnitsChoice.getSelectedItem() == "hours")
			multiplier = 3600;
		else if (periodUnitsChoice.getSelectedItem() == "minutes")
			multiplier = 60;
		
		return Integer.parseInt(periodField.getText())*multiplier;
	}

	

	void OKButton_ActionPerformed(java.awt.event.ActionEvent event)
	{

		boolean changed = false;
			
		Enumeration eFields = changeableItems.keys();
		while (eFields.hasMoreElements()) {
			String name = (String) eFields.nextElement();
			Object field = changeableItems.get(name);
			String txt=null;
			if (field instanceof TextField) {
				TextField tf = (TextField) field;
				txt = tf.getText();
				if (name.equals("~Name")) {
					if (pnode.getName().equals(txt) ) 
						continue;
					if ( parentApp.getPDIFNode(txt) != null) {
						// the name is already in use - make them go back and fix it.
						MessageDlg.showMessageDlg(this, "The name\"" + txt + "\" already exists, please rename.");
						return;
					}	
					parentApp.renamePDIFNode(pnode.getName(), txt);
					continue;
				}
			}
			else if (field instanceof Checkbox) {
				Checkbox cb = (Checkbox) field;
				txt = cb.getState()?"true":"false";
			}
			else if (field instanceof Choice) {
				Choice choice = (Choice) field;
				txt = choice.getSelectedItem();
			}
			else if (field == PropertySet.DELETED) {
				txt = PropertySet.DELETED;
//				System.out.println(" deleted: "+name);
			}
			else {
				System.err.println("NodeEditor doesn't understand type of field "+name);
			}
			
			if (txt != null) {
				ValueList vl = pnode.getPropertyValues(name);
				if (vl == null || vl.size() == 0 || !txt.equals(vl.toString())) {
//				System.out.println("attr: "+name+" was "+vl.toString());
					changed = true;
					Vector values = ValueList.parseValues(txt);
					if (values.size() == 0) {
						pnode.setProperty(name, "");
//						System.out.println("  -- changed <>:");
					}
					else if (values.size() == 1) {
						pnode.setProperty(name, (String)values.elementAt(0));
//						System.out.println("  -- changed st : "+(String)values.elementAt(0));
					}
					else {
						ValueList newvl = new ValueList(values);
						pnode.setPropertyValues(name, newvl);
//						System.out.println("  -- changed vl : "+newvl);
					}
				}
			}
		}

		// check the SchedulePanel for changes to the schedule
		if (schedPaneChanged() ) {
			copyBackScheduleAttributes(pnode);
			changed = true;
		} 
		if (changed) {
			pnode.touch();
		}
		NodeEditor_WindowClosing(null);
	}

	
	private boolean schedPaneChanged() {
		return schedulePaneChanged;
	}
	
	private void copyBackScheduleAttributes(IpamPDIFNode pnode) {
	/*
	* remove all schedule attributesPanel
	*/
		Enumeration attributesPanel = knownScheduleAttributes.keys();
		while (attributesPanel.hasMoreElements()) {
			String attribute = (String)attributesPanel.nextElement();
			pnode.removePropertyAndRemoteCopies(attribute);
//			System.out.println("copyback remove attr="+attribute);
		}
	/*
	* copy back appropriate attributesPanel
	*/
		if (onceSchedRadio.getState()) {
			pnode.setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_ONCE);
		}
		else if (noSchedRadio.getState()) {
			pnode.setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_NONE);
		}
		else if (periodicSchedRadio.getState()) {
			pnode.setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_PERIODIC);
			try {
				pnode.setProperty(ScheduleAttributes.PERIOD, Integer.toString(getPeriod()));
			}
			catch (NumberFormatException e) {
				pnode.setProperty(ScheduleAttributes.PERIOD, "3600");
			}
		}
		else {
			String schedType = scheduleChoice.getSelectedItem();
			
			if (schedType == "Hourly") {
				pnode.setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_HOURLY);
				pnode.setProperty(ScheduleAttributes.MINUTE_OF_HOUR, minOfHour.getText());
			}
			else if (schedType == "Daily") {
				pnode.setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_DAILY);
				pnode.setProperty(ScheduleAttributes.MINUTE_OF_HOUR, minOfHour.getText());
				pnode.setProperty(ScheduleAttributes.HOUR_OF_DAY, hourOfDay.getSelectedItem());
			}
			else if (schedType == "Weekly") {
				pnode.setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_WEEKLY);
				pnode.setProperty(ScheduleAttributes.MINUTE_OF_HOUR, minOfHour.getText());
				pnode.setProperty(ScheduleAttributes.HOUR_OF_DAY, hourOfDay.getSelectedItem());
				pnode.setProperty(ScheduleAttributes.DAY_OF_WEEK, dayOfWeek.getSelectedItem());
			}
/*			else if (schedType == "Monthly") {
				pnode.setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_MONTHLY);
				pnode.setProperty(ScheduleAttributes.MINUTE_OF_HOUR, minutOfHour.getText());
				pnode.setProperty(ScheduleAttributes.HOUR_OF_DAY, hourOfDay.getSelectedItem());
				pnode.setProperty(ScheduleAttributes.DAY_OF_WEEK, dayOfWeek.getText());
				pnode.setProperty(ScheduleAttributes.DAY_OF_MONTH, dayOfMonth.getText());
			}
*/
		}
	}
		
	void cancelButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		NodeEditor_WindowClosing(null);
	}

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosed(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == NodeEditor.this)
				NodeEditor_WindowClosed(event);
		}

		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == NodeEditor.this)
				NodeEditor_WindowClosing(event);
		}
	}

	

	void NodeEditor_WindowClosed(java.awt.event.WindowEvent event)
	{
//		targetNode.doEditorDone();
//		System.out.println("window closed");
	}

	void NodeEditor_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);
		dispose();
//		System.out.println("window closing");
	}

	void scheduleChoice_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		schedulePaneChanged = true;
		String value = scheduleChoice.getSelectedItem();
		scheduleChoice.setEnabled(true);
		if (value == "Hourly") {
			minuteLabel.setEnabled(true);
			minOfHour.setEnabled(true);
			hourLabel.setEnabled(false);
			hourOfDay.setEnabled(false);
			dayOfWeek.setEnabled(false);
		}
		if (value == "Daily") {
			minuteLabel.setEnabled(true);
			minOfHour.setEnabled(true);
			hourLabel.setEnabled(true);
			hourOfDay.setEnabled(true);
			dayOfWeek.setEnabled(false);
		}
		if (value == "Weekly") {
			minuteLabel.setEnabled(true);
			minOfHour.setEnabled(true);
			hourLabel.setEnabled(true);
			hourOfDay.setEnabled(true);
			dayOfWeek.setEnabled(true);
		}
	}

	void dayOfWeek_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		schedulePaneChanged = true;
	}

	void hourOfDay_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		schedulePaneChanged = true;
	}

	class SymText implements java.awt.event.TextListener
	{
		public void textValueChanged(java.awt.event.TextEvent event)
		{
			Object object = event.getSource();
			if (object == minOfHour)
				minOfHour_TextValueChanged(event);
			else if (object == periodField)
				periodField_TextValueChanged(event);
		}
	}

	void minOfHour_TextValueChanged(java.awt.event.TextEvent event)
	{
		schedulePaneChanged = true;
	}

	void periodField_TextValueChanged(java.awt.event.TextEvent event)
	{
		schedulePaneChanged = true;
	}

	void periodUnitsChoice_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		schedulePaneChanged = true;
	}

	

	

	void schedulePanel_MouseEntered(java.awt.event.MouseEvent event)
	{
		setStatus("");
	}

	void topPanel_MouseEntered(java.awt.event.MouseEvent event)
	{
		setStatus("");
	}

	void attributesPanel_MouseEntered(java.awt.event.MouseEvent event)
	{
		setStatus("Right mouse click to add or remove attributes");
	}

	void noSchedRadio_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		schedulePaneChanged = true;
		boolean state = noSchedRadio.getState();
		
		if (state) {
			setContainerEnable(fixedSchedStuff, false);
			setContainerEnable(periodicSchedStuff, false);
		}
	}

	void onceSchedRadio_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		schedulePaneChanged = true;
		boolean state = onceSchedRadio.getState();
		
		if (state) {
			setContainerEnable(fixedSchedStuff, false);
			setContainerEnable(periodicSchedStuff, false);
		}
	}
/**
* display help text
*/
	private void setHelp(String msg) {
		setStatus(msg);
	}

}
