/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.smtp;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;

import org.mitre.pam.util.PrintCRLFWriter;

/**
* This mail composer interface must be implemented by any class that composes
* mail messages.
*/
public interface MailComposer {

  /**
  * This function will write a properly formatted mail message to the provided
  * output stream.  A mail composer must be responsible for knowing how to
  * format itself.
  *
  * The idea behind this function is to allow a class that uses a mail composer
  * to be able to use this function to save the mail message to the hard drive,
  * or to send the message across a socket.
  *
  * @param out  The output stream to write the message to.
  *
  * @throws IOException  Thrown if there is an error writing to the stream, or
  *                      if there is an error reading the file attachments.
  */
  public void writeMessage(BufferedWriter out) throws IOException;

  public void writeMessage(PrintCRLFWriter out) throws IOException;

  public void setTo(String to);

  public void setFrom(String from);

  public void setSubject(String subject);

  public void setBody(String body);

  /**
  * Mail messages may contain attachments, also known as enclosures.  This function
  * represents the minimum amount of support that any mail composer must have for
  * adding attachments to a mail message.
  */
  public void addAttachment(File attachment);

  /**
  * @return The "to" field of the mail message.
  */
  public String getTo();

  /**
  * @return The "from" field of the mail message.
  */
  public String getFrom();
}