/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.util.smtpIn;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Vector;
import org.mitre.pam.interfaces.ExecCallbacks;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.executive.ExecGlobal;

/**
* An iPAM instance of an SMTP server.
* @author Nazario Irizarry
*/
public class PAMSMTPServer extends SMTPserver{
//	ExecCallbacks exec;
	Reporter reporter;

/**
* A reference to the single server
*/

	protected static PAMSMTPServer theServer=null;

	protected PAMSMTPServer(String hostname, File workingDir, ExecCallbacks exec){
		super(hostname, workingDir);
		this.reporter = exec.getExecAsAgent().getReporter();
	}

/**
* This is a global method to create only one instance of an SMTP server if needed
* or to return a reference to an existing one
*/
	static public PAMSMTPServer getServer(ExecCallbacks exec){
		if (theServer != null) {
			return theServer;
		}
	/*
	* Does not exist yet, create it
	*/
        try {
// 		int threadNum = Integer.parseInt(getProperty(THREAD_NUM));
//		int timeout = Integer.parseInt(getProperty(TIMEOUT));
        URL url = new URL(ExecGlobal.getProtocolHostNoPort());
		File execDir = ExecGlobal.g_getWorkingDir("exec");
        theServer = new PAMSMTPServer(url.getHost(), new File(execDir,"smtpIn"), exec);
        	theServer.start();
        }
        catch (Exception e) {
        	exec.getExecAsAgent().getReporter().reportException(e);
        }
        
        return theServer;
	}
  /**
  * This function overrides the parent method to augment the mail list.
  * For every name in the mail list that has a simple form (such
  * as "john" an additional entry is made of the form
  * "john@mailhost"
  *
  * @param registrant   An SMTPlistener object.
  * @param mailingListNames  A list of email addresses that the registrant wants
  *                     to receive mail for.
  */
  public void addSMTPeventListener(Object registrant, Vector mailingListNames) {
  	String hostName="none";
  	Vector mailingList = new Vector(mailingListNames.size()*2);
  	
  	try {
  		hostName = "@"+(new URL(ExecGlobal.getProtocolHostNoPort())).getHost();
  	}
  	catch (MalformedURLException e) {
  		reporter.reportException(e);
  	}
  	int len = mailingListNames.size();
    for(int i=0; i<len; i++) {
    	String name = (String)mailingListNames.elementAt(i);
    	mailingList.addElement(name);
//    	System.out.println(" added to mail subscribtion -> "+name);
    	if (name.indexOf('@') < 0) {
    		mailingList.addElement(name+hostName);
//    		System.out.println("       to mail subscribtion -> "+name+hostName);
    	}
    }
    super.addSMTPeventListener(registrant, mailingList);
  }

}