/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pdif.eventParser;

import java.util.Date;


/** 
* A time/date representation that imports from or exports to PDIF. 
* The string version of this class is YYYYMMDDHHMMSS.
* This can grow in the future to include fractions of seconds.
*
* @version  2, 15 Dec 1997
* @author   Nazario Irizarry, Jr.
*/
public class PDIFDate extends java.util.Date{
	
/** 
 * Initialize to current time/date. <P>
 * @see java.util.Date
 */
	public PDIFDate() {
		super();
	}
/** 
 * @see java.util.Date
 */
    public PDIFDate (long date) {
    	super();
    }
/** 
 * @see java.util.Date
 */
	public PDIFDate (int year, int month, int date) {
		super(year, month, date, 0, 0, 0);
	}
/** 
 * @see java.util.Date
 */
	public PDIFDate (int year, int month, int date, int hrs, int min) {
		super(year, month, date, hrs, min, 0);
	}
/** 
 * @see java.util.Date
 */
    public PDIFDate (int year, int month, int date, int hrs, int min, int sec) {
		super(year, month, date, hrs, min, sec);
	}
/** 
 * Initialize based on String in YYYYMMDDHHMMSS format.
 */
	public PDIFDate(String dateStr) throws BadPDIFException{
		setFromPDIF(dateStr);
	}
/** 
 * Set to the time/date specified by the string.  Format is "YYYYMMDDHHMMSS"
 *
 * @param when specifies the time/date
 */
	public void setFromPDIF(String when) throws BadPDIFException{
		
		try {
			int yrs = stoi(sub(when,0,3))-1900;
			setYear(yrs);
			
			int mos = stoi(sub(when,4,5))-1;
			setMonth(mos);
			
			int dys = stoi(sub(when,6,7));
			setDate(dys);
	
			int hrs = stoi(sub(when,8,9));
			setHours(hrs);
	
			int mis = stoi(sub(when,10,11));
			setMinutes(mis);
	
			int ses = stoi(sub(when,12,13));
			setSeconds(ses);
		}
		catch (Exception e) {
			throw new BadPDIFException("while parsing ["+when+"] as PDIFDate");
		}
	}
	
/** 
 * Convenience function to convert string to integer.  Makes porting easier. No error checking.
 *
 * @param s the string number to convert
 * @return integer value of string
 */

	private int stoi(String s) {
		return Integer.parseInt(s);
	}

/** 
 * Convenience substring function to facilitate ports. No error checking.
 *
 * @param s the string
 * @param from the beginning index (0 is first char)
 * @return to the final char
 */

	private String sub(String s, int from, int to) {
		return s.substring(from,to+1);		// in java the ending index is exclusive
	}
/**
* same as toPDIF()
*/
	public String toString() {
		return toPDIF();
	}
/** 
 * Returns a string version of time/date.  The string is always 14 chars long and properly set up
 * so that string compares will work.  Currently in local time.  (Defieciency 115)
 *
 * @return string version of this time/date
 */

	public String toPDIF() {
		return toPDIF(this);
	}
	
/** 
 * Formats any Date into a PDIF string representation
 *
 * @return string version of this time/date
 */

	static public String toPDIF(Date date) {
		String result = "";
		
		int yr = date.getYear()+1900;	// java years start at 1900
		int mo = date.getMonth()+1;	// java months start at 0
		int dy = date.getDate();

		int hr = date.getHours();
		int mi = date.getMinutes();
		int se = date.getSeconds();

		result = Integer.toString(yr);
		
		if (mo <= 9)
			result = result + '0';
		result = result + Integer.toString(mo);

		if (dy <= 9)
			result = result + '0';
		result = result + Integer.toString(dy);
		
//		result = result + ' ';
		
		if (hr <= 9)
			result = result + '0';
		result = result + Integer.toString(hr);

		if (mi <= 9)
			result = result + '0';
		result = result + Integer.toString(mi);

		if (se <= 9)
			result = result + '0';
		result = result + Integer.toString(se);

		
		return result;
	}
	
/** 
 * Updates to the current time.
 *
 */
	public void touch() {		
		setTime(System.currentTimeMillis());
	}
}
