/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pdif.eventParser;

import java.util.Vector;
import java.util.Enumeration;
import java.io.StreamTokenizer;
import java.io.StringReader;
import java.io.IOException;
import java.io.Serializable;

/**
* A vector of strings wrapped in a distinctive class. Built such that
* the values are immutable.
*
* @version  1, 12 Mar 1998
* @author Nazario Irizarry, Jr.
* @author John R. Fantry - added Java serialization
*/
public final class ValueList implements Serializable {
/**
* The EMPTY_VALUE_LIST is a ValueList of size==0
*/
	public static ValueList EMPTY_VALUE_LIST = new ValueList();
	Vector valuesVector = new Vector(4);
/**
* Build from a vector of Strings. Copies the references of
* the vector such that the argument vector can then be
* modified without impacting the list of values.
*/
	public ValueList(Vector values) {
		if (values.size() == 0) {
			valuesVector = EMPTY_VALUE_LIST.valuesVector;
			return;
		}
		for (int i = 0; i< values.size(); i++) {
			valuesVector.addElement(values.elementAt(i).toString());
		}
	}
/**
* Build from one String with a single value
*/
	public ValueList(String value) {
		if (value == null) {
			valuesVector = EMPTY_VALUE_LIST.valuesVector;
			return;
		}
		valuesVector.addElement(value);
	}
/**
* Build from a String array
*/
	public ValueList(String[] value) {
		for (int i = 0; i< value.length; i++) {
			valuesVector.addElement(value[i]);
		}
		if (valuesVector.size() == 0) {
			valuesVector = EMPTY_VALUE_LIST.valuesVector;
		}
	}
/**
* Build from an Object array
*/
	public ValueList(Object[] value) {
		for (int i = 0; i< value.length; i++) {
			valuesVector.addElement(value[i].toString());
		}
		if (valuesVector.size() == 0) {
			valuesVector = EMPTY_VALUE_LIST.valuesVector;
		}
	}
/**
* Build from an enumeration
*/
	public ValueList(Enumeration enumeration) {
		while (enumeration.hasMoreElements()) {
			valuesVector.addElement(enumeration.nextElement().toString());
		}
		if (valuesVector.size() == 0) {
			valuesVector = EMPTY_VALUE_LIST.valuesVector;
		}
	}
/**
* Build an empty valuelist
*/
	ValueList() {
	}

/**
* Returns a String that displays the multiple values
* For example: "value one" "two" "value three" would be returned 
* if there were three values: "value one", "two", and "value three"
*
*/
	public String toString() {
		int n = valuesVector.size();
		if (n == 0)
			return "";
			
		if (n == 1)
			return (String) valuesVector.elementAt(0);
			
		StringBuffer out = new StringBuffer(30*n);
		for (int i = 0; i<n; i++) {
			if (i>0)
				out.append(' ');
			String s = (String) valuesVector.elementAt(i);
			out.append('"');
			out.append(s);
			out.append('"');
		}
		
		return out.toString();
	}
/**
* @return an enumeration of the contents
*/
	public Enumeration elements() { 
		return valuesVector.elements();
	}
/**
* @return a vector that contains a clone of the elements of this
* valueList
*/
	public Vector getVectorOfElements() {
		return (Vector)valuesVector.clone();
	}
/**
* @return the i'th String
*/
	public String elementAt(int i) {
		return (String)valuesVector.elementAt(i);
	}
/**
* @return number of elements
*/
	public int size() {
		if (this == EMPTY_VALUE_LIST)
			return 0;
		return valuesVector.size();
	}
/**
* Parse the input stream into muiltiple values. The
* inputString is of the form "one" "second value" "third value".
* If the input string has not double-quotes then the string
* is interpreted as a single value
*
* @param inputString 
* @return a Vector of size >= 0 containing the input strings
*/
	static public Vector parseValues(String inputString) {
		Vector v = new Vector(10);
		if (inputString == null)
			return v;
		if (inputString.indexOf("\"") < 0) {
			v.addElement(inputString);
			return v;
		}
		StreamTokenizer st = new StreamTokenizer(new StringReader(inputString));
		st.resetSyntax();
		st.lowerCaseMode(false);
		st.whitespaceChars(0,32);
		st.wordChars(0x21,0x7E);
		st.whitespaceChars('#','#');
		st.whitespaceChars(128,255);
		st.quoteChar('\"');
		
		int tok;
		try {
			while ((tok = st.nextToken()) != StreamTokenizer.TT_EOF){
//				System.out.println("see : "+st.sval);
				v.addElement(st.sval);
			}
		}
		catch (IOException ignore){}
		
		return v;
	}
/**
* Two vectors are .equals if their contents are .equals
*/
	public boolean equals(Object other) {
		if (other == this)
			return true;
		if (! (other instanceof ValueList))
			return false;
		
		ValueList vOther = (ValueList) other;
		if (valuesVector.size() != vOther.valuesVector.size())
			return false;
		
		for (int i = 0; i<valuesVector.size(); i++) {
			if (!valuesVector.elementAt(i).equals(vOther.valuesVector.elementAt(i)))
				return false;
		}
		
		return true;
	}
}