/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.daemon.rules;

import jess.Context;
import jess.Funcall;
import jess.ReteException;
import jess.RU;
import jess.Userfunction;
import jess.Value;
import jess.ValueVector;

import org.mitre.pam.interfaces.ExecToEditor;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.store.IntegrityException;

/**
* Jess userfunction to create a PAM agent of type agentKind
* optionally with a name prefix of agentNamePrefix.
*
*    (buildUniqueAgent <agentKind> [agentNamePrefix] )
*
* @returns the agent id of the created agent if successful otherwise FALSE.
*
* @author Jason Mathews
*/

public class BuildUniqueAgent implements Userfunction {

    protected ExecToEditor exec;
    protected Reporter reporter;

	public BuildUniqueAgent(Reporter reporter, ExecToEditor exec) {
      this.exec = exec;
      this.reporter = reporter;
    }

    public String name() { 
	return "buildUniqueAgent"; 
    }

  /**
   * returns an unique PAM agent name given a prefix String
   */
  protected String getUniqueAgentName(String prefix) {
    for (int count=1;; count++) {
      String name = prefix + Integer.toString(count);
      Agent agent = exec.getActiveAgent(name);
      if (agent == null) return name;
    }
  }

  protected Agent buildUniqueAgent(String kind, String prefix) throws IntegrityException {
	String name = getUniqueAgentName(prefix);

	exec.declareNode(kind, name, EditAgent.getDefaultAttributes());

	Agent agent = exec.getActiveAgent(name);
	if (agent == null) throw new IntegrityException("Unable to create agent " + name);
	return agent;
  }

  public Value call(ValueVector vv, Context context) throws ReteException {

      if (vv.size() < 2) {
	reporter.warning("Bad call to " + name());
	return Funcall.FALSE();
      }

      String kind = vv.get(1).stringValue(); // get agent kind

	if (kind == null || kind.length() == 0)
		return Funcall.FALSE();

      String name = kind;

      if (vv.size() == 3) {
	String prefix = vv.get(2).stringValue();
	if (prefix != null && prefix.length() > 0)
		name = prefix + kind;
      }

	try {
		Agent agent = buildUniqueAgent(kind, name);
		return new Value(agent.getId(), RU.ATOM);
	}
	catch (IntegrityException ie) {
		reporter.reportException(ie);
		return Funcall.FALSE();
	}
    } 
}
