/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.daemon.rules;

import java.io.File;

import jess.Context;
import jess.Funcall;
import jess.ReteException;
import jess.Rete;
import jess.RU;
import jess.Userfunction;
import jess.Value;
import jess.ValueVector;

import org.mitre.pam.store.FileRef;
import org.mitre.pam.interfaces.Reporter;

/**
* Jess function to Copy a given file
*
* (copyfile source dest [minfilesize])
*
* The file size must be > minfilesize in order to copy the file.
* 
* @returns TRUE if successful otherwise FALSE.
*
* @author Jason Mathews
*/

class CopyFile implements Userfunction {

    protected Reporter reporter;

    CopyFile(Reporter reporter) {
	this.reporter = reporter;
    }

    public String name() {
	return "copyFile"; 
    }

    // excepts arguments to an ordered fact as input

    public Value call(ValueVector vv, Context context) throws ReteException
    {

	// System.out.println("copyFile size=" + vv.size() + " call=" + vv);

	if (vv.size() < 3) {
	    return Funcall.FALSE();
	}

	File source = new File(vv.get(1).stringValue());

	long minFileSize = 0;

	if (vv.size() == 4)
	  try {
	    minFileSize = Long.parseLong(vv.get(3).stringValue());
	  } catch (NumberFormatException ignore) {}

	// System.out.println("minFileSize=" + minFileSize);

	if (!source.exists()) {
	    reporter.warning("Cannot copy: file " + source + " does not exist");
	} else if (source.length() <= minFileSize) {
	    reporter.warning("Cannot copy: file " + source +
			     " has short length " + source.length());
	} else {
	    String targetPath = vv.get(2).stringValue();
	    File target = new File(targetPath);
	     if (target.exists() && !targetPath.endsWith(File.pathSeparator))
		target.renameTo(new File(targetPath + '~'));
	    try {
		FileRef.copyFile(source, target);
		return Funcall.TRUE();
	    } catch(java.io.IOException e) {
		reporter.reportException(e);
	    }
	}
	
	return Funcall.FALSE();
    }
}
