/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.daemon.rules;

import java.io.*;
import java.util.Enumeration;

import jess.Context;
import jess.Funcall;
import jess.ReteException;
import jess.Rete;
import jess.RU;
import jess.Userfunction;
import jess.Value;
import jess.Fact;
import jess.ValueVector;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.Reporter;

/**
* Jess function to list facts in a given file
*
* (dump-facts file-name)
*
* @returns TRUE if successful otherwise FALSE.
*
* @author Jason Mathews
*/

class DumpFacts implements Userfunction {

  protected Rete rete;

  DumpFacts(Rete rete) {
    this.rete = rete;
  }

  public String name() {
    return "dump-facts"; 
  }

  public Value call(ValueVector vv, Context context) throws ReteException
    {
      // (dump-facts file)

      if (vv.size() < 2) {
	return Funcall.FALSE();
      }

      File dest = new File(vv.get(1).stringValue());

      PrintWriter pw = null;
      try {
	pw = new PrintWriter(new FileWriter(dest));
	int count = 0;
	for (Enumeration e = rete.listFacts(); e.hasMoreElements();)
	  {
	    ValueVector f = (ValueVector) e.nextElement();
	    Fact fact = new Fact(f, rete);
	    pw.print("f-");
	    pw.print(f.get(RU.ID).factIDValue() + "   ");
	    pw.println(fact.toString());
	    count++;
	  }
	pw.println("For a total of " + count + " facts.");
	return Funcall.TRUE();
      } catch (IOException ioe) {
	Reporter reporter = ExecGlobal.getExec().getExecAsAgent().getReporter();
	reporter.reportException(ioe);
      }
      finally {
	if (pw != null)
	    pw.close();
      }

      return Funcall.FALSE();
    }

}
