/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.daemon.rules;

import java.util.Enumeration;

import jess.Context;
import jess.Funcall;
import jess.ReteException;
import jess.Rete;
import jess.RU;
import jess.Userfunction;
import jess.Value;
import jess.ValueVector;

/**
* Finds the fact that matches the given arguments
*
* @returns the fact id of the matching fact if found otherwise FALSE.
*
* @author Jason Mathews
*/

class FindFact implements Userfunction {

    Rete rete;

    FindFact(Rete rete) {
	this.rete = rete;
    }

    public String name() {
	return "findFact"; 
    }

    // excepts arguments to an ordered fact as input

    public Value call(ValueVector vv, Context context) throws ReteException
    {

	// System.out.println("findFact size=" + vv.size() + " call=" + vv);

	if (vv.size() < 2) {
	    return Funcall.FALSE();
	}

	int fsize = vv.size() + 1; // expected size should be # elements + 2
				   // for fact_id + ordered_fact type
				   // add one because the target fact already has an extra field
				   // for the userfunction name.

	Value factType = vv.get(1);
	ValueVector tf;

	Enumeration e = rete.listFacts();

    outer_loop:
	while (e.hasMoreElements()) {
	    tf = (ValueVector) e.nextElement();

	    if (fsize != tf.size()) {
		// System.out.println("1 wrong size: " + tf.size() + " " + fsize + " : " + tf);
		continue;
		}
	    if (! factType.equals(tf.get(RU.CLASS))) {
		// System.out.println("2 wrong class: " + tf);
		continue;
	    }

	Value d = tf.get(RU.DESC);
	if (d.type() != RU.DESCRIPTOR || d.descriptorValue() != RU.ORDERED_FACT) {
		// System.out.println("3-wrong type: " + tf);
		continue;
	}

	    for (int j=RU.FIRST_SLOT; j < fsize; j++) 
		{
		    if (!vv.get(j-1).equals(tf.get(j))) {
//System.out.println("4-wrong value, pos=" + j + " [" + vv.get(j-1) + " vs " + tf.get(j) + "] fact: " + tf);
			continue outer_loop;
		    }
		}
//	    System.out.println("Found FACT: " + tf);

	    return new Value(tf.get(RU.ID).factIDValue(), RU.FACT_ID);
	} // while

	return Funcall.FALSE();
   }

}
