package org.mitre.pam.daemon.rules;
//package jess;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;

/**
* Creates an input frame with room for a single line
* of "console-like" input at the bottom.  Scrolls a history
* of prior commands above.  Allows user to temporarily
* redirect input stream to a file input.
*
* @author Nazario Irizarry, Jr.
* @version 1.0 3/14/1998
*
*/

public class ScrolledInputStream extends InputStream implements ActionListener, KeyListener{

	InputStream empty = new StringBufferInputStream("");
	InputStream ins = empty;
	Frame frame;
	List history;
	TextArea input;
	MenuBar menuBar = new MenuBar();
	Menu file = new Menu("File");
	MenuItem loadMenuItem = new MenuItem("Load");
	MenuItem eofMenuItem = new MenuItem("EOF");
	boolean eofFlag;
	Label promptLabel = new Label(">");
	Panel southPanel = new Panel();
	String prompt = ">";
	int numberHistoryLines = 50;

	
/**
* @param title the name of the Java frame
*/
	public ScrolledInputStream(String title) {
		Font stdFont = new Font("Monospaced",Font.PLAIN, 12);
		
		history = new List();
		history.addActionListener(this);
		history.setFont(stdFont);

		input = new TextArea("()", 2, 60, TextArea.SCROLLBARS_NONE);
//		input.addActionListener(this);
		input.addKeyListener(this);
		
		southPanel.setLayout(new BorderLayout());
		promptLabel.setFont(stdFont);
		southPanel.add("West",promptLabel);
		input.setFont(stdFont);
		southPanel.add("Center",input);

		frame = new Frame(title);
		frame.reshape(20,20,400,300);
		frame.setLayout(new BorderLayout());
		frame.add("Center",history);
		frame.add("South", southPanel);
		
		file.add(loadMenuItem);
		file.add(eofMenuItem);
		menuBar.add(file);
		frame.setMenuBar(menuBar);
		file.addActionListener(this);
		
//		frame.show();
	}
	
	public int available() throws IOException{
		if (eofFlag)
			return 1;
			
		waitForLine();
		
		return ins.available();
	}
	
	public int read() throws IOException {
		
		if (eofFlag)
			return -1;

		waitForLine();
		
		if (ins.available() > 0) {
			return ins.read();
		}
		
		throw new IOException("ScrolledInputStream.read() - shouldn't be here");
	}
	
	public void setPrompt(String prompt) {
		this.prompt = prompt;
		promptLabel.setText(prompt);
		promptLabel.invalidate();
		frame.validate();
	}
	
	private void waitForLine() throws IOException {
		while (true) {
			if (eofFlag)
				return;
				
			if (ins instanceof FileInputStream) {
				FileInputStream fis = (FileInputStream) ins;
				if (fis.available() > 0) {
					return ;
				}
				try {
					fis.close();
					ins = empty;
				}
				catch (Exception e) {}
			}
			else {
				if (ins.available() > 0)
					return;
				try {
					Thread.sleep(500);
				}
				catch (Exception e) {}
			}
		}
	}
	

    /**
     * Invoked when a key has been typed.
     * This event occurs when a key press is followed by a key release.
     */
    public void keyTyped(KeyEvent e){
    }

    /**
     * Invoked when a key has been pressed.
     */
    public void keyPressed(KeyEvent e){
    }

    /**
     * Invoked when a key has been released.
     */
    public void keyReleased(KeyEvent e){
    	char c = e.getKeyChar();
		if (c == '\r' || c == '\n') {
			String newLine = input.getText();
			
			StringTokenizer st = new StringTokenizer(newLine);
			String toadd = "";
			while (st.hasMoreTokens()) {
				toadd += st.nextToken("\r\n");
			}
			ins = new StringBufferInputStream(toadd+"\n");
			appendHistoryLine(newLine);
			history.select(history.getItemCount()-1);
		}
    }

	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
	/*
	* Selection from history buffer
	*/
		if (source == history){
			input.setText(history.getSelectedItem());
		}
	/*
	* Menu selection
	*/
		else if (source == file){
			String com = e.getActionCommand();
//			System.out.println("menu item "+com);
		/*
		* On eof set the flag
		*/
			if (com.equals("End of File")) {
				eofFlag = true;
			}
		/*
		* Prompt user for a file to redirect input to
		*/
			else if (com.equals("Load")) {
				FileDialog fd = new FileDialog(frame, "Select input file", FileDialog.LOAD);
				fd.show();
				
				String dirname = fd.getDirectory();
				String fname = fd.getFile();
				if (fname == null || fname.equals(""))
					return;
					
				try {
					File file = new File(new File(dirname), fname);
					appendHistoryLine("[input from "+file.getPath()+"]");
					ins = new FileInputStream(file);
				}
				catch (Exception e2) {
					e2.printStackTrace();
					return;
				}
			}
		}
/*		else {
			System.out.println("source "+e.getSource());
			System.out.println("com "+e.getActionCommand());
		}
*/
	}
	
	private void appendHistoryLine(String line) {

		history.addItem(line);
		while (history.getItemCount() > numberHistoryLines) {
			history.remove(0);
		}
		input.setText("()");
		input.select(1,1);
	}

/**
* A test driver for this class
*/
	public static void main(String args[]){
		ScrolledInputStream sin = new ScrolledInputStream("test input");
		try {
			while (true) {
				char c = (char) sin.read();
				if (c == -1) {
					System.out.println("[EOF]");
					break;
				}
				else
					System.out.print(c);
			}
		} 
		catch (IOException e) {
		}
	}
/**
* show or hide the correspongind window
*/
  public void setVisible(boolean vis) {
  	frame.setVisible(vis);
  }
/**
* show or hide the correspongind window
*/
  public boolean isVisible() {
  	return frame.isVisible();
  }
}
