package org.mitre.pam.daemon.rules;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.ExecCallbacks;
import org.mitre.pam.interfaces.Reporter;

import jess.Context;
import jess.Funcall;
import jess.ReteException;
import jess.RU;
import jess.Userfunction;
import jess.Value;
import jess.ValueVector;

/**
* Jess userfunction to write to PAM log file
*
*    (writeToLog <category> <msg>)
*
* argument <category> must be an atom or a string with the values log/warn/warning
* argument <msg> may be single or multiple values/lists.
*
* @returns TRUE if the log written, FALSE otherwise
*
* @author Nazario Irizarry
* @author Jason Mathews Added multiple string values
*/
class WriteToLog implements Userfunction {

  protected final static String LOG = "log";
  protected final static String LOG_CATEGORY = "infer";
  protected final static String WARNING = "warning";
  protected final static String WARN = "warn";
  protected final static String WARNING_CATEGORY = "infer";

  private final static String m_name = "writeToLog";

  protected ExecCallbacks exec;
  protected Reporter reporter;

  public WriteToLog(Reporter reporter, ExecCallbacks exec) {
		this.exec = exec;
		this.reporter = reporter;
    }

  public String name() {
	return m_name;
  }

  public Value call(ValueVector vv, Context context) throws ReteException {
    if(vv.size() < 3) {
      return Funcall.FALSE();
    }

    String type = vv.get(1).stringValue();

    String msg;
    if (vv.size() == 3)
      msg = Util.valueToString(vv.get(2));	// single value log message
    else {
      // handle multi-value log message
      StringBuffer buf = new StringBuffer();
	buf.append(Util.valueToString(vv.get(2)));
      int size = vv.size();
      for (int i=3; i < size; i++) {
	buf.append(' ');
	buf.append(Util.valueToString(vv.get(i)));
      }
      msg = buf.toString();	
    }

    if(type.equalsIgnoreCase(LOG)) {
      reporter.log(msg);
    }
    else if(type.equalsIgnoreCase(WARNING)) {
      reporter.warning(msg);
    }
    else if(type.equalsIgnoreCase(WARN)) {
      reporter.warning(msg);
    }
    else {
      // ignore bad category type (assume default log) and print the log entry
      reporter.log(msg);
    }

    return Funcall.TRUE();
  }
}
