/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import java.util.*;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ProductFilter;
import org.mitre.pam.interfaces.ContainerProduct;

import org.mitre.pam.product.view.BasicProduct;

/**
* This Juggler: 1) Creates an aggregate product from one or more sources
*               2) Deletes expired product from the aggregate product
*               3) Passes on the resulting aggregate product
* Garbage collection will remove cached files from the local host which
* no longer have FileRefs pointing to them.
*
*
* @author Jones Leung
*
* @author John R. Fantry, added RemoveExpiredFilter
*/
public class CheckExpiration extends NoSchedBase {
	
/**
* This channel is not scheduled.
*/
	public CheckExpiration () {
//        setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_NONE);
		setDescription("Deletes expired products (based on product_lifetime)");
	}
/**
* Expired product is removed from the composite product.  Garbage collection
* will remove the files cached on the local host which no longer have
* FileRefs pointing to them.
*/
	protected Product buildProduct(Product jugglersProduct) {
	    if (jugglersProduct == null || isExpired (jugglersProduct)) {     
	        return null;
	        
    /*
    * check contents of container product
    */
	    } else if (jugglersProduct instanceof ContainerProduct) {
        Product deepCopy = jugglersProduct.cloneDeeper();

        ProductFilter pf = new RemoveExpiredFilter();
        deepCopy.apply(pf);

        if(deepCopy == null) {
          return null;
        }

        return deepCopy;
      }

	return jugglersProduct; 
	}
/*
* Is this product expired?  Check the lifetime attribute which is in seconds.
*/
    private boolean isExpired (Product prod) {
        long lifetime = prod.getLifetime()*1000;
        long modificationLong = prod.getModificationDate().getTime();
        
//        System.out.println ("Current date: " + new Date (System.currentTimeMillis()));
//        System.out.println ("Expiration date: " + new Date (lifetime + modificationLong));
        
        return (System.currentTimeMillis() > lifetime + modificationLong) ;
    }



    public class RemoveExpiredFilter implements ProductFilter {

      public boolean filter(ContainerProduct parent, Product productToTest) {

      if(parent == null) {
        return true;
      }

      if(isExpired(productToTest)) {
        parent.removeElement(productToTest);
      }

      return true;
    }
  }
}