/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ProductFilter;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.VectorProduct;

/**
* A Juggler that flattens the hierarchical structure of its product to a vector product of
* basic products it contains more than one basic product or a single basic product if it
* contains only one item.
*
* @author Jason Mathews
*/
public class FlattenHierarchy extends NoSchedBase {

  /**
   * A ProductFilter that adds all basic products to the parent container
   */
	class WalkProductFilter implements ProductFilter {
	
		VectorProduct container;
		
		WalkProductFilter(VectorProduct vp) {
			container = vp;
		}
		
		public boolean filter ( ContainerProduct parent, Product p ) {
			
			if ( p instanceof BasicProduct ) {
			container.addElement(p);
			}
			
			return true; // continue search
		}
	}//WalkProductFilter

    protected Product buildProduct(Product jugglersProduct) {
	
		if (jugglersProduct == null) {
		    return null;
		}
	
		VectorProduct interimProduct = new VectorProduct(getId(), getDescription());
	
		jugglersProduct.apply( new WalkProductFilter(interimProduct) );
	
		if (interimProduct.size() == 0)
		    return null;
		else if (interimProduct.size() == 1)
		    return interimProduct.elementAt(0);
		else
		    return interimProduct;
    }//buildProduct
}
