/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import java.net.URL;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.product.view.BasicProduct;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.ProductFilter;
import org.mitre.pam.product.view.VectorProduct;

/**
* A Juggler that forwards any URLs and FileRef products. Products that have FileRef content
* are converted to URLs if they are local. URLs are forwarded without change.
*
* @author Jason Mathews
*/
public class ForwardAsUrl extends NoSchedBase {

    class FilterUrlProducts implements ProductFilter {

      VectorProduct container;
      Reporter reporter;

      FilterUrlProducts(VectorProduct vp, Reporter reporter) {
	  container = vp;
	  this.reporter = reporter;
      }

      public boolean filter ( ContainerProduct parent, Product p ) {
	  if ( !(p instanceof BasicProduct) ) {
	      return true;
	  }

	  Object content = p.getContent();

	  if (content != null) {
	      if (content instanceof URL) {
		  //	BasicProduct bp = (BasicProduct) p.cloneDeeper();
		  //	BasicProduct bp = new BasicProduct(p.getId(), p.getDescription());
		  //	bp.setContent( (URL) content );
		  //	System.out.println("URL: " + p);
		  container.addElement(p);
	      } else if (content instanceof FileRef) {
		  try {
		      String url = ((FileRef)content).getReferenceAsFullURLString();
		      BasicProduct bp = (BasicProduct) p.cloneShallow(p.getId(), p.getDescription());
		      bp.setContent(new URL(url));
		      // System.out.println("FileRef: " + bp);
		      container.addElement(bp);
		  } catch (Exception e) {
		      reporter.reportException(e);
		  }
	      }
	  }

	  return true; // continue search
      }
  }

  public ForwardAsUrl() {
	setDescription("Changes file contents into URLs that point back to PAMs copy of the files");
  }

/**
* A channel can build and change the product that it's pims provide
*/
	protected Product buildProduct(Product jugglersProduct) {


		if (jugglersProduct == null)
			return null;

		VectorProduct interimProduct = new VectorProduct(getId(), getDescription());

		jugglersProduct.apply( new FilterUrlProducts(interimProduct, reporter) );

		if (interimProduct.size() == 0)
			return null;
		else if (interimProduct.size() == 1)
			return interimProduct.elementAt(0);
		else
			return interimProduct;
	}

}
