/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import java.util.Enumeration;
import java.util.Random;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.productVisitor.AsTree;

/**
* A Juggler that builds an aggregate product and passes along only a single
* basic product chosen at random. Used to generate simple basic products from
* vector products.
*
* @author Jason Mathews
*/
public class PassOneRandomProduct extends ChanSchedBase {

  // target child index choosen at random
  private int target;

  // count of children visited so far
  private int childnum;

  private Product returnProduct;

  private static Random rand = new Random();

  public PassOneRandomProduct () {
    setDescription("Passes one basic product choosen at random from its input");
//    setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_NONE);
  }


  protected Product buildProduct(Product inProduct) {

    // If product is null or already a basicProduct then no random selection is needed
    if (inProduct == null || inProduct instanceof BasicProduct) {
      return inProduct;
    }

    AsTree asTree = new AsTree();
    asTree.visit(inProduct);
    int count = asTree.getLeafCount();

    if (count == 0) {
	return null; // no leaves -> no product
    }

    /* Select a random child from 0..n-1 */
    target = rand.nextInt() % count;
    if (target < 0) target = -target;

    /*
     * Traverse the product and select the i-th element (where i=target)
     */
    childnum = 0;
    returnProduct = null;
    visit(inProduct);
    if (returnProduct != null)
    	return returnProduct;

    reporter.assert(false, "Couldn't find element " , Integer.toString(target) , " in product " , inProduct);
    return null;
  }

  /**
   * Recursively visit product and inner products to determine if they match given keywords
   */

  private void visit (Product productToTest) {

    if (productToTest == null)
    	return;

    if (productToTest instanceof ContainerProduct) {
      // get the enumeration of products inside VectorProduct
      ContainerProduct vp = (ContainerProduct) productToTest;
      Enumeration enum = (Enumeration) vp.getContent();
      // check content on each of the top-level inner products
      while ((returnProduct == null) && enum.hasMoreElements()) {
		Product element = (Product) enum.nextElement();
		visit(element);
      }
    } else {
		if (childnum++ >= target) {
			// System.out.println("found childnum = " + childnum);
			returnProduct = productToTest;
		}
	}
  }

}
