/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import java.util.*;

import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ProductFilter;
import org.mitre.pam.product.view.BasicProduct;

/**
 * This juggler passes products if the product has attributes that
 * match certain specified attributes.
 *
 * @author Michael DeCorpo 7/21/98
 * @author Jones Leung, changed description and commented out default property,
 *  updated so that it uses a ProductFilter to check inner-products instead of
 *  just the outer-product.
 * @author Jason Mathews allow filter to handle HashtableProducts,
 *  combine filter match tests into a single function productMatch().
 */
public class PassProductAttributes extends NoSchedBase {

    public static String ATTRIBUTE_PREFIX = "productAttribute_";

    public PassProductAttributes () {
		setDescription("Passes products with matching attributes of the form: productAttribute_ + attrname, attrvalue.");
//		setProperty(ATTRIBUTE_PREFIX+"xxx","yyy");
    }

    /**
     * Go through the jugglers properties one by one looking for ones that begin
     * with the value of ATTRIBUTE_PREFIX.  If we find one, the product
     * must have an attribute with this name and the value must be the same.
     */
  protected boolean productMatch(Product productToTest) {
    Enumeration e = getProperties();
    while (e.hasMoreElements()) {
      String prop = (String) e.nextElement();
      if (prop.startsWith(ATTRIBUTE_PREFIX)) {
        // remove the prefix and check the jugglers product
        String property = prop.substring(ATTRIBUTE_PREFIX.length());
        String val = getProperty(prop);

        // check if product matches
        String productsval = productToTest.getProperty(property);
		reporter.debugMsg(property, val, "-->",productsval, getId());
        if ( productsval==null || !productsval.equals(val)) {
          return false;
        }
      }
    }
    return true;
  }

    /**
     * This channel builds a composite product from the product that it's sources provide.
     * It augments the topic vector of the source product -- which is wrong when two channels
     * end up using the same product, but this will have to do for now.
     */
    protected Product buildProduct(Product productIn) {
		if (productIn == null)
		    return null;

		Product jugglersProduct = productIn.cloneDeeper(); /* working here */
		
		// filter out a BasicProduct
		if (jugglersProduct instanceof BasicProduct) {
        if (!productMatch(jugglersProduct))
          return null;

			return jugglersProduct;
		} 
	/*
	* Do container products
	*/
	    ProductFilter filter =  new filterProducts();
	    jugglersProduct.apply (filter);
	    if (jugglersProduct instanceof ContainerProduct) {
	        ContainerProduct  cp = (ContainerProduct) jugglersProduct;
	        if (cp.size() == 0) {
	            return null;
	        }
	        if (cp.size() == 1) {
            Enumeration e = (Enumeration) cp.getContent();
            return (Product) e.nextElement();
	        }
	    }    
	    else {
	    	assert(false, "Unknown product type",jugglersProduct,null,null);
	    }
		return jugglersProduct;    
    }

/**
* Filters out products which don't have the correct attribute-value pair.
* Only a deep-copy of a Product should be filtered, not the original Product.
*/
    public class filterProducts implements ProductFilter {
        public boolean filter (ContainerProduct parent, Product productToTest) {
            
            // this is to get by the test in apply() so that mapply() will be called
            if (parent == null) {
                return true;
            }
            
            // go through the jugglers properties 1by1 looking for ones that begin
        	// with the value of ATTRIBUTE_PREFIX.  If we find one, the product 
        	// must have an attribute with this name and the value must be the same
            if (!productMatch(productToTest))
                    parent.removeElement(productToTest);
            
            // do not stop iteration until we have gone through the entire product
            return true;  
        }    
    }
}
