/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import org.mitre.pam.interfaces.Product;

/**
* A Handler that does nothing but keep a counter of the number of products it
* has seen. And their cumulative size. The counter is incremented when a non-null product is received.
* This juggler can be watched by observers to take action upon a new product
* via a change in the counter property. Other than keeping count it passes
* the product to its ouputs (just like a passThru juggler) without any
* modification to its input product.
*
* @author Jason Mathews
*/
public class ProductCounter extends NoSchedBase {

	protected static final String COUNTER = "productCount";
	protected static final String SIZE = "productSize";
	protected static final String RESET = "resetCounts";
	
	public ProductCounter () {
		setDescription("Increments a counter of the number of received products");
		setProperty(COUNTER, "0");
		metaIntegerInfo(COUNTER, 0,"Count of the number of products passed");

		setProperty(SIZE, "0");
		metaIntegerInfo(SIZE, 0,"Total size of products passed");

		setProperty(RESET, "false");
		metaBooleanInfo(RESET, false,"If set, will reset the counts to zero");
	}
	
	protected Product buildProduct(Product inProduct) {
		if (inProduct == null) {
			return null;
		}
		super.setProperty(COUNTER, getIntegerProperty(COUNTER, 0)+1);
		super.setProperty(SIZE, (int) (getDoubleProperty(COUNTER, 0.0) + inProduct.getSizeInBytesAsDouble()));
		return inProduct;
	}
	
	public void setProperty(String property, String value) {
		if (property.equals(RESET)) {
			if (value.equalsIgnoreCase("true")) {
				super.setProperty(COUNTER, 0);
				super.setProperty(SIZE, 0);
			}
		}
		super.setProperty(property, value);
	}
}
