/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import java.util.*;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ManagesOwnInputQueue;
import org.mitre.pam.interfaces.AsynchronousAgent;
import org.mitre.pam.interfaces.Task;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.executive.CommandParallelOutputs;
import org.mitre.pam.executive.ExecGlobal;

import org.mitre.pam.executive.QueueModuleHelper;
import org.mitre.pam.executive.Command;
import org.mitre.pam.interfaces.Handler;
import org.mitre.pdif.eventParser.ValueList;

/**
 * A handler that does nothing but act as a queue between

 * input and output
 *
 * @author Nazario Irizarry
 */

final public class QueueThru extends NoSchedBase implements ManagesOwnInputQueue, AsynchronousAgent {

	private static final String CLEAR_QUEUE = "clearQueue";
	protected QueueModuleHelper queueHelper;

	public QueueThru () {
		queueHelper = new QueueModuleHelper(this);
		setProperty(LOG_ACTIVITY, false);
//		setProperty(CLEAR_QUEUE, "false");
//		metaBooleanInfo(CLEAR_QUEUE,false,"Write only, select to clear queue");
	}

	public void setProperty(String property, String value) {
		if (queueHelper != null && queueHelper.setProperty(property, value))
			return;  // don't handle if the queueHelper handled it
		super.setProperty(property, value);
	}

/**
* Callback to receive a command.  There
* is only one thing we respond to - send product.
*/
	public synchronized void doCommittedTask(){
		declareStart(Handler.STATE_DIMS_OUTSTANDING);
		Task task = taskHelper.getCommittedTask();
		if (task.getCommand() == Command.SEND_PRODUCT) {
			putProduct(task.getProduct());
		}
		else
			declareEnd(false); // clears committed task
	}

/**
* Accept a new product.
*/
	public void putProduct(Product p) {
		if (enabled)
			queueHelper.queueProduct(p);
		declareEnd(false); // clears committed task
	}


	public void putQueuedProduct(Product p){
		pushProductToAllClients(p);
	}

/**
* Callback that indicates when the command was done.  
* Since all we do is output we don't really care that the output
* command is done.
*/
	public void taskDone(Task doneTask){
	}

  //From AsynchronousAgent interface:
	public void enableService() {
		queueHelper.enableService();
	}

	public void disableService() {
		queueHelper.disableService();
	}

	public void pauseService() {
		queueHelper.pauseService();
	}

	public void resumeService() {
		queueHelper.resumeService();
	}

	public Product getPersistentProduct(){
		return null;
	}
  
	public void setPersistentProduct(Product p){
	}
  
	
/**
* Log the fact that we are starting and switch our execution state
*/
	final public void declareStart(String newState) {
		setState(newState);
	}

/**
* set the conditions at end of update
*/
	final public void declareEnd(boolean ignored) {
		setState(ExecGlobal.IDLE); // must do this to clear committed task
	}

	public String getProperty(String property) {
		queueHelper.updateStats(property);
		return super.getProperty(property);
	}
	public Memento getMemento() {
		queueHelper.updateStats(null);
		return super.getMemento();
	}
	public ValueList getPropertyValues(String property) {
		queueHelper.updateStats(property);
		return super.getPropertyValues(property);
	}
}
