/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import java.util.*;
import org.mitre.pam.interfaces.Sink;
import org.mitre.pam.interfaces.Product;
//import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.util.EnabledTable;
import org.mitre.pam.executive.Module;
import org.mitre.pdif.eventParser.ValueList;


/**
* A juggler that sends output to one or more Sinks which are specified in
* a configuration variable -- listOfSinks <p>
*
* Its purpose is to send product to different DIMs which are connected to it
* depending on which DIMs are temporarily "enabled" or "disabled". <p>
*
* Called the Switch juggler because it is supposed to be
* able to re-route its product to DIMs which are "enabled" in case certain
* DIMs are "disabled" and unable to broadcast the product. <p>
*
* Typically, this juggler would be connected to more DIMs than are in its
* 'listOfSinks' configuration variable because the 'listOfSinks' configuration
* variable is expected to be changed by an administrator without warning
* in case certain DIMs are not able to broadcast or if it is not desirable
* to broadcast over certain DIMs due to bandwidth or other considerations. <p>
*
* This juggler functions as a "multiplexer" by connecting inputs to outputs
* using the 'listOfSinks' configuration variable as the select code. <p>
*
* User Configuration Variables: 
*<pre>
*    listOfSinks -- a space-delimited list of DIM ID's which are each enclosed in
*                   double quotes
*
*    e.g. Here is some sample PDIF:
*    (listOfSinks "putDIM15" "putMail27" "putLocalPublisher9" "putBackWebLocal6")
*</pre>
*
* @author Jones Leung
*/
public class Switch extends ChanSchedBase {
/**
* What is the list of sinks that I need to send product to?
*/
    private static final String SINKS = "selectedSinks";
/**
* This is the list of "temporarily" enabled sink id's from the PDIF
* configuration file
*/
    protected Hashtable sinkIds = new Hashtable();
/**
* Debug key to turn on verbose messages
*/
    private static final String DEBUG_KEY = "switch";

/*
* Unique channel attributes -- none.
*/
	public Switch () {
		String[] defSinks = {"sinkId1", "sinkId2"};
		ValueList vl = new ValueList(defSinks);
		
		setDescription("Sends to selected subset of sinks");
		setPropertyValues(SINKS,vl);				
		metaMultiStringInfo(SINKS,vl,"list of enabled sinks such as : \"sinkId1\" \"sinkId2\"");				
	}
/**
* Initialize variables on startup.
*/
    protected void initialize () {
        reporter.debugMsg("Switch.initialize()",null,null,null, DEBUG_KEY);
        
        ValueList temp_sinks = getPropertyValues (SINKS);
                
        // the temporarily enabled sink IDs from a PDIF file
        reporter.debugMsg("Sinks \"temporarily\" enabled : (" 
            , temp_sinks , ")", null, DEBUG_KEY);        
        
        // create the hashtable of temporarily enabled sinks
        if (temp_sinks.size() > 0) {
        	Enumeration enum = temp_sinks.elements();
        	while (enum.hasMoreElements()) {
        		String name = (String) enum.nextElement();
        		if (name != null && name.length() > 0) {
	        		Module agent = (Module) sinkList.get(enum.nextElement());
	        		if (agent != null && agent.isEnabled() && (agent instanceof Sink)) {
	        			productRecepient = (Sink) agent;
	        			return;
	        		}
	        	}
        	}      
            warning ("Switch juggler: No sinks enabled for "+toString());
        }                
    }
/**
*
*/
	protected Product buildProduct(Product inProd) {
	    reporter.debugMsg("Switch.buildProduct()", null,null,null, DEBUG_KEY);  
        initialize();
        return inProd;
	}
}
