/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

import java.util.Enumeration;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Schedulable;

/**
* This agent accepts an input but does nothing with it. 

* When an input is present it will cause Jugglers at its

* output to trigger.
*
* @author Jason Mathews
*/
public class Trigger extends NoSchedBase {

	protected static final String DELAY = "activateDelayInSeconds";
	protected static final String ACTIVATE_OUTPUTS = "activateOutputs";
	protected static final String ENABLE_OUTPUTS = "enableOutputs";
	protected static final String DISABLE_OUTPUTS = "disableOutputs";
	
	public Trigger () {
		setDescription("When an input is present, agents at the output are activated");
		metaIntegerInfo(DELAY, 0,"Number of seconds to delay before taking action");
		metaBooleanInfo(ACTIVATE_OUTPUTS, true, "Activate handlers connected to output");
		metaBooleanInfo(ENABLE_OUTPUTS, false, "Enable handlers connected to output (overrides "+DISABLE_OUTPUTS+")");
		metaBooleanInfo(DISABLE_OUTPUTS, false, "Disable handlers connected to output");
	}
	
	protected Product buildProduct(Product inProduct) {

		boolean activate = getBooleanProperty(ACTIVATE_OUTPUTS, true);
		boolean enable = getBooleanProperty(ENABLE_OUTPUTS, false);
		boolean disable = getBooleanProperty(DISABLE_OUTPUTS, false);
		if (enable && disable) {
			setProperty(DISABLE_OUTPUTS, false);
		}

		if (enable || disable) {
			Enumeration sinks = getFullSinkElementEnumeration();
			while (sinks.hasMoreElements()) {
				Agent agentToEnable = (Agent) sinks.nextElement();
				if (logActivity) {
					if (enable)
						reporter.log("\""+agentToEnable.getId()+"\" enabled");
					else
						reporter.log("\""+agentToEnable.getId()+"\" disabled");
				}
				agentToEnable.setEnabled(enable);
			}			
		}
		
		if (activate) {
			long timeToTrigger = System.currentTimeMillis() + 1000 * (long)getIntegerProperty(DELAY,0);
			Enumeration sinks = getSinkList().elements(); // will only see enabled outputs
			while (sinks.hasMoreElements()) {
				Agent agentToTrigger = (Agent) sinks.nextElement();
				if (agentToTrigger instanceof Schedulable){
					if (logActivity)
						reporter.log("\""+agentToTrigger.getId()+"\" triggered");
					((Schedulable) agentToTrigger).activateAt(timeToTrigger);
				}
				else {
					reporter.error("Can not trigger agent \""+agentToTrigger+"\" because it is not scheduleable");
				}
			}
		}
		return null;
	}
	
}
