/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator;

//import java.util.*;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.productVisitor.AsZipFile;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.productVisitor.AsFlatFileList;



/**
* A basic Juggler that unwraps its input if it was a zip file wrapped
* by @see WrapAsZip.
*
* @author Nazario Irizarry, Jr.
*/
public class UnwrapFromZip extends NoSchedBase {

	private static final String DEBUG_KEY = "UnwrapFromZip";
	
	public UnwrapFromZip () {
		super();
//        setProperty(ScheduleAttributes.SCHEDULE_VERB, ScheduleAttributes.SCHEDULE_NONE);
		setDescription("Expand a from zip file from another iPAM into product");
	}
/**
* This channel builds a composite product from the product that it's sources provide.
* It augments the topic vector of the source product -- which is wrong when two channels
* end up using the same product, but this will have to do for now.
*/
	protected Product buildProduct(Product jugglersProduct) {
		if (jugglersProduct == null)
			return null;
		
		AsFlatFileList asf = new AsFlatFileList();
		asf.visit(jugglersProduct);
		Enumeration list = asf.getFileProductList();
		
		VectorProduct results = new VectorProduct(getId(), getDescription());
		Product lastProduct=null;
		
		if (!list.hasMoreElements()) {
			reporter.debugMsg("no files in input ",null,null,null, DEBUG_KEY);
		}
		while (list.hasMoreElements()) {
			File zipFile = candidateFileContent((Product)list.nextElement());
			reporter.debugMsg("looking at file ",zipFile.getName(),null,null, DEBUG_KEY);
			if (zipFile != null) {
				String fileName = zipFile.getName();
				int ind = fileName.lastIndexOf(WrapAsZip.WRAP_SUFFIX);
				String prodStoreName = fileName.substring(0,ind) + "ps";
				Product p=null;
				try {
					p = AsZipFile.buildProduct(getWorkingDir(), zipFile, prodStoreName, reporter);
				}
				catch (IOException e) {
					reporter.reportException(e);
				}
				reporter.assert(p != null, "UnwrapFromZip expanded to null product",null,null,null);
				lastProduct = p;
				results.addElement(p);
			}
		}
		
		if (results.size() == 0) {
			return null;
		}
		if (results.size() == 1) {
			return lastProduct;
		}
		return results;
	}
	
	private File candidateFileContent(Product p) {
		if (p == null)
			return null;
			
		Object content = p.getContent();
		if (!(content instanceof FileRef)) 
			return null;

		FileRef fref = (FileRef) content;
		File zipFile = fref.getFile();
		String fileName = zipFile.getName();
		if (fileName.startsWith(WrapAsZip.WRAP_PREFIX) && fileName.endsWith('.'+WrapAsZip.WRAP_SUFFIX)){
			return zipFile;
		}
		return null;
	}
}
