/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.aggregator.approve;

import java.util.Enumeration;
import java.util.Vector;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.aggregator.ChanSchedBase;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.Source;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.aggregator.PassThru;
import org.mitre.pam.store.FileRef;

/**
* A Juggler that keeps adding new products onto the channel.  Products
* are only removed from the channel when the they are removed or approved
* using the ProductApproveServlet servlet.
*
* @author Michael DeCorpo
*/
public class Approve extends ChanSchedBase {
	static final String APPROVE_ID_TAG = "approveId";
	static final String AUTHORIZATION = "authUserColonPassword";
	VectorProduct approvedProduct;
	VectorProduct unapprovedProduct;
	
/*
* 
*/
	public Approve () {
		setDescription("Requires approval before forwarding");
		setProperty(AUTHORIZATION,"*:*");
		unapprovedProduct = new VectorProduct("unapproved","");
		approvedProduct = new VectorProduct("approved","");
		
		metaStringInfo(AUTHORIZATION,"*:*","\"Name:password\" to be used to access and administer the approved products" );
	}


/**
* For the DIMs.  Only give them the approved Product.
*/
	public synchronized Product getProduct() 
	{
		if (approvedProduct == null)
			return null;
			
		if (approvedProduct.size() == 0)
			return null;

		Product theProduct;

		if (approvedProduct.size() == 1)
			theProduct = approvedProduct.elementAt(0);
		else
			theProduct = approvedProduct;
			
		copyAttributesToProduct(getMemento(), theProduct);

		return theProduct;
	} 
	/**
	* Set the Product that is waiting to be approved. Used by SrvProductApprove
	*/
	void setApprovedProduct(VectorProduct p)
	{
		approvedProduct = p;
		copyAttributesToProduct(getMemento(), p);
		newContent();
	}
	/**
	* Set the Product that is waiting to be approved. Used by SrvProductApprove
	*/
	VectorProduct getApprovedProduct()
	{
		return approvedProduct;
	}

	/**
	* Get the Product that is waiting to be approved.
	*/
	public VectorProduct getWaitingProduct()
	{
		return unapprovedProduct;
	}

	/**
	* Set the Product that is waiting to be approved. Used by SrvProductApprove
	*/
	void setWaitingProduct(VectorProduct p)
	{
		unapprovedProduct = p;
	}

/*
* Notification of new product so we can tell the DIMs.
*/
	private void newContent() {
		pushProductToAllClients(getProduct());
	}


/**
* Add a new input product to the wait list
*/
	private void addToWaitList(Product inputProduct) {
		if (inputProduct == null)
			return;
		assignApproveIds(inputProduct);
		unapprovedProduct.addElement(inputProduct);
  }

/**
* Adds the product to the waiting list and returns
* null so that no product is automatically forwarded.
*/
	protected Product buildProduct(Product p) {
		addToWaitList(p);
		return null;
	}
  
/**
 * Set the channels Product;
 */
  public void setPersistentProduct(Product p) {
	if ((p == null) || !(p instanceof VectorProduct)) {
    	reporter.assert(false,"Approve.setPersistentProduct restored with unrecognized product type=",p,null,null);
	}
	
	VectorProduct vp = (VectorProduct) p;

    unapprovedProduct = (VectorProduct)vp.elementAt(0);
    approvedProduct = (VectorProduct)vp.elementAt(1);
  }

/**
 * for Exec.
 */
  public Product getPersistentProduct()
  {
  	VectorProduct vp = new VectorProduct("persistent","");
  	vp.addElement(unapprovedProduct);
  	vp.addElement(approvedProduct);
    return vp;
  }



/**
* Assigns a unique id to each incoming product and subproduct
*/
	private void assignApproveIds(Product p) {
		p.setProperty(APPROVE_ID_TAG, FileRef.getNewFileId("apprv"));
		if (p instanceof ContainerProduct) {
			Enumeration enum = (Enumeration) p.getContent();
			while (enum.hasMoreElements()) {
				Product subProduct = (Product) enum.nextElement();
				assignApproveIds(subProduct);
			}
		}
	}

}
