/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
/*
  Problems
    The servlet will not behave properly if:
      1) Two users are approving products on the the same channel and 
         at the same time.
      2) A user doesn't reload the web page after approving and then
         trying to approve again using the same form.
      
*/

package org.mitre.pam.aggregator.approve;

import java.io.IOException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.NoSuchElementException;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpUtils;


import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.ProductFilter;
import org.mitre.pam.interfaces.ScheduleAttributes;
import org.mitre.pam.monitor.SrvPAMBase;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.EnumeratedProduct;
import org.mitre.pam.product.view.HashtableProduct;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.store.FileRef;
import org.mitre.pam.util.PAMDate;
import org.mitre.pam.monitor.SrvPAMBase;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.util.Codecs;

/**
 * A servlet that works with Approve channels to allow a the approving
 * and rejecting of products before sending them to the dims or sinks.
 *
 * @author Michael DeCorpo
 * @author Nazario Irizarry 2/10/98 many changes
 */


public class SrvProductApprove extends SrvPAMBase {	
	/**
	* A ProductFilter that looks to match a specified
	* approveId and returns/stops iteration when found.
	*/
	class MatchProductId implements ProductFilter{
		String id;
		Product parent = null;
		Product product = null;
		String key = null;
		
		MatchProductId(String id) {
			this.id = id;
		}
		
		public boolean filter(ContainerProduct parent, Product p) {
			String pid = p.getProperty(Approve.APPROVE_ID_TAG);
			if (pid == null)
				return true; // continue search

			if (pid.equals(id)) {
				this.parent = parent;
				this.product = p;
				if (parent instanceof HashtableProduct) {
					key = ((HashtableProduct)parent).getKey(p);
				}
				return false; // halt search
			}
			return true; // continue search
		}
	}


	/**
	* A ProductFilter that checks leaves in the product tree
	* and deletes leaves whos id is specified
	*/
	class Disconnecter implements ProductFilter{
		Hashtable ids;
		Reporter reporter;
		boolean logMessage;
		String postMessage="";
		
		Disconnecter(Hashtable ids, Reporter reporter, boolean logMessage, String postMessage) {
			this.ids = ids;
			this.reporter = reporter;
			this.logMessage = logMessage;
			if (postMessage != null)
				this.postMessage = postMessage;
		}
		
		public boolean filter(ContainerProduct parent, Product p) {
			if ((p instanceof ContainerProduct) || parent == null) 
				return true;

			String pid = p.getProperty(Approve.APPROVE_ID_TAG);
			if (pid == null)
				return true; // continue search

			reporter.assert(pid != null,"SrvProductApprove.KeepOnly pic==null for ",p,null,null);
			
			if (pid != null && ids.containsKey(pid)) {
				parent.removeElement(p);
				if (logMessage)
					log("Reject \""+p.getId()+"\" "+postMessage);
			}
			return true; // continue search
		}
	}

	/**
	* A ProductFilter that checks leaves in the product tree
	* and deletes leaves whos id is not specified
	*/
	class KeepOnly implements ProductFilter{
		Hashtable ids;
		Reporter reporter;
		boolean logMessage;
		String postMessage="";
		
		KeepOnly(Hashtable ids, Reporter reporter, boolean logMessage, String postMessage) {
			this.ids = ids;
			this.reporter = reporter;
			this.logMessage = logMessage;
			if (postMessage != null)
				this.postMessage = postMessage;
		}
		
		public boolean filter(ContainerProduct parent, Product p) {
			if ((p instanceof ContainerProduct) || parent == null) 
				return true;

			String pid = p.getProperty(Approve.APPROVE_ID_TAG);
			reporter.assert(pid != null,"SrvProductApprove.KeepOnly pic==null for ",p,null,null);
			if (!ids.containsKey(pid)) {
				parent.removeElement(p);
			}
			else {
				if (logMessage)
					log("Approve \""+p.getId()+"\" "+postMessage);
			}

			return true; // continue search
		}
	}
	
  
  public static String APPROVE_QUERY_BASE = "/PAM/Approve.query";

	public void doGet(HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
	try { Thread.sleep(2000); } catch (Exception e) {}
		Context context = new Context();
		
		context.res = res;
		context.req = req;
		context.res.setContentType("text/html");
		ServletOutputStream out = res.getOutputStream();
		
	/*
	* List of channels?
	*/
		String channelname = req.getParameter( "channel" );
		if (channelname == null) {
			getApproveChannelList(out);
			return;
		}
		
		context.realm = channelname;
		
	/*
	* List representation of one product?
	*/
		Approve channel = getApproveChannel(channelname);
		if (channel == null) {
			printErrorMsg(out,"The Approve Channel: " + channelname + " was not found.");
			return;
		}
		
		context.authList = channel.getPropertyValues(Approve.AUTHORIZATION);
		context.channel = channel;
		
		if (!context.verifyAccess(reporter,this,"approve channel "+channelname)) 
			return;

		String tmp = req.getParameter("index");
		if (tmp == null) {
			getListForm(context);
		}
	/*
	* Edit a product?
	*/
		else {
			getEditForm(context,tmp); 
		}
		out.flush();
	}
/**
* UUDECODES the input string.
*/
	private final static String uudecode(String data)
	{
		StringBuffer result = new StringBuffer(180);
		
		
		int src = 0;
		int len = data.length();

		while (src+1 < len)
		{
			int a = (0xFF & data.charAt(src++)) - 32;
			int b = (0xFF & data.charAt(src++)) - 32;
			result.append( (char)((a << 2) | (b >> 4)));

			if ( src < len) 
			{
				int c = (0xFF & data.charAt(src++)) - 32;
				result.append( (char)((b  << 4) | ((c >> 2) & 0x0F)));

				if (src < len) 
				{
					int d = (0xFF & data.charAt(src++)) - 32;
					result.append( (char)(((c << 6) & 192) | d ));
				}
			}
		}
		return result.toString();
	}
/**
* Builds and returns a nicely formatted page
* that presents the "product" to the user as
* an HTML page
*/
	private void getListForm(Context context) throws IOException {
		VectorProduct vectprod;
		vectprod = context.channel.getWaitingProduct();
	
		ServletOutputStream out = context.res.getOutputStream();
		
		if (vectprod == null) {
			out.println("Agent <strong>"+context.realm+"</strong> is empty.");
			out.println("<br>It was last updated "+context.channel.getProperty(ScheduleAttributes.LAST_UPDATE_TIME));
			out.println("<br>Next scheduled updated is "+context.channel.getProperty(ScheduleAttributes.NEXT_UPDATE_TIME));
			return;
		}
	
		listTopOfPage(out, context.realm);	
		listBotOfPage(out, vectprod, context.realm);
		
		out.println("<br><br>");
		out.println("<input type=\"submit\" value=\"Submit!\"></form>");
		out.println("</body></html>");
		}

	public void doPost(HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		Context context = new Context();
		context.res = res;
		context.req = req;
		res.setContentType("text/html");
		ServletOutputStream out = res.getOutputStream();
		Hashtable h;
		try {
			h = HttpUtils.parsePostData(req.getContentLength(),req.getInputStream());
		}
		catch (IllegalArgumentException iae) {
			printErrorMsg(out,"Bad Post data");
			return;
		}
		
		String cname[] = (String []) h.get("channel");
		if (cname[0] == null) {
			printErrorMsg(out,"There was no channel specified");
			return;
		}
		context.realm = cname[0];
		
	/*
	* Find approver juggler
	*/
		context.channel = getApproveChannel(context.realm);
		if (context.channel == null) {
			printErrorMsg(out,"The Approve Channel: " + context.realm + " was not found.");
			return;
		}
		context.authList = context.channel.getPropertyValues(Approve.AUTHORIZATION);

		if (!context.verifyAccess(reporter, this, "approve")) 
			return;

		String index[] = (String []) h.get("index"); 
	/*
	* Edit one specific product
	*/
		if (index != null) {
			doEditProduct(context,h,index[0]);
		}
	/*
	* Handle approves and rejects
	*/
		else {
			doApprovesAndRejects(context, h);
		}
	
	} 
    

	private void getApproveChannelList(ServletOutputStream out) throws IOException
	{
		Enumeration e = exec.getListOfActiveModules(ExecToMonitor.JUGGLER);
		out.println("<html><head><title>List of Approve Channels</title>");
		out.println("<body bgcolor=\"white\">");
		out.println("<H1>List of Approve channels:</H1><br><br>");
		while (e.hasMoreElements()) {
			Agent c = (Agent) e.nextElement();
			if (c instanceof Approve) {
				String lastUpdate = c.getProperty(ScheduleAttributes.LAST_UPDATE_TIME);
				if (lastUpdate == null)
					lastUpdate = "";
				String nextUpdate = c.getProperty(ScheduleAttributes.NEXT_UPDATE_TIME);
				if (nextUpdate == null)
					nextUpdate = "";
				out.print("<a href=\"");
				out.print(APPROVE_QUERY_BASE);
				out.print("?channel=");
				out.print(URLEncoder.encode(((Approve) c).getId()));
				out.print("\"><font size=+1>" + ((Approve) c).getId() + "</font></a>");
				
				boolean paren = false;
				if (lastUpdate.length() > 0) {
					out.print( " ( Last Update: "+lastUpdate);
					paren = true;
				}
				if (nextUpdate.length() > 0) {
					if (paren)
						out.print(", ");
					else
						out.print(" ( ");
					out.print( " Next Update: "+nextUpdate);
					paren = true;
				}
				if (paren)
					out.print(" )");
				out.println("<br>");
			}
		}
		out.println("</body></html>");
	}

/**
* Find a subproduct given its unique approveId
*
* @return null if not found
*/
	private Product findProductWithApproveId(ContainerProduct root, String approveId) {
		MatchProductId matcher = new MatchProductId(approveId);
		root.apply(matcher);
		return matcher.product;
	}
/**
* This method finds the product that the user wants to edit and then calls
* another method to print out the form to edit the product.
*/
	private void getEditForm(Context context, String index) throws IOException
	{
		ServletOutputStream out = context.res.getOutputStream();
		VectorProduct original = context.channel.getWaitingProduct();
		if (original == null) {
			printErrorMsg(out,"The channel is empty");
			return;
		}

		Product p = findProductWithApproveId(original, index);
		if (p == null) {
			printErrorMsg(out,"Can't edit, the product doesn't exist <i>[SrvProductApprove.getEditForm approve id="+index+"]</i>");
			return;
		}
		
		out.println("<html><head>");
		out.println("<META HTTP-EQUIV=\"expires\" CONTENT=\"Wed, 11 Feb 1988 01:01:01 GMT\"" );
		out.println("<title>Edit Product data</title></head>");
		out.print("<body><h2>Edit Product " + index + " of channel ");
		out.print(context.realm + "</h2>");
		out.print("<form method=\"POST\" action=\"" );
			out.print(APPROVE_QUERY_BASE);
			out.print( "\" ENCTYPE=");
		out.println("\"application/x-www-form-urlencoded\">");
		out.print("<input type=\"hidden\" value=\"" + context.realm);
		out.println("\" name=\"channel\">"); 
		out.print("<input type=\"hidden\" value=\"" + index);
		out.println("\" name=\"index\">"); 
		out.println("<table border=0>");
		out.println("<tr><td valign=top><font size=+1>ID:</b></font></td><td>");
		out.println("<TEXTAREA NAME=\"id\" ROWS=3 COLS=50 WRAP=VIRTUAL>" + p.getId() + "</TEXTAREA>");
		out.println("</td></tr><tr><td valign=top>");
		out.print("<br><font size=+1>Desc: </b></font></td><td>");
		out.print("<TEXTAREA NAME=\"desc\" ROWS=3 COLS=50 WRAP=VIRTUAL>");
		out.print(p.getDescription());
		out.println("</TEXTAREA></td></tr></table><br><br>");
		out.println("<input type=\"submit\" value=\"Submit!\"></form></body></html>");
		out.flush();
  }


/**
 * This method prints out the HTML heading of the page we are creating in 
 * response to the servlet request.
 */
  private void listTopOfPage(ServletOutputStream out, String channelname) throws IOException
  {
    out.println("<html>");
    out.println("<head><title>Agent " + channelname + "</title></head>");
    out.println("<body>\n<font size=+1>");
    out.println("Approve, Reject, or Edit products on <b>"+channelname+"</b>.");
    out.println("</font>");
    out.println("Submit button is at bottom.<br><BR>");
    out.print("<form method=\"POST\" action=\"" );
    	out.print(APPROVE_QUERY_BASE);
    	out.print( "\" ENCTYPE=");
    out.println("\"application/x-www-form-urlencoded\">");
    out.print("<input type=\"hidden\" value=\"" + channelname);
    out.println("\" name=\"channel\">"); 
  }

/**
 * This method builds an HTML table
 * that allows the user to see and modify whats 
 * in the channels containerProduct.
 */
	private void listBotOfPage(ServletOutputStream out, ContainerProduct containerProduct, String chname) throws IOException
	{
		Enumeration e = (Enumeration) containerProduct.getContent();
		
		out.println("<table border=0>");
		int count = 0;
		while (e.hasMoreElements()) {
			Product prod = (Product) e.nextElement();
			Object o = prod.getContent();
			String tag = prod.getProperty(Approve.APPROVE_ID_TAG);
			
			if (o instanceof String) {
				printNormalProduct(out,prod,tag);
				out.print((String) o);
				out.println("</td>\n<td>");
				printChangeLink(out, tag, chname);
			}
			else if (o instanceof URL) {
				printNormalProduct(out,prod,tag);
				out.print("<a href=\"" + ((URL) o).toString() + "\">");
				out.print( ((URL) o).toString() + "</a>");
				out.println("</td>\n<td>");
				printChangeLink(out, tag, chname);
			}
			else if (o instanceof FileRef) {
				printNormalProduct(out,prod,tag);
				out.println("FileRef<br>");
				out.print("<a href=\"");
				try {
					out.print( ((FileRef) o).getReferenceAsFullURLString() );
					out.println("\">" + ((FileRef) o).getFile().getName() + "</a><br>");
				}
				catch (Exception ex1) { 
					out.println("\"></a><br>");
				}
				try {
					out.println("MIMEType: " + ((FileRef) o).getMIMEType() + "<br>");
					out.println("Size: " + ((FileRef) o).getFile().length() + "<br>");
					Date d = new PAMDate(((FileRef) o).getFile().lastModified());
					out.println("Last Modified: " + d.toString() + "<br><br>");
				} 
				catch (Exception ex2) {
					out.println("<br><br>");
				}
				out.println("</td>\n<td>");
				printChangeLink(out, tag, chname);
			}
			else if (o instanceof Enumeration) {      
				listEnumeration(out, prod, (Enumeration)o, chname);
			}
			else { // unknown content type should not happen
				reporter.assert(false,"SrvProductApprove.listBotOfPage unknown content type ",o,null,null);
				printNormalProduct(out,prod,tag);
				out.println("Error");
				out.println("</td>\n<td>");
				printChangeLink(out, tag, chname);
			}
			
			count++;
		}
		out.println("</table>");
	}


/**
 * This method decides what to display if the content of the product is an
 * Enumeration.  
 */
	private void listEnumeration(ServletOutputStream out, Product prod, Enumeration e, String chname) throws IOException
	{
		boolean website = false;
		while (e.hasMoreElements()) {
			Product p = (Product) e.nextElement();
			String tag = p.getProperty(Approve.APPROVE_ID_TAG);
			
			if (p.getId().equals("root page")) {
				// A web page, don't need to print everything
				Object obj = p.getContent();
				if (obj instanceof FileRef) {
					// a web site so give link to index.html
					printNormalProduct(out,prod,tag);
					out.println("Web Site<br>\nIndex: ");
					out.print("<a href=\"");
					try {
						out.print(((FileRef) obj).getReferenceAsFullURLString());
						out.println("\">" +((FileRef) obj).getFile().getName() + "</a><br>");
					}
					catch (Exception ex) {
						out.println("\"></a><br>");
					}
					
					out.println("<br>");
					
					out.println("</td>\n<td>");
					printChangeLink(out, tag, chname);
					website = true;
					break;
				}
			}
		}
		if (website)
			return;
		
		printEnumerationProduct(out, prod, chname);
		listBotOfPage(out, (ContainerProduct)prod, chname); // go as far deep as needed
		
	}
	
	
	/**
	 * This method prints out a link to edit a certain product on the channel
	 */
	  private void printChangeLink(ServletOutputStream out, String tag, String chname) throws IOException
	  {
	    out.print("<a href=\"");
	    	out.print( APPROVE_QUERY_BASE);
	    	out.print( "?index=" + tag );
	    out.println("&channel=" + URLEncoder.encode(chname) + "\">Edit</a><br><br></td>\n</tr>");
	  } 
	
	
	/**
	 * This method prints the format of a Product with an Enumeration as it's 
	 * content.
	 */
	  private void printEnumerationProduct(ServletOutputStream out, Product prod, String chname) throws IOException
	  {
	  	String tag = prod.getProperty(Approve.APPROVE_ID_TAG);
	    out.println("<tr><td valign=top><B>ID:</B></TD>");
	    out.println("<TD valign=top width=50%>");
	    out.println(prod.getId() + "</td>\n<td valign=top>");
	    out.print("<a href=\"");
	    	out.print( APPROVE_QUERY_BASE);
	    	out.print( "?index=");
	    	out.print( tag );
	    out.println("&channel=" + URLEncoder.encode(chname) + "\">Edit</a></td>\n</tr>");
	    out.println("<tr><td valign=top><b>Desc:</b></td>");
	    out.print("<td valign=top width=50%>" + prod.getDescription());
	    out.println("</td>\n<td valign=top>");
	    out.println("&nbsp;</td>\n</tr>\n<tr>");
	    out.println("<td valign=top><b>Content:</b></td>\n<td valign=top colspan=2>");
	  }
	
	
	/**
	 * This method prints the id, description, and content for a product without
	 * an Enumeration as content.
	 */
	  private void printNormalProduct(ServletOutputStream out, Product prod, String tag) throws IOException
	  {
	    out.println("<tr><td valign=top><B>ID:</B></TD>");
	    out.println("<TD valign=top width=50%>");
	    out.println(prod.getId() + "</td>\n<td valign=top>");
	    out.print("<INPUT TYPE=\"radio\" NAME=\"" + tag + "\" ");
	    out.println("VALUE=\"approve\">Approve</td>\n</tr>");
	    out.println("<tr><td valign=top><b>Desc:</b></td>");
	    out.print("<td valign=top width=50%>" + prod.getDescription());
	    out.println("</td>\n<td valign=top>");
	    out.print("<INPUT TYPE=\"radio\" NAME=\"" + tag + "\" ");
	    out.println("VALUE=\"reject\">Reject</td></tr>");
	    out.println("<td valign=top><b>Content:</b></td>\n<td valign=top>");
	    
	  }
	    
	
	/**
	 * This method handles the approving and rejection of projects.
	 */
	  private void doApprovesAndRejects(Context context, Hashtable htmlVals) throws IOException
	  {
	    
	    if (context.channel == null) {
	      printErrorMsg(context.res.getOutputStream(),"The Approve Channel: " + context.realm + " was not found.");
	      return;
	    }

	    VectorProduct original = context.channel.getWaitingProduct();
	    if (original == null) {
	      printErrorMsg(context.res.getOutputStream(),"The channel is empty");
	      return;
	    }
	
		Hashtable approveList = new Hashtable();
		Hashtable rejectList = new Hashtable();
		
		makeApproveRejectLists(htmlVals, approveList, rejectList);

		String authString = "on "+context.realm+", Authorization: "+context.username + " [" + context.req.getRemoteHost()+"]";
	/*
	* First delete rejected products
	*/
		if (rejectList.size() > 0) {
			Disconnecter disconnector = new Disconnecter(rejectList, reporter, true, authString);
			original.apply(disconnector);
			original.prune();
		}
		
	/*
	* Second clone the original to extract the approved only
	*/
		if (approveList.size() > 0) {
			VectorProduct approved = (VectorProduct) original.cloneDeeper(context.channel.getId(), context.channel.getDescription());
			KeepOnly keepOnly = new KeepOnly(approveList, reporter, true, authString);
			approved.apply(keepOnly);
			approved.prune();
			context.channel.setApprovedProduct(approved);
		
	/*
	* Third remove whatever has already been approved
	*/
			Disconnecter disconnector = new Disconnecter(approveList, reporter, false, null);
			original.apply(disconnector);
			original.prune();
		}
		printRefreshHTML(context);
	    context.res.getOutputStream().flush();    
	  }
	  
	/**
	 * This method changes the id and description of a product given by the edit
	 * form. Because the id may be used by the parent container it can't be just changed,
	 * the channels product has to be rebuilt 
	 * from scratch changing only the product being edited.
	 */
		private void doEditProduct(Context context, Hashtable h, String index) throws IOException
		{
			ServletOutputStream out = context.res.getOutputStream();
			String id = ((String []) h.get("id"))[0];
			String description = ((String []) h.get("desc"))[0];
					
			VectorProduct original = context.channel.getWaitingProduct();
			if (original == null) {
			  printErrorMsg(out,"The channel is empty");
			  return;
			}
			
			if (index == null || index.length() == 0) {
				reporter.assert(false,"SrvProductApprove.doEditProduct bad index",null,null,null);
				printErrorMsg(out,"SrvProductApprove.doEditProduct bad index");
				return;
			}
			
			Product p = findProductWithApproveId(original, index);

			if (p == null ) {
				reporter.assert(false,"SrvProductApprove.doEditProduct couldnt find product ",index,null,null);
				printErrorMsg(out,"SrvProductApprove.doEditProduct couldnt find product "+index);
				return;
			}
			

			p.setId(id);
			p.setDescription(description);
						
			printRefreshHTML(context);
			out.flush();    
		}	
	

/**
* Makes a hashtable list of approve/reject ids.  The approveId
* is used as the key in the hashtable to support more efficient look ups.
*/
	private void makeApproveRejectLists(Hashtable htmlVals, Hashtable approveList, Hashtable rejectList)
	{
		Enumeration keys = htmlVals.keys();
		Enumeration vals = htmlVals.elements();
		while (keys.hasMoreElements()) {
			String k = (String) keys.nextElement();
			String v[] = (String []) vals.nextElement();
			
			if (v[0].equals("approve")) {
				approveList.put(k,k);
			}
			else if (v[0].equals("reject")) {
				rejectList.put(k,k);
			}
		}
	}
	
	
	/**
	* Print html to cause refresh of channel page
	*/
	private void printRefreshHTML( Context context) throws IOException
	{
		ServletOutputStream out = context.res.getOutputStream();
	    out.print("<META HTTP-EQUIV=\"REFRESH\" CONTENT=\"0;URL=");
    	out.print( APPROVE_QUERY_BASE );
    	out.print( "?channel=");
    	out.print( URLEncoder.encode(context.realm) );
    	out.print( "\">");
	}	
	
	/**
	* This method prints an error msg
	*/
	private void printErrorMsg(ServletOutputStream out, String msg) throws IOException
	{
		out.println("<html>");
		out.println("<head><title>Product Approve Servlet Error</title></head>");
		out.println("<body bgcolor=\"#FFFFFF\"><h2>Product Approve Servlet Error:");
		out.println("</h2><blockquote><font color=\"darkred\" size=+1>" + msg);
		out.println("</body></html>");
	}
	
		
	/**
	* This method gives us a handle on the Approve channel that is being used.
	*/
	private Approve getApproveChannel(String channelid)
	{
		Approve approve;
		try {
			approve = (Approve) exec.getActiveSourceById("channel", channelid);
		}
		catch (ClassCastException cce) {
			return null;
		}
		return approve;
	}

/**
* A context record for authorization of user
*/
	class Context extends org.mitre.pam.util.AuthContext{
			Approve channel;
		}

	
}
