/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.remote;

import java.util.*;
import java.io.*;
import java.net.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;

import org.mitre.pam.interfaces.ExecCallbacks;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.AsynchronousAgent;
import org.mitre.pam.executive.Exec;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.getter.PIMShell1;
import org.mitre.pam.interfaces.ScheduleAttributes;

/**
* A Juggler that receives a product from another iPAM system, reconstructs it, and
*  passes it on.
*
* @author Srinivas Inguva
*/

public class Connector extends PIMShell1 implements AsynchronousAgent {
  
  public static String OBJECT_NAME = "proxyServiceName";
  public static final String PRODUCT_FILE_NAME = "prodStore.txt";
  public static String defServiceName = null;
  

  public Connector() {
    setDescription("Receives and reconstructs a product from another iPAM using Java RMI");
	buildDefServiceName();
    setProperty(OBJECT_NAME, defServiceName);

    metaStringInfo(OBJECT_NAME, "proxyForPAM", "The rmi service name used by this connector iPAM (must be unique)");
  }
  
  private void buildDefServiceName() {
	if (defServiceName != null)
		return;
	URL oururl=null;
	try {
		oururl = new URL(ExecGlobal.getProtocolHostNoPort());
		defServiceName = "rmi://"+oururl.getHost()+"/proxyForPAM";
	}
	catch (Exception e) {
		defServiceName="proxyForPAM";
	}
  }
  
/**
* All of the work is done in ConnectorServer.notify()!
*/
  public Product doFetch() {
    return null;
  }

/*
* On init, register for RMI
*/
  private void initialize() {
    //Need to set a security manager before using RMI services!

    try {
      ConnectorServer cs = new ConnectorServer(this, reporter);
      cs.setProdFile(PRODUCT_FILE_NAME);
      String obj = getStringProperty(OBJECT_NAME,defServiceName);
      Naming.rebind(obj,cs);
    } catch (Exception e) {
      reporter.reportException(e);
      reporter.warning("Failed to bind to RMIRegistry!");
    }
  }

/**
* Called to distribute the product that "comes in" over
* this connector
*/
	public void pushProduct(Product p) {
		pushProductToAllClients(p);
	}

/**
* Called after the exec has loaded everything when it is ready
* to start real work or when this agent is created its 
* properties have been initialized.
*/
	public void startService() {
		initialize();
	}


/**
* Called when the exec is ready to shutdown or when this agent
* is going to be deleted.  
*/
	public void stopService(){}
	
/**
* Called to request that the agent stop injecting asynchronous events into the system.
* However, the agent does not need to stop listening and responding to
* sockets.  An agent should have a queue so that it can process its "stimulus"
* without stimulating the system during a pause. Do not assume that pauseService
* is called before stopService.  The exec may decide to stop and may not issue a pause.
*/
	public void pauseService(){}
	
/**
* Called when it is ok to pass stimulus to the exec or other agents.
*/
	public void resumeService(){}
}
