/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.altavista;

import java.util.Hashtable;

import org.mitre.pam.getter.search.parse.LoggingHTMLParser;
import org.mitre.pam.getter.search.parse.ParseException;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.getter.search.ThreePart;

/**
* Parse results from Altavista Search Intranet v2.3A (1999a)
*
* @author Jason Mathews 9/23/99
* @since iPAM 2.4.84
*/

public class GetOnePage99a extends GetOnePage97 {

	public GetOnePage99a(Reporter reporter, String id, String urlPart0, String urlPart1) {
		super(reporter, id, urlPart0, urlPart1);
	}

	protected void scan(LoggingHTMLParser parser) throws ParseException {
	      String description, url, title;
	      for (int i=1; i <= 10; i++) {
/*

query results format:

<item>
...
<A href="url"><ititle>title</ititle></A>
...
<abstract>description</abstract>
...
<TD>
<FONT..>
[Mod date]
<A hef="url">url</A>[page size]<P>
</item>

*/

			if (! parser.SkipToAfterTag("<item>") || ! parser.SkipToOpenTag(LoggingHTMLParser.A))
				return;

			Hashtable params = parser.ProcessParameters();
			url = (String) params.get("HREF");         // get link
//System.out.println("url = "+url);
			if (url == null || url.length() == 0) {
				reporter.debugMsg("WARNING: missing value for HREF tag" , null , null, null, DEBUG_KEY);
				return;
			}

			// get title
			if (! parser.SkipToAfterTag("<ititle>")) {
 			        reporter.debugMsg("WARNING: <ititle> tag not found in input" , null , null, null, DEBUG_KEY);
				return;
			}

			title = parser.GetUntilNextTag();
			if (title == null || title.length()==0) {
 			        reporter.debugMsg("WARNING: missing value for <ititle> tag" , null , null, null, DEBUG_KEY);
				return;
			}
			title = LoggingHTMLParser.stripHTML(title);

			// get abstract
			if (! parser.SkipToAfterTag("<abstract>")) {
 			        reporter.debugMsg("WARNING: <abstract> tag not found in input" , null , null, null, DEBUG_KEY);
				return;
			}
			description = parser.GetUntilTag("</abstract>");
			//System.out.println("description = " + description);
			if (description == null || description.length()==0)
				description = "no description";
			else {
				description = LoggingHTMLParser.removeAllStrings(description, "&nbsp;");
				description = LoggingHTMLParser.stripHTML(description);
			}

			ThreePart s = new ThreePart();
			s.url = url;
			s.id = title;
			s.description = description;
			reporter.debugMsg("Results:", s.id, s.url, s.description, DEBUG_KEY);		
			results.addElement(s);
		}
	}
 
}
