/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.bnn;

import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import java.text.SimpleDateFormat;
import java.io.*;
import java.net.*;

import org.mitre.pam.getter.search.parse.ParseException;
import org.mitre.pam.getter.search.parse.LoggingHTMLParser;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pdif.eventParser.ValueList;

/**
 * This class connects to the BNN and puts all of the stories found into a 
 * Vector.
 *
 * @author Michael DeCorpo
 * @author Srinivas Inguva (added basic search capability)
 * @author Jason Mathews (many changes including multi-values on people/topic searches)
 *
 */

public class GetBNN {
  private Reporter reporter;
  private Date start_search;
  private Date end_search;
  private String urlstring;
  private String searchString;
//  private String personString;
  private String orgString;
  private String locString;
  private boolean doingWordSearch = false;
  private boolean doingTopicSearch = false;
  private boolean doingPersonSearch = false;
  private boolean doingOrgSearch = false;
  private boolean doingLocSearch = false;

  Vector stories = new Vector(50);

  //The 'BNN_rev' part of bnnRoot changes periodically...
  private String bnnRoot;

  private String url_prefix;
  private Vector sources = null;
  private Vector topics = null;
  private Vector people = null;

  public GetBNN(String host, ValueList sourceList, Reporter reporter) {
    this.reporter = reporter;
    this.bnnRoot = host;
    if (sourceList != null) {
      sources = new Vector();
      for (Enumeration e=sourceList.elements(); e.hasMoreElements(); ) {
        String source = (String) e.nextElement();
        if (source.length() != 0 && !source.equals("dummy"))
          sources.addElement(URLEncoder.encode(source));
      }
    }
  }

  public Vector getStories() {
    return stories;
  }

  public String getURLString() {
    return urlstring;
  }

  //Return a URL pointing to the given story
  public String getURLfromStory(BNNStory s) {
    return makeURL(s.id, false);
  }

/**
 *
 * Make the correct url string for the stories we want.
 *
 */
  protected String makeURL(String startRec, boolean initialSearch) {

    boolean doingSearch = doingWordSearch || doingTopicSearch || doingPersonSearch ||
		doingOrgSearch || doingLocSearch;

    if (url_prefix == null) {

      // build the url_prefix once and save for later recall

      StringBuffer sb = new StringBuffer(80);

      sb.append(bnnRoot);
      sb.append("?NewsAgency=dummy"); // defaults to all sources
      if (sources != null)
       for (Enumeration e=sources.elements(); e.hasMoreElements(); ) {
	String s = (String) e.nextElement();
		sb.append("&NewsAgency=");
		sb.append(s);
       }

    sb.append("&FromDate=");
    sb.append(makeDateString(start_search));
    sb.append("&ToDate=");
    sb.append(makeDateString(end_search));
    sb.append("&DTheme=dummy&DPerson=dummy&DOrg=dummy&DLoc=dummy");
      sb.append("&Qty=10");

      if (doingSearch)
        sb.append("&ret_url=bnn_revh.bnn_search");
      else
        sb.append("&ret_url=bnn_revh.bnn_browse");

      if (doingTopicSearch) {
        for (Enumeration e=topics.elements(); e.hasMoreElements(); ) {
          sb.append("&DTheme=");
          sb.append(e.nextElement());
        }
      }

      if (doingPersonSearch) {
       for (Enumeration e=people.elements(); e.hasMoreElements(); ) {
	 sb.append("&DPerson=");
	 sb.append(e.nextElement());
       }
      }

      if (doingOrgSearch) {
        sb.append("&DOrg=");
        sb.append(orgString);
      }

      if (doingLocSearch) {
        sb.append("&DLoc=");
        sb.append(locString);
      }

      if (doingWordSearch) {
        //require that all words appear (i.e. 'AND' them together)
        sb.append("&qop=ALL");
	sb.append("&qterm=");
	sb.append(searchString);
      }
      else
        sb.append("&qop=N");

      url_prefix = sb.toString();
    }

    StringBuffer sb = new StringBuffer(url_prefix);

    sb.append("&StartRec=");
    sb.append(startRec);

    sb.append("&MetaTags=");
    sb.append(initialSearch ? "True" : "False");

    if (initialSearch) {
      if (doingSearch)
        sb.append("&qsub=Search");  // searching
      else
        sb.append("&qsub=Story");   // browsing
    } else
      sb.append("&qsub=Next+Story");	// use for individual stories
      
    return sb.toString();
  }

/**
 * Connect to the BNN, filter and parse the output stream from the BNN URL
 *
 */
  public void getNow(Date startDate) throws IOException, ParseException {
    start_search = startDate;
    end_search = new Date();

    url_prefix = null;
    urlstring = makeURL("1", true);

    reporter.debugMsg("BNN query: " ,urlstring, null,null,
      org.mitre.pam.getter.PIMShell1.LOG_QUERIES);

    URL bnnURL = new URL(urlstring);

    InputStream urlStream = bnnURL.openStream();

    LoggingHTMLParser parser = new LoggingHTMLParser(urlStream, reporter);
    String temp;

    try {
      while (parser.SkipToAfterTag("<X-BNN-Summary>")) {
	BNNStory s = new BNNStory();
	temp = parser.GetUntilTag("</X-BNN-Summary>");
	temp.trim();
	if (temp.startsWith("<!--") && temp.endsWith("-->"))
	  temp = temp.substring(4,temp.length()-3);
	s.summary = temp;
	parser.SkipToAfterTag("</X-BNN-Summary>");

	parser.SkipToAfterTag("<X-BNN-StoryId>");
	temp = parser.GetUntilTag("</X-BNN-StoryId>");
	temp.trim();
	if (temp.startsWith("<!--") && temp.endsWith("-->"))
	  temp = temp.substring(4,temp.length()-3);
	s.id = temp;
	parser.SkipToAfterTag("</X-BNN-StoryId>");

	parser.SkipToAfterTag("<X-BNN-Source>");
	temp = parser.GetUntilTag("</X-BNN-Source>");
	temp.trim();
	if (temp.startsWith("<!--") && temp.endsWith("-->"))
	  temp = temp.substring(4,temp.length()-3);
	s.source = temp;
	parser.SkipToAfterTag("</X-BNN-Source>");

	parser.SkipToAfterTag("<X-BNN-Date>");
	temp = parser.GetUntilTag("</X-BNN-Date>");
	temp.trim();
	if (temp.startsWith("<!--") && temp.endsWith("-->"))
	  temp = temp.substring(4,temp.length()-3);
	s.date = temp;
	parser.SkipToAfterTag("</X-BNN-Date>");

	parser.SkipToAfterTag("<X-BNN-Length>");
	temp = parser.GetUntilTag("</X-BNN-Length>");
	temp.trim();
	if (temp.startsWith("<!--") && temp.endsWith("-->"))
	  temp = temp.substring(4,temp.length()-3);
	s.length = temp;
	parser.SkipToAfterTag("</X-BNN-Length>");

	parser.SkipToAfterTag("<X-BNN-PERSON>");
	temp = parser.GetUntilTag("</X-BNN-PERSON>");
	temp.trim();
	if (temp.startsWith("<!--") && temp.endsWith("-->"))
	  temp = temp.substring(4,temp.length()-3);
	s.people = temp;
	parser.SkipToAfterTag("</X-BNN-PERSON>");

	parser.SkipToAfterTag("<X-BNN-ORGANIZATION>");
	temp = parser.GetUntilTag("</X-BNN-ORGANIZATION>");
	temp.trim();
	if (temp.startsWith("<!--") && temp.endsWith("-->"))
	  temp = temp.substring(4,temp.length()-3);
	s.organizations = temp;
	parser.SkipToAfterTag("</X-BNN-ORGANIZATION>");

	parser.SkipToAfterTag("<X-BNN-LOCATION>");
	temp = parser.GetUntilTag("</X-BNN-LOCATION>");
	temp.trim();
	if (temp.startsWith("<!--") && temp.endsWith("-->"))
	  temp = temp.substring(4,temp.length()-3);
	s.locations = temp;
	parser.SkipToAfterTag("</X-BNN-LOCATION>");

	parser.SkipToAfterTag("<X-BNN-Story>");
	temp = parser.GetUntilTag("</X-BNN-Story>");
	temp.trim();
	if (temp.startsWith("<!--") && temp.endsWith("-->"))
	  temp = temp.substring(4,temp.length()-3);
	s.story = temp;
	parser.SkipToAfterTag("</X-BNN-Story>");

	stories.addElement(s);
      }
    } catch(Exception ex) {
      reporter.reportException( ex);
    } catch(Error er) {
      reporter.reportException( er);
      throw new ParseException(er.getMessage());
    } finally {
      urlStream.close();
    }
  }

/**
 * Convert the date ino the BNN format (e.g. 02-Nov-1998)
 *
 */
  protected String makeDateString(Date d)
  {
    return new SimpleDateFormat("dd-MMM-yyyy").format(d);
  }

  public void setSearchString(String ss) {
    searchString = URLEncoder.encode(ss);
    doingWordSearch = true;
  }

  public void setTopics(ValueList vl) {
    topics = new Vector(vl.size());
    for (Enumeration e=vl.elements(); e.hasMoreElements(); ) {
      String topic = (String) e.nextElement();
      if ( topic.length() != 0 ) {
        topics.addElement(URLEncoder.encode(topic));
    doingTopicSearch = true;
  }
    }
  }

  public void setPerson(ValueList vl) {
    people = new Vector(vl.size());
    for (Enumeration e=vl.elements(); e.hasMoreElements(); ) {
      String person = (String) e.nextElement();
      if ( person.length() != 0 ) {
        people.addElement(URLEncoder.encode(person));
    doingPersonSearch = true;
  }
    }
  }

  public void setOrg(String os) {
    this.orgString = URLEncoder.encode(os);
    doingOrgSearch = true;
  }

  public void setLoc(String ls) {
    this.locString = URLEncoder.encode(ls);
    doingLocSearch = true;
  }
/*
  public static void main ( String args[] ) {
    try {
      GetBNN gb = new GetBNN("http://videoserver3.mitre.org/tr/owa/BNN_revh.BnnFindStories", null, null);
      System.out.println("getting now");
      gb.getNow(new Date(1996,0,1));
      System.out.println("got now");
      Vector stories = gb.getStories();
      System.out.println(stories.size());
      for (int i=0; i < stories.size(); i++) {
	BNNStory s = (BNNStory) stories.elementAt(i);
	System.out.println(gb.getURLfromStory(s));
	System.out.println("Story " + (i+1));
	System.out.println("date: " + s.date);
	System.out.println("source: " + s.source);
	System.out.println("summary: " + s.summary);
	System.out.println("length: " + s.length);
	System.out.println("story: " + s.story);
	System.out.println();
      }
    } catch(Exception ex) {
    } catch(Error er) {
    }
  }
*/
}//end GetBNN
