/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.excite;

import java.io.InputStream;
import java.io.IOException;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.getter.PIMShell1;

import org.mitre.pam.getter.search.parse.LoggingHTMLParser;
import org.mitre.pam.getter.search.parse.ParseException;

import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.getter.search.ThreePart;


/**
* The Excite Search PIM will perform a search for news articles using the Excite search engine
* at www.excite.com.
*/
public class ExciteKeywordPIM extends PIMShell1 {

  private static final String AND = "AND";
  private static final String NOT = "NOT";
  private static final String MUST_HAVE = "mustHaveWords";
  private static final String MUST_NOT_HAVE = "mustNotHaveWords";
  private static final String EXCITE_HOST = "search.excite.com";

  protected Vector results = new Vector();
	private static final String[] DEFAULT_NOT_HAVE = {"sex","nude","erotic","horny"};
	private static final String[] DEFAULT_HAVE = {"word1","word2"};

  public ExciteKeywordPIM() {
    setDescription("Excite search at search.excite.com");
    
    ValueList vlHave = new ValueList(DEFAULT_HAVE);
    ValueList vlNotHave = new ValueList(DEFAULT_NOT_HAVE);
    
    setPropertyValues(MUST_HAVE, vlHave);
	setPropertyValues(MUST_NOT_HAVE, vlNotHave);

    metaMultiStringInfo(MUST_HAVE, vlHave,"list of words that must ALL be in documents");
	metaMultiStringInfo(MUST_NOT_HAVE, vlNotHave,"list of words that can not be in documents");
  }

  public Product doFetch() {
    results = new Vector();

    try {
      URL searchURL = buildURL();
      InputStream in = searchURL.openStream();
      LoggingHTMLParser parser = new LoggingHTMLParser(in, reporter);
      scan(parser);

      in.close();

      if(results.size() == 0) {
        return null;
      }
      else {
        VectorProduct container = new VectorProduct(getId(), getDescription());
        Enumeration enum = results.elements();

        while(enum.hasMoreElements()) {
          ThreePart tp = (ThreePart)enum.nextElement();
          BasicProduct bp = new BasicProduct(tp.id, tp.description);
          try {
            bp.setContent(new URL(tp.url));
            container.addElement(bp);
          } catch(MalformedURLException mue) {
            reportException(mue);
          }
        }
        return container;
      }
    } catch(MalformedURLException mue) {
      reportException(mue);
    } catch(IOException ioe) {
      reportException(ioe);
    }
    return null;
  }

  /**
  * This function will attempt to build a URL using the searchWords and notSearchWords
  * fields.
  */
  protected URL buildURL() throws MalformedURLException {
    String URLstring = new String("http://" + EXCITE_HOST + "/search.gw?c=timely&sort=date&search=");

    StringBuffer searchString = new StringBuffer();

    // AND words
    ValueList words = getPropertyValues(MUST_HAVE);
    if(words.size() > 0) {
      searchString = searchString.append((String)words.elementAt(0));
      for(int i=1; i<words.size(); i++) {
        searchString = searchString.append("+" + AND + "+" + (String)words.elementAt(i));
      }
    }

    // NOT words
    words = getPropertyValues(MUST_NOT_HAVE);
    if(words.size() > 0) {
      if(((String)words.elementAt(0)).length() > 0) {
        searchString = searchString.append("+" + AND + "+" + NOT + "+" + (String)words.elementAt(0));
      }
      for(int i=1; i<words.size(); i++) {
        if(((String)words.elementAt(i)).length() > 0) {
          searchString = searchString.append("+" + AND + "+" + NOT + "+" + (String)words.elementAt(i));
        }
      }
    }

    URL searchURL = new URL(URLstring + searchString.toString());

    return searchURL;
  }

  /**
  * This function attempts to extract the useful info out of the html page that is
  * returned by the Excite engine.
  *
  * The structure of the html that we are interested in parsing looks like this:
  * <SMALL>64% </SMALL> <A HREF="http://www.wired.com/news/news/technology/story/13098.html">Technology News from Wired News</A>
  */
  protected void scan(LoggingHTMLParser parser) {

    try {
      while(parser.SkipToOpenTag(LoggingHTMLParser.SMALL)) {
        ThreePart tp = new ThreePart();

        String line = parser.GetUntilClosedTagOfType(LoggingHTMLParser.SMALL);
        if(line.indexOf("%") > -1) {
          if(parser.SkipToOpenTag(LoggingHTMLParser.A)) {
            Hashtable h = parser.ProcessParameters();
	    String value = (String) h.get("HREF");         // get link
	    if (value != null) tp.url = value;
            line = parser.GetUntilClosedTagOfType(LoggingHTMLParser.A);
            tp.id = line;

            tp.description = new String();

            results.addElement(tp);
          }
        }
      }
    } catch(ParseException pe) {
      reportException(pe);
    }
  }
}