/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.excite;

import java.util.Date;
import java.util.Vector;
import java.util.Hashtable;
import java.io.*;
import java.lang.*;
import java.net.*;
import org.mitre.pam.getter.search.parse.LoggingHTMLParser;
import org.mitre.pam.interfaces.Reporter;

/**
* This class connects to a custom Excite Channel and puts all of the stories 
* found into a Vector.
*  
* @author Nazario Irizarry, Jr (updates)
*/


public class GetNewsTracker2 implements GetNewsTracker {
  
  Vector stories = new Vector(20);  
  private String exciteURLString;
  Reporter reporter;
  private static final String DEBUG_KEY = "watchNewstrackerParse";

/** 
 * The constructor 
 */
  public GetNewsTracker2(String urlString, Reporter reporter) 
  {
  	this.reporter = reporter;
  	
    if (urlString == null)
      exciteURLString = "";
    else
      exciteURLString = urlString;
  }

  

/**
 * Return a Vector of Excite Stories
 *
 */
  public Vector getStories()
  {
    return stories;
  }

/**
 * Connect to Excite, filter and parse the output stream from the Excite URL
 * Note that the returned date on hits may be null if the date can't be parsed.
 *
 */
  public void getNow() throws IOException, Exception
  {
    URL exciteURL = new URL(exciteURLString);

    InputStream urlStream = exciteURL.openStream();
    LoggingHTMLParser parser = new LoggingHTMLParser(urlStream, reporter);
	int storyCount = 1;
    try {
		while (parser.SkipToOpenTag(LoggingHTMLParser.A)) 
		{
			reporter.debugMsg("looking...",null,null,null,DEBUG_KEY);
			ExciteStory s = new ExciteStory();
			
			Hashtable params = parser.ProcessParameters();
			s.url = (String) params.get("HREF");         // get link
			reporter.debugMsg(" potential url:",s.url,null,null,DEBUG_KEY);
			if (s.url == null) {
				reporter.log("null story detection based on URL on query "+exciteURLString+" story index = "+storyCount);
				continue;
			}
			if (!s.url.startsWith("http://")) // valid urls start with http://
				continue;
			if (s.url.indexOf("excite") >= 0) // local excite references don't count
				continue;

			
			parser.SkipToAfterOpenTag(LoggingHTMLParser.B);
			s.title = parser.GetUntilNextTag();          // get title
			reporter.debugMsg(" potential title: ",s.title,null,null,DEBUG_KEY);
			if (s.title == null) {
				reporter.log("null story detection based on TITLE on query "+exciteURLString+" story index = "+storyCount);
				continue;
			}
			parser.SkipToAfterOpenTag(LoggingHTMLParser.NOBR);
			parser.SkipToAfterString("(");
			s.source = parser.GetUntilString(")");       // get source
			reporter.debugMsg(" source:",s.source,null,null,DEBUG_KEY);
			
			parser.SkipToAfterToken(LoggingHTMLParser.BR);
			parser.SkipToAfterCloseTag(LoggingHTMLParser.FONT);
			s.story = parser.GetUntilNextTag();          // get story
			if (s.story == null) {
				reporter.log("null story detection based on STORY on query "+exciteURLString+" story index = "+storyCount);
				continue;
			}
			s.story = parser.removeAllStrings(s.story, "&nbsp;");
			reporter.debugMsg(" story:",s.story,null,null,DEBUG_KEY);
			
			parser.SkipToAfterString("First found: ");
			s.date = null;
			String dateStr = parser.GetUntilNextTag();           // get date
			/*
			* remove "Universal time"
			*/
			if (dateStr != null && dateStr.length() > 4) {
				int i = dateStr.indexOf(" Universal");
				if (i >= 0) {
					s.date = dateStr.substring(0,i);
				}
				else
					s.date = dateStr;
			}
			reporter.debugMsg(" date:",s.date,null,null,DEBUG_KEY);
			storyCount++;
			stories.addElement(s);
		}
    }
    catch (Exception e) {
    	reporter.assert(false,"The following error occured on this url - ",exciteURLString," story index = ",Integer.toString(storyCount));
    	reporter.reportException(e);
    }   // go on if found parse error.
    catch (Error e) {
    	reporter.assert(false,"The following error occured on this url - ",exciteURLString," story index = ",Integer.toString(storyCount));
    	reporter.reportException(e);
    }
    finally {
    	urlStream.close(); 
    }
  
  }
		

}



