/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.search.smarTraveler;

import java.util.Date;
import java.util.Vector;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.io.*;
import java.net.*;

import org.mitre.pam.getter.search.parse.LoggingHTMLParser;
import org.mitre.pam.getter.search.parse.ParseException;
import org.mitre.pam.getter.search.parse.Token;
import org.mitre.pam.interfaces.Reporter;

/**
 * This class connects to a specfic SmarTraveler web page and puts the traffic info
 * into a vector with each paragraph in a separate BasicProduct.
 *
 * @author Jason Mathews
 */

public class GetTravelerBos extends GetTraveler {

  protected String key;
  private String id;
  private String sectionSummary;

  private static final String DEBUG_KEY = "TravelerPIMBos";

/**
 * The constructor
 */
  public GetTravelerBos(String urlString, Reporter reporter, String key, String id)
  {
    super(reporter, urlString);
    this.key = key == null || key.length()==0 ? null : key;
    this.id = id;
    reporter.debugMsg("Traveler Key: " , key , " " , urlString, DEBUG_KEY);
  }

/*
  public void setShortLabels(boolean useShortSectionLabels) {
	super.setShortLabels(useShortSectionLabels);
	reporter.debugMsg("set Short labels " + this.useShortSectionLabels, null, null, null, DEBUG_KEY);
  }
*/

  /* Tests whether a string is a word (only composed of letters) */
  private static boolean isWord(String s) {
	if (s == null || s.length() == 0) return false;
	for (int i=0; i < s.length(); i++) {
	  if (! Character.isLetter(s.charAt(i))) return false;
	}
	return true;
  }

// Sun's JDK substring method barfs if the end or beginning positions exceed the length
// of the string in which we want the entire string or an empty one.

  private static final String subString (String s, int pos, int end) {
	int len = s.length();
	if (pos >= len) return "";	// return empty string
	if (end >= len) {
		if (pos == 0) return s;
		else end = len;
	}
	return s.substring(pos, end);
  }

  private static final String subString(String s, int pos) {
	return subString(s, pos, s.length());
  }

  private String getSectionText(String text) {
	// extract image src summary icon from smarTraveler road report
	int i, j;
	// sectionStatus = UNKNOWN;
	sectionSummary = null;
	if ((i = text.indexOf(".gif")) > 0 && (j=text.lastIndexOf('/', i)) > 0) {
	 String image = text.substring(j+1, i);
	// image tag should be ok|slow|delay|alert
	if (image.length() != 0 && isWord(image))
		sectionSummary = image;
	}
	return LoggingHTMLParser.stripHTML(text);
  }

// private void debug(String s) { exec.debugMsg(s, DEBUG_KEY); System.out.println(s); }

/*
static boolean documentHeader(String s) {
// match pattern: \s*<HTML>\s*<HEAD>
  int i = 0;
  int len = s.length();
  while (i < len && Character.isWhitespace(s.charAt(i))) i++;
  if (i+12 > len || !s.substring(i, i+6).equals("<HTML>")) return false;
  i += 6;
  while (i < len && Character.isWhitespace(s.charAt(i))) i++;
  return i+6 <= len && s.substring(i, i+6).startsWith("<HEAD>");
}
*/

/**
 * Connect to Traveler, filter and parse the output stream from the Traveler URL
 * Note that the returned date on hits may be null if the date can't be parsed.
 *
 */
  public void getNow() throws IOException, ParseException
  {
    InputStream is;

//    if (travelerURLString.startsWith("file://")) {
//      // System.out.println("file = " + travelerURLString.substring(7));
//      is = new FileInputStream(travelerURLString.substring(7));
//    }
      URL travelerURL = new URL(travelerURLString);
      //    InputStream urlStream = travelerURL.openStream();
      is = travelerURL.openStream();
//    }

   if (useShortSectionLabels)
	reporter.debugMsg("use Short labels", null, null, null, DEBUG_KEY);
    
    BufferedInputStream urlStream = new BufferedInputStream(is);

    try {
      LoggingHTMLParser parser = new LoggingHTMLParser(urlStream, reporter);

      if (!parser.SkipToAfterOpenTag(LoggingHTMLParser.CENTER) ||
	  !parser.SkipToAfterOpenTag(LoggingHTMLParser.FONT) ||
	  !parser.SkipToAfterOpenTag(LoggingHTMLParser.B) ||
	  (title = parser.GetUntilClosedTagOfType(LoggingHTMLParser.B)) == null)
	{
	  throw new ParseException("Unable to parse smarTraveler content");
	}

      reporter.debugMsg("Title: " , title,null,null, DEBUG_KEY);

      // Get Date --> mm/dd/yy hh:mm pm
      // expected date format: "3/21/98 1:12 pm" minimum 14 char length
      parser.SkipToAfterOpenTag(LoggingHTMLParser.FONT);
      String date = parser.GetUntilNextTag();
      if (date != null && date.length() < 14)
	date = null;

      reporter.debugMsg("Date: " , date,null,null, DEBUG_KEY);

      String sectionTitle = null;
      String text;
      TravelerStory lastStory = null;

      while (parser.SkipToAfterOpenTag(LoggingHTMLParser.TD)) {
	text = parser.GetUntilClosedTagOfType(LoggingHTMLParser.TD).trim();
	
	reporter.debugMsg("TEXT: " , text,null,null, DEBUG_KEY);
	
		if (text.startsWith("<img src")) {
			sectionTitle = getSectionText(text);
			continue; // get next TD element
		} else if (text.startsWith("<P><BR><img src")) {
			sectionTitle = getSectionText(text);
			// generic final road status (other sections, etc.)
			// add default text to product ???
			// add canned response "all others roads normal" only if no other
			// responses have been added
			if (useShortSectionLabels && sectionTitle != null)
			  reporter.debugMsg("debug title: " , abbreviateRoute(sectionTitle), null,null, DEBUG_KEY);
			// sectionTitle = abbreviateRoute(sectionTitle);
			break; // done
		} else if (text.startsWith("<HTML>") || text.startsWith("<TABLE")) {
			/* This marks the outer boundary of a nested HTML document
			 * so skip to next inner TD element (if present)
			 */
			int pos = text.indexOf("<TD", 3);
			int rpos = -1;

			if (pos > 0 && (rpos = text.indexOf('>', pos+1)) > 0) {
				// jump ahead to the inner TD block
				text = text.substring(rpos+1).trim();
				// debug("TEXT2: " + subString(text,0,40));
			} else {
				if ((pos = text.indexOf("<CENTER><IMG SRC")) != -1) {
					break;
				}
				continue; // next TD element
			}
		} else if (text.startsWith("<")) {

		if (text.indexOf("<CENTER><IMG SRC") >= 0) {
			// There should be no further info after this tag is found
			break; // end
		}
		// otherwise we're in an unknown state

			reporter.assert(false, "Stop parsing page: UNKNOWN tags found in smartTraveler page: ",
				text, null, null);
/*
// save unparsed HTML page for later debugging
try {
 URL u = new URL(travelerURLString);
 InputStream in = u.openStream();
 FileOutputStream os = new FileOutputStream(
   new File("D:\\pam\\" + org.mitre.pam.store.FileRef.getNewFileId("boston", "asp")));
 org.mitre.pam.util.StreamCopy.copyFully(in, os, true);
} catch (Exception e) {}
*/
			break;
		}

		if (key == null || sectionTitle != null && sectionTitle.indexOf(key) != -1) {
			//	debug("["+sectionTitle+"]");
			//	debug("STORY: " + text);
			// remove any double spaces from the text
			StringTokenizer st = new StringTokenizer(text, " \t");
			StringBuffer buf = new StringBuffer(256);
			for (int p =0; st.hasMoreElements(); p++) {
				if (p > 0) buf.append(' ');
				buf.append((String)st.nextElement());
			}
			int i, j;
			if (sectionTitle == null) sectionTitle = "";
			else if (useShortSectionLabels && sectionTitle.length()!=0) {
				sectionTitle = abbreviateRoute(sectionTitle);
				reporter.debugMsg("use Short labels", sectionTitle, null, null, DEBUG_KEY);
			}

/*
			sectionTitle.length()!=0 && (i=sectionTitle.indexOf('(')) > 0 &&
				(j=sectionTitle.indexOf(')', i+1)) > i) {
				// use what's in ()'s for the title
				sectionTitle = sectionTitle.substring(i+1, j);
			}
/*
			if (sectionSummary == null) sectionSummary = "";

			/*
			* Check for continuation of last traffic area report
			* and append to last product, otherwise create a new one.
			*/

			if (lastStory != null && sectionTitle.length()!=0 && lastStory.title.equals(sectionTitle)) {

			 // assumes that a different road summary is not defined
/*
			 if (!lastStory.description.equals(sectionSummary))
				System.out.println("*** Same section: " + sectionTitle + " (DIFFERENT FLAG)");
*/
			 lastStory.story = lastStory.story.concat("\n" + buf.toString());

			}
			else {
				TravelerStory ts = new TravelerStory();
				ts.date  = date;
				ts.title = sectionTitle;
				ts.description = sectionSummary;
				ts.story = buf.toString();
				stories.addElement(ts);
				lastStory = ts;
			}
		}

	} // while more TD's

	/*
	* Catch the generic "no problem" message with no text blocks:
	*
	* <TD><img src=/newbos/images/ok.gif ALIGN=ABSMIDDLE> There are no problems or delays to report at this time.</TD></TR>
	* <TD><P><BR><img src=/newbos/images/ok.gif align=left> Traffic on all other sections of the road is moving at or near speed limit.</TD></TR>
	*/

	if (stories.size() == 0) {
		if (sectionTitle == null)
			throw new ParseException("Page not parsed properly. No content found");
		if (sectionSummary == null) {
			sectionSummary = "";
			reporter.assert(false, "Boston smartTraveler sectionSummary is null",null,null,null);
		}
		TravelerStory ts = new TravelerStory();
		ts.date = date;
		ts.title = title;
		ts.description = sectionSummary; // ok/slow/delay
		ts.story = sectionTitle;
		stories.addElement(ts);
	}
    }
    catch (ParseException pe) {
      // log it and rethrow it allowing caller to try again in case server has temporary problems
      reporter.reportException(pe);
      throw pe;
    }
    finally {
	try {
	  urlStream.close();
	}
	catch (IOException ignore) {}
    }
  }

/*

  // for debugging
  static void testFile(Reporter reporter, String url) {
    System.out.println("======================================================");
    GetTravelerBos gt = new GetTravelerBos(url,	reporter, "", "128 NB");
    gt.setShortLabels(true);

    try {
      gt.getNow();
      Vector v = gt.getStories();
      if (v.size() == 0)
	System.out.println("No stories");
      else {
	System.out.println("");
	for (java.util.Enumeration e=v.elements(); e.hasMoreElements(); )
	  {
	    TravelerStory ts = (TravelerStory) e.nextElement();
	    System.out.println("["+ts.title + "] " + ts.description + "\n" + ts.story);
	  } // for
      }
    } catch (Exception e) {
      System.out.println(e);
    }
  }

  public static void main (String args[]) throws Exception {

    new org.mitre.pam.executive.Exec();
    Reporter reporter = org.mitre.pam.executive.ExecGlobal.getExec().getExecAsAgent().getReporter();
    
    if (args.length !=0 )
      for (int i=0; i < args.length; i++)
        testFile(reporter, "file://" + args[i]);
    else
      for (int i=1; i <= 39; i++) {
	File file = new File("boston" + Integer.toString(i) + ".asp");
	if (file.exists()) {
	  testFile(reporter, "file://" + file.getName());
	}
	} // for
	System.exit(0);
 }

*/

  
}
