/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.getter.webcopy;

/*
  Definitions:

  Relative URL - a URL which does not contain a protocol or host and only
                 contains a file.
                 Example: index.html
                          /index.html
                          ../index.html
                          /cgi-bin/123.cgi
                          formX

  Absolute URL - a URL which contains a protocol and host and optionally
                 contains a file.
                 Example: http://www.yahoo.com/
                          http://www.widget.com/cgi-bin/
                          http://www.widget.com/cgi-bin/formX

  Context URL - a URL which identifies the context in which to parse and
                construct new URLs.

*/
import java.net.*;
import java.io.*;

import org.mitre.pam.store.FileRef;

/**
* An extended FileRef that can identify itself in various forms
*
* @author Nazario Irizarry, Jr.
*/
class LinkObject extends FileRef{
    /** The URL object from which we can open a connection to copy files */
    protected URL url = null;

    boolean isSupportComponent = false;
    boolean isIndexComponent = false;
    boolean isRootComponent = false;

    /**
     * Constructor for LinkObject.
     * If context URL is null, then it is ignored and spec must be an
     * absolute URL (i.e. one which contains a protocol and host)
     */
    public LinkObject (File file, URL url) {
      super(file, System.currentTimeMillis(), url.getFile());
      this.url = url;
    }

    /**
     * Returns URL object for this LinkObject.
     */
    public URL getURL () {
        return (url);
    }

/**
 * Returns just the filename part with anchor
 */
    public String getLocalRelative () {
		String firstPart = getFile().getName();
		String secondPart = url.getRef();
		if (secondPart != null)
			return firstPart+'#'+secondPart;
		else
			return firstPart;
    }

/**
* Returns absolute reference
*/
    public String getRemoteAbsolute () {
        return url.toString();
    }

/**
* Returns fully qualified URL (e.g. http://www.yahoo.com/index.html)
* with no anchor.
*/
    public String getFullyQualified () {
    	return getFullyQualifiedFromURL(url);
    }
/**
* Returns fully qualified URL (e.g. http://www.yahoo.com/index.html)
* with no anchor.
*/
    static public String getFullyQualifiedFromURL (URL url) {
      StringBuffer sb = new StringBuffer();
      sb.append(url.getProtocol());
      sb.append("://");
      sb.append(url.getHost());
      int port = url.getPort();
      if (port != -1 && port != 80) {
        sb.append(':');
        sb.append(port);
      }
      sb.append(url.getFile());
      return sb.toString();
    }

/**
 * Sets the Date last modified for this LinkObject.
 */
    public void setLastModified (long date) {
        modificationDate = date;
    }
}