/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;

import org.mitre.pam.util.PAMDate;
import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.Identifiable;
import org.mitre.pam.interfaces.Memento;
import org.mitre.pam.interfaces.ScheduleAttributes;
import org.mitre.pam.interfaces.Schedulable;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.AdminTool;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.HasProperties;

import org.mitre.pam.util.alarm.Alarm;
import org.mitre.pam.executive.Exec;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pdif.eventParser.ValueList;
import org.mitre.pam.util.pdifHashtables.IpamPDIFNode;
import org.mitre.pam.util.pdifHashtables.ParentServices;
import org.mitre.pam.util.pdifHashtables.NodeEditor;

/**
* A basic gui for developing, debugging, and administering PAM
*
* @author Nazario Irizarry, Jr.
*/

public class MonitorFrame extends Frame implements Observer, AdminTool {

	static ExecToController exec;
	static Reporter reporter;
	static Vector monitorList = new Vector(10);
	static MonitorFrame keeper = null;
	MyItemListener myItemListener;
	
/**
* static so that multiple instances of MonitorFrame will not create multiple editor panes for an agent
*/
	static Hashtable outstandingEditors; 
	static ParentServer parentServer;
	static EditorCloser editorCloser;
	
	public MonitorFrame() {
	}
	
	public void setExec(ExecToController executive) {
		if (exec == null) {
			exec = executive;
			reporter = exec.getExecAsAgent().getReporter();
			outstandingEditors = new Hashtable(); 
			parentServer = new ParentServer();
			editorCloser = new EditorCloser();
		}
		initMonitorFrame();
		exec.addObserver(this);
		doShowExec();
	}
		
	public void initMonitorFrame() {
		monitorList.addElement(this);
		
		//{{INIT_CONTROLS
		GridBagLayout gridBagLayout;
		gridBagLayout = new GridBagLayout();
		setLayout(gridBagLayout);
		setSize(460,296);
		setFont(new Font("Dialog", Font.PLAIN, 12));
		setForeground(new Color(0));
		setBackground(new Color(12632256));
		viewPanelTop = new java.awt.Panel();
		viewPanelTop.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		viewPanelTop.setBounds(0,1,451,45);
		GridBagConstraints gbc;
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		gbc.ipady = 10;
		((GridBagLayout)getLayout()).setConstraints(viewPanelTop, gbc);
		add(viewPanelTop);
		displayChoice = new java.awt.Choice();
		displayChoice.addItem("Exec");
		displayChoice.addItem("Daemons");
		displayChoice.addItem("Inputs");
		displayChoice.addItem("Handlers");
		displayChoice.addItem("Outputs");
		displayChoice.addItem("Input Queue");
		displayChoice.addItem("Handler Queue");
		displayChoice.addItem("Output Queue");
		displayChoice.addItem("Debug Keys");
		displayChoice.setBounds(27,8,138,19);
		viewPanelTop.add(displayChoice);
		labelTime = new java.awt.Label("012345678901234567890123456789");
		labelTime.setBounds(170,5,254,25);
		viewPanelTop.add(labelTime);
		commandPanelBottom = new java.awt.Panel();
		commandPanelBottom.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		commandPanelBottom.setBounds(0,253,451,40);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 9;
		gbc.gridwidth = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		gbc.ipady = 5;
		((GridBagLayout)getLayout()).setConstraints(commandPanelBottom, gbc);
		add(commandPanelBottom);
		checkPointBtn = new java.awt.Button();
		checkPointBtn.setLabel("Ckpt");
		checkPointBtn.setBounds(70,5,44,20);
		commandPanelBottom.add(checkPointBtn);
		showNetworkBtn = new java.awt.Button();
		showNetworkBtn.setLabel("Show Network");
		showNetworkBtn.setBounds(119,5,110,20);
		commandPanelBottom.add(showNetworkBtn);
		showNetworkBtn.setEnabled(false);
		newMonitorBtn = new java.awt.Button();
		newMonitorBtn.setLabel("New Monitor");
		newMonitorBtn.setBounds(234,5,99,20);
		commandPanelBottom.add(newMonitorBtn);
		stopBtn = new java.awt.Button();
		stopBtn.setLabel("Stop");
		stopBtn.setBounds(338,5,43,20);
		stopBtn.setBackground(new Color(16468562));
		commandPanelBottom.add(stopBtn);
		middlePanel = new java.awt.Panel();
		middlePanel.setLayout(new CardLayout(0,0));
		middlePanel.setBounds(0,0,20,40);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridwidth = 2;
		gbc.gridheight = 8;
		gbc.weightx = 1.0;
		gbc.weighty = 8.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)getLayout()).setConstraints(middlePanel, gbc);
		add(middlePanel);
		splitPanel = new java.awt.Panel();
		gridBagLayout = new GridBagLayout();
		splitPanel.setLayout(gridBagLayout);
		splitPanel.setBounds(0,0,20,40);
		splitPanel.setFont(new Font("SansSerif", Font.PLAIN, 12));
		middlePanel.add("split", splitPanel);
		attributePanel = new java.awt.Panel();
		gridBagLayout = new GridBagLayout();
		attributePanel.setLayout(gridBagLayout);
		attributePanel.setBounds(225,48,226,204);
		attributePanel.setFont(new Font("SansSerif", Font.PLAIN, 10));
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.gridheight = 8;
		gbc.weightx = 10.0;
		gbc.weighty = 5.0;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0,3,0,0);
		((GridBagLayout)splitPanel.getLayout()).setConstraints(attributePanel, gbc);
		splitPanel.add(attributePanel);
		fieldModuleShown = new java.awt.TextField();
		fieldModuleShown.setEditable(false);
		fieldModuleShown.setBounds(0,0,226,17);
		fieldModuleShown.setFont(new Font("Dialog", Font.PLAIN, 12));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)attributePanel.getLayout()).setConstraints(fieldModuleShown, gbc);
		attributePanel.add(fieldModuleShown);
		attributesList = new java.awt.List(0);
		attributesList.setBounds(0,17,226,86);
		attributesList.setBackground(new Color(16777215));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.gridwidth = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 5.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)attributePanel.getLayout()).setConstraints(attributesList, gbc);
		attributePanel.add(attributesList);
		labelInputs = new java.awt.Label("Inputs",Label.CENTER);
		labelInputs.setBounds(28,103,57,25);
		labelInputs.setFont(new Font("Dialog", Font.PLAIN, 12));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)attributePanel.getLayout()).setConstraints(labelInputs, gbc);
		attributePanel.add(labelInputs);
		labelOutputs = new java.awt.Label("Outputs",Label.CENTER);
		labelOutputs.setBounds(137,103,65,25);
		labelOutputs.setFont(new Font("Dialog", Font.PLAIN, 12));
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 2;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)attributePanel.getLayout()).setConstraints(labelOutputs, gbc);
		attributePanel.add(labelOutputs);
		inputList = new java.awt.List(0);
		inputList.setBounds(0,128,113,75);
		inputList.setBackground(new Color(16777215));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)attributePanel.getLayout()).setConstraints(inputList, gbc);
		attributePanel.add(inputList);
		outputList = new java.awt.List(0);
		outputList.setBounds(113,128,113,75);
		outputList.setBackground(new Color(16777215));
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 3;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)attributePanel.getLayout()).setConstraints(outputList, gbc);
		attributePanel.add(outputList);
		layerPanel = new java.awt.Panel();
		layerPanel.setLayout(new CardLayout(1,0));
		layerPanel.setBounds(0,48,222,204);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridheight = 8;
		gbc.weightx = 5.0;
		gbc.weighty = 5.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)splitPanel.getLayout()).setConstraints(layerPanel, gbc);
		splitPanel.add(layerPanel);
		execPanel = new java.awt.Panel();
		gridBagLayout = new GridBagLayout();
		execPanel.setLayout(gridBagLayout);
		execPanel.setBounds(1,0,220,204);
		layerPanel.add("Exec", execPanel);
		lExec = new java.awt.Label("Exec");
		lExec.setBounds(0,13,54,30);
		lExec.setFont(new Font("Dialog", Font.PLAIN, 16));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)execPanel.getLayout()).setConstraints(lExec, gbc);
		execPanel.add(lExec);
		checkpointingLabel = new java.awt.Label("checkpointing");
		checkpointingLabel.setBounds(31,70,189,21);
		checkpointingLabel.setFont(new Font("Courier", Font.PLAIN, 12));
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 1;
		gbc.weightx = 10.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)execPanel.getLayout()).setConstraints(checkpointingLabel, gbc);
		execPanel.add(checkpointingLabel);
		stateLabel = new java.awt.Label("runState");
		stateLabel.setBounds(31,119,189,21);
		stateLabel.setFont(new Font("Courier", Font.PLAIN, 12));
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 2;
		gbc.weightx = 10.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)execPanel.getLayout()).setConstraints(stateLabel, gbc);
		execPanel.add(stateLabel);
		checkpointCanvas = new java.awt.Canvas();
		checkpointCanvas.setBounds(5,70,20,20);
		checkpointCanvas.setBackground(new Color(16776960));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(2,2,2,2);
		((GridBagLayout)execPanel.getLayout()).setConstraints(checkpointCanvas, gbc);
		execPanel.add(checkpointCanvas);
		runStateCanvas = new java.awt.Canvas();
		runStateCanvas.setBounds(5,119,20,20);
		runStateCanvas.setBackground(new Color(16776960));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(2,2,2,2);
		((GridBagLayout)execPanel.getLayout()).setConstraints(runStateCanvas, gbc);
		execPanel.add(runStateCanvas);
		lGarbageCollect = new java.awt.Label("Garbage collect");
		lGarbageCollect.setBounds(31,168,189,21);
		lGarbageCollect.setFont(new Font("Courier", Font.PLAIN, 12));
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 3;
		gbc.weightx = 10.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)execPanel.getLayout()).setConstraints(lGarbageCollect, gbc);
		execPanel.add(lGarbageCollect);
		gcCanvas = new java.awt.Canvas();
		gcCanvas.setBounds(5,168,20,20);
		gcCanvas.setBackground(new Color(16776960));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(2,2,2,2);
		((GridBagLayout)execPanel.getLayout()).setConstraints(gcCanvas, gbc);
		execPanel.add(gcCanvas);
		modulePanel = new java.awt.Panel();
		modulePanel.setLayout(new BorderLayout(0,0));
		modulePanel.setBounds(1,0,220,204);
		layerPanel.add("Module", modulePanel);
		moduleLabel = new java.awt.Label("DIM");
		moduleLabel.setBounds(0,0,220,30);
		moduleLabel.setFont(new Font("Dialog", Font.PLAIN, 16));
		modulePanel.add("North", moduleLabel);
		moduleList = new java.awt.List(0);
		moduleList.setBounds(0,30,220,143);
		moduleList.setFont(new Font("SansSerif", Font.PLAIN, 12));
		moduleList.setBackground(new Color(16777215));
		modulePanel.add("Center", moduleList);
		modActPanel = new java.awt.Panel();
		modActPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		modActPanel.setBounds(0,173,220,31);
		modulePanel.add("South", modActPanel);
		checkboxModuleEnabled = new java.awt.Checkbox("Enabled");
		checkboxModuleEnabled.setBounds(4,5,68,21);
		checkboxModuleEnabled.setFont(new Font("Dialog", Font.PLAIN, 9));
		modActPanel.add(checkboxModuleEnabled);
		checkboxModuleEnabled.setEnabled(false);
		btnActivateNow = new java.awt.Button();
		btnActivateNow.setLabel("Activate Now");
		btnActivateNow.setBounds(77,7,78,16);
		btnActivateNow.setFont(new Font("Dialog", Font.PLAIN, 9));
		modActPanel.add(btnActivateNow);
		btnActivateNow.setEnabled(false);
		btnEditModule = new java.awt.Button();
		btnEditModule.setLabel("Edit");
		btnEditModule.setVisible(true);
		btnEditModule.setBounds(0,0,60,23);
		btnEditModule.setFont(new Font("Dialog", Font.PLAIN, 9));
		modActPanel.add(btnEditModule);
		refreshBtn2 = new java.awt.Button();
		refreshBtn2.setLabel("Refresh");
		refreshBtn2.setBounds(160,7,57,17);
		refreshBtn2.setFont(new Font("Dialog", Font.PLAIN, 10));
		modActPanel.add(refreshBtn2);
		queuePanel = new java.awt.Panel();
		gridBagLayout = new GridBagLayout();
		queuePanel.setLayout(gridBagLayout);
		queuePanel.setBounds(1,0,201,210);
		layerPanel.add("Q", queuePanel);
		queueLabel = new java.awt.Label("DIM Queue");
		queueLabel.setBounds(0,0,109,30);
		queueLabel.setFont(new Font("Dialog", Font.PLAIN, 16));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 10.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)queuePanel.getLayout()).setConstraints(queueLabel, gbc);
		queuePanel.add(queueLabel);
		lexecutingTasks = new java.awt.Label("Executing");
		lexecutingTasks.setBounds(4,30,62,21);
		lexecutingTasks.setFont(new Font("Dialog", Font.PLAIN, 9));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,4,0,0);
		((GridBagLayout)queuePanel.getLayout()).setConstraints(lexecutingTasks, gbc);
		queuePanel.add(lexecutingTasks);
		refreshBtn = new java.awt.Button();
		refreshBtn.setLabel("Refresh");
		refreshBtn.setBounds(144,32,57,17);
		refreshBtn.setFont(new Font("Dialog", Font.PLAIN, 10));
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,0,0);
		((GridBagLayout)queuePanel.getLayout()).setConstraints(refreshBtn, gbc);
		queuePanel.add(refreshBtn);
		executingTaskList = new java.awt.List(0);
		executingTaskList.setBounds(4,51,197,158);
		executingTaskList.setFont(new Font("Helvetica", Font.PLAIN, 9));
		executingTaskList.setBackground(new Color(13762520));
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.gridwidth = 2;
		gbc.gridheight = 4;
		gbc.weightx = 1.0;
		gbc.weighty = 50.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0,4,0,0);
		((GridBagLayout)queuePanel.getLayout()).setConstraints(executingTaskList, gbc);
		queuePanel.add(executingTaskList);
		((CardLayout) layerPanel.getLayout()).show(layerPanel,"Exec");
		singlePanel = new java.awt.Panel();
		singlePanel.setLayout(new BorderLayout(0,0));
		singlePanel.setBounds(0,0,20,40);
		middlePanel.add("single", singlePanel);
		dbgBtnPane = new java.awt.ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
		dbgBtnPane.setBounds(0,0,150,120);
		singlePanel.add("Center", dbgBtnPane);
		dbgPanel = new java.awt.Panel();
		dbgPanel.setLayout(new GridLayout(4,1,0,2));
		dbgPanel.setBounds(0,0,20,40);
		dbgBtnPane.add(dbgPanel);
		dbgKeyBtn = new java.awt.Button();
		dbgKeyBtn.setLabel("Update List");
		dbgKeyBtn.setBounds(0,0,60,23);
		dbgPanel.add(dbgKeyBtn);
		dbgKeyBox = new java.awt.Checkbox("Show Debug Keys");
		dbgKeyBox.setBounds(0,0,100,40);
		dbgPanel.add(dbgKeyBox);
		((CardLayout) middlePanel.getLayout()).show(middlePanel,"split");
		setTitle("Untitled");
		//}}

		if (!reporter.isDebugKey("showDebugKeys")) {
			displayChoice.remove("Debug Keys");
		}
		//{{INIT_MENUS
		//}}

		checkboxModuleEnabled.setEnabled(false);
		btnActivateNow.setEnabled(false);
		btnEditModule.setEnabled(false);

		if (keeper == null) {
			keeper = this;
		}
		else {
			stopBtn.setEnabled(false);
			stopBtn.setBackground(this.getBackground());
		}

	
		//{{REGISTER_LISTENERS
		SymAction lSymAction = new SymAction();
		stopBtn.addActionListener(lSymAction);
		refreshBtn.addActionListener(lSymAction);
		checkPointBtn.addActionListener(lSymAction);
		newMonitorBtn.addActionListener(lSymAction);
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		moduleList.addActionListener(lSymAction);
		SymItem lSymItem = new SymItem();
		moduleList.addItemListener(lSymItem);
		SymMouse aSymMouse = new SymMouse();
		modulePanel.addMouseListener(aSymMouse);
		displayChoice.addItemListener(lSymItem);
		btnActivateNow.addActionListener(lSymAction);
		checkboxModuleEnabled.addItemListener(lSymItem);
		SymFocus aSymFocus = new SymFocus();
		modulePanel.addFocusListener(aSymFocus);
		showNetworkBtn.addActionListener(lSymAction);
		refreshBtn2.addActionListener(lSymAction);
		inputList.addActionListener(lSymAction);
		outputList.addActionListener(lSymAction);
		dbgKeyBtn.addActionListener(lSymAction);
		dbgKeyBox.addItemListener(lSymItem);
		btnEditModule.addActionListener(lSymAction);
		//}}
		
		myItemListener = new MyItemListener();
	}

	public MonitorFrame(String title) {
	    this();
	    setTitle(title);
	}

	//{{DECLARE_CONTROLS
	java.awt.Panel viewPanelTop;
	java.awt.Choice displayChoice;
	java.awt.Label labelTime;
	java.awt.Panel commandPanelBottom;
	java.awt.Button checkPointBtn;
	java.awt.Button showNetworkBtn;
	java.awt.Button newMonitorBtn;
	java.awt.Button stopBtn;
	java.awt.Panel middlePanel;
	java.awt.Panel splitPanel;
	java.awt.Panel attributePanel;
	java.awt.TextField fieldModuleShown;
	java.awt.List attributesList;
	java.awt.Label labelInputs;
	java.awt.Label labelOutputs;
	java.awt.List inputList;
	java.awt.List outputList;
	java.awt.Panel layerPanel;
	java.awt.Panel execPanel;
	java.awt.Label lExec;
	java.awt.Label checkpointingLabel;
	java.awt.Label stateLabel;
	java.awt.Canvas checkpointCanvas;
	java.awt.Canvas runStateCanvas;
	java.awt.Label lGarbageCollect;
	java.awt.Canvas gcCanvas;
	java.awt.Panel modulePanel;
	java.awt.Label moduleLabel;
	java.awt.List moduleList;
	java.awt.Panel modActPanel;
	java.awt.Checkbox checkboxModuleEnabled;
	java.awt.Button btnActivateNow;
	java.awt.Button btnEditModule;
	java.awt.Button refreshBtn2;
	java.awt.Panel queuePanel;
	java.awt.Label queueLabel;
	java.awt.Label lexecutingTasks;
	java.awt.Button refreshBtn;
	java.awt.List executingTaskList;
	java.awt.Panel singlePanel;
	java.awt.ScrollPane dbgBtnPane;
	java.awt.Panel dbgPanel;
	java.awt.Button dbgKeyBtn;
	java.awt.Checkbox dbgKeyBox;
	//}}

	//{{DECLARE_MENUS
	//}}
	static String JUGGLER = "Juggler";
	static String PIM = "PIM";
	static String DIM = "DIM";
	static String DAEMON = "Daemon";
	static String DIMQ = ExecGlobal.DIM_EXECUTER;
	static String PIMQ = ExecGlobal.PIM_EXECUTER;
	static String CHANQ = ExecGlobal.JUGGLER_SCHEDULER;
	static String EXEC = "Exec";
	static String MODULE = "Module";
	static String DEBUG_KEYS = "Debug Keys";

	String showing;
	
	void doShowDebugKeys() {
		showing = DEBUG_KEYS;
		updateDebugKeys();
//		((CardLayout)layerPanel.getLayout()).show(layerPanel, "dbg");
		((CardLayout)middlePanel.getLayout()).show(middlePanel, "single");
//		RefreshBtn_Clicked(null);
	}
	
	void doShowJugglers() {
		// to do: place event handler code here.
		showing = JUGGLER;
		moduleLabel.setText("Handlers");
		((CardLayout)layerPanel.getLayout()).show(layerPanel, MODULE);
		((CardLayout)middlePanel.getLayout()).show(middlePanel, "split");
		RefreshBtn_Clicked(null);
	}

	void doShowPIMs() {
		// to do: place event handler code here.
		showing = PIM;
		moduleLabel.setText("Inputs");
		((CardLayout)layerPanel.getLayout()).show(layerPanel, MODULE);
		((CardLayout)middlePanel.getLayout()).show(middlePanel, "split");
		RefreshBtn_Clicked(null);
	}

	void doShowDIMs() {
		// to do: place event handler code here.
		showing = DIM;
		moduleLabel.setText("Outputs");
		((CardLayout)layerPanel.getLayout()).show(layerPanel, MODULE);
		((CardLayout)middlePanel.getLayout()).show(middlePanel, "split");
		RefreshBtn_Clicked(null);
	}

	void doShowExec() {
		// to do: place event handler code here.
		showing = EXEC;
		((CardLayout)layerPanel.getLayout()).show(layerPanel, EXEC);
		((CardLayout)middlePanel.getLayout()).show(middlePanel, "split");
		RefreshBtn_Clicked(null);
	}

	void doShowDaemons() {
		// to do: place event handler code here.
		showing = DAEMON;
		moduleLabel.setText("Daemons");
		((CardLayout)layerPanel.getLayout()).show(layerPanel, MODULE);
		((CardLayout)middlePanel.getLayout()).show(middlePanel, "split");
		RefreshBtn_Clicked(null);
	}

	void stopBtn_Clicked(java.awt.event.ActionEvent event) {
		// to do: place event handler code here.
		exec.halt();
		killNonkeepers();
        	monitorList.removeElement(this);
        	setVisible(false);         // hide the Frame
        	dispose();
	}

	void RefreshBtn_Clicked(java.awt.event.ActionEvent event) {
		// to do: place event handler code here.
		checkboxModuleEnabled.setEnabled(false);
		btnActivateNow.setEnabled(false);
		btnEditModule.setEnabled(false);

		if ( showing == EXEC ) {
			drawExecLights();
			displayProperties("exec");
		} else if ( showing == PIM || 
					showing == DIM || 
					showing == JUGGLER || 
					showing == DAEMON ) {

			moduleList.removeAll();
			Enumeration modules=null;
			if ( showing == PIM ) {
				modules = exec.getListOfActiveModules(ExecToMonitor.PIM);
				}
			else if ( showing == DIM ) {
				modules = exec.getListOfActiveModules(ExecToMonitor.DIM);
				}
			else if ( showing == JUGGLER ) {
				modules = exec.getListOfActiveModules(ExecToMonitor.JUGGLER);
				}
			else  {
				modules = exec.getListOfActiveModules(ExecToMonitor.DAEMON);
				}
			while ( modules.hasMoreElements() ) {
				Identifiable module = (Identifiable) modules.nextElement();
				moduleList.add(module.getId());
			}
			
		} else if ( showing == DIMQ || showing == PIMQ || showing == CHANQ ) {
		
			
			Vector running = new Vector(10);
			Vector pending = new Vector(50);
			
			int selector = ExecToMonitor.DIM;
			if (showing == PIMQ) 
				selector =  ExecToMonitor.PIM;
			if (showing == CHANQ) 
				selector =  ExecToMonitor.JUGGLER;
			exec.getQueState( selector, running, pending);
			
			attributesList.setEnabled(false);
			attributesList.removeAll();
			executingTaskList.setEnabled(false);
			executingTaskList.removeAll();
			for (int i=0; i<pending.size(); i++) {
				attributesList.add((String)pending.elementAt(i));
			}

			for (int i=0; i<running.size(); i++) {
				String item = (String) running.elementAt(i);
				executingTaskList.add(item);
			}
			attributesList.setEnabled(true);
			executingTaskList.setEnabled(true);
		} 
		
	}

	long updateDisplayBeat = 0;
	boolean checkpointing = false;
	boolean garbageCollecting = false;
	String execState = ExecGlobal.WORKING;
	AdminTool netTool;
	private boolean resetGarbageCollecting;
	private boolean resetCheckpointing;
	
	private void drawExecLights() {
		if (garbageCollecting) {
			gcCanvas.setBackground(Color.green);
			gcCanvas.repaint(1);
			resetGarbageCollecting = true;
		}
		if (checkpointing) {
			checkpointCanvas.setBackground(Color.green);
			checkpointCanvas.repaint(1);
			resetCheckpointing = true;
		}
		if (execState == ExecGlobal.TICK) {	
			if (resetGarbageCollecting) {
				gcCanvas.setBackground(getBackground());
				gcCanvas.repaint();
				resetGarbageCollecting = false;
			}
			if (resetCheckpointing) {
			checkpointCanvas.setBackground(getBackground());
			checkpointCanvas.repaint();
				resetCheckpointing = false;
			}
		}
		else if (execState == ExecGlobal.STARTING) {
			runStateCanvas.setBackground(Color.yellow);
			stateLabel.setText(execState);
			runStateCanvas.repaint(1);
		}
		else if (execState == ExecGlobal.WORKING) {
			runStateCanvas.setBackground(Color.green);
			stateLabel.setText(execState);
			runStateCanvas.repaint();
		}
		else if (execState == ExecGlobal.SHUTTING_DOWN) {
			runStateCanvas.setBackground(Color.magenta);
			stateLabel.setText(execState);
			runStateCanvas.repaint(1);
		}
		else {		//execState == ExecGlobal.IDLE
			runStateCanvas.setBackground(Color.yellow);
			stateLabel.setText(execState);
			runStateCanvas.repaint(1);
		}
	}
	
	public void update(Observable o, Object arg) {
		
		String updateEvent = null;
		if (arg instanceof String)
			updateEvent = (String) arg;
		
		long now = System.currentTimeMillis();
		if (now > updateDisplayBeat) {
			updateDisplayBeat = now + 1000;
			labelTime.setText((new PAMDate()).toString());
		}
	
		if (showing == arg) {
			RefreshBtn_Clicked(null);
		}
		else if (updateEvent != null && (showing == EXEC)){

			if (updateEvent == ExecGlobal.START_GC) {
				garbageCollecting = true;
			}
			else if (updateEvent == ExecGlobal.END_GC) {
				garbageCollecting = false;
			}
			else if (updateEvent == ExecGlobal.START_CHECKPOINT) {
				checkpointing = true;
			}
			else if (updateEvent == ExecGlobal.END_CHECKPOINT) {
				checkpointing = false;
			}
			else if (updateEvent.startsWith("executive.")) {
				execState = updateEvent;
			}
			drawExecLights();
		}
	}
	
	
	void doShowDIMQueue() {
		showing = DIMQ;
		queueLabel.setText("Output Queue");
		((CardLayout)layerPanel.getLayout()).show(layerPanel, "Q");
		((CardLayout)middlePanel.getLayout()).show(middlePanel, "split");
		RefreshBtn_Clicked(null);
	}

	
	void doShowPIMQueue() {
		// to do: place event handler code here.
		showing = PIMQ;
		queueLabel.setText("Input Queue");
		((CardLayout)layerPanel.getLayout()).show(layerPanel, "Q");
		((CardLayout)middlePanel.getLayout()).show(middlePanel, "split");
		RefreshBtn_Clicked(null);
	}

	void doShowJugglerQueue() {
		showing = CHANQ;
		queueLabel.setText("Handler Queue");
		((CardLayout)layerPanel.getLayout()).show(layerPanel, "Q");
		((CardLayout)middlePanel.getLayout()).show(middlePanel, "split");
		RefreshBtn_Clicked(null);
	}

	void newMonitorBtn_Clicked(java.awt.event.ActionEvent event) {
		MonitorFrame f = new MonitorFrame();
		f.setExec(exec);
		f.setVisible(true);
	}

	void checkPointBtn_Clicked(java.awt.event.ActionEvent event) {
		((ExecToController) exec).checkPoint();
	}

	public void addNetworkTool(AdminTool netTool) {
		this.netTool = netTool;
		if (netTool != null) {
			showNetworkBtn.setEnabled(true);
		}
	}
	
	void showNetworkBtn_ActionPerformed(java.awt.event.ActionEvent event)
	{
		if (netTool != null) {
			netTool.setEnabled(true);
		}
		
	}

	void killNonkeepers() {
		for (Enumeration e = monitorList.elements(); e.hasMoreElements(); ) {
			MonitorFrame otherFrame = (MonitorFrame) e.nextElement();
			otherFrame.MonitorFrame_WindowClosing(null);
		}
	}
	
	void DIMList_DblClicked(java.awt.event.ItemEvent event) {
		// to do: place event handler code here.
		displayProperties(moduleList.getSelectedItem());
	}

/**
* Display properties for item with selected name
*/
	void displayProperties(String itemId){
		if (itemId == null)
			return;

	/*
	* Display name
	*/
		fieldModuleShown.setText(itemId);
	/*
	* Display attributes
	*/
		attributesList.removeAll();
		Vector properties = new Vector(20);
		Vector values = new Vector(20);
		exec.getItemStateList(itemId, properties, values);
		int cacheItemCount = 0;
		int metaItemCount = 0;
		if (properties.size() > 0) {
			for (int i = 0; i< properties.size(); i++) {
				String p = (String) properties.elementAt(i);
				if (p.startsWith("cache.")) {
						cacheItemCount++;
				}
				else if (p.startsWith("meta_")) {
						metaItemCount++;
				}
				else {
					if (cacheItemCount > 0) {
						attributesList.add("cache. items not listed ("+cacheItemCount+")");
						cacheItemCount = 0; // this assumes the cache. items are contiguous and results in 1 summary line					
					}
					if (metaItemCount > 0) {
						attributesList.add("meta_ items not listed ("+metaItemCount+")");
						metaItemCount = 0; // this assumes the meta_ items are contiguous and results in 1 summary line					
					}
					attributesList.add(p+": "+values.elementAt(i).toString());
				}
			}
		}
		
	/*
	* Display output connections
	*/
		Enumeration sinks = exec.getListOfAgentSinks(itemId);
		outputList.removeAll();
		while (sinks.hasMoreElements()) {
			outputList.addItem(((Agent)sinks.nextElement()).getId());
		}

	/*
	* Display input connections
	*/
		Enumeration sources = exec.getListOfAgentSources(itemId);
		inputList.removeAll();
		while (sources.hasMoreElements()) {
			inputList.addItem(((Agent)sources.nextElement()).getId());
		}
		
		
	/*
	* Toggle enable/disable button
	*/
		Agent module = exec.getActiveAgent(itemId);
		if (module == null) {
			checkboxModuleEnabled.setEnabled(false);
			btnActivateNow.setEnabled(false);
			btnEditModule.setEnabled(false);
		}
		else {
			Memento mem = module.getMemento();
			boolean enabled;
			String enabledString = mem.getProperty("enabled");
			enabled = ((enabledString != null) && (enabledString.equals("true")));
			checkboxModuleEnabled.setEnabled(true);
			checkboxModuleEnabled.setState(enabled);
			
			boolean canActivate = enabled && (
					module instanceof org.mitre.pam.interfaces.Juggler ||
					module instanceof org.mitre.pam.interfaces.Daemon) ;
					
			btnActivateNow.setEnabled( canActivate ); 

			btnEditModule.setEnabled(true);
		}
	}
	
	protected void finalize() {
		dispose();
	}

	void displayChoice_Action(java.awt.event.ItemEvent event) {
		// to do: place event handler code here.
		int n = displayChoice.getSelectedIndex();

		switch ( n ) {
			case 0:			// Exec
				doShowExec();
				break;
			case 1:			// Daemons
				doShowDaemons();
				break;
			case 2:			// PIMs
				doShowPIMs();
				break;
			case 3:			// Jugglers
				doShowJugglers();
				break;
			case 4:			// DIMs
				doShowDIMs();
				break;
			case 5:			// PIM Queue
				doShowPIMQueue();
				break;
			case 6:			// Juggler Queue
				doShowJugglerQueue();
				break;
			case 7:			// DIM Queue
				doShowDIMQueue();
				break;
			case 8:			// Debug Keys
				doShowDebugKeys();
				break;
		}
	}

	void btnActivateNow_Clicked(java.awt.event.ActionEvent event) {
		String module = moduleList.getSelectedItem();
		if (module == null ) {
			btnActivateNow.setEnabled(false);
			}
		else {
			Agent agent = exec.getActiveAgent(module);
			if (agent != null && (agent instanceof Schedulable) ) {
				((Schedulable)agent).activateASAP();
			}
			else {
			}
		}
	}

	void checkboxModuleEnabled_Action(java.awt.event.ItemEvent event) {
		boolean state = checkboxModuleEnabled.getState();
//		checkboxModuleEnabled.setState(state);
		String itemId = moduleList.getSelectedItem();
		if (itemId != null) {
			Agent module =  exec.getActiveAgent(itemId);
	
			if (module == null) {
				checkboxModuleEnabled.setEnabled(false);
				checkboxModuleEnabled.setState(false);
				}
			else {
				Agent agent = exec.getActiveAgent(itemId);
				if (agent != null)
					agent.setProperty( "enabled", state?"true":"false");
				boolean canActivate = state && (
						module instanceof org.mitre.pam.interfaces.Juggler ||
						module instanceof org.mitre.pam.interfaces.Daemon) ;
						
				btnActivateNow.setEnabled( canActivate ); 
			}
		}
	}


	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == stopBtn)
				stopBtn_Clicked(event);
			else if (object == refreshBtn)
				RefreshBtn_Clicked(event);
			else if (object == checkPointBtn)
				checkPointBtn_Clicked(event);
			else if (object == newMonitorBtn)
				newMonitorBtn_Clicked(event);
			else if (object == moduleList)
				DIMList_DblClicked(null);
			else if (object == btnActivateNow)
				btnActivateNow_Clicked(event);
			else if (object == showNetworkBtn)
				showNetworkBtn_ActionPerformed(event);
			else if (object == refreshBtn2)
				refreshBtn2_ActionPerformed(event);
			else if (object == inputList)
				inputList_DblClicked(event);
			else if (object == outputList)
				outputList_DblClicked(event);
			else if (object == dbgKeyBtn)
				dbgKeyBtn_ActionPerformed(event);
			else if (object == btnEditModule)
				btnEditModule_ActionPerformed(event);
		}
	}

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == MonitorFrame.this)
				MonitorFrame.this.MonitorFrame_WindowClosing(event);
		}
	}

	void MonitorFrame_WindowClosing(java.awt.event.WindowEvent event)
	{
        if (this == keeper) {
        	setVisible(false);         // hide the Frame
        }
        else {
        	monitorList.removeElement(this);
        	setVisible(false);         // hide the Frame
        	dispose();
        }
	}

	class MyItemListener implements java.awt.event.ItemListener
	{
		public void itemStateChanged(java.awt.event.ItemEvent event)
		{
			Object object = event.getSource();
			if (object instanceof Checkbox) {
				String name = ((Checkbox)object).getLabel();
				boolean state = ((Checkbox)object).getState();
				Alarm.setDebug(name, state);
			}
		}
	}


	class SymItem implements java.awt.event.ItemListener
	{
		public void itemStateChanged(java.awt.event.ItemEvent event)
		{
			Object object = event.getSource();
			if (object == moduleList)
				DIMList_DblClicked(null);
			else if (object == displayChoice)
				displayChoice_Action(event);
			else if (object == checkboxModuleEnabled)
				checkboxModuleEnabled_Action(event);
			else if (object == dbgKeyBox)
				dbgKeyBox_ItemStateChanged(event);
		}
	}

	class SymMouse extends java.awt.event.MouseAdapter
	{
		public void mouseExited(java.awt.event.MouseEvent event)
		{
			Object object = event.getSource();
			if (object == modulePanel)
				modulePanel_Exited(event);
		}
	}

	class SymFocus extends java.awt.event.FocusAdapter
	{
		public void focusLost(java.awt.event.FocusEvent event)
		{
			Object object = event.getSource();
			if (object == modulePanel)
				modulePanel_FocusLost(event);
		}
	}

	void modulePanel_Exited(java.awt.event.MouseEvent event) {
	}

	void modulePanel_FocusLost(java.awt.event.FocusEvent event)
	{
		checkboxModuleEnabled.setEnabled(false);
		btnActivateNow.setEnabled(false);
		btnEditModule.setEnabled(false);
	}

/**
* Tell the tool to become active and to work
*/
	public void setEnabled(boolean enabled){
		super.setEnabled(enabled);
	}
/**
* Tell tool to display state for named agent
*/
	public void displayAgent(String agent){
		displayProperties(agent);
	}
/**
* Get a list of debug keys 
*/
	void updateDebugKeys(){
	/*
	* Remove existing keys
	*/
		for (int i = dbgPanel.getComponentCount()-1; i>1 ;i--) {
			Checkbox cb = (Checkbox)dbgPanel.getComponent(i);
			cb.removeItemListener( myItemListener );
			dbgPanel.remove(i);
		}
	/*
	* Get new list
	*/
		Enumeration eKey = Alarm.getAvailableDebugKeyList();
	/*
	* Add new keys
	*/
		while (eKey.hasMoreElements()) {
			String keyName = (String) eKey.nextElement();
			boolean keyState = reporter.isDebugKey(keyName);
			Checkbox newKeyBox = new Checkbox(keyName);
			newKeyBox.setState(keyState);
			newKeyBox.setBounds(dbgKeyBox.getBounds());
			dbgPanel.add(newKeyBox);
			newKeyBox.addItemListener(myItemListener);
		}
		int items = dbgPanel.getComponentCount();  // the number of actual items
		((GridLayout)dbgPanel.getLayout()).setColumns(2);
		((GridLayout)dbgPanel.getLayout()).setRows((items+1)/2);
		dbgPanel.invalidate();
		dbgBtnPane.validate();
		
	}

	void refreshBtn2_ActionPerformed(java.awt.event.ActionEvent event)
	{
		RefreshBtn_Clicked(null);
	}

	void inputList_DblClicked(java.awt.event.ActionEvent event)
	{
		displayAgent(inputList.getSelectedItem());
	}

	void outputList_DblClicked(java.awt.event.ActionEvent event)
	{
		displayAgent(outputList.getSelectedItem());
	}

	void dbgKeyBtn_ActionPerformed(java.awt.event.ActionEvent event)
	{
		updateDebugKeys();
	}

	void dbgKeyBox_ItemStateChanged(java.awt.event.ItemEvent event)
	{
		// to do: code goes here.
	}
	public void setTitle(String title) {
		super.setTitle("iPAM Admin., "+title);
	}

	void btnEditModule_ActionPerformed(java.awt.event.ActionEvent event)
	{
		String module = moduleList.getSelectedItem();
		if (module == null ) {
			btnEditModule.setEnabled(false);
			}
		else {
			Agent agent = exec.getActiveAgent(module);
			if (agent != null ) {
				NodeEditor editor = (NodeEditor) outstandingEditors.get(agent.getId());
				if (editor == null) {
					IpamPDIFNode pn = new IpamPDIFNode(agent.getKind(), agent.getId(), agent);
					pn.buildMetaDataFromExistingAttributes();
					editor = new NodeEditor(pn, parentServer);
					outstandingEditors.put(agent.getId(), editor);
					editor.addWindowListener(editorCloser);
				}
				editor.setVisible(true);
			}
		}
	}

	class ParentServer implements ParentServices {
		public boolean isAdminSession(){
			return true;
		}
		public HasProperties getPDIFNode(String name){
			HasProperties agent = exec.getAvailableAgent(name);
			if (agent != null)
				return agent;
			return exec.getActiveAgent(name);
		}
		public String getUserID(){
			return exec.getExecAsAgent().getStringProperty(ExecGlobal.BERT_ADMINS_NAME, "Administrator");
		}
		public void renamePDIFNode(String oldName, String newName){
		}
		public boolean isDebugSession(){
			return false;
		}
		public boolean isRenameSupported(){
			return false;
		}
	}
	class EditorCloser implements WindowListener {
	    /**
	     * Invoked the first time a window is made visible.
	     */
	    public void windowOpened(WindowEvent e){}
	
	    /**
	     * Invoked when the user attempts to close the window
	     * from the window's system menu.  If the program does not 
	     * explicitly hide or dispose the window while processing 
	     * this event, the window close operation will be cancelled.
	     */
	    public void windowClosing(WindowEvent e){}
	
	    /**
	     * Invoked when a window has been closed as the result
	     * of calling dispose on the window.
	     */
	    public void windowClosed(WindowEvent e){
	    	Frame c =  (Frame) e.getSource();
	    	String name = c.getTitle();
			c.removeWindowListener(this);
			outstandingEditors.remove(name);
		}
	
	    /**
	     * Invoked when a window is no longer the user's active
	     * window, which means that keyboard events will no longer
	     * be delivered to the window or its subcomponents.
	     */
	    public void windowDeactivated(WindowEvent e){}
	    /**
	     * Invoked when a window is changed from a normal to a
	     * minimized state. For many platforms, a minimized window 
	     * is displayed as the icon specified in the window's 
	     * iconImage property.
	     * @see Frame#setIconImage
	     */
	    public void windowIconified(WindowEvent e){}
	
	    /**
	     * Invoked when a window is changed from a minimized
	     * to a normal state.
	     */
	    public void windowDeiconified(WindowEvent e){}
	
	    /**
	     * Invoked when the window is set to be the user's
	     * active window, which means the window (or one of its
	     * subcomponents) will receive keyboard events.
	     */
	    public void windowActivated(WindowEvent e){}

	}

/*
* Todo: fix edit button, font, and default show
* Change "Juggler" to "Handler"
* set size to 460,296
* set title to iPAM admin
*/
}
