/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

import java.util.Hashtable;

import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.util.AuthContext;

/**
* A servlet that gets PAMs state as PDIF (under development)
*
* @author Nazario Irizarry, Jr.
*/
public class SrvGetExecState extends SrvPAMBase {

	public static String PAM_STATE_QUERY = "/PAM/State.query";
	public static String PAM_STATE_POST = "/PAM/State.post";
	
    public String getServletInfo() {
		return "Return iPAM state as PDIF";
	}

/**
* Services a single request.
*
* @param req the servlet request
* @param req the servlet response
* @exception ServletException when an exception has occurred
*/
	protected void doGet (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException {
		String specificConfiguration; // the specific configuration to get


		/*
		* Begin access check +++++++++
		*/
		AuthContext context = getAuthContext(req, res);
	
		if (!context.verifyAccess(reporter, this,"get exec state"))
			return;
		/*
		* End access check ------------
		*/

		/*
		* check if this is a specific get
		*/
		specificConfiguration = req.getParameter("configuration");
		String noRedundant = req.getParameter("noRedundant");
		if (specificConfiguration != null) {
			String adminName = exec.getExecAsAgent().getStringProperty("bertAdministratorName","");
			if (adminName.length() != 0) {
				if (adminName.equals(specificConfiguration)) {
					specificConfiguration = null; // null means get all
				}
			}
		}
		
		
		res.setStatus( HttpServletResponse.SC_OK );
		res.setContentType( "text/html" );
		ServletOutputStream p = res.getOutputStream();

// -----------  preamble

		p.println( "<HTML><HEAD>" );
		p.println( "<TITLE>iPAM State</TITLE>" );
		p.println( "</HEAD><BODY>" );
		p.println( "<H1>Current iPAM State</H1>" );

// -----------  state

		p.println( "<P><PRE>");
		Hashtable selectors = new Hashtable();
		if (specificConfiguration != null)
			selectors.put(exec.SELECTOR_CONFIGURATION_NAME, specificConfiguration);
		if (noRedundant != null)
			selectors.put(exec.SELECTOR_NO_REDUNDANCY_WITH_PARENT, "true");
		exec.getPAMState(new PrintWriter(p), selectors);
		p.println( "</PRE>");

// ----------- change form

		p.print( "<FORM action=\"");
			p.print( PAM_STATE_POST );
			p.println( "\" method=\"POST\">");
		p.println( "<P><TEXTAREA NAME=\"changeText\" ROWS=7 COLS=80>(pdif (node\n\n))</TEXTAREA></P>");
		p.println( "<P><INPUT TYPE=\"submit\" NAME=\"submitChange\" VALUE=\"Submit Change\"></P>");
		p.println( "</FORM>");
		
// ----------- postamble
		p.println( "</BODY></HTML>" );
		p.flush();
	}
/**
* Services a single request.
*
* @param req the servlet request
* @param req the servlet response
* @exception ServletException when an exception has occurred
*/
	protected void doPost (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException {

	/*
	* Begin access check +++++++++
	*/
		AuthContext context = getAuthContext(req, res);		

		if (!context.verifyAccess(reporter, this, "change configuration"))
			return;
	/*
	* End access check ------------
	*/

		res.setStatus( HttpServletResponse.SC_OK );
		res.setContentType( "text/html" );
		ServletOutputStream p = res.getOutputStream();
		p.println( "<HTML><HEAD>" );
		p.println( "<TITLE>iPAM State</TITLE>" );
		p.println( "</HEAD><BODY>" );
		p.println( "<H1>Change requested</H1>" );
		p.println( "<P><PRE>");
		
		String feedback=null;

		String changeText= req.getParameter("changeText");
		p.println( "Change - "+changeText);
		if (changeText == null ) {
			feedback = "null change text";
		}
		else {
			feedback = exec.modifyFromPDIFString(changeText);
		}


		if (feedback == null)
			p.println("Change accepted");
		else
			p.println(feedback);
		p.print( "</PRE>");
		p.println( "</BODY></HTML>" );
		p.flush();
	}
}
