/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.net.URLEncoder;
import org.mitre.pam.interfaces.ExecToMonitor;
import org.mitre.pam.interfaces.Identifiable;
import org.mitre.pam.interfaces.Juggler;
import org.mitre.pam.interfaces.PIM;
import org.mitre.pam.interfaces.DIM;
import org.mitre.pam.executive.Exec;
import org.mitre.pam.monitor.SrvAgentEditor;
import org.mitre.pam.util.AuthContext;
import org.mitre.pam.interfaces.Agent;

/**
* Prints out the agents in iPAM with a link to the SrvAgentEditor servlet to
* edit it's properties.
*
* @author Nazario Irizarry, Jr.
*
* @author Michael DeCorpo  -added links to servlet
*
* @author Jason Mathews    iPAM 2.4.84 10/15/99
*                          -add Daemons to list if specified in query
*                          -allow multiple agent types to be specified in query
*/

public class SrvPAMServlet extends SrvPAMBase {

	public static String QUERY_URL = "/PAM/List.query";

  public void doGet(HttpServletRequest req, HttpServletResponse res)
                    throws ServletException, IOException
  {

	/*
	* Begin access check +++++++++
	*/
	AuthContext context = getAuthContext(req, res);

	if (!context.verifyAccess(reporter, this, QUERY_URL))
		return;
	/*
	* End access check ------------
	*/

    ServletOutputStream out = res.getOutputStream();
    res.setContentType("text/html");

    out.println("<html>");
    out.print("<head><title>iPAM Agent List");

    String qString = req.getQueryString();

    if (qString != null) {
	if (qString.length() > 0) {
	out.print(": ");
    	out.print(qString);
	        qString = qString.toUpperCase();
	}
	// if at least one query parameter is not specified with the ACTIVE_<agentType> pattern
	// then all agent types [except DAEMONS] will be shown by default.
	if (qString.indexOf("ACTIVE_") == -1) qString = null;
    }

    String id = exec.getExecAsAgent().getProperty("id");

    if (id != null && id.length() > 0)
    	out.print( " on " + id);

    out.println("</title></head>");
//    out.println("<body>");
    out.println("<body bgcolor=\"#FFFFFF\">");

//    out.println("<ul type=disc>");
//    out.println("<lh>" + qString + ":</lh><br>");

    Enumeration e;

    // hide Daemons by default
    if (qString != null && qString.indexOf("ACTIVE_DAEMONS") != -1) {
      out.println("<H2>Daemons</H2>");
      e = exec.getListOfActiveModules(ExecToMonitor.DAEMON);
      out.println("<ul type=disc>");
      while (e.hasMoreElements()) {
        Identifiable daemon = (Identifiable)e.nextElement();
	String query = "?agent=" + URLEncoder.encode(daemon.getId()) + "&uid=1";
        out.println("<li><a href=\"" + SrvAgentEditor.QUERY_BASE + query + "\">" + daemon.getId() + "</a>");
      }
      out.println("</ul>");
    }

    if (qString == null || qString.indexOf("ACTIVE_PIMS") != -1) {
      out.println("<H2>Inputs</H2>");
      e = exec.getListOfActiveModules(ExecToMonitor.PIM);
      out.println("<ul type=disc>");
      while (e.hasMoreElements()) {
        Identifiable pim = (Identifiable)e.nextElement();
	String query = "?agent=" + URLEncoder.encode(pim.getId()) + "&uid=1";
        out.println("<li><a href=\"" + SrvAgentEditor.QUERY_BASE + query + "\">" + pim.getId() + "</a>");
      }
      out.println("</ul>");
    }
    
    
    if (qString == null || qString.indexOf("ACTIVE_JUGGLER") != -1) {
      out.println("<H2>Handlers</H2>");
      e = exec.getListOfActiveModules(ExecToMonitor.JUGGLER); 
      out.println("<ul type=disc>");
      while (e.hasMoreElements()) {
        Identifiable juggler = (Identifiable)e.nextElement();
        String query = "?agent=" + URLEncoder.encode(juggler.getId()) + "&uid=1";
        out.println("<li><a href=\"" + SrvAgentEditor.QUERY_BASE + query + "\">" + juggler.getId() + "</a>");
      }
      out.println("</ul>");
    }

    if (qString == null || qString.indexOf("ACTIVE_DIMS") != -1) {
      out.println("<H2>Outputs</H2>");
      e = exec.getListOfActiveModules(ExecToMonitor.DIM); 
      out.println("<ul type=disc>");
      while (e.hasMoreElements()) {
        Identifiable dim = (Identifiable)e.nextElement();
        String query = "?agent=" + URLEncoder.encode(dim.getId()) + "&uid=1";
	out.println("<li><a href=\"" + SrvAgentEditor.QUERY_BASE + query + "\">" + dim.getId() + "</a>");	
      }
      out.println("</ul>");
    }
    
    out.println("</body></html>");
  }
}
