/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
/**
* Serves one specific file.  This is a non-standard servlet.  The constructor
* has an argument which makes it easy to write this servlet but not usable
* on any other server other than PAM.
*
* @author Nazario Irizarry, <naz@mitre.org> modifications to original code, 1 Jan 1998
*/

public class SrvServeOneFile extends SrvPAMBase {

	File fileToGet;

    public SrvServeOneFile(File fileToGet) {
    	if (fileToGet == null) {
    		log("can't serve null file");
    		return;
    	}

    	if (fileToGet.isDirectory()) {
    		log("can't serve directory file:"+fileToGet.getPath());
    		return;
    	}
    		
    	this.fileToGet = fileToGet;
	}

    public String getServletInfo() {
		return "servlet to serve one specific file";
	}

/**
* Implement the GET method
*/
	protected void doGet (HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException {

		if (fileToGet == null){
			res.setStatus( HttpServletResponse.SC_NOT_FOUND );
		}
		doGetFile( req, res, false, fileToGet );
	}

    private void doGetFile( 
    		HttpServletRequest req, 
    		HttpServletResponse res, 
    		boolean headOnly, 
    		File file ) throws IOException
	{
	log( "getting " + fileToGet.getName() );
	if ( ! file.canRead() )
	    {
	    res.sendError( HttpServletResponse.SC_FORBIDDEN );
	    return;
	    }

	res.setStatus( HttpServletResponse.SC_OK );

	res.setContentType( getServletContext().getMimeType( file.getName() ) );
	res.setContentLength( (int) file.length() );
	res.setDateHeader( "Last-Modified", file.lastModified() );
	OutputStream out = res.getOutputStream();
	if ( ! headOnly ) {
	    InputStream in = new FileInputStream( file );
	    copyStream( in, out );
	    in.close();
	    }
	
	}
    
    private void copyStream( InputStream in, OutputStream out ) throws IOException
	{
		byte[] buf = new byte[4096];
		int len;
		while ( ( len = in.read( buf ) ) != -1 ) {
		    if (len > 0)
		    	out.write( buf, 0, len );
		}
	}
    
}
