/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.monitor.ssi;

import java.io.*;
import java.util.Properties;
import java.util.Date;
import java.util.*;

/**
 * A class that is used in conjunction with the SSIParser.
 *
 * @author Naz Irizarry
 *
 * @modifications Michael DeCorpo
 */

public class ResultFile extends PrintStream{

  Properties substitutionVar;   // all of the properties that may be substituted
  Properties substitutionFile;
  Properties remaining;        // a list of properties that should be used to build
                               // to build the additional table.  As a variable is
                               // substituted for, it is removed from this object 
  
  ResultFile (
	      PrintStream out,
	      Properties substitutionVar, 
	      Properties substitutionFile,
	      Properties remaining) throws IOException{
		
		super(out);
		
		this.substitutionVar = substitutionVar;
		this.substitutionFile = substitutionFile;
		this.remaining = remaining;
		if (this.remaining == this.substitutionVar)
		  this.remaining = (Properties) this.substitutionVar.clone();
  }

  /**
   * This method prints quotes around the msg string if the parameter is true;
   */
  private void Print(String msg, boolean useQuotes)
  {
    if (useQuotes) {
      // if the string contains quotes, print single quotes around it
      if ((msg.indexOf("\"") >= 0) && (msg.indexOf("\'") < 0))
	print("\'" + msg + "\'");
      else
	print("\"" + msg + "\"");
    }
    else
      print(msg);
  }
  
  /**
   * This method takes @param variableName and prints the value of that property
   * in the substitutionVar object if it exist.  If @param useQuotes is true,
   * quotes are printed around the result.
   */
  void substituteVariable(String variableName, boolean useQuotes) {

    if(variableName.equals("DATE_LOCAL")) {
      String defaultDate=null;
      if (substitutionVar != null) {
		defaultDate = substitutionVar.getProperty("DATE_LOCAL");
      }
      if (defaultDate == null)
      	defaultDate = (new Date()).toString();
	  Print(defaultDate,useQuotes);
      return;
    }
    
    if (substitutionVar == null) {
      return;
    }
    // get the value of the property.  If it exists, remove it from the remaining
    // object and print it
    String sub = substitutionVar.getProperty(variableName);
    if (sub != null) {
      remaining.remove(variableName);
      Print(sub,useQuotes);
    }
  }
  
  final static int BSIZE = 2048;
  
  void substituteFile(String requestFileName) {
    //		print("<"+requestFileName+">");
    FileInputStream source = null;
    
    /*
     * Try to translate the requested name.
     * If it does not translate then use it as is.
     */
    
    String fileName = substitutionFile.getProperty(requestFileName, requestFileName);
    
    byte buffer[] = new byte[BSIZE];
    
    try {
      source = new FileInputStream(new File(fileName));	/* use File to accept a generic path */
    }
    catch (IOException e) {
      System.err.println("file open error during output merge while opening "+fileName);
      e.printStackTrace(System.err);
      return;
    }
    
    try {
      while (true) {
	int count = source.read(buffer, 0, BSIZE);
	this.write(buffer, 0, count);
      }
    }
    catch (Exception e) {}
    
    try {
      source.close();
    }
    catch (IOException e) {}
  }

  /**
   * This method takes a property name and prints out an html select structure
   * containing options true and false.
   */
  void buildBoolean(String variableName) 
  {
    if (substitutionVar == null) {
      return;
    }
    String val = substitutionVar.getProperty(variableName);

    // remove from list
    remaining.remove(variableName);

    print("<select name=\"" + variableName + "\">");
    if (val.equals("true"))
      print("<OPTION SELECTED>true<OPTION>false</select>");
    else
      print("<OPTION>true<OPTION SELECTED>false</select>");
  }

  /** 
   * This method takes all of the properties that are left in the remaining 
   * object and prints out them out in an html table.
   */
  void buildAdditionalTable()
  {
    if (remaining.size() <= 0)
      return;

    println("<table border>");
    println("<tr><td colspan=2 align=CENTER><b>Additional Attributes</b></td></tr>");
    Enumeration e = remaining.propertyNames();
    while (e.hasMoreElements()) {
      String prop = (String) e.nextElement();
      String val = remaining.getProperty(prop);

      println("<tr><td width=150>" + prop + "</td>");

      // if it a boolean value, print out a select otherwise print out a
      // text field.
      if ((val.equals("true")) || (val.equals("false"))) {
	println("<td><select name=\"" + prop + "\">");
	if (val.equals("true"))
	  println("<OPTION SELECTED>true<OPTION>false");
	else
	  println("<OPTION>true<OPTION SELECTED>false");
	println("</select>");
      }
      else {
	if (val.indexOf("\"") >= 0)
	  val = "\'" + val + "\'";
	else
	  val = "\"" + val + "\"";
	println("<td><input type=\"text\" name=\"" + prop + "\" size=39 value=" + val + "></td></tr>");
      }
    }
    println("</table>");
  }
}

