/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.productVisitor;

import java.io.IOException;
import java.io.File;
import java.util.Enumeration;
import java.util.Vector;

import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.Agent;
import org.mitre.pam.interfaces.Reporter;
import org.mitre.pam.interfaces.ProductVisitor;
import org.mitre.pam.product.view.BasicProduct;
import org.mitre.pam.product.view.VectorProduct;
import org.mitre.pam.product.view.HashtableProduct;
import org.mitre.pam.store.FileRef;


/**
 * Visits a product to determine if the file has all local

 * file references. Can also make a copy of the product with
 * all local references.
 * 
 * @author N. Irizarry
 */
public class AsLocalProduct implements ProductVisitor{

	File workingDir;
	Product reviewProduct;
	Reporter reportingAgent;

/**
* @param workingDir
* @param reportingAgent is the parent which can report on errors
*/
	public AsLocalProduct(File workingDir, Reporter reportingAgent) {
		this.workingDir = workingDir;
		this.reportingAgent = reportingAgent;
	}
	
	public void visit(Product product) {
		reviewProduct = product;
	}
/**
* Identifies whether the product and all its subproducts are completly
* within the http accessible areas
*/
	public boolean isAllLocal() {
		AsFlatFileList lister = new AsFlatFileList();
		lister.visit(reviewProduct);
		Enumeration list = lister.getFileProductList();
		
		if (!list.hasMoreElements())
			return true;
		
		while (list.hasMoreElements()) {
			BasicProduct p = (BasicProduct) list.nextElement();
			FileRef fr = (FileRef) p.getContent();
			if (!fr.hasURL())
				return false;
		}
		
		return true;
	}
/**
* Copies over the product and makes a local copy of any
* file references if needed
*/
	public Product makeLocalClone(Product source) throws IOException {
		if (source == null) {
			return null;
		}
		else if (source instanceof BasicProduct) {
			BasicProduct clone = (BasicProduct) source.cloneShallow();
			Object content = source.getContent();
			if (content instanceof FileRef) {
				FileRef fr = (FileRef) content;
				if (!fr.hasURL()) {
			/*
			* need to make a local copy
			*/
					File from = fr.getFile();
					File to = new File(workingDir, from.getName());
					if (to.exists())
						to.delete();
					reportingAgent.assert(!to.exists(),"Trying to localize an existing file -",from.getName(),null,null);
					
					FileRef.copyFile(from, to);
					FileRef dupRef = new FileRef(to, fr.getModificationDateLong(), fr.getSourceName());
					clone.setContent(dupRef);
			
					return clone;
				}
			}
		}
		else if (source instanceof VectorProduct) {
			VectorProduct clone = (VectorProduct) source.cloneShallow();
			Enumeration e = (Enumeration) source.getContent();
			while (e.hasMoreElements()) {
				Product p = (Product) e.nextElement();
				clone.addElement(makeLocalClone(p));
			}
			return clone;
		}
		else if (source instanceof HashtableProduct) {
			HashtableProduct table = (HashtableProduct) source;
			
			HashtableProduct clone = (HashtableProduct) source.cloneShallow();
			Enumeration keys =  table.keys();
			while (keys.hasMoreElements()) {
				String key = (String) keys.nextElement();
				Product p = table.getElement(key);
				clone.addElement(key, makeLocalClone(p));
			}
			return clone;
		}

		reportingAgent.assert(false,"Don't understand class of -",source.toString(),null,null);
		return null;
	}
}