/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.productVisitor;

import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.PrintWriter;
import java.net.URL;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.ProductWriter;
import org.mitre.pam.store.FileRef;

/**
* Formats a product for Output using a tickertext style (assuming an output device
* with a single line of text).
*
* 1) Titles are printed in UPPERCASE and prefixed with " -- " for emphasis.
* 2) Multiple lines are separated with "|" delimiter.
*
* @author Jason Mathews
*/
public class AsTickerText implements ProductWriter
{
        protected int depthOfProduct;
        protected Product inputProduct = null;

        // suppresses id from being displayed (duplicated for email also in subject)
        protected boolean rootIdSuppress = false;

	public AsTickerText() {
	}

	public AsTickerText(Vector options) {
	  for (int i=0; i < options.size(); i++) {
	    String opt = ((String) options.elementAt(i)).toLowerCase();
	    if (opt.equals("norootid")) {
		rootIdSuppress = true;
	    }
	  }
	}

        public void writeResults(PrintWriter out) {
                process(out, inputProduct, 0);
                out.println();
                out.flush();
        }
        
        public void setRootIdSuppress(boolean suppress) {
                rootIdSuppress = suppress;
        }
        
        public void visit(Product inputProduct) {
// System.out.println("AsTickerText.visit()");
                this.inputProduct = inputProduct;
                AsTree asTree = new AsTree();
                asTree.visit(inputProduct);
                depthOfProduct = asTree.getDepth();
// System.out.println("Depth: "+ depthOfProduct);
        }

        protected void process(PrintWriter out, Product product) {
// System.out.println("AsTickerText.process()");
                process(out, product, 0);
        }

        protected void process(PrintWriter out, Product product, int level) {
                // System.out.println(" level= "+level+" product= "+product);
                if (product == null)
                        return;
        /*
        * "name, date"
        */
                if (level == 0 && depthOfProduct > 1) {
                        if (!rootIdSuppress) // suppress the ID from displaying
                                out.print(product.getId().toUpperCase());
                }
                else if (level > 0) {
                        out.print(" -- " + product.getId().toUpperCase());
                }
                else if (!rootIdSuppress) // suppress the ID from displaying
                { // if (level == 0 && depthOfProduct <= 1) ...
                        System.out.println(" level= "+level+" product= "+product);
			// user OPTION for uppercase ??
                        out.print(product.getId().toUpperCase());
                }

        /*
        * " - description"
        */
                StringTokenizer st = new StringTokenizer(product.getDescription(), "\n");
		if (!rootIdSuppress || level > 0)
			out.print("|");
                for (int count=0; st.hasMoreElements(); count++) {
                        String sub = (String) st.nextElement();
                        if (count != 0) out.print(" "); // separator
                        out.print(sub);
                }

                Object content = product.getContent();
                /*
                * vector contents
                */
                if (content instanceof Enumeration) {
                    Product root = ((ContainerProduct)product).getRoot();
                    if (root == null) {
                                Enumeration enum = (Enumeration) content;
                                while (enum.hasMoreElements()) {
                                        process(out, (Product)enum.nextElement(), level+1);
                                }
                    }
                    else {
                                process(out, root, level);
                    }
                }
                /*
                * String contents
                */
                else if (content instanceof String) {
                        String s = (String)content;
//                      System.out.println(" AsTickerText, string content "+s);
                        StringTokenizer std = new StringTokenizer(s, "\n");
                        out.print("|");
                        for (int count=0; std.hasMoreElements(); count++) {
                                String sub = (String) std.nextElement();
                                if (count != 0) out.print(" "); // separator
                                out.print(sub);
                        }
                        // out.println();
                }
                /*
                * byte array contents
                */
                else if (content instanceof byte[]) {
//                      System.out.println(" AsTickerText, byte array content ");
                        out.print("|");
                        AsPlainText.processByteArray(out, (byte[]) content);
                        out.println();
                }
                /*
                * FileRef contents
                */
                else if (content instanceof FileRef) {
                        String name = ((FileRef) content).getFile().getName();
                        out.print("|File: "+name);
                }
                /*
                * URL contents
                */
                else if (content instanceof URL) {
                        String s = ((URL)content).toString();
                        out.print("|URL - "+s);
                }
                /*
                * unknown contents
                */
                else {
                      throw new IllegalArgumentException("AsTickerText: unrecognized content "+content);
                }
        }

}
