/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.putter;

import java.util.Enumeration;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.awt.*;
import java.util.Vector;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.productVisitor.AsPlainText;


/**
* Simple DIM that takes its product and displays to a window.
* For test purposes.  <P>
*
* @author Nazario Irizarry, Jr.
*/
public class TestDIM1 extends DIMShell1 implements WindowListener {

	Frame window;
	TextArea text;
	private final String META_DATA = "showMetaData";
	private final static String SLEEP_TIME = "sleepTimeMillis";
	private final static String FONT_SIZE = "fontSize";
	int sleepTime = 3000;

	public TestDIM1() {
		setDescription("displays summary in a local window for development purposes");
		setProperty(META_DATA,"false");
		metaBooleanInfo(META_DATA,false,"allows display of product metadata");
		setProperty(SLEEP_TIME, "3000");
		metaIntegerInfo(SLEEP_TIME, 3000,"number of milliseconds to sleep after every window update");

		setProperty(FONT_SIZE, "9");
		metaIntegerInfo(FONT_SIZE, 9,"Size of font used by the display window");
	}

	public void setProperty(String property, String value) {
		if (property.equals(SLEEP_TIME)) {
			super.setProperty(property, value);
			sleepTime = getIntegerProperty(SLEEP_TIME,3000);
			return;
		}
		super.setProperty(property, value);
	}
/**
* Accept a product for output.  
*/
	public void putProduct(Product prod) {
		showWindow();
		if (prod == null) {
			text.setText("<nothing provided>");
		}
		else {
			StringWriter sw = new StringWriter();
			AsPlainText renderer;
			if (getProperty(META_DATA).equals("true")) {
				Vector options = new Vector();
				options.addElement("showMetaData");
				renderer = new AsPlainText(options);
			}
			else {
				renderer = new AsPlainText();
			}
			renderer.visit(prod);
			renderer.writeResults(new PrintWriter(sw));
			text.setText(sw.toString());
		}
		try {
			Thread.sleep(sleepTime);
		}
		catch (Exception e) {
		
		}
	}

/**
* This dim does not create any transient FileRefs
*/
	public Enumeration getCurrentFileRefs(){
		return null;
	}
/**
* Create feedback window
*/
	private void showWindow() {
		if (window != null) {
			window.setVisible(true);
			return;
		}
		
		window = new Frame(name);
		window.setBackground(Color.white);
		text = new TextArea(10,60);
		text.setFont(new Font("Monospaced",Font.PLAIN, getIntegerProperty(FONT_SIZE,9)));
		
		window.setBounds(new Rectangle(10,10,250,150));
		window.setLayout(new BorderLayout());
		window.add("Center",text);
//		window.pack();
		window.show();
		window.addWindowListener(this);
	}

/**
* We don't do anything with this event
*/
    public void windowOpened(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowClosed(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowIconified(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowDeiconified(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowActivated(WindowEvent e) {}
/**
* We don't do anything with this event
*/
    public void windowDeactivated(WindowEvent e) {}

/**
* Close the window.
*/
	public void windowClosing(WindowEvent e) {
		window.removeWindowListener(this);
		Frame closingWindow = window;
		window = null;
		closingWindow.setVisible(false);
		closingWindow.dispose();
	}

}
