/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.putter;

import java.util.Enumeration;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.awt.*;
import java.util.Vector;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.productVisitor.AsPlainText;

/**
* Test DIM to excercise the QueuedDIMShell base class.
*
* @author Nazario Irizarry, Jr.
*/
public class TestQueuedDIM extends QueuedDIMShell {

	Frame window;
	TextArea text;
	private final static String META_DATA = "showMetaData";
	private final static String SLEEP_TIME = "sleepTimeMillis";
	private final static String DEBUG_KEY = "testQueuedDIM";
	private final static String WINDOW_SIZE = "windowSize";
	int sequenceNumber;
	Vector history = new Vector(10);
	int sleepTime = 3000;
	StringBuffer windowText = new StringBuffer(1000);

	public TestQueuedDIM() {
		setDescription("displays input product summary (for test purposes)");
		setProperty(SLEEP_TIME, "3000");
		setProperty(WINDOW_SIZE, "15");
		metaIntegerInfo(SLEEP_TIME, 3000,"number of milliseconds to sleep after every window update");
	}
	
	public void setProperty(String property, String value) {
		if (property.equals(SLEEP_TIME)) {
			super.setProperty(property, value);
			sleepTime = getIntegerProperty(SLEEP_TIME,3000);
			return;
		}
		super.setProperty(property, value);
	}
/**
* Accept a product for output.  
*/
	public void putQueuedProduct(Product prod) {
		showWindow();
		sequenceNumber++;
		String body;
		if (prod == null) {
			body = "<nothing provided>";
		}
		else {
			body = prod.getId();
		}
		int maxSize = getIntegerProperty(WINDOW_SIZE, 15);
		if (maxSize < 1) maxSize = 1;
		while (history.size() >= maxSize) {
			history.removeElementAt(0);
		}
		history.addElement(sequenceNumber+" --> "+body);
		windowText.setLength(0);
		for (int i = 0; i < history.size(); i++) {
			windowText.append((String)history.elementAt(i));
			windowText.append("\n");
		}
		text.setText(windowText.toString());
		if (sleepTime > 0) {
		reporter.debugMsg("before sleep ",null,null,null,DEBUG_KEY);
			try {
				Thread.sleep(sleepTime);
			}
			catch (Exception e) {
			}
			reporter.debugMsg("after sleep",null,null,null,DEBUG_KEY);
		}
	}

/**
* Create feedback window
*/
	private void showWindow() {
		if (window != null) {
			window.show();
			return;
		}
		
		window = new MyFrame(name);
		window.setBackground(Color.white);
		text = new TextArea(10,60);
		text.setFont(new Font("Monospaced",Font.PLAIN, 9));
		
		window.setBounds(new Rectangle(10,10,250,150));
		window.setLayout(new BorderLayout());
		window.add("Center",text);
//		window.pack();
		window.show();
	}


	class MyFrame extends Frame {
		MyFrame(String name) {
			super(name);
		}
		
		public boolean handleEvent(Event e) {
			if (e.id == Event.WINDOW_DESTROY) {
				hide();
				return true;
			}
			
			return false;
		}
	}

}
