/* Copyright 1998 The MITRE Corporation. ALL RIGHTS RESERVED */
package org.mitre.pam.service.resource;

/**
* A resource pool is somewhat analogous to an AWT
* Container. <p>
* Resources may be multidimensional. Initially
* most resources will probably have one dimension.
*
* @author Nazario Irizarry, Jr.
*/
public interface ResourcePool extends ResourceConsumer {

/**
* Get the current pool limit <p>
*
* @param resourceDimension for multidimensional pools this is the name of the dimension
* @return the current budget number.
*/
	public Number getBudget(String resourceDimension);
/**
* Set the allowed pool limit. 
* Simple pools have only one dimension.
* This is analogous to 
* size on an AWT container.  
*
* More complicated pools may have multiple dimensions. <p>
* setBudget() should have the side effect
* of causing any specified constraints on members 
* of the pool to be reevaluated.  This
* is a bit like an AWT Container invalidate().
*
* @param resourceDimension for multidimensional pools this is the name of the dimension
* @param newBudget the value of this dimension
*/
	public void setBudget(String resourceDimension, Number newBudget);
/**
* Set the name of the manager.  The manager is intimately
* associated with one or more
* "algorithms" used to execute the budget.
* There must be a default manager used by each resource pool.
* Simple pools will probably ignore this method.
*/
	public void setManager(String name);
}