/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.wizard;

import java.awt.*;
import java.util.Stack;
import org.mitre.pam.interfaces.ExecToController;
import org.mitre.pam.interfaces.Wizard;


/**
* The wizard base class knows how to display and sequence through wizard panes.
* A subclass must implement a few abstract methods.
*/
abstract public class WizardBase1 extends Frame implements Wizard
{
	public WizardBase1(String displayName)
	{
		super(displayName);
		// This code is automatically generated by Visual Cafe when you add
		// components to the visual environment. It instantiates and initializes
		// the components. To modify the code, only use code syntax that matches
		// what Visual Cafe can generate, or Visual Cafe may be unable to back
		// parse your Java file into its visual environment.
		//{{INIT_CONTROLS
		GridBagLayout gridBagLayout;
		gridBagLayout = new GridBagLayout();
		setLayout(gridBagLayout);
		setVisible(false);
		setSize(305,174);
		setFont(new Font("Dialog", Font.PLAIN, 12));
		setForeground(new Color(0));
		setBackground(new Color(16777215));
		navigation = new java.awt.Panel();
		navigation.setLayout(new GridLayout(1,2,20,2));
		navigation.setBounds(55,132,194,25);
		GridBagConstraints gbc;
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(0,0,10,0);
		gbc.ipady = 5;
		((GridBagLayout)getLayout()).setConstraints(navigation, gbc);
		add(navigation);
		previousButton = new java.awt.Button();
		previousButton.setLabel("<< Previous");
		previousButton.setBounds(0,0,87,25);
		navigation.add(previousButton);
		nextButton = new java.awt.Button();
		nextButton.setLabel("Next >>");
		nextButton.setBounds(107,0,87,25);
		navigation.add(nextButton);
		currentFrame = new java.awt.Panel();
		currentFrame.setLayout(new BorderLayout(0,0));
		currentFrame.setBounds(4,4,297,115);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1.0;
		gbc.weighty = 10.0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(4,4,8,4);
		((GridBagLayout)getLayout()).setConstraints(currentFrame, gbc);
		add(currentFrame);
		//}}
	
		//{{REGISTER_LISTENERS
		SymAction lSymAction = new SymAction();
		previousButton.addActionListener(lSymAction);
		nextButton.addActionListener(lSymAction);
		//}}
		//{{INIT_MENUS
		//}}


}


	//{{DECLARE_CONTROLS
	java.awt.Panel navigation;
	java.awt.Button previousButton;
	java.awt.Button nextButton;
	java.awt.Panel currentFrame;
	//}}
	
	private int step = 0;
	private Stack stepsSoFar = new Stack();
	private Object finishFlag = new Object();
		
	private WizardFrame[] script;
	protected ExecToController exec;
	
/**
* Called from a using program to display the wizard and allow
* the user to sequence through its display panes.
*
*/
	final public void runScript(ExecToController exec) {
		this.exec = exec;
		
		showScriptFrame(0);
		setVisible(true);

		synchronized (finishFlag) {
			try {
				finishFlag.wait();
			}
			catch (InterruptedException e) {}
		}

		setVisible(false);
		dispose();
		
		implementWizardResults(exec);
	}
	

	protected void showScriptFrame(int n) {
//		System.out.println("show frame "+n);
		currentFrame.removeAll();
		Panel newPanel = script[n].getPanel();
		newPanel.setVisible(false);
		currentFrame.add(newPanel, BorderLayout.CENTER);
		newPanel.setVisible(true);
		validate();
		
		previousButton.setEnabled(n > 0);
		if (n == script.length -1)
			nextButton.setLabel("Finish");
		else
			if (nextButton.getLabel() != "Next >>")
				nextButton.setLabel("Next >>");
		
		if (n > step)
			stepsSoFar.push(new Integer(step));
		step = n;
	}
	
	void previousButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
		if (stepsSoFar.empty()) {
			showScriptFrame(0);
		}
		else {
			Integer i = (Integer) stepsSoFar.pop();
			showScriptFrame(i.intValue());
		}
	}

	void nextButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
//		System.out.println("next after frame "+step);
		if (step >= script.length-1) {
			finish();
		}
		else {
			int next = step;
			if ( script[step] instanceof WizardSequencer) {
				int maybeNext = ((WizardSequencer) script[step]).getNext();
				if (maybeNext >= 0)
					next = maybeNext;
				else
					next = step + 1;
			}
			else {
				next = step + 1;
			}

			if (next > script.length-1) {
				next = script.length-1;
			}
			showScriptFrame(next);		
		}
	}
	
	private void finish() {
		synchronized (finishFlag) {
			finishFlag.notify();
		}
	}

	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == previousButton)
				previousButton_ActionPerformed(event);
			else if (object == nextButton)
				nextButton_ActionPerformed(event);
		}
	}

	
/**
* Invoked when the wizard object is constructed.
* Calls buildScriptBody and keeps a reference to the
* script array returned by that method.
*
* @param exec provided for the benefit of the base class, not used by WizardBase1.
*/
	public final void buildScript(ExecToController exec) {
		this.script = buildScriptBody(exec);
	}
/**
* Builds a sequence of display panes.
* The subclass must keep references to its display panes so
* that it can "extract" appropriate information later, when the
* wizard "session" is complete.
*
* @param exec provided for the benefit of the base class, not used by WizardBase1.
*
*/
	abstract protected WizardFrame[] buildScriptBody(ExecToController exec);
/**
* Called by the base class when the user has finished going though
* the interactive "session".  The subclass takes all responsibility
*
*/
	abstract protected void implementWizardResults(ExecToController exec);


	//{{DECLARE_MENUS
	//}}
}
