/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.putter.mapdim;

import org.mitre.pam.putter.mapdim.PAMSpatialObject;
import org.mitre.pam.putter.mapdim.PAMSpatialObjectRenderer;

import java.io.StringWriter;
import java.io.PrintWriter;
import java.io.File;

import java.awt.*;
import java.awt.event.*;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Date;
import java.util.Enumeration;

import org.mitre.pam.interfaces.Product;
import org.mitre.pam.interfaces.ContainerProduct;
import org.mitre.pam.interfaces.WantsConnectionChanges;
import org.mitre.pam.interfaces.ConnectedAgent;
import org.mitre.pam.productVisitor.AsPlainText;
import org.mitre.pam.putter.DIMShell1;
import org.mitre.pam.executive.ExecGlobal;
import org.mitre.pam.interfaces.BasicProductAttributes;

import com.ofx.component.*;
import com.ofx.mapViewer.*;
import com.ofx.base.*;
import com.ofx.repository.*;
import com.ofx.geometry.*;
import com.ofx.query.SxQueryResultInterface;

/*
* An iPAM output agent that can display
* "track" data on a map.  Tracks are usually real-time reports on
* things that have a position in, on, or above the earth.
* In iPAM lingo this output agent displays a simple icon
* for products that have lattitude, longitude and several other attributes.
* The positions for these objects is updated when a newer report with the
* same id is delivered.
*
* @author Ayal Spitz.
* @author Nazario Irizarry, Jr.
*/
public class MapDIM extends DIMShell1 implements Runnable, WantsConnectionChanges {
// -------------------------------------------------------------------------
// --- Constructor ---------------------------------------------------------
	public MapDIM() {
		setDescription("Display objects on a map");
		setProperty(DISPOSE_WINDOW,"true");
		setProperty(MAP_WINDOW_NAME,"");
		
	}

// -------------------------------------------------------------------------
// --- Methods -------------------------------------------------------------
	private void setup() {
		// Connect an Objects ID to the Layer it exists on
		objectLayerTable = new Hashtable();
		// Connect an Objects ID to the amount of time it has left to live
		objectTimeToLiveTable = new Hashtable();
		// Instantiate a Date object
		
		initSxMap();
		try {
			SxClass sc = SxClass.retrieve("City");
			sc.getLabelSpec().setIsDisplayable(false);
			sc.getSymbology().setIsDisplayable(false);
		}
		catch (SxRepositoryException e) {
			reporter.reportException(e);
		}
		initMapLayers();
	}
	public void setProperty(String property, String value) {
		if (property.equals(MAP_WINDOW_NAME) && mainFrame != null) {
			mainFrame.setTitle(value);
		}
		super.setProperty(property, value);
	}
	public synchronized void putProduct(Product prod) {
		showWindow();
		// LA - Long : -118.411201, Lat : 34.112101
		
		
		if (prod == null) {
			reporter.warning("Null product");
			//throw new IllegalArgumentException();
		} else {
			// If the product is a ContainerProduct brake out the individual products and process
			if (prod instanceof ContainerProduct){
				processProducts(prod);
			} else {
				// If it's a single product process it	
				processOneProduct(prod);
			}
		}
	}

	private void processProducts(Product prod){
		ContainerProduct	cp = (ContainerProduct)prod;
		Product				tempProduct = null;
		
		for (Enumeration e = (Enumeration)(cp.getContent()); e.hasMoreElements() ;) {
			tempProduct = (Product)(e.nextElement());
			if (tempProduct instanceof ContainerProduct){
				processProducts(tempProduct);
			} else {
				processOneProduct(tempProduct);
			}
		}			
	}
	
// -------------------------------------------------------------------------
	public Enumeration getCurrentFileRefs(){
		return null;
	}

// -------------------------------------------------------------------------
	private int settingUp = 0;
	
	private void showWindow() {
		if (objectLayerTable == null) {
			setup();
		}
		if (mainFrame != null) {
			mainFrame.show();
			return;
		}
		settingUp++;
//		reporter.assert(false,getId(),"setting up",Integer.toString(settingUp),null);
		String theName = getStringProperty(MAP_WINDOW_NAME,"");
		if (theName.length() == 0) {
			theName = getId();
		}
		mainFrame = new Frame(theName);
		
		mainFrame.addWindowListener(new HideFrameAction());
		
		mainFrame.setBackground(Color.white);
		
		mainFrame.setBounds(new Rectangle(10,10,250,150));
		mainFrame.setLayout(new BorderLayout());
		mainFrame.add(mainMapView,BorderLayout.CENTER);

        MapToolbar aMapToolbar = new MapToolbar();
        aMapToolbar.setSize( 200,65);
		mainFrame.add(aMapToolbar,BorderLayout.NORTH);

		mainFrame.setSize(375,325);
		mainFrame.show();
	}

// -------------------------------------------------------------------------
	private String buildDBURL() {
		File supportDirRoot = ExecGlobal.getSupportDir("localofxdim"+File.separator+"Data"+File.separator+"DemoDB"+File.separator);
		String url = "file:/"+supportDirRoot.getAbsolutePath();
		if (File.separatorChar != '/')
			url = url.replace(File.separatorChar,'/');


		reporter.debugMsg("Using ofx database:",url,null,null,DEBUG_KEY);
//		System.out.println("\nUsing ofx database:"+url+"\n");
		return url;
	}
// -------------------------------------------------------------------------
	private void initSxMap(){
		String		dataBase = null;
		
		// Get a MapView, and SxMapView, what the difference is I don't know
		mainMapView = new MapViewer();
		mainSxMapView = mainMapView.getSxMapViewer();
		
		try{
			// Try to connect the SxMapView to a MapDatabase
			mainSxMapView.setDBName(buildDBURL());
			// Pick a map
			mainSxMapView.setMapName("US");
			// Set the scale to Miles
			mainSxMapView.setUnits(SxDistance.MILES);
		}catch(SxInvalidNameException e){
			reporter.reportException(e);
		}
		
		mainSxMapView.addMouseListener(new DoubleClickAction());
	}

// -------------------------------------------------------------------------
	private void initMapLayers(){
		/*SxClass	test = new SxClass("test","testClass","Point");

		try{
			// Try to connect a layer to the SxMapView
			SxClassLayer	layerClass = new SxClassLayer(test);

			//layerClass.setRenderer(renderer);
			mainSxMapView.addLayerBelow(SxMapLayerInterface.MARKER,"testLayer",layerClass);
		}catch(SxMapLayerException e){e.printStackTrace();}*/

		// Load up layers
		initLayer(PAMSpatialObject.LAYER_AIR);
		initLayer(PAMSpatialObject.LAYER_GROUND);
		initLayer(PAMSpatialObject.LAYER_SURFACE);
		initLayer(PAMSpatialObject.LAYER_SUBSURFACE);
		initLayer(PAMSpatialObject.LAYER_SPACE);
		
		// Tell the SxMapView to update the map
		mainSxMapView.postPaintEvent();		
	}
	
	private void initLayer(String layer){
        PAMSpatialObjectRenderer	renderer = new PAMSpatialObjectRenderer();
		try{
			// Try to connect a layer to the SxMapView
			SxClassLayer	layerClass = new SxClassLayer(layer);

			layerClass.setRenderer(renderer);
			mainSxMapView.addLayerBelow(SxMapLayerInterface.MARKER,layer + "Layer",layerClass);
		}catch(SxMapLayerException e){
			reporter.reportException(e);
		}
	}
	
// -------------------------------------------------------------------------
	
	private void processOneProduct(Product prod){
		// Process the new product
		String		strLat = 		null;
		String		strLong = 		null;
		String		strTrackNum = 	null;

		strLat = prod.getProperty(BasicProductAttributes.LATITUDE);	// Location
		strLong = prod.getProperty(BasicProductAttributes.LONGITUDE);

		// Try to find a trackNumber
		strTrackNum = prod.getProperty(BasicProductAttributes.TRACK_IDENTIFIER);// Uniqu ID
	

		if (strTrackNum == null){
			strTrackNum = prod.getProperty("trackId");// Uniqu ID

				if (strTrackNum == null){
					strTrackNum = prod.getId();// Uniqu ID
				}
		}

		if ((strLat != null) && (strLong != null) && (strTrackNum != null)){
			double		dblLat = 0;
			double		dblLong = 0;	
			String		strCategory = null;

			dblLat = new Double(strLat).doubleValue();
			dblLong = new Double(strLong).doubleValue();
			
			strCategory = prod.getProperty("category");	// Layer
			if (strCategory == null){
				strCategory = "GROUND";
			}

			reporter.debugMsg("Point: ",strTrackNum, ",lat="+dblLat,",lon="+dblLong,DEBUG_KEY);
			
			// Deal with catagory conversion
			String		layer;
			
			if (strCategory.equals(BasicProductAttributes.CATEGORY_AIR)){
				layer = PAMSpatialObject.LAYER_AIR;
			} else
			if (strCategory.equals(BasicProductAttributes.CATEGORY_SURFACE)){
				layer = PAMSpatialObject.LAYER_SURFACE;
			} else
			if (strCategory.equals(BasicProductAttributes.CATEGORY_SUBSURFACE)){
				layer = PAMSpatialObject.LAYER_SUBSURFACE;
			} else
			if (strCategory.equals(BasicProductAttributes.CATEGORY_SPACE)){
				layer = PAMSpatialObject.LAYER_SPACE;
			} else {
				layer = PAMSpatialObject.LAYER_GROUND;
			}
					
			// Deal with lifeTime of product issues
			long		lifeTime = computeLifeTime(prod);

			String		strFOF = prod.getProperty(BasicProductAttributes.FRIEND_OR_FOE);
			int		friendFoe = PAMSpatialObject.FRIEND_FOE_UNKNOWN;
			if (strFOF == null){
				friendFoe = PAMSpatialObject.FRIEND_FOE_UNKNOWN;
			} else if (strFOF.equalsIgnoreCase("friend")) {
				friendFoe = PAMSpatialObject.FRIEND_FOE_FRIEND;
			} else if (strFOF.equalsIgnoreCase("foe")) {
				friendFoe = PAMSpatialObject.FRIEND_FOE_FOE;
			}

			double	course = prod.getDoubleProperty(BasicProductAttributes.COURSE,Double.MAX_VALUE);
			boolean hasCourse = course != Double.MAX_VALUE;
			if (!hasCourse) {
				course = prod.getDoubleProperty("heading",Double.MAX_VALUE);
				hasCourse = course != Double.MAX_VALUE;
			}
			if (hasCourse) {
				course *= Math.PI/180.0;
			}
						
			// Check if we're updating an existing object or creating a new one
			if (objectLayerTable.containsKey(strTrackNum)){
				removeSpatialObject(strTrackNum);
			} 
			addPAMSpatialObject(strTrackNum,layer,dblLong,dblLat,lifeTime,prod, friendFoe, hasCourse, course);			
		}
	}

	private void sweepForTheLivingDead(){
		long		currTime = System.currentTimeMillis();
		String		key = null;
		Long		value = null;
		long		lifeTime = 0;
		
		for (Enumeration e = objectTimeToLiveTable.keys() ; e.hasMoreElements() ;) {
			key = (String)(e.nextElement());
			value = (Long)(objectTimeToLiveTable.get(key));
			
			lifeTime = value.longValue();
			
			if (lifeTime <= currTime){
				removeSpatialObject(key);
			}
		}
	}
	
// -------------------------------------------------------------------------
	private long computeLifeTime(Product prod){
		long		lifeTime = prod.getLifetime()*1000;
		long		modTime = prod.getModificationDate().getTime();
			
		return lifeTime + modTime;
	}
	
// -------------------------------------------------------------------------
	private void addPAMSpatialObject(String strLabel,String layer,double dblLong,double dblLat,
										long time,Product prod, int friendFoe, boolean hasCourse, double course){
										
		if (thread == null) {
			thread = new Thread(this);
			thread.setPriority(Thread.NORM_PRIORITY -1);
			thread.start();
		}
		SxDoublePoint		dblPoint = mainSxMapView.convertLatLongToDB(dblLat,dblLong);
		SxPoint				sxPoint = new SxPoint(dblPoint);
		SxClass				classDef = mainSxMapView.getSxClass(layer);
		
		//System.out.println(">" + layer);
		
		PAMSpatialObject	spatialObj = new PAMSpatialObject(classDef,"ID_" + strLabel);
		
		spatialObj.spatialObjProd = prod;
		spatialObj.lat = dblLat;
		spatialObj.lon = dblLong;
		spatialObj.layer = layer;
		spatialObj.friendFoe = friendFoe;
		spatialObj.hasCourse = hasCourse;
		spatialObj.course = course;

		spatialObj.setGeometry(sxPoint);
		spatialObj.setLabel(strLabel);
		boolean a = mainSxMapView.getQuery().add(spatialObj);
		
		//mainSxMapView.postPaintEvent();		

		objectLayerTable.put(strLabel,layer);
		objectTimeToLiveTable.put(strLabel,new Long(time));

		repaintLayer(layer);
	}
	
	
	private void removeSpatialObject(String label){
		String		layer = (String)(objectLayerTable.get(label));
		
		if (layer != null){
			mainSxMapView. deleteMapObject(layer,"ID_" + label);

			objectLayerTable.remove(label);
			objectTimeToLiveTable.remove(label);
			
			repaintLayer(layer);
		}
	}

	private void repaintLayer(String layer){
	/*	SxClassLayer	tempLayer = new SxClassLayer(layer);
		tempLayer.postDirtyEvent();
		tempLayer.postPaintEvent();*/
		synchronized(repaintSync) {
			needRepaint = true;
			repaintSync.notify();
		}
	}
	
	Object repaintSync = new Object();
	boolean needRepaint = false;
	
	public void run() {
		long nextDeathSweep = System.currentTimeMillis() + 30000; // every 30 seconds
		try {
			while (exec.getState() != ExecGlobal.SHUTTING_DOWN && mainFrame != null && mainFrame.isVisible()) {
				synchronized(repaintSync) {
					try {
						repaintSync.wait(30); // wait till something to repaint
					}
					catch (InterruptedException ignore) {
					}
				}
				if (needRepaint) {
					try {
						Thread.sleep(3000); // let a bunch of updates happen before we repaint
					}
					catch (InterruptedException e) {}
					needRepaint = false;
					mainMapView.clear();
					}
				if (System.currentTimeMillis() > nextDeathSweep) {
					sweepForTheLivingDead();
					nextDeathSweep = System.currentTimeMillis() + 29000;
				}
			}
		}
		finally {
			thread = null;
		}
	}
	
// -------------------------------------------------------------------------
// --- Memebers ------------------------------------------------------------
	private Frame 			mainFrame = null;
	private TextArea 		text = null;

	private MapViewer		mainMapView = null;
	private SxMapViewer		mainSxMapView = null;

	private Hashtable		objectLayerTable = null;
	private Hashtable		objectTimeToLiveTable = null;
	
	
// -------------------------------------------------------------------------
	private static final String			DEBUG_KEY			= "ofxMapDIM";
	private Thread						thread = null; // clear thread
	private static final String			DISPOSE_WINDOW = "autoDisposeWindow";
	private static final String			MAP_WINDOW_NAME = "mapWindowName";

// -------------------------------------------------------------------------
// --- Inner Classes -------------------------------------------------------
	class HideFrameAction extends WindowAdapter{
		public void windowClosing(WindowEvent e){
			e.getWindow().hide();
		}
	}

	class DisposeFrameAction extends WindowAdapter{
		public void windowClosing(WindowEvent e){
			e.getWindow().hide();
			e.getWindow().dispose();
		}
	}
	
	class DoubleClickAction extends MouseAdapter{
		public DoubleClickAction(){
			buildFrame();
		}
		
		private Frame buildFrame() {
			Frame dataFrame = null;

			dataFrame = new Frame();
			dataFrame.setLocation(20,20);
			dataFrame.setSize(200,200);
			text = new TextArea(10,60);
			text.setFont(new Font("Monospaced",Font.PLAIN, 9));
			
			dataFrame.setBounds(new Rectangle(10,10,250,150));
			dataFrame.setLayout(new BorderLayout());
			dataFrame.add("Center",text);

			dataFrame.addWindowListener(new DisposeFrameAction());
			return dataFrame;
		}
		
		public void mouseClicked(MouseEvent e){
			if (e.getClickCount() == 2){
				SxMarkerLayer	markLayer = mainSxMapView.getMarkerLayer();

				Point			scrnPt = e.getPoint();

				Point			ul = new Point(scrnPt.x - 5, scrnPt.y - 5);
				Point			lr = new Point(scrnPt.x + 5, scrnPt.y + 5);
				SxDoublePoint	upperLeft = markLayer.convertScreenToMap(ul);
				SxDoublePoint	lowerRight = markLayer.convertScreenToMap(lr);
				
				if (markLayer.getMapProjectionEnabled()){
					markLayer.convertMapToDB(upperLeft);
					markLayer.convertMapToDB(lowerRight);
				}

				SxRectangle		clickZone = new SxRectangle(upperLeft,lowerRight);

				SxQueryResultInterface 	stuff = null;
				
				stuff = mainSxMapView.getObjectsInGeometry(PAMSpatialObject.LAYER_AIR,clickZone);
				processDoubleClick(stuff.asVector());
				stuff = mainSxMapView.getObjectsInGeometry(PAMSpatialObject.LAYER_GROUND,clickZone);
				processDoubleClick(stuff.asVector());
				stuff = mainSxMapView.getObjectsInGeometry(PAMSpatialObject.LAYER_SURFACE,clickZone);
				processDoubleClick(stuff.asVector());
				stuff = mainSxMapView.getObjectsInGeometry(PAMSpatialObject.LAYER_SUBSURFACE,clickZone);
				processDoubleClick(stuff.asVector());
				stuff = mainSxMapView.getObjectsInGeometry(PAMSpatialObject.LAYER_SPACE,clickZone);
				processDoubleClick(stuff.asVector());
			}
		}
	
		private void processDoubleClick(Vector spatialObjects){
			if ((spatialObjects == null) || (spatialObjects.size() == 0)){
			} else {
				PAMSpatialObject	space = null;
				Frame objectDataFrame = buildFrame();
				
				for (Enumeration e = spatialObjects.elements(); e.hasMoreElements();) {
					space = (PAMSpatialObject)(e.nextElement());

					Product			prod = space.spatialObjProd;
					StringWriter	sw = new StringWriter();
					AsPlainText		renderer;
					Vector 			options = new Vector();

					// Build window to show off the contents of the spatialObject
					objectDataFrame.setTitle("Platform: " + prod.getId());

					//options.addElement("showMetaData");
					renderer = new AsPlainText(options);

					renderer.visit(prod);
					renderer.writeResults(new PrintWriter(sw));
					text.setText(sw.toString());

					objectDataFrame.show();
				}
			}
		}
		
	}

	/**
	* An input to the agent was removed
	*/
	public void aSourceWasRemoved(ConnectedAgent source){}
			
	/**
	* An input to the agent was added
	*/
	public void aSourceWasAdded(ConnectedAgent source){}

	/**
	* An output to the agent was removed
	*/
	public void aSinkWasRemoved(ConnectedAgent sink){}
			
	/**
	* An output to the agent was added
	*/
	public void aSinkWasAdded(ConnectedAgent sink){}
	
	/**
	* Notification that the instance is no longer being used
	*/
	public void youHaveBeenDiscarded(){
		if (mainFrame != null && getBooleanProperty(DISPOSE_WINDOW,true)) {
			mainFrame.setVisible(false);
			mainFrame.dispose();
			mainFrame = null;
		}
	}
	
	protected void finalize() {
		if (mainFrame != null) {
			mainFrame.setVisible(false);
			mainFrame.dispose();
			mainFrame = null;
		}
	}
}
