/* Copyright 1999 The MITRE Corporation. ALL RIGHTS RESERVED */

package org.mitre.pam.putter.mapdim;

import org.mitre.pam.interfaces.Product;

import java.awt.Graphics;
import java.awt.Color;

import com.ofx.mapViewer.SxRendererInterface;
import com.ofx.mapViewer.SxMapLayerInterface;
import com.ofx.mapViewer.SxProjection;

import com.ofx.repository.SxSpatialObject;
import com.ofx.repository.SxSymbology;

import com.ofx.geometry.SxGeometryInterface;
import com.ofx.geometry.SxDoublePoint;
import com.ofx.geometry.SxPoint;

/**
* @author Ayal Spitz.
* @author Nazario Irizarry, Jr.
*/
public class PAMSpatialObjectRenderer implements SxRendererInterface{
	public PAMSpatialObjectRenderer(){
	}
	
	public void setGeometry(SxGeometryInterface gi){
		geometry = gi;
	}
	
	
	// Map legend graphic
	public void display(Graphics g,int width,int height,double rotation, SxSymbology sym){
	}

	// Symbol on map
    public void display(Graphics g, double aScale, SxDoublePoint aTranslation,
						SxSymbology sd,SxSpatialObject so,SxMapLayerInterface aMli){
		SxDoublePoint		location = ((SxPoint)geometry).getLocation();
		int					x = (int)((location.x - aTranslation.x) / aScale);
		int					y = (int)((aTranslation.y - location.y) / aScale);
		
		int					s = 15;
		int					r = s / 2;
		Color				currColor = g.getColor();
		
		PAMSpatialObject	spatialObj = (PAMSpatialObject)so;
				
		if (spatialObj.friendFoe == PAMSpatialObject.FRIEND_FOE_FRIEND){
			int[]	x1 = { x-r,x  ,x+r,x  ,x-r};
			int[]	y1 = { y  ,y-r,y  ,y+r,y  };

			int[]	x2 = { x-r,x  ,x+r,x-r};
			int[]	y2 = { y  ,y-r,y  ,y  };
			
			int[]	x3 = { x-r,x+r,x  ,x-r};
			int[]	y3 = { y  ,y  ,y+r,y  };

			if ((spatialObj.layer == PAMSpatialObject.LAYER_AIR) || (spatialObj.layer == PAMSpatialObject.LAYER_AIR)){
				g.setColor(Color.white);
				g.fillPolygon(x3,y3,4);
				g.setColor(FRIEND_COLOR);
				g.fillPolygon(x2,y2,4);
				g.setColor(FRIEND_COLOR);
				g.drawPolygon(x1,y1,5);
			} else
			if (spatialObj.layer == PAMSpatialObject.LAYER_SUBSURFACE){
				g.setColor(Color.white);
				g.fillPolygon(x2,y2,4);
				g.setColor(FRIEND_COLOR);
				g.fillPolygon(x3,y3,4);
				g.setColor(FRIEND_COLOR);
				g.drawPolygon(x1,y1,5);
			} else {
				g.setColor(Color.white);
				g.fillPolygon(x1,y1,5);
				g.setColor(FRIEND_COLOR);
				g.drawPolygon(x1,y1,5);
			}
			
		} else
		if (spatialObj.friendFoe == PAMSpatialObject.FRIEND_FOE_FOE){
			if ((spatialObj.layer == PAMSpatialObject.LAYER_AIR) || (spatialObj.layer == PAMSpatialObject.LAYER_AIR)){
				g.setColor(Color.white);
				g.fillOval(x - r,y - r,s,s);
				g.setColor(FOE_COLOR);
				g.fillArc(x - r,y - r,s,s,0,180);
				g.setColor(FOE_COLOR);
				g.drawOval(x - r,y - r,s,s);
			} else
			if (spatialObj.layer == PAMSpatialObject.LAYER_SUBSURFACE){
				g.setColor(FOE_COLOR);
				g.fillOval(x - r,y - r,s,s);
				g.setColor(Color.white);
				g.fillArc(x - r,y - r,s,s,0,180);
				g.setColor(FOE_COLOR);
				g.drawOval(x - r,y - r,s,s);
			} else {
				g.setColor(Color.white);
				g.fillOval(x - r,y - r,s,s);
				g.setColor(FOE_COLOR);
				g.drawOval(x - r,y - r,s,s);
			} 
		} else {
			if ((spatialObj.layer == PAMSpatialObject.LAYER_AIR) || (spatialObj.layer == PAMSpatialObject.LAYER_AIR)){
				g.setColor(UNKNOWN_COLOR);
				g.fillRect(x - r,y - r,s,r);
				g.setColor(Color.white);
				g.fillRect(x - r,y,s,r);
				g.setColor(UNKNOWN_COLOR);
				g.drawRect(x - r,y - r,s,s);
			} else
			if (spatialObj.layer == PAMSpatialObject.LAYER_SUBSURFACE){
				g.setColor(UNKNOWN_COLOR);
				g.fillRect(x - r,y,s,r);
				g.setColor(Color.white);
				g.fillRect(x - r,y - r,s,r);
				g.setColor(UNKNOWN_COLOR);
				g.drawRect(x - r,y - r,s,s);
			} else {
				g.setColor(Color.white);
				g.fillRect(x - r,y - r,s,s);
				g.setColor(UNKNOWN_COLOR);
				g.drawRect(x - r,y - r,s,s);
			} 
		}
		

		if (spatialObj.hasCourse) {
			g.setColor(Color.black);
			g.drawLine( x,y,(int)(x+(s*Math.sin(spatialObj.course))),
						    (int)(y-(s*Math.cos(spatialObj.course))) );
		}

		
		g.setColor(currColor);
	}
	
	// Projection on the fly
	public void display(Graphics g, double aScale, SxDoublePoint aTranslation,
						SxSymbology sd, SxSpatialObject so,SxMapLayerInterface mli,
						SxProjection dbProjection, SxProjection mapProjection){
	}

	public void release(){
	}

	private SxGeometryInterface		geometry = null;
	
	private final static Color		FRIEND_COLOR 	= Color.green.darker();
	private final static Color		FOE_COLOR 		= Color.red.darker();	
	private final static Color		UNKNOWN_COLOR	= Color.yellow.darker();	
}
